// BasisView.cpp : implementation of the CBasisView class
//

#include "stdafx.h"
#include "Basis.h"
#include "BasisDoc.h"
#include "TViewingAgent.h"
#include "TVector.h"
#include "TDirection.h"
#include "BasisView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// CBasisView

IMPLEMENT_DYNCREATE(CBasisView, CEditView)

BEGIN_MESSAGE_MAP(CBasisView, CEditView)
	//{{AFX_MSG_MAP(CBasisView)
	ON_WM_TIMER()
	ON_WM_DESTROY()
	ON_COMMAND(ID_MSG_ACTIONS, OnMsgActions)
	ON_UPDATE_COMMAND_UI(ID_MSG_ACTIONS, OnUpdateMsgActions)
	ON_COMMAND(ID_MSG_BALL_POSITION, OnMsgBallPosition)
	ON_UPDATE_COMMAND_UI(ID_MSG_BALL_POSITION, OnUpdateMsgBallPosition)
	ON_COMMAND(ID_MSG_HIGHLEVEL, OnMsgHighlevel)
	ON_UPDATE_COMMAND_UI(ID_MSG_HIGHLEVEL, OnUpdateMsgHighlevel)
	ON_COMMAND(ID_MSG_PERCEPTION, OnMsgPerception)
	ON_UPDATE_COMMAND_UI(ID_MSG_PERCEPTION, OnUpdateMsgPerception)
	ON_COMMAND(ID_MSG_POSITION, OnMsgPosition)
	ON_UPDATE_COMMAND_UI(ID_MSG_POSITION, OnUpdateMsgPosition)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CEditView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CEditView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CEditView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBasisView construction/destruction

CBasisView::CBasisView()
{
	m_bNoGraphics = FALSE;
	m_bViewPerception = FALSE;
	m_bViewAction = TRUE;
	m_bViewHighlevelAction = TRUE;
	m_bViewPosition = FALSE;
	m_bViewBallPosition = FALSE;

	m_bTimerRunning = FALSE;
	m_iTimerID = 0;
	m_bAgentRunning = FALSE;
	m_iAgentTimerID = 0;
	m_iTimeoutTimerID = 0;
	m_bReceivedData = FALSE;
	m_iNoOfTries = 0;

#ifdef WRITE_MESSAGES_TO_DISK
	m_pTextFile = NULL;
	m_pTextArchive = NULL;
	m_pBinaryFile = NULL;
	m_pBinaryArchive = NULL;

	m_bViewPerception = TRUE;
	m_bViewAction = TRUE;
	m_bViewHighlevelAction = TRUE;
	m_bViewPosition = TRUE;
	m_bViewBallPosition = TRUE;

	// initialize the time counting for messages
	if (QueryPerformanceFrequency(&m_frequency))
		QueryPerformanceCounter(&m_startTime);
#endif
}

CBasisView::~CBasisView()
{
#ifdef WRITE_MESSAGES_TO_DISK

	// close the files
	if (m_pTextArchive)
	{
		m_pTextArchive->Close();
		delete m_pTextArchive;
	}

	if (m_pTextFile)
	{
		m_pTextFile->Close();
		delete m_pTextFile;
	}

	if (m_pBinaryArchive)
	{
		BYTE	type = TYPE_NO_MORE_DATA;
		*m_pBinaryArchive << type;
		m_pBinaryArchive->Close();
		delete m_pBinaryArchive;
	}

	if (m_pBinaryFile)
	{
		m_pBinaryFile->Close();
		delete m_pBinaryFile;
	}

#endif
}

BOOL CBasisView::PreCreateWindow(CREATESTRUCT& cs)
{
	BOOL ret = CEditView::PreCreateWindow(cs);
	cs.style = AFX_WS_DEFAULT_VIEW | WS_VSCROLL | WS_HSCROLL | ES_AUTOHSCROLL |
		ES_AUTOVSCROLL | ES_MULTILINE | ES_NOHIDESEL | ES_READONLY;
	return ret;
}

/////////////////////////////////////////////////////////////////////////////
// CBasisView drawing

void CBasisView::OnDraw(CDC* pDC)
{
	// TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////////////
// CBasisView printing

BOOL CBasisView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CBasisView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CBasisView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CBasisView diagnostics

#ifdef _DEBUG
void CBasisView::AssertValid() const
{
	CEditView::AssertValid();
}

void CBasisView::Dump(CDumpContext& dc) const
{
	CEditView::Dump(dc);
}

CBasisDoc* CBasisView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CBasisDoc)));
	return (CBasisDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CBasisView message handlers

void CBasisView::DisplayDebug(LPCTSTR lpszMessage, int category, int level)
{
	ASSERT((level < DEBUG_LEVEL_MAX) && (category < KATEGORY_MAX));

	switch(category)
	{
	case KATEGORY_ACTION:
		if (level == DEBUG_HIGHACTION)
		{
			if (!m_bViewHighlevelAction)
				return;
		}
		else if (!m_bViewAction)
			return;
		break;

	case DEBUG_PERCEPTION:
		if (!m_bViewPerception)
			return;
		break;
	}

	CString msg = lpszMessage;

#ifdef WRITE_MESSAGES_TO_DISK
	double time;

	if (QueryPerformanceCounter(&m_stopTime))
	{
		CString	theTime;
		time = (double)(m_stopTime.LowPart - m_startTime.LowPart) / (double) m_frequency.LowPart;
		theTime.Format("%4.3f: ", time);
		msg = theTime + msg;
	}

	if (level <= MAX_DEBUG_LEVEL)
	{
		BYTE	type = TYPE_DEBUG_MSG + level;
		BYTE	cat = category;
		int	time = m_pAgent->GetSenseTime();
		
		if (m_pBinaryArchive == NULL)
			OpenFiles();
		if (m_pBinaryArchive != NULL)
			*m_pBinaryArchive << type << time << cat << msg;

	}
#endif

	if (m_bNoGraphics)
		return;

	Message(msg);
}

void CBasisView::DisplayPosition(TVector& pos, int bodyDir, int dir, BOOL bSee)
{
	if (!m_bViewPosition)
		return;

	CString	temp;
	int		time = m_pAgent->GetSenseTime();
	double	x = pos.GetX();
	double	y = pos.GetY();

#ifdef WRITE_MESSAGES_TO_DISK
	// write the position of the agent machine readable
	BYTE	type = TYPE_AGENT_POSITION;
	if (m_pBinaryArchive != NULL)
		*m_pBinaryArchive << type << time << x << y << bodyDir;
	else
		OpenFiles();
#endif

	if (m_bNoGraphics)
		return;

	temp.Format("Agent Position: x: %4.2f y: %4.2f  body: %d face: %d seen:%d"
					, x, y, bodyDir, dir, bSee);
	Message(temp);
}

void CBasisView::DisplayBallPosition(TVector& pos, TVector& speed, BOOL bSee)
{
	if (!m_bViewBallPosition)
		return;

	CString	temp;
	int		time = m_pAgent->GetSenseTime();
	double	x = pos.GetX();
	double	y = pos.GetY();
	double	sx = speed.GetX();
	double	sy = speed.GetY();
	double	len = speed.GetLength();
	int		dir = (int) speed.GetDirection();

#ifdef WRITE_MESSAGES_TO_DISK
	// write the position of the agent machine readable
	BYTE	type = TYPE_BALL_POSITION;
	if (m_pBinaryArchive != NULL)
		*m_pBinaryArchive << type << time << x << y << sx << sy;
	else
		OpenFiles();
#endif

	if (m_bNoGraphics)
		return;

	temp.Format("Ball Position: x: %4.2f y: %4.2f  speed: %4.2f sdir:%d seen:%d"
					, x, y, len, dir, bSee);
	Message(temp);

}

void CBasisView::DisplayPlayerPosition(TVector& pos, int number, BOOL bSee)
{
#ifdef WRITE_MESSAGES_TO_DISK
	CString	temp;
	int		time = m_pAgent->GetSenseTime();
	double	x = pos.GetX();
	double	y = pos.GetY();
	BYTE		type = TYPE_PLAYER_POSITION;
	BYTE		id = number;

	// write the position of the agent machine readable
	if (m_pBinaryArchive != NULL)
		*m_pBinaryArchive << type << time << id << x << y;
	else
		OpenFiles();
#endif
}


void CBasisView::Erase()
{
	GetEditCtrl().SetSel(0, -1);
	GetEditCtrl().ReplaceSel("");
}

void CBasisView::Message(LPCTSTR lpszMessage)
{
	CString	strTemp;
	int		time = m_pAgent->GetSenseTime();
	
	strTemp.Format("%d: ", time);
	strTemp += lpszMessage;
	strTemp += _T("\r\n");

	// if the message is from network, it is appended
	// store the string
	int len = GetWindowTextLength();
	if (len > 30000)
		// erase the middle
		GetEditCtrl().SetSel(500,8000);
	else
		// append
		GetEditCtrl().SetSel(len,len);
	GetEditCtrl().ReplaceSel(strTemp);

#ifdef WRITE_MESSAGES_TO_DISK
/*
	// is it the first message ?
	if (m_pBinaryFile  == NULL)
		// open the files and archives. This is done here and not in the
		// constructor, because the filename contains the number of the agent
		// which is not know once the constructor is called
		OpenFiles();

	if (m_pTextArchive == NULL)
		return;

	m_pTextArchive->WriteString(strTemp);
*/
#endif
}

void CBasisView::OpenFiles()
{
#ifdef WRITE_MESSAGES_TO_DISK
	CString strFilename;
/*
	// open text file for logging
	strFilename.Format("Player%d.txt", m_pAgent->GetNumber());
	m_pTextFile = new CFile();
	if (!m_pTextFile->Open(strFilename, CFile::modeCreate | CFile::modeWrite))
	{
		delete m_pTextFile;
		m_pTextFile = NULL;
	}
	else
		m_pTextArchive = new CArchive(m_pTextFile, CArchive::store);
*/
	// open binary file for logging machine readable
	strFilename.Format("Player%d.dat", m_pAgent->GetNumber());
	m_pBinaryFile = new CFile();
	if (!m_pBinaryFile->Open(strFilename, CFile::modeCreate | CFile::modeWrite))
	{
		delete m_pBinaryFile;
		m_pBinaryFile = NULL;
	}
	else
		m_pBinaryArchive = new CArchive(m_pBinaryFile, CArchive::store);
#endif
}

void CBasisView::StartConnectionTimer()
{
	if (!m_bTimerRunning)
	{
		m_bTimerRunning = TRUE;
		m_iTimerID = CONNECTION_TIMER;
		m_iTimerID = SetTimer(m_iTimerID, CONNECTION_TIMEOUT * 1000, NULL);
	}
}

void CBasisView::StopConnectionTimer()
{
	if (m_bTimerRunning)
	{
		KillTimer(m_iTimerID);
		m_bTimerRunning = FALSE;
		m_iTimerID = 0;
	}
}
void CBasisView::StartTimeoutTimer()
{
	if (m_iTimeoutTimerID == 0)
	{
		m_iTimeoutTimerID = TIMEOUT_TIMER;
		m_iTimeoutTimerID = SetTimer(m_iTimeoutTimerID, SERVER_TIMEOUT, NULL);
	}
}

void CBasisView::StopTimeoutTimer()
{
	if (m_iTimeoutTimerID != 0)
	{
		KillTimer(m_iTimeoutTimerID);
		m_iTimeoutTimerID = 0;
	}
}

void CBasisView::OnDestroy() 
{
	CEditView::OnDestroy();
	
	StopConnectionTimer();
	StopAgentTimer();
	StopTimeoutTimer();
}


void CBasisView::StartAgentTimer()
{
	#ifndef SYNCHRONIZE_ON_SENSE_BODY
		if (!m_bAgentRunning)
		{
			// start the agent acting
			m_iAgentTimerID = AGENT_TIMER;
			m_iAgentTimerID = SetTimer(m_iAgentTimerID, 100, NULL);
			m_bAgentRunning = TRUE;
		}
	#else
		m_bAgentRunning = TRUE;
	#endif
}

void CBasisView::StopAgentTimer()
{
	if (m_bAgentRunning)
	{
		KillTimer(m_iAgentTimerID);
		m_iAgentTimerID = 0;
		m_bAgentRunning = FALSE;
	}
}

void CBasisView::OnTimer(UINT nIDEvent) 
{
	if ((int)nIDEvent == m_iAgentTimerID)
	{
		// it is time for the agent to do anything
		if (m_bAgentRunning)
			GetDocument()->OnActionTimer();
	}
	
	else if ((int)nIDEvent == m_iTimeoutTimerID)
	{
		if (!m_bReceivedData)
		{
			// no message from the soccerserver
			if (m_iNoOfTries > MAX_SOCKET_KICKS)
			{
				StopTimeoutTimer();
				GetDocument()->SetModifiedFlag(FALSE);
				theApp.m_pMainWnd->SendMessage(WM_CLOSE);
				return;
			}
			else
			{
				// due to a bug in WSA under NT, sockets can get stuck
				// a read seems to recover the socket
				CString message;
				GetDocument()->KickSocket();		
				message.Format("\r\n\r\n Socket stuck %d!!\r\n\r\n", m_iNoOfTries + 1);
				Message(message);
				m_iNoOfTries++;
			}
		}
		m_bReceivedData = FALSE;
	}

	else if ((int)nIDEvent == m_iTimerID)
	{
		// we got a timeout from connecting to server; tell the document
		StopConnectionTimer();
		GetDocument()->OnConnectionTimeout();
	}

	CEditView::OnTimer(nIDEvent);
}



void CBasisView::OnMsgActions() 
{
	m_bViewAction = !m_bViewAction;
}

void CBasisView::OnUpdateMsgActions(CCmdUI* pCmdUI) 
{
	if (m_bViewAction)
		pCmdUI->SetCheck(TRUE);
	else
		pCmdUI->SetCheck(FALSE);
}

void CBasisView::OnMsgBallPosition() 
{
	m_bViewBallPosition = !m_bViewBallPosition;
}

void CBasisView::OnUpdateMsgBallPosition(CCmdUI* pCmdUI) 
{
	if (m_bViewBallPosition)
		pCmdUI->SetCheck(TRUE);
	else
		pCmdUI->SetCheck(FALSE);
}

void CBasisView::OnMsgHighlevel() 
{
	m_bViewHighlevelAction = !m_bViewHighlevelAction;
}

void CBasisView::OnUpdateMsgHighlevel(CCmdUI* pCmdUI) 
{
	if (m_bViewHighlevelAction)
		pCmdUI->SetCheck(TRUE);
	else
		pCmdUI->SetCheck(FALSE);
}

void CBasisView::OnMsgPerception() 
{
	m_bViewPerception = !m_bViewPerception;
}

void CBasisView::OnUpdateMsgPerception(CCmdUI* pCmdUI) 
{
	if (m_bViewPerception)
		pCmdUI->SetCheck(TRUE);
	else
		pCmdUI->SetCheck(FALSE);
}

void CBasisView::OnMsgPosition() 
{
	m_bViewPosition = !m_bViewPosition;
}

void CBasisView::OnUpdateMsgPosition(CCmdUI* pCmdUI) 
{
	if (m_bViewPosition)
		pCmdUI->SetCheck(TRUE);
	else
		pCmdUI->SetCheck(FALSE);
}
