// BasisView.h : interface of the CBasisView class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_BASISVIEW_H__54CD01D1_4540_11D1_B242_0000C09B5CBE__INCLUDED_)
#define AFX_BASISVIEW_H__54CD01D1_4540_11D1_B242_0000C09B5CBE__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

// !!! should always be disabled for real games
#define MAX_DEBUG_LEVEL 6
/*
#ifdef _DEBUG
	#define WRITE_MESSAGES_TO_DISK
#endif
*/
#ifdef _RELEASE_WITH_LOG
	#define WRITE_MESSAGES_TO_DISK
#endif

#define CONNECTION_TIMEOUT	5
#define SERVER_TIMEOUT		150
#define MAX_SOCKET_KICKS	30

#define CONNECTION_TIMER	6
#define AGENT_TIMER			7
#define TIMEOUT_TIMER		8


#define TYPE_BALL_POSITION		11
#define TYPE_AGENT_POSITION	12
#define TYPE_NO_MORE_DATA		13
#define TYPE_PLAYER_POSITION	14
#define TYPE_DEBUG_MSG			100
// 100 - 110 are reserved for different debug levels
#define DEBUG_LEVEL_0			0
#define DEBUG_LEVEL_1			1
#define DEBUG_LEVEL_2			2
#define DEBUG_LEVEL_3			3
#define DEBUG_LEVEL_4			4
#define DEBUG_LEVEL_5			5
#define DEBUG_LEVEL_6			6
#define DEBUG_LEVEL_MAX			7

#define DEBUG_HIGHACTION	DEBUG_LEVEL_0
#define DEBUG_MIDACTION		DEBUG_LEVEL_1
#define DEBUG_LOWACTION		DEBUG_LEVEL_2
#define DEBUG_ACTION			DEBUG_LEVEL_3

#define DEBUG_HIGHPERCEPTION		DEBUG_LEVEL_0
#define DEBUG_MIDPERCEPTION		DEBUG_LEVEL_1
#define DEBUG_LOWPERCEPTION		DEBUG_LEVEL_2
#define DEBUG_PERCEPTION			DEBUG_LEVEL_3

#define KATEGORY_ACTION			0
#define KATEGORY_PERCEPTION	1
#define KATEGORY_MAX				2

class CBasisDoc;
class TViewingAgent;
class TVector;

class CBasisView : public CEditView
{
protected: // create from serialization only
	CBasisView();
	DECLARE_DYNCREATE(CBasisView)

// Attributes
public:
	BOOL	m_bNoGraphics;
	BOOL	m_bViewPerception;
	BOOL	m_bViewAction;
	BOOL	m_bViewHighlevelAction;
	BOOL	m_bViewPosition;
	BOOL	m_bViewBallPosition;

	BOOL	m_bTimerRunning;
	int	m_iTimerID;
	BOOL	m_bAgentRunning;
	int	m_iAgentTimerID;
	int	m_iTimeoutTimerID;
	BOOL	m_bReceivedData;
	int	m_iNoOfTries;
	TViewingAgent*	m_pAgent;

#ifdef WRITE_MESSAGES_TO_DISK
	CFile*		m_pTextFile;
	CArchive*	m_pTextArchive;
	CFile*		m_pBinaryFile;
	CArchive*	m_pBinaryArchive;

	// for debugging
	LARGE_INTEGER	m_frequency;
	LARGE_INTEGER	m_startTime;
	LARGE_INTEGER	m_stopTime;
#endif

// Operations
public:
	CBasisDoc*	GetDocument();

	void	DisplayDebug(LPCTSTR lpszMessage, int category, int level);
	void	DisplayPosition(TVector& pos, int bodyDir, int dir, BOOL bSee);
	void	DisplayBallPosition(TVector& pos, TVector& speed, BOOL bSee);
	void	DisplayPlayerPosition(TVector& pos, int number, BOOL bSee);

	void	Erase();
	void	OpenFiles();
	
	void	SetNoGraphics(BOOL val) { m_bNoGraphics = val; };
	void	StartConnectionTimer();
	void	StopConnectionTimer();
	void	StartAgentTimer();
	void	StopAgentTimer();
	void	StartTimeoutTimer();
	void	StopTimeoutTimer();
	BOOL	GetRunning() { return m_bAgentRunning; };
	void	SetReceivedData(BOOL m_bVal = TRUE) { m_bReceivedData = TRUE; m_iNoOfTries = 0; };
	void	SetAgent(TViewingAgent* pAgent) { m_pAgent = pAgent; };

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CBasisView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	protected:
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CBasisView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:
	void	Message(LPCTSTR lpszMessage);

// Generated message map functions
protected:
	//{{AFX_MSG(CBasisView)
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnDestroy();
	afx_msg void OnMsgActions();
	afx_msg void OnUpdateMsgActions(CCmdUI* pCmdUI);
	afx_msg void OnMsgBallPosition();
	afx_msg void OnUpdateMsgBallPosition(CCmdUI* pCmdUI);
	afx_msg void OnMsgHighlevel();
	afx_msg void OnUpdateMsgHighlevel(CCmdUI* pCmdUI);
	afx_msg void OnMsgPerception();
	afx_msg void OnUpdateMsgPerception(CCmdUI* pCmdUI);
	afx_msg void OnMsgPosition();
	afx_msg void OnUpdateMsgPosition(CCmdUI* pCmdUI);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

#ifndef _DEBUG  // debug version in SoccermonitorView.cpp
inline CBasisDoc* CBasisView::GetDocument()
   { return (CBasisDoc*)m_pDocument; }
#endif


//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_BASISVIEW_H__54CD01D1_4540_11D1_B242_0000C09B5CBE__INCLUDED_)
