// TBehaviorNetwork.h: interface for the TBehaviorNetwork class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TBehaviorNetwork_H__76B7A467_8821_11D1_B242_0000C09B5CBE__INCLUDED_)
#define AFX_TBehaviorNetwork_H__76B7A467_8821_11D1_B242_0000C09B5CBE__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#define MAX_CYCLES 100


class TAct;
class TPercept;
class TAction;
class TCompetence;
class TRuleView;
class TNetworkView;
class TScreenParamsNetwork;
class TNetworkParams;
class TParseString;
class TGoal;
class TEnergySpreader;

/*-----------------------------------------------------------------------------
Class   : TBehaviorNetwork
Date    : 08.01.98
-------------------------------------------------------------------------------
Base Class: CObject
Description: represents the knowledge of an agent containing the three levels
				 of cognition: 
				 cognitivly inpenetrable perception and action
				 competence modules
				 goals and goal management
-----------------------------------------------------------------------------*/

class DLL_CLASS TBehaviorNetwork : public CObject  
{
	public:
		TBehaviorNetwork(TNetworkView* pNView, TRuleView* pRView);
		virtual ~TBehaviorNetwork();

		// public methods
		virtual void	Initialize();
		virtual double	DoActivationSpreading();
		virtual void	InitializeActionSelection() {};
		virtual BOOL	DoActionSelection();
		virtual BOOL	ParseRules(LPCTSTR rules);
		virtual BOOL	ParseRulesWithVariables(LPCTSTR rules);
		virtual BOOL	ParseCompetenceRule(TCompetence* pCompetence, LPCTSTR rule);
		virtual void	DrawNetwork(CDC* pDC, TScreenParamsNetwork* pScreen);
		virtual void	SetNormCoordinates();
		virtual void	SetCoordinates(TScreenParamsNetwork* pScreen);
		virtual void	ParamsDialog();
		virtual void	ReduceThreshold();

		virtual void	UpdatePerception(BOOL bFromNetwork);
		virtual void	AddResource(LPCTSTR name, CObject* pObj, PCallBackPerception pFn, int iAmount) {};
		void				AddPerception(LPCTSTR name, CObject* pObj, PCallBackPerception pFn);
		void				AddAction(LPCTSTR name, CObject* pObj, PCallBackAction pFn);
		void				SetCombo(CComboBox* pCombo, BOOL bWhich);

		int				GetNoOfRules() const;
		CString			GetActionPerformedToString();
		CString			GetResults() const;
		CString			GetResultsInitial() const;
		void				SetThreshold(double val);
		TEnergySpreader* GetModuleFromCoordinates(CPoint point, TScreenParamsNetwork* pScreen);
		TNetworkParams* GetNetworkParams();
		void				SetUseFuzzyValues(BOOL val);
		BOOL				GetUseFuzzyValues() const; 
		void				SetFuzzyRound(double val);

		// for blocksworld
		BOOL	PerformCycle(TNetworkParams* params, int iNoOfCycles);
		void	ResetNetwork();
		BOOL	IsBlocksworld();

	protected:
		virtual TCompetence*	CreateNewCompetence();
		virtual TGoal*			CreateNewGoal();
		virtual void			CalculateSpreading();
		virtual double			GetActivationSum() const;

		void	DeleteNetwork();
		void	ConnectCompetence(TCompetence* newCompetence);
		void	DrawNetworkStructure(CDC* pDC, TScreenParamsNetwork* pScreen);
		void	DrawActionHistory(CDC* pDC, TScreenParamsNetwork* pScreen);
		
		// for blocksworld
		virtual BOOL	ParseGoals(TParseString &str);			
		BOOL	ParseObjects(TParseString &str);			
		BOOL	ParseAttributes(TParseString &str);			
		BOOL	ParseInitial(TParseString &str);			
		BOOL	ParseRule(TParseString &str);
		void	SortLinks();
		BOOL	CreatePropositions(int depth, LPCTSTR name);
		BOOL	CreateCompetences(int depth);
		BOOL	CheckGoalReached();

		TAct*				m_pAct;				// action factory

		TPercept*		m_pPercept;			// perception factory
		CObList*			m_pGoalList;
		CObList*			m_pCompetenceList;
		TRuleView*		m_pRulesView;
		TNetworkView*	m_pNetworkView;
		TNetworkParams* m_pNetworkParams;

		double			m_dThreshold;

		// for statistics

		TAction*			m_pLastAction;
		long				m_lNoOfActions;			// no of actions performed
		long				m_lNoOfActionChanges;	// no of changes in action selection
		double			m_dAvgNoOfExecutableModules;
		int				m_aiActionExecuted[MAX_CYCLES]; // history of executed actions

		// for blocksworld
		CStringList*	m_pObjects;		
		BOOL				m_bHaveConst;
		CString			m_strConst;
		CStringList		m_strlConditions;
		CStringList		m_strlEffects;
		CStringList		m_strlVariables;
		CStringList		m_strlObjects;
		CString			m_strAction;
		BOOL				m_bBlocksworld;

	private:
		
		void		InsertSorted(CObList& list, TCompetence* module);
};

#endif // !defined(AFX_TBehaviorNetwork_H__76B7A467_8821_11D1_B242_0000C09B5CBE__INCLUDED_)
