// TCompetence.cpp: implementation of the TCompetence class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "NetworkHeader.h"
#include "TConnection.h"
#include "TAct.h"
#include "TPercept.h"
#include "TAction.h"
#include "TWorldState.h"
#include "TProposition.h"
#include "TParseString.h"
#include "TScreenParamsNetwork.h"
#include "TWorldState.h"
#include "TGoal.h"
#include "TGoalTrack.h"
#include "TNetworkParams.h"
#include "TCompetence.h"

#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TCompetence::TCompetence(TNetworkParams* params, LPCTSTR name, int index)
					:TEnergySpreader(name, index)
{
	int	i;

	m_pNetworkParams = params;

	// create the member variables
	m_pPreconditions = new CObList(5);
	m_pEffects = new CObList(5);
	m_pAction = NULL;
	m_iNoOfSources = 0;
	
	// !!! vieleicht flexible anzahl links
	for (i = 0; i < NO_OF_CONNECTION_TYPES; i++)
		m_pLinks[i] = new CObList();
	
	m_dSpreadingActivation = 0.0;
	m_dTransferedActivation = 0.0;
	m_dExternActivation = 0.0;
	m_dExecutability = 0.0;

	m_bExecuted = FALSE;
}

TCompetence::~TCompetence()
{
	int	i;

	// delete all
	while (!m_pPreconditions->IsEmpty())
		delete m_pPreconditions->RemoveHead();
	delete m_pPreconditions;

	while (!m_pEffects->IsEmpty())
		delete m_pEffects->RemoveHead();
	delete m_pEffects;

	for (i = 0; i < NO_OF_CONNECTION_TYPES; i++)
	{
		while (!m_pLinks[i]->IsEmpty())
			delete m_pLinks[i]->RemoveHead();
		delete m_pLinks[i];
	}
}

/*-----------------------------------------------------------------------------
Function: TConnection* TCompetence::AddEnergySource(TProposition* srcProp, TProposition* destProp
														, int iSourceType)
Date    : 08.01.98
-------------------------------------------------------------------------------
Parameter: pointer to the new connection
			  pointer to the proposition of the source module
			  type of the link (nicht schoen)
Returnval: void
Description: creates a new connection to the passed energy source.
-----------------------------------------------------------------------------*/
void TCompetence::AddEnergySource(TConnection* newConnection, TProposition* srcProp, int iSourceType)
{
	// add it to the list of connections
	m_pLinks[iSourceType]->AddTail(newConnection);

	// inform the source, that it has been added to another destination
	srcProp->GetModule()->OnAddedToDestination();
	
	m_iNoOfSources++;
}

/*-----------------------------------------------------------------------------
Function: void TCompetence::ConnectCompetence(CObList* pCompetenceList, CObList* pGoalList)
Date    : 16.01.98
-------------------------------------------------------------------------------
Parameter: pointer to the newly created competence
Returnval: (void)
Description: connects this competence which means connect it to
					- its preconditions (perceptions)
					- its goals
					- successor competence modules
					- predecessor modules
					- conflictor modules
-----------------------------------------------------------------------------*/
void TCompetence::ConnectCompetence(CObList* pCompetenceList, CObList* pGoalList)
{
	POSITION				pos;
	POSITION				pos2;
	POSITION				pos3;
	CObList*				pPreconditions;
	CObList*				pEffects;
	TProposition*		pSource;
	TProposition*		pDest;
	TCompetence*		pCompetence;
	TGoal*				pGoal;
	BOOL					bInhibition;
	BOOL					bExcitation;

	// connections due to the preconditions of this module 
	// (proposition, predecessor, conflictor)
	pos = m_pPreconditions->GetHeadPosition();
	while (pos != NULL)
	{
		pDest = (TProposition*) m_pPreconditions->GetNext(pos);

		// add as energy source
		AddPropositionLink(pDest->GetWorldState()->GetProposition(), pDest);
		// tell the proposition, that it is within another modules preconditions
		pDest->GetWorldState()->OnAddedToPreconditions();
		
		// look for predecessor and conflictor links
		pos2 = pCompetenceList->GetHeadPosition();
		while (pos2 != NULL)
		{
			pCompetence = (TCompetence*) pCompetenceList->GetNext(pos2);
			pEffects = pCompetence->GetEffects();

			// go through the effects of the module
			pos3 = pEffects->GetHeadPosition();
			while (pos3 != NULL)
			{
				pSource = (TProposition*) pEffects->GetNext(pos3);
				if (pSource->IsIdentic(pDest))
				{
					// is it a predecessor
					AddPredecessor(pSource, pDest);
					pCompetence->AddSuccessor(pDest, pSource);
					// !!! this is not completely correct
					pDest->GetWorldState()->OnAddedToExcitation();
				}
				else if (pSource->IsInvers(pDest))
				{
					// the other module is a conflictor of the new one
					pCompetence->AddConflictor(pDest, pSource);
					// !!! this is not correct, because it might have been added for this 
					// proposition before
					pSource->GetWorldState()->OnAddedToInhibition();
				}
			}
		}
	}

	// connections due to the effects of this module 
	// (goals, protected goals, successors)
	pEffects = m_pEffects;
	pos = pEffects->GetHeadPosition();
	while (pos != NULL)
	{
		bInhibition = FALSE;
		bExcitation = FALSE;
		pDest = (TProposition*) pEffects->GetNext(pos);

		// look for successor and conflictor links
		pos2 = pCompetenceList->GetHeadPosition();
		while (pos2 != NULL)
		{
			pCompetence = (TCompetence*) pCompetenceList->GetNext(pos2);
			pPreconditions = pCompetence->GetPreconditions();

			// go through the preconditions of the module
			pos3 = pPreconditions->GetHeadPosition();
			while (pos3 != NULL)
			{
				pSource = (TProposition*) pPreconditions->GetNext(pos3);
				// is it a successor
				if (pSource->IsIdentic(pDest))
				{
					AddSuccessor(pSource, pDest);
					pCompetence->AddPredecessor(pDest, pSource);
					bExcitation = TRUE;
				}
				else if (pSource->IsInvers(pDest))
				{
					AddConflictor(pSource, pDest);
					bInhibition = TRUE;
				}
			}
		}

		// add goals, protected goals
		pos2 = pGoalList->GetHeadPosition();
		while (pos2 != NULL)
		{
			pGoal = (TGoal*) pGoalList->GetNext(pos2);
			pSource = pGoal->GetCondition();
			if (pSource->IsIdentic(pDest))
			{
				AddGoalLink(pSource, pDest);
				bExcitation = TRUE;
			}
			else if (pSource->IsInvers(pDest))
			{
				AddProtectedGoalLink(pSource, pDest);
				bInhibition = TRUE;
			}
		}

		if (bExcitation)
			// tell the proposition, that it is within another modules excitation links
			pDest->GetWorldState()->OnAddedToExcitation();

		if (bInhibition)
			// tell the proposition, that it is within another modules inhibition links
			pDest->GetWorldState()->OnAddedToInhibition();
	}
}


/*-----------------------------------------------------------------------------
Function: void TCompetence::DrawModule(CDC* pDC, TScreenParamsNetwork* pScreen)
Date    : 03.02.98
-------------------------------------------------------------------------------
Parameter: pointer to the device context
			  pointer to the screen parameters (translation, scaling, ...)
			  maximal reachable activation (for color scaling)
Returnval: (void)
Description: draws the connections of the competence module and the module itself
-----------------------------------------------------------------------------*/
void TCompetence::DrawModule(CDC* pDC, TScreenParamsNetwork* pScreen)
{
	POSITION		pos;
	int			i;

	// set the screen coordinates of this module
	SetCoordinates(pScreen);
	if ((m_tScreenCoord.x < 0) || (m_tScreenCoord.x > pScreen->GetSize().cx))
		return;
	if ((m_tScreenCoord.y < 0) || (m_tScreenCoord.y > pScreen->GetSize().cy))
		return;

	// draw all connections
	if (!pScreen->GetJustDrawSelectedConnections() || IsSelected())
	{
		for (i = 0; i < NO_OF_CONNECTION_TYPES; i++)
		{
			pos = m_pLinks[i]->GetHeadPosition();
			while (pos != NULL)
				((TConnection*) m_pLinks[i]->GetNext(pos))->DrawConnection(pDC, pScreen);
		}
	}

	// Draw the executability
	COLORREF cCol;
	CBrush	brush;
	CBrush	*oldBrush;
	int		size;

	// set the color red = active, blue = not active
	cCol = RGB((int) (m_dExecutability * 220.0), 0, 0);
	brush.CreateSolidBrush(cCol);
	oldBrush = pDC->SelectObject(&brush);

	// draw a Ellipse at the modules position
	size = (int) (MODULE_SIZE * pScreen->GetScale() * 1.5);
	pDC->Ellipse(m_tScreenCoord.x - size, m_tScreenCoord.y - size
					, m_tScreenCoord.x + size, m_tScreenCoord.y + size);

	pDC->SelectObject(oldBrush);
	brush.DeleteObject();


	// draw the module itself
	TEnergySpreader::DrawModule(pDC, pScreen);
}

COLORREF TCompetence::GetNameBackgroundColor()
{
	// highlight the executed module with
	// the more deciseiv, the more red the name's background
	if (m_bExecuted)
	{
		if (m_pAction == NULL)
			return RGB(255, 50, 50);
		else
			return RGB(50 + 200 * m_pAction->GetDecisiveness(), 50, 50);
	}
	else
		return TEnergySpreader::GetNameBackgroundColor();
}

/*-----------------------------------------------------------------------------
Function: double TCompetence::SetToNewActivation() 
Date    : 08.01.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: new activation of the module
Description: sets the activation to the previously calculated new activation
				 it is neccessary to first calculate all new activations of each
				 module and then set it to the new value, because otherwise the
				 algorithm would depend on the order of calculation
-----------------------------------------------------------------------------*/
double TCompetence::SetToNewActivation() 
{ 
	m_dTransferedActivation = Transfer(m_dSpreadingActivation);
	return (SetActivation(m_dSpreadingActivation));
}

/*-----------------------------------------------------------------------------
Function: BOOL TCompetence::CheckForExecution(double dThreshold, BOOL bBlocksworld)
Date    : 08.03.99
-------------------------------------------------------------------------------
Parameter: activation threshold
			  flag indicating if this is blocksworld mode (dirty!)
Returnval: (void)
Description: checks this module if it can be executed
-----------------------------------------------------------------------------*/
BOOL TCompetence::CheckForExecution(double dThreshold, BOOL bBlocksworld)
{
	double	activity;

	activity = GetActivationAndExecutability();
	if (activity > dThreshold)
	{
		// execute the action
		if (bBlocksworld)
			ExecuteBlocksworldModule();
		else
			ExecuteModule(activity);

		return TRUE;
	}
	else
		// module not executed
		return FALSE;
}

/*-----------------------------------------------------------------------------
Function: void TCompetence::ExecuteModule(double activation)
Date    : 16.01.98
-------------------------------------------------------------------------------
Parameter: activation*executability of the module
Returnval: (void)
Description: executes the action connected with this module
-----------------------------------------------------------------------------*/
void TCompetence::ExecuteModule(double activation)
{
	// execute action
	m_pAction->PerformAction(activation, m_pNetworkParams->GetPhi());
	m_bExecuted = TRUE;
}

/*-----------------------------------------------------------------------------
Function: void TCompetence::ParseRule(LPCTSTR rule)
Date    : 14.01.98
-------------------------------------------------------------------------------
Parameter: string containing the rule
			  pointer to the action factory of the network for access to the action function
			  pointer to the perception factory for access to the truth value functions
Returnval: (void)
Description: parses a rule string like
					if    klausi 1.0
					  and not susi 1.0
					then  heigaus
					effects 
					 	   rot 1.0
					  and not weg 0.9
-----------------------------------------------------------------------------*/
BOOL TCompetence::ParseRule(LPCTSTR rule, TAct* pActionFactory, TPercept* pPerceptionFactory)
{
	TParseString	parse(rule);
	CString			temp;
	CString			message;
	BOOL				bReady;
	double			value;
	BOOL				bNot;
	TProposition*	pProposition;
	TWorldState*	pState;
	BOOL				bFirstConjunct;

	// read the if
	if (!parse.ParseString(temp))
	{
		message.Format("No rule detected ! Rule ignored \r\n \r\n %s", rule);
		AfxMessageBox(message);
		return FALSE;
	}
	if (temp.CompareNoCase(STRING_IF) != 0)
	{
		message.Format("No if detected ! Rule ignored \r\n\r\n %s", rule);
		AfxMessageBox(message);
		return FALSE;
	}

	// read preconditions
	bReady = FALSE;
	bFirstConjunct = TRUE;
	while (!bReady)
	{
		// not or name
		bNot = FALSE;
		parse.ParseString(temp);
		if (temp.CompareNoCase(STRING_THEN) == 0)
			bReady = TRUE;
		else if (bFirstConjunct || (temp.CompareNoCase(STRING_AND) == 0))
		{
			// it is another precondition
			if (!bFirstConjunct)
				// overread and
				parse.ParseString(temp);
			
			if (temp.CompareNoCase(STRING_NOT) == 0)
			{
				bNot = TRUE;
				parse.ParseString(temp);
			}
			
			pState = GetState(temp, pPerceptionFactory);
			if (pState == NULL)
				return FALSE;

			// value 
			value = parse.ParseDouble();
			if (parse.HasErrorOccured())
				// value not written, so default is 1.0
				value = 1.0;

			// create the new precondition
			pProposition = new TProposition(this, pState, value, bNot);
			m_pPreconditions->AddTail(pProposition);
			bFirstConjunct = FALSE;
			pState->SetUsed();
		}
		else
			return FALSE;
	}

	// read action (the 'then' has been read above
	parse.ParseString(temp);
	m_pAction = pActionFactory->GetAction(temp);
	if (m_pAction == NULL)
	{
		message.Format("Action '%s' not existing! \r\n Rule ignored.", temp);
		AfxMessageBox(message);
		return FALSE;
	}
	SetName(temp);

	// read effects
	parse.ParseString(temp);
	if (temp.CompareNoCase(STRING_EFFECT) != 0)
	{
		message.Format("No effect detected ! Rule ignored \r\n\r\n %s", rule);
		AfxMessageBox(message);
		return FALSE;
	}

	bReady = FALSE;
	bFirstConjunct = TRUE;
	while (!bReady)
	{
		// not or name
		bNot = FALSE;
		parse.ParseString(temp);
		if (temp.CompareNoCase(STRING_END_IF) == 0)
			bReady = TRUE;
		else if (bFirstConjunct || (temp.CompareNoCase(STRING_AND) == 0))
		{
			// it is another effect
			if (!bFirstConjunct)
				// overread and
				parse.ParseString(temp);

			if (temp.CompareNoCase(STRING_NOT) == 0)
			{
				bNot = TRUE;
				parse.ParseString(temp);
			}
			
			pState = GetState(temp, pPerceptionFactory);
			if (pState == NULL)
				return FALSE;

			// value 
			value = parse.ParseDouble();
			if (parse.HasErrorOccured())
				// value not written, so default is 1.0
				value = 1.0;

			// create the new effect
			pProposition = new TProposition(this, pState, value, bNot);
			m_pEffects->AddTail(pProposition);

			bFirstConjunct = FALSE;
		}
	}
	return TRUE;
}

/*-----------------------------------------------------------------------------
Function: CString TCompetence::ToString()
Date    : 08.01.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: the connections of the module converted to a string
Description: converts the connections of the module to a rule string
-----------------------------------------------------------------------------*/
CString TCompetence::ToString()
{
	int					i;
	CString				ret;
	POSITION				pos;
	TProposition*		pSource;

	ret.Format("%s ", STRING_IF);

	// add all preconditions
	i = 0;
	pos = m_pPreconditions->GetHeadPosition();
	while (pos != NULL)
	{
		pSource = (TProposition*) m_pPreconditions->GetNext(pos);
		if (i == 0)
			ret += "     ";
		else
		{
			ret += " ";
			ret += STRING_AND;
			ret += " ";
		}
		ret += pSource->ToString();
		i++;
	}
	
	// add the action
	ret += m_pAction->GetName();

	// add the effects

	return ret;
}

/*-----------------------------------------------------------------------------
Function: CString TCompetence::GetResults() const
Date    : 30.06.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: string containing statistical results of this module
Description: returns statistical results of this module:
					- no the module has been executed
-----------------------------------------------------------------------------*/
CString TCompetence::GetResults() const
{
	CString	res;

	res.Format(" %5d", m_pAction->GetNoOfActionPerformed());
	return res;
}

CString TCompetence::GetResultsInitial() const
{
	CString	res;

	res.Format(" %s", m_pAction->GetName());
	return res;
}



// for blocksworld
/*-----------------------------------------------------------------------------
Function: void TCompetence::ExecuteBlocksworldModule()
Date    : 16.01.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: worldstate is changed to effects of this module
-----------------------------------------------------------------------------*/
void TCompetence::ExecuteBlocksworldModule()
{
	// run through all the effects and change the world state
	POSITION pos = m_pEffects->GetHeadPosition();
	while (pos != NULL)
		((TProposition*) m_pEffects->GetNext(pos))->SetWorldState();
	
	m_bExecuted = TRUE;
}

/*-----------------------------------------------------------------------------
Function: _
Date    : 23.10.98
-------------------------------------------------------------------------------
Parameter: _
Returnval: (void)
Description: instantiates the condition and effect variables with constants
-----------------------------------------------------------------------------*/
BOOL TCompetence::Instantiate(CStringList &objects, CStringList &variables, CStringList& conditions
									, CString action, CStringList& effects, TPercept* pPercept)
{
	POSITION			pos;
	TParseString	temp;
	BOOL				bNegated;
	TProposition*	pProposition;
	TWorldState*	pState;

	// instantiate the conditions
	pos = conditions.GetHeadPosition();
	while(pos != NULL)
	{
		temp = (TParseString) conditions.GetNext(pos);

		// create a new proposition
		bNegated = DecodeProposition(temp, objects, variables);
		pState = GetState(temp, pPercept);
		if (pState == NULL)
			return FALSE;

		pProposition = new TProposition(this, pState, 1.0, bNegated);
		m_pPreconditions->AddTail(pProposition);
		pState->SetUsed();
	}

	// action
	temp = (TParseString) action;
	DecodeProposition(temp, objects, variables);
	SetName(temp);

	// effects
	pos = effects.GetHeadPosition();
	while(pos != NULL)
	{
		temp = (TParseString) effects.GetNext(pos);

		// create a new proposition
		bNegated = DecodeProposition(temp, objects, variables);
		pState = GetState(temp, pPercept);
		if (pState == NULL)
			return FALSE;

		pProposition = new TProposition(this, pState, 1.0, bNegated);
		m_pEffects->AddTail(pProposition);
	}
	
	return TRUE;
}
									
/*-----------------------------------------------------------------------------
Function: BOOL TCompetence::DecodeNegated(CString &str)
Date    : 26.10.98
-------------------------------------------------------------------------------
Parameter: reference to the string to decode
Returnval: TRUE, if the string represents a negated proposition
Description: a negated proposition is encoded by a '0' at the end of the name,
				 positive propositions are encoded by a '1'
-----------------------------------------------------------------------------*/
BOOL TCompetence::DecodeNegated(CString &str)
{
	char	flag = str.GetAt(str.GetLength() - 1);

	str = str.Left(str.GetLength() - 1);
	if (flag == '0')
		return TRUE;
	else
		return FALSE;
}


/*-----------------------------------------------------------------------------
Function: int TCompetence::DecodeProposition(BOOL &bNegated, CString &prop, LPCTSTR src, LPCTSTR dest)
Date    : 23.10.98
-------------------------------------------------------------------------------
Parameter: reference to a flag indicating if proposition is negated
			  reference to the proposition's name
			  variable, that needs to be replaced
			  name, with which to replace
Returnval: negated or not negated
Description: decodes a propositions negation, index and instantiates the variables
-----------------------------------------------------------------------------*/
BOOL TCompetence::DecodeProposition(TParseString &prop, CStringList &objs, CStringList &vars)
{
	BOOL		bNegated;
	POSITION	pos1;
	POSITION	pos2;
	CString	src;
	CString	dest;

	ASSERT(objs.GetCount() == vars.GetCount());

	// decode negated, which is a tailing 0 for FALSE, 1 for TRUE
	bNegated = DecodeNegated(prop);
	
	prop.Replace('(', '-');
	prop.Replace(',', '-');

	// replace each occurence of the variable by the constant
	pos1 = objs.GetHeadPosition();
	pos2 = vars.GetHeadPosition();
	while (pos1 != NULL)
	{
		dest = objs.GetNext(pos1);
		src = vars.GetNext(pos2);

		while (prop.Find(src) >= 0)
			prop.Replace(src, dest);
	}
	
	return bNegated;
}

/*-----------------------------------------------------------------------------
Function: TWorldstate* TCompetence::GetState(LPCTSTR prop, TPercept* pPerceptionFactory)
Date    : 22.02.99
-------------------------------------------------------------------------------
Parameter: name of the proposition
			  pointer to the perception factory for access to the truth value functions
Returnval: pointer to the worldstate
Description: gets the world state and shows error
-----------------------------------------------------------------------------*/
TWorldState* TCompetence::GetState(LPCTSTR prop, TPercept* pPerceptionFactory)
{
	CString			message;
	TWorldState*	pState;
	
	// get the proposition
	pState = pPerceptionFactory->GetPerception(prop);
	if (pState == NULL)
	{
		message.Format("Proposition '%s' not existing! \r\n Rule ignored.", prop);
		AfxMessageBox(message);
	}
	return pState;
}

/*-----------------------------------------------------------------------------
Function: 
Date    : 23.10.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: calls each competence modules sort routine, to sort the links, so
				 that links to the same module are next to each other for drawing
				 using bubblesort
-----------------------------------------------------------------------------*/
void	TCompetence::SortLinks()
{
	int					i;
	POSITION				pos0;
	POSITION				pos1;
	POSITION				pos2;
	POSITION				pos3;
	TEnergySpreader	*pComp1;
	TEnergySpreader	*pComp2;
	TConnection			*pCon1;
	TConnection			*pCon2;

	// just neccessary for successors and conflictors
	for (i = SUCCESSOR_CONNECTION; i < NO_OF_CONNECTION_TYPES; i++)
	{
		pos1 = m_pLinks[i]->GetHeadPosition();
		while (pos1 != NULL)
		{
			pos0 = pos1;
			pCon1 = (TConnection*) m_pLinks[i]->GetNext(pos1);
			pComp1 = pCon1->GetSourceProp()->GetModule();
			pos2 = pos1;
			while (pos2 != NULL)
			{
				pos3 = pos2;
				pCon2 = (TConnection*) m_pLinks[i]->GetNext(pos2);
				pComp2 = pCon2->GetSourceProp()->GetModule();
				if (pComp1 == pComp2)
				{
					// we have the same source module, so remove link from list
					// at set it behind the other
					m_pLinks[i]->RemoveAt(pos3);
					m_pLinks[i]->InsertAfter(pos0, pCon2);
				}
			}
		}
	}
}

