// TConnection.h: interface for the TConnection class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TCONNECTION_H__76B7A462_8821_11D1_B242_0000C09B5CBE__INCLUDED_)
#define AFX_TCONNECTION_H__76B7A462_8821_11D1_B242_0000C09B5CBE__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000


class TProposition;
class TScreenParamsNetwork;
class TEnergySpreader;
class TNetworkParams;

/*-----------------------------------------------------------------------------
Class   : TConnection
Date    : 08.01.98
-------------------------------------------------------------------------------
Base Class: CObject

Description: class describing an energy connection between two modules. The
				 TEnergySpreader object is the source of the energy, the object
				 containing this connection is the destination.
				 May be it is more efficient to have separate classes for conflictor
				 connections, not connections.
-----------------------------------------------------------------------------*/

class TConnection : public CObject  
{
	public:
		TConnection();
		TConnection(TNetworkParams* params, TProposition* src, TProposition *dest);
		virtual ~TConnection();

		// energy functions
		virtual void		DrawConnection(CDC* pDC, TScreenParamsNetwork* pScreen);
		virtual double		GetActivation() { return m_dActivation; };
		virtual BOOL		IsDrawn(TScreenParamsNetwork* pScreen) { return TRUE; };
		virtual void		SetMaxActivation() { m_dMaxActivation = 1.0; };
		virtual void		DrawLink(CDC* pDC, TScreenParamsNetwork* pScreen) { DrawExternalLink(pDC); };
		virtual void		DoCreatePen(CPen& pen, COLORREF cCol) { pen.CreatePen(PS_SOLID, 1, cCol); };
		virtual	int		GetBending();

		TEnergySpreader*	GetSourceModule();
		TEnergySpreader*	GetDestinationModule();
		TProposition*		GetSourceProp() { return m_pSourceProp; };
		TProposition*		GetDestinationProp() { return m_pDestinationProp; };
		void					SetNetworkParams(TNetworkParams* params) { m_pNetworkParams = params; };

	protected:
		void		DrawExternalLink(CDC* pDC);
		void		DrawInternalLink(CDC* pDC, TScreenParamsNetwork* pScreen);
		void		DrawConnectionActivations(CDC* pDC, TScreenParamsNetwork* pScreen
													,CPoint& start, CPoint& end, int bending, BOOL bLeftToRight);

		TProposition*		m_pSourceProp;			// proposition of the energy source

		TProposition*		m_pDestinationProp;	// proposition of the destination module 
		double				m_dActivation;
		double				m_dMaxActivation;
		TNetworkParams*	m_pNetworkParams;
};


#endif // !defined(AFX_TCONNECTION_H__76B7A462_8821_11D1_B242_0000C09B5CBE__INCLUDED_)
