// TDorer2Competence.cpp: implementation of the TDorer2Competence class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "NetworkHeader.h"
#include "TParseString.h"
#include "TScreenParamsNetwork.h"
#include "TProposition.h"
#include "TWorldState.h"
#include "TResourceProposition.h"
#include "TResource.h"
#include "TResourceFactory.h"
#include "TGoal.h"
#include "TAct.h"
#include "TPercept.h"
#include "TNetworkParams.h"
#include "TDorer2GoalTrack.h"
#include "TDorer2Connection.h"
#include "TDorer2Competence.h"

#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TDorer2Competence::TDorer2Competence(TNetworkParams* params
											 , LPCTSTR name, int index, int iNoOfGoals)
					:TCompetence(params, name, index)
{
	m_dSigmaCompetence = m_pNetworkParams->GetSigma();
	m_dGainCompetence = m_pNetworkParams->GetGain();
	
	#ifdef NO_GOALTRACKING
		m_pGoalTrack = new TDorer2NoGoalTrack(iNoOfGoals);
	#else
		m_pGoalTrack = new TDorer2GoalTrack(iNoOfGoals);
	#endif
	m_pResources = new CObList();
	m_pResourceLinks = new CObList();
	m_dSumTaup = 0.0;
}

TDorer2Competence::~TDorer2Competence()
{
	delete m_pGoalTrack;

	// remove all resource links
	while (!m_pResourceLinks->IsEmpty())
		delete m_pResourceLinks->RemoveHead();
	delete m_pResourceLinks;

	// remove all resource propositions
	while (!m_pResources->IsEmpty())
		delete m_pResources->RemoveHead();
	delete m_pResources;
}

/*-----------------------------------------------------------------------------
Function: void TDorer2Competence::AddPropositionLink(TProposition* source, TProposition* dest)
Date    : 29.01.98
-------------------------------------------------------------------------------
Parameter: pointer to the proposition that is a precondition of this module
			  pointer to the destination proposition
Returnval: (void)
Description: adds an energysource that is a situation proposition
-----------------------------------------------------------------------------*/
void TDorer2Competence::AddPropositionLink(TProposition* source, TProposition* dest)
{
	// create a new connection
	TConnection* newConnection = new TDorer2SituationLink(m_pNetworkParams, source, dest);
	AddEnergySource(newConnection, source, SITUATION_CONNECTION);
}

/*-----------------------------------------------------------------------------
Function: void TDorer2Competence::AddGoalLink(TProposition* source, TProposition* dest)
Date    : 29.01.98
-------------------------------------------------------------------------------
Parameter: pointer to the source proposition
			  pointer to the destination proposition
Returnval: (void)
Description: adds an energysource that is a situation proposition
-----------------------------------------------------------------------------*/
void TDorer2Competence::AddGoalLink(TProposition* source, TProposition* dest)
{
	// create a new connection
	TConnection* newConnection = new TDorer2GoalLink(m_pNetworkParams, source, dest);
	AddEnergySource(newConnection, source, GOAL_CONNECTION);
}

/*-----------------------------------------------------------------------------
Function: void TDorer2Competence::AddProtectedGoalLink(TProposition* source, TProposition* dest)
Date    : 29.01.98
-------------------------------------------------------------------------------
Parameter: pointer to the source proposition
			  pointer to the destination proposition
Returnval: (void)
Description: adds an energysource that is a situation proposition
-----------------------------------------------------------------------------*/
void TDorer2Competence::AddProtectedGoalLink(TProposition* source, TProposition* dest)
{
	// create a new connection
	TConnection* newConnection = new TDorer2ProtectedGoalLink(m_pNetworkParams, source, dest);
	AddEnergySource(newConnection, source, PROTECTED_GOAL_CONNECTION);
}

/*-----------------------------------------------------------------------------
Function: void TDorer2Competence::AddSuccessor(TProposition* source, TProposition* dest)
Date    : 29.01.98
-------------------------------------------------------------------------------
Parameter: pointer to the source proposition
			  pointer to the destination proposition
Returnval: (void)
Description: adds an energysource that is a situation proposition
-----------------------------------------------------------------------------*/
void TDorer2Competence::AddSuccessor(TProposition* source, TProposition* dest)
{
	// create a new connection
	TConnection* newConnection = new TDorer2SuccessorLink(m_pNetworkParams, source, dest);
	AddEnergySource(newConnection, source, SUCCESSOR_CONNECTION);
}

/*-----------------------------------------------------------------------------
Function: void TDorer2Competence::AddPredecessor(TProposition* source, TProposition* dest)
Date    : 29.01.98
-------------------------------------------------------------------------------
Parameter: pointer to the source proposition
			  pointer to the destination proposition
Returnval: (void)
Description: adds an energysource that is a situation proposition
-----------------------------------------------------------------------------*/
void TDorer2Competence::AddPredecessor(TProposition* source, TProposition* dest)
{
	// create a new connection
//	TConnection* newConnection = new TDorer2PredecessorLink(m_pNetworkParams, source, dest);
//	AddEnergySource(newConnection, source, PREDECESSOR_CONNECTION);
}

/*-----------------------------------------------------------------------------
Function: void TDorer2Competence::AddConflictor(TProposition* source, TProposition* dest)
Date    : 29.01.98
-------------------------------------------------------------------------------
Parameter: pointer to the proposition (effect) of the passed module
			  pointer to the proposition (precondition) of this module
Returnval: (void)
Description: adds an energysource of which this module is a conflictor
-----------------------------------------------------------------------------*/
void TDorer2Competence::AddConflictor(TProposition* source, TProposition* dest)
{
	// add the module
	TConnection* newConnection = new TDorer2ConflictorLink(m_pNetworkParams, source, dest);
	AddEnergySource(newConnection, source, CONFLICTOR_CONNECTION);
}

/*-----------------------------------------------------------------------------
Function: void TDorer2Competence::AddPropositionLink(TProposition* source, TProposition* dest)
Date    : 29.01.98
-------------------------------------------------------------------------------
Parameter: pointer to the proposition that is a precondition of this module
			  pointer to the destination proposition
Returnval: (void)
Description: adds an energysource that is a situation proposition
-----------------------------------------------------------------------------*/
void TDorer2Competence::AddResourceLink(TProposition* source, TProposition* dest)
{
	// create a new connection
	TConnection* newConnection = new TResourceLink(m_pNetworkParams, source, dest);
	m_pResourceLinks->AddTail(newConnection);
}

/*-----------------------------------------------------------------------------
Function: void TDorer2Competence::DrawModule(CDC* pDC, TScreenParamsNetwork* pScreen)
Date    : 03.02.98
-------------------------------------------------------------------------------
Parameter: pointer to the device context
			  pointer to the screen parameters (translation, scaling, ...)
			  maximal reachable activation (for color scaling)
Returnval: (void)
Description: draws the connections of the competence module and the module itself
-----------------------------------------------------------------------------*/
void TDorer2Competence::DrawModule(CDC* pDC, TScreenParamsNetwork* pScreen)
{
	POSITION			pos;
	TConnection*	pCon;

	// draw the module itself
	TCompetence::DrawModule(pDC, pScreen);

	// draw resource links
	pos = m_pResourceLinks->GetHeadPosition();
	while (pos != NULL)
	{
		pCon = (TConnection*) m_pResourceLinks->GetNext(pos);
		if (!pScreen->GetJustDrawSelectedConnections() || IsSelected())
			pCon->DrawConnection(pDC, pScreen);
	}

	// draw goaltracking information
	if (pScreen->GetDrawGoalTracking())
		m_pGoalTrack->Draw(pDC, pScreen, m_tScreenCoord);
}

/*-----------------------------------------------------------------------------
Function: void TDorer2Competence::CalculateActivation(int iNoOfModules, double dSumOfActivation)
Date    : 08.01.98
-------------------------------------------------------------------------------
Parameter: pointer to the class containing the network parameters such as phi, gamma, ...
Returnval: (void)
Description: first the old activation is decayed
				 calculates the activation of this module. This is done by summing
				 up the energy of each incoming connection to this module according
				 to Maes. 
-----------------------------------------------------------------------------*/
void TDorer2Competence::CalculateActivation(int iNoOfModules, double dSumOfActivation)
{
	double			dPreviousActivation;

	// previous activation
	dPreviousActivation = CalculatePreviousActivation();

	// extern activation 
	CalculateExternActivation();

	// spreading activation
	CalculateSpreadingActivation();

	// and sum them up
	m_dSpreadingActivation = dPreviousActivation + m_pGoalTrack->GetSumActivation();
}

/*-----------------------------------------------------------------------------
Function: double TDorer2Competence::CalculatePreviousActivation()
Date    : 15.07.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: activation of the module, that's source is the previous activation
			  of the module
Description: does the decay of activation
-----------------------------------------------------------------------------*/
double TDorer2Competence::CalculatePreviousActivation()
{
	return m_dMainActivation * m_pNetworkParams->GetBeta();
}

/*-----------------------------------------------------------------------------
Function: void TDorer2Competence::CalculateExternActivation()
Date    : 15.07.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: void
Description: receives the extern activation from goals and does goaltracking
-----------------------------------------------------------------------------*/
void TDorer2Competence::CalculateExternActivation()
{
	POSITION					pos;
	TConnection*			pCon;
	int						i;
	double					activation;

	// situation connections (!!! only needed for drawing)
	pos = m_pLinks[SITUATION_CONNECTION]->GetHeadPosition();
	while (pos != NULL)
	{
		// add the activation of this kind of link
		pCon = (TConnection*) m_pLinks[SITUATION_CONNECTION]->GetNext(pos);
		activation = pCon->GetActivation();
	}

	// goal connections
	pos = m_pLinks[GOAL_CONNECTION]->GetHeadPosition();
	while (pos != NULL)
	{
		// add the activation of this kind of link
		pCon = (TConnection*) m_pLinks[GOAL_CONNECTION]->GetNext(pos);
		activation = pCon->GetActivation();
		i = pCon->GetSourceModule()->GetIndex();
		m_pGoalTrack->SetActivation(i, activation);
	}

	// protected goal connections
	pos = m_pLinks[PROTECTED_GOAL_CONNECTION]->GetHeadPosition();
	while (pos != NULL)
	{
		// add the activation of this kind of link
		pCon = (TConnection*) m_pLinks[PROTECTED_GOAL_CONNECTION]->GetNext(pos);
		activation = pCon->GetActivation();
		i = pCon->GetSourceModule()->GetIndex();
		m_pGoalTrack->SetActivation(i, activation);
	}
}

/*-----------------------------------------------------------------------------
Function: double TDorer2Competence::CalculateSpreadingActivation()
Date    : 28.01.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: activation spread to this module by other modules
			  if goal tracking is on, the influence of direct goals may be within this
Description: calculates the new activation that is gained by activation spreading
				 among the modules
-----------------------------------------------------------------------------*/
void TDorer2Competence::CalculateSpreadingActivation()
{

#ifdef ACTIVATION_FAN

#define MAX_COMPETENCES	50
#define MAX_GOALS			10
	
	POSITION					pos;
	TConnection*			pCon;
	int						i, j;
	int						iNoOfGoals;
	double					dGoal[MAX_GOALS][MAX_COMPETENCES]; // MAX_GOALS
	int						iCompetence;

	// successors
	iNoOfGoals = m_pGoalTrack->GetNoOfGoals();
	for ( i = 0; i < iNoOfGoals; i++)
		for ( j = 0; j < MAX_COMPETENCES; j++)
			dGoal[i][j] = 0.0;

	pos = m_pLinks[SUCCESSOR_CONNECTION]->GetHeadPosition();
	while (pos != NULL)
	{
		pCon = (TDorer2SuccessorLink*) m_pLinks[SUCCESSOR_CONNECTION]->GetNext(pos);
		iCompetence = pCon->GetSourceModule()->GetIndex();
		ASSERT((iCompetence >= 0) && (iCompetence < MAX_COMPETENCES));
		for ( i = 0; i < iNoOfGoals; i++)
			dGoal[i][iCompetence] += ((TDorer2SuccessorLink*) pCon)->GetGoalActivation(i);
	}

	// conflictors
	pos = m_pLinks[CONFLICTOR_CONNECTION]->GetHeadPosition();
	while (pos != NULL)
	{
		pCon = (TDorer2ConflictorLink*) m_pLinks[CONFLICTOR_CONNECTION]->GetNext(pos);
		iCompetence = pCon->GetSourceModule()->GetIndex();
		ASSERT((iCompetence >= 0) && (iCompetence < MAX_COMPETENCES));
		for ( i = 0; i < iNoOfGoals; i++)
		{
			dGoal[i][iCompetence] += ((TDorer2ConflictorLink*) pCon)->GetGoalActivation(i);
		}
	}

	for ( i = 0; i < iNoOfGoals; i++)
		for ( j = 0; j < MAX_COMPETENCES; j++)
			m_pGoalTrack->SetActivation(i, dGoal[i][j]);

#else
	
	POSITION					pos;
	TConnection*			pCon;
	int						i;
	int						iNoOfGoals;

	// successors
	iNoOfGoals = m_pGoalTrack->GetNoOfGoals();
	pos = m_pLinks[SUCCESSOR_CONNECTION]->GetHeadPosition();
	while (pos != NULL)
	{
		pCon = (TDorer2SuccessorLink*) m_pLinks[SUCCESSOR_CONNECTION]->GetNext(pos);
		for ( i = 0; i < iNoOfGoals; i++)
			m_pGoalTrack->SetActivation(i, ((TDorer2SuccessorLink*) pCon)->GetGoalActivation(i));
	}

	// conflictors
	pos = m_pLinks[CONFLICTOR_CONNECTION]->GetHeadPosition();
	while (pos != NULL)
	{
		pCon = (TDorer2ConflictorLink*) m_pLinks[CONFLICTOR_CONNECTION]->GetNext(pos);
		for ( i = 0; i < iNoOfGoals; i++)
			m_pGoalTrack->SetActivation(i, ((TDorer2ConflictorLink*) pCon)->GetGoalActivation(i));
	}
#endif
}
/*-----------------------------------------------------------------------------
Function: void TDorerCompetence::CalculateExecutability()
Date    : 08.01.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: calculates the executability of this module. This is done by multiplying
				 the value of each precondition. The result is powered to 1/number of 
				 preconditions
-----------------------------------------------------------------------------*/
void TDorer2Competence::CalculateExecutability()
{
	POSITION		pos;
	int			n;
	double		value;

	n = 0;
	m_dExecutability = 1.0;
	m_dSumTaup = 0.0;
	pos = m_pPreconditions->GetHeadPosition();
	// step through each precondition
	while (pos != NULL)
	{
		value = ((TProposition*) m_pPreconditions->GetNext(pos))->GetValue();
		#ifdef FUZZY_MIN_AND
			// this version uses min as fuzzy and	
			if (value < m_dExecutability)
				m_dExecutability = value;
		#else 
			// this version uses product as fuzzy min
			m_dExecutability *= value;
		#endif
		n++;

		m_dSumTaup += 1.0 - value;
	}

	// m_dSumTaup = max (1, Sum(taup))
	if (m_dSumTaup < 1.0)
		m_dSumTaup = 1.0;

	// the executability is powered by 1 / number of preconditions
	#ifdef FUZZY_SQRT_AND
		if (n > 2)
			m_dExecutability = pow(m_dExecutability, 1.0 / (double) n);
		else if (n == 2)
			m_dExecutability = sqrt(m_dExecutability);
	#endif
}

/*-----------------------------------------------------------------------------
Function: double TDorerCompetence::Transfer(double dVal)
Date    : 21.04.98
-------------------------------------------------------------------------------
Parameter: activation of a module
Returnval: sigmoid of the value
Description: calculates the transfer function of a modules activity (the activity
				 that is spread). Current transfer function is a sigmoid function
				 (see Goetz 1997)
-----------------------------------------------------------------------------*/
double TDorer2Competence::Transfer(double dVal)
{
#ifdef NO_TRANSFER_FUNCTION
	return dVal;
#else
	// 1 / 1 + e ^ (m * (s - x))
	return 1.0 / (1.0 + exp(m_dGainCompetence * (m_dSigmaCompetence - dVal)));
#endif
}

/*-----------------------------------------------------------------------------
Function: void TDorer2Competence::SetToNewActivation()
Date    : 23.11.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: sets the new activation of this module after completion of the
				 activation spreading
-----------------------------------------------------------------------------*/
double TDorer2Competence::SetToNewActivation()
{
	m_pGoalTrack->SetToNewActivation();
	return	TCompetence::SetToNewActivation();
}

/*-----------------------------------------------------------------------------
Function: double TDorerCompetence::GetActivationAndExecutability()
Date    : 18.11.98
-------------------------------------------------------------------------------
Parameter: activation of the module
Returnval: (void)
Description: _
-----------------------------------------------------------------------------*/
double TDorer2Competence::GetActivationAndExecutability()
{
	// Dorer uses Product
	return m_dExecutability * GetActivation();
}

double TDorer2Competence::GetTransferedActivation(int iGoalIndex)
{
	return Transfer(m_pGoalTrack->GetActivation(iGoalIndex));
}

/*-----------------------------------------------------------------------------
Function: BOOL TDorer2Competence::CheckForExecution(double dThreshold, BOOL bBlocksworld)
Date    : 08.03.99
-------------------------------------------------------------------------------
Parameter: activation threshold
			  flag indicating if this is blocksworld mode (dirty!)
Returnval: (void)
Description: checks this module if it can be executed
-----------------------------------------------------------------------------*/
BOOL TDorer2Competence::CheckForExecution(double dThreshold, BOOL bBlocksworld)
{
	double				activity;
	POSITION				pos;
	TResourceLink*		pResourceLink;
	TResource*			pResource;
	int					iAmountUsed;

	activity = GetActivationAndExecutability();

	// check if activity surpasses all resources thresholds
	pos = m_pResourceLinks->GetHeadPosition();
	while (pos != NULL)
	{
		pResourceLink = (TResourceLink*) m_pResourceLinks->GetNext(pos);
		pResource = (TResource*) pResourceLink->GetSourceModule();

		// check activity against resource threshold
		if (activity <= pResource->GetActivation())
			return FALSE;

		// check if enough resource units are available
		iAmountUsed = ((TResourceProposition*) pResourceLink->GetDestinationProp())->GetAmountUsed();
		if (iAmountUsed > pResource->GetAmountAvailable())
			return FALSE;
	}

	// found no resource conflicts, execute the action
	if (bBlocksworld)
	{	
		if (activity < dThreshold)
			return FALSE;
		ExecuteBlocksworldModule();
	}
//	else if (activity < 0.0001)
	else if (activity < 0.01)
		// check if a module without using resources is at least executable
		return FALSE;
	else
		ExecuteModule(activity);

	// update resources
	pos = m_pResourceLinks->GetHeadPosition();
	while (pos != NULL)
	{
		pResourceLink = (TResourceLink*) m_pResourceLinks->GetNext(pos);
		pResource = (TResource*) pResourceLink->GetSourceModule();
	
		// reduce resource
		iAmountUsed = ((TResourceProposition*) pResourceLink->GetDestinationProp())->GetAmountUsed();
		pResource->ReduceAmountAvailable(iAmountUsed);
		pResource->ResetActivationLevel(dThreshold);
	}

	return TRUE;
}

/*-----------------------------------------------------------------------------
Function: void TDorer2Competence::UpdateResourceUsage()
Date    : 16.04.99
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: releases the resources used by this module, if it has been executed
-----------------------------------------------------------------------------*/
void TDorer2Competence::UpdateResourceUsage()
{
	POSITION			pos;
	TResourceLink*	pLink;
	int				iUsage;

	if (GetExecuted())
	{
		// this module has been executed in the previous cycle, so release resources
		pos = m_pResourceLinks->GetHeadPosition();
		while (pos != NULL)
		{
			pLink = (TResourceLink*) m_pResourceLinks->GetNext(pos);
			iUsage = ((TResourceProposition*) pLink->GetDestinationProp())->GetAmountUsed();
			((TResource*) pLink->GetSourceModule())->IncreaseAmountAvailable(iUsage);
		}
		
		SetExecuted(FALSE);
	}
}

/*-----------------------------------------------------------------------------
Function: BOOL TDorer2Competence::ParseRule(LPCTSTR rule, TAct* pActionFactory
											 , TPercept* pPerceptionFactory, TResourceFactory* pResourceFactory)
Date    : 14.01.98
-------------------------------------------------------------------------------
Parameter: string containing the rule
			  pointer to the action factory of the network for access to the action function
			  pointer to the perception factory for access to the truth value functions
			  pointer to the resource factory	
Returnval: (void)
Description: parses a rule string like
					if    klausi 1.0
					  and not susi 1.0
					then  heigaus
					effects 
					 	   rot 1.0
					  and not weg 0.9
					using resource 1
					endif
				 resources have been added in dorer2
-----------------------------------------------------------------------------*/
BOOL TDorer2Competence::ParseRule(LPCTSTR rule, TAct* pActionFactory
											 , TPercept* pPerceptionFactory, TResourceFactory* pResourceFactory)
{
	TParseString	parse(rule);
	CString			temp;
	CString			message;
	BOOL				bReady;
	double			value;
	int				iAmount;
	BOOL				bNot;
	TProposition*	pProposition;
	BOOL				bFirstConjunct;
	BOOL				bUsing = FALSE;
	TWorldState*	pState;
	TResource*		pResource;
	TResourceProposition* pResourceProp;

	// read the if
	if (!parse.ParseString(temp))
	{
		message.Format("No rule detected ! Rule ignored \r\n \r\n %s", rule);
		AfxMessageBox(message);
		return FALSE;
	}
	if (temp.CompareNoCase(STRING_IF) != 0)
	{
		message.Format("No if detected ! Rule ignored \r\n\r\n %s", rule);
		AfxMessageBox(message);
		return FALSE;
	}

	// read preconditions
	bReady = FALSE;
	bFirstConjunct = TRUE;
	while (!bReady)
	{
		// not or name
		bNot = FALSE;
		parse.ParseString(temp);
		if (temp.CompareNoCase(STRING_THEN) == 0)
			bReady = TRUE;
		else if (bFirstConjunct || (temp.CompareNoCase(STRING_AND) == 0))
		{
			// it is another precondition
			if (!bFirstConjunct)
				// overread and
				parse.ParseString(temp);
			
			if (temp.CompareNoCase(STRING_NOT) == 0)
			{
				bNot = TRUE;
				parse.ParseString(temp);
			}
			
			pState = GetState(temp, pPerceptionFactory);
			if (pState == NULL)
				return FALSE;

			// value 
			value = parse.ParseDouble();
			if (parse.HasErrorOccured())
				// value not written, so default is 1.0
				value = 1.0;

			// create the new precondition
			pProposition = new TProposition(this, pState, value, bNot);
			m_pPreconditions->AddTail(pProposition);
			bFirstConjunct = FALSE;
			pState->SetUsed();
		}
		else
			return FALSE;
	}

	// read action (the 'then' has been read above
	parse.ParseString(temp);
	m_pAction = pActionFactory->GetAction(temp);
	if (m_pAction == NULL)
	{
		message.Format("Action '%s' not existing! \r\n Rule ignored.", temp);
		AfxMessageBox(message);
		return FALSE;
	}
	SetName(temp);

	// read effects
	parse.ParseString(temp);
	if (temp.CompareNoCase(STRING_EFFECT) != 0)
	{
		message.Format("No effect detected ! Rule ignored \r\n\r\n %s", rule);
		AfxMessageBox(message);
		return FALSE;
	}

	bReady = FALSE;
	bFirstConjunct = TRUE;
	while (!bReady)
	{
		// not or name
		bNot = FALSE;
		if (!parse.ParseString(temp))
			return FALSE;

		if (temp.CompareNoCase(STRING_USING) == 0)
		{
			bReady = TRUE;
			bUsing = TRUE;
		}
		else if (temp.CompareNoCase(STRING_END_IF) == 0)
			bReady = TRUE;
		else if (bFirstConjunct || (temp.CompareNoCase(STRING_AND) == 0))
		{
			// it is another effect
			if (!bFirstConjunct)
				// overread and
				parse.ParseString(temp);

			if (temp.CompareNoCase(STRING_NOT) == 0)
			{
				bNot = TRUE;
				parse.ParseString(temp);
			}
			
			pState = GetState(temp, pPerceptionFactory);
			if (pState == NULL)
				return FALSE;

			// value 
			value = parse.ParseDouble();
			if (parse.HasErrorOccured())
				// value not written, so default is 1.0
				value = 1.0;

			// create the new effect
			pProposition = new TProposition(this, pState, value, bNot);
			m_pEffects->AddTail(pProposition);

			bFirstConjunct = FALSE;
		}
	}

	// parse ressources
	if (bUsing)
	{
		bReady = FALSE;
		bFirstConjunct = TRUE;
		while (!bReady)
		{
			if (!parse.ParseString(temp))
				return FALSE;

			if (temp.CompareNoCase(STRING_END_IF) == 0)
				bReady = TRUE;
			else if (bFirstConjunct || (temp.CompareNoCase(STRING_AND) == 0))
			{
				// it is another effect
				if (!bFirstConjunct)
					// overread and
					parse.ParseString(temp);

				// read the amount
				iAmount = parse.ParseInteger();
				if (parse.HasErrorOccured())
					// value not written, so default is 1
					iAmount = 1;

				// store the new resource
				pResource = (TResource*) GetState(temp, pResourceFactory);
				if (pResource == NULL)
					return FALSE;
				pResourceProp = new TResourceProposition(this, pResource, iAmount);
				m_pResources->AddTail(pResourceProp);
				pResource->SetUsed();

				bFirstConjunct = FALSE;
			}
		}
	}
	
	return TRUE;
}

/*-----------------------------------------------------------------------------
Function: void TDorer2Competence::ConnectCompetence(CObList* pCompetenceList, CObList* pGoalList)
Date    : 08.03.99
-------------------------------------------------------------------------------
Parameter: behavior networks list of competences
			  behavior networks list of goals
Returnval: (void)
Description: creates a connection between each resource proposition and resource object
-----------------------------------------------------------------------------*/
void TDorer2Competence::ConnectCompetence(CObList* pCompetenceList, CObList* pGoalList)
{
	POSITION						pos;
	TResourceProposition*	pProp;

	TCompetence::ConnectCompetence(pCompetenceList, pGoalList);

	// connect the module to the resources
	pos = m_pResources->GetHeadPosition();
	while (pos != NULL)
	{
		pProp = (TResourceProposition*) m_pResources->GetNext(pos);

		// add resource link
		AddResourceLink(pProp->GetWorldState()->GetProposition(), pProp);
	}
}