// TConnection.cpp: implementation of the TConnection class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "NetworkHeader.h"
#include "TEnergySpreader.h"
#include "TProposition.h"
#include "TScreenParamsNetwork.h"
#include "TNetworkParams.h"
#include "TDorer2Competence.h"
#include "TDorer2Connection.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//--------------------------------------------------------------------
// Dorer
//--------------------------------------------------------------------

//////////////////////////////////////////////////////////////////////
// TDorerSituationLink
//////////////////////////////////////////////////////////////////////

TDorer2SituationLink::TDorer2SituationLink(TNetworkParams* params, TProposition* src, TProposition* dest)
						: TConnection(params, src, dest)
{
}

double TDorer2SituationLink::GetActivation() 
{ 
	double			dProposition;

	ASSERT(m_pDestinationProp != NULL);

	dProposition = m_pDestinationProp->GetValue();
	return m_dActivation = dProposition;
}

BOOL TDorer2SituationLink::IsDrawn(TScreenParamsNetwork* pScreen)
{
	return pScreen->GetDrawPredecessors();
}

void TDorer2SituationLink::SetMaxActivation()
{
	m_dMaxActivation = 1.0;
}

//////////////////////////////////////////////////////////////////////
// TDorer2GoalLink
//////////////////////////////////////////////////////////////////////

TDorer2GoalLink::TDorer2GoalLink(TNetworkParams* params, TProposition* src, TProposition* dest)
						: TConnection(params, src, dest)
{
}

double TDorer2GoalLink::GetActivation() 
{ 
	ASSERT(m_pSourceProp != NULL);
	ASSERT(m_pDestinationProp != NULL);

	double dProposition;

#ifdef ACTIVATION_FAN
	dProposition = (1.0 - m_pSourceProp->GetValue());
#else
	dProposition = 1.0;
#endif

	m_dActivation = m_pNetworkParams->GetGamma() 
						* (m_pSourceProp->GetModule()->GetActivation()
						* dProposition
						* m_pDestinationProp->GetWeigh());
	return m_dActivation;
}

BOOL TDorer2GoalLink::IsDrawn(TScreenParamsNetwork* pScreen)
{
	return pScreen->GetDrawSuccessors();
}

void  TDorer2GoalLink::SetMaxActivation()
{
	m_dMaxActivation = m_pNetworkParams->GetGamma();
}

//////////////////////////////////////////////////////////////////////
// TDorer2ProtectedGoalLink
//////////////////////////////////////////////////////////////////////

TDorer2ProtectedGoalLink::TDorer2ProtectedGoalLink(TNetworkParams* params, TProposition* src, TProposition* dest)
							: TConnection(params, src, dest)
{
}

double TDorer2ProtectedGoalLink::GetActivation() 
{ 
	ASSERT(m_pSourceProp != NULL);
	ASSERT(m_pDestinationProp != NULL);

	double dProposition;

#ifdef ACTIVATION_FAN
	dProposition = m_pSourceProp->GetValue();
#else
	dProposition = 1.0;
#endif

	m_dActivation = -m_pNetworkParams->GetDelta() 
						* (m_pSourceProp->GetModule()->GetActivation() 
						* dProposition
						* m_pDestinationProp->GetWeigh());
	return m_dActivation;
}

BOOL TDorer2ProtectedGoalLink::IsDrawn(TScreenParamsNetwork* pScreen)
{
	return pScreen->GetDrawConflictors();
}

void  TDorer2ProtectedGoalLink::SetMaxActivation()
{
	m_dMaxActivation = m_pNetworkParams->GetDelta();
}

//////////////////////////////////////////////////////////////////////
// TDorer2SuccessorLink
//////////////////////////////////////////////////////////////////////

TDorer2SuccessorLink::TDorer2SuccessorLink(TNetworkParams* params, TProposition* src, TProposition* dest)
							: TConnection(params, src, dest)
{
}

double TDorer2SuccessorLink::GetActivation() 
{ 
	return m_dActivation;
}

double TDorer2SuccessorLink::GetGoalActivation(int iGoalIndex) 
{
	double dActivation;
	double dProposition;

	ASSERT(m_pSourceProp != NULL);
	ASSERT(m_pDestinationProp != NULL);

	dActivation = ((TDorer2Competence*) m_pSourceProp->GetModule())->GetTransferedActivation(iGoalIndex);
	
#ifdef ACTIVATION_FAN
	dProposition = ((TDorer2Competence*) m_pSourceProp->GetModule())->GetSumTaup();
	ASSERT(dProposition >= 1.0);
	dProposition = (1.0 - m_pSourceProp->GetValue()) / dProposition;
#else
	dProposition = (1.0 - m_pSourceProp->GetValue());
#endif

	// difference to Dorer: not gamma square but gamma
	dActivation = m_pNetworkParams->GetGamma() 
						* dProposition
						* dActivation
						* m_pDestinationProp->GetWeigh();
	if (iGoalIndex == 0)
		m_dActivation = dActivation;
	else
		m_dActivation += dActivation;
	
	return dActivation;
}

BOOL TDorer2SuccessorLink::IsDrawn(TScreenParamsNetwork* pScreen)
{
	return pScreen->GetDrawSuccessors();
}

void TDorer2SuccessorLink::SetMaxActivation()
{
	m_dMaxActivation = m_pNetworkParams->GetGammaSquare();
}

//////////////////////////////////////////////////////////////////////
// TDorer2ConflictorLink
//////////////////////////////////////////////////////////////////////

TDorer2ConflictorLink::TDorer2ConflictorLink(TNetworkParams* params, TProposition* src, TProposition* dest)
							: TConnection(params, src, dest)
{
}

double TDorer2ConflictorLink::GetActivation() 
{
	return m_dActivation;
}

double TDorer2ConflictorLink::GetGoalActivation(int iGoalIndex) 
{ 
	double dProposition;
	double dActivation;

	ASSERT(m_pSourceProp != NULL);
	ASSERT(m_pDestinationProp != NULL);

	dActivation = ((TDorer2Competence*) m_pSourceProp->GetModule())->GetTransferedActivation(iGoalIndex);

	// if the precondition is false anyway, there is no inhibition: m_pSourceProp->GetValue()

#ifdef ACTIVATION_FAN
	dProposition = ((TDorer2Competence*) m_pSourceProp->GetModule())->GetSumTaup();
	ASSERT(dProposition >= 1.0);
	dProposition = m_pSourceProp->GetValue() / dProposition;
#else
	dProposition = m_pSourceProp->GetValue();
#endif


	// difference to Dorer: not delta by gamma but only delta
	dActivation = -m_pNetworkParams->GetDelta() 
					* dProposition 
					* dActivation
					* m_pDestinationProp->GetWeigh();
	if (iGoalIndex == 0)
		m_dActivation = dActivation;
	else
		m_dActivation += dActivation;
	
	return dActivation;
}

BOOL TDorer2ConflictorLink::IsDrawn(TScreenParamsNetwork* pScreen)
{
	return pScreen->GetDrawConflictors();
}

void TDorer2ConflictorLink::SetMaxActivation()
{
	m_dMaxActivation = m_pNetworkParams->GetDeltaTimesGamma();
}

//////////////////////////////////////////////////////////////////////
// TResourceLink
//////////////////////////////////////////////////////////////////////

TResourceLink::TResourceLink(TNetworkParams* params, TProposition* src, TProposition* dest)
							: TConnection(params, src, dest)
{
}

BOOL TResourceLink::IsDrawn(TScreenParamsNetwork* pScreen)
{
	return pScreen->GetDrawResourceLinks();
}