// TDorer2Goal.cpp: implementation of the TDorer2Goal class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "NetworkHeader.h"
#include "TProposition.h"
#include "TPercept.h"
#include "TParseString.h"
#include "TScreenParamsNetwork.h"
#include "TWorldState.h"
#include "TDorer2Network.h"
#include "TNetworkParams.h"
#include "TDorer2Goal.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TDorer2Goal::TDorer2Goal() 
{
	m_pRelevanceList = new CObList();
	m_pEffectsList = new CObList();
	SetRelevance(1.0);
	m_pNetwork = NULL;
}

TDorer2Goal::~TDorer2Goal()
{
	while (!m_pRelevanceList->IsEmpty())
		delete m_pRelevanceList->RemoveHead();
	delete m_pRelevanceList;

	while (!m_pEffectsList->IsEmpty())
		delete m_pEffectsList->RemoveHead();
	delete m_pEffectsList;
}


/*-----------------------------------------------------------------------------
Function: BOOL TDorer2Goal::ParseGoal(LPCTSTR goal, TPercept* pPerceptionFactory)
Date    : 29.04.98
-------------------------------------------------------------------------------
Parameter: string containing the goal (Goal name importance Endgoal)
Returnval: (void)
Description: parses a goalstring and sets the parameters of the goal
-----------------------------------------------------------------------------*/
BOOL TDorer2Goal::ParseGoal(LPCTSTR goal, TPercept* pPerceptionFactory)
{	
	TParseString	parse(goal);
	CString			temp;
	CString			message;
	BOOL				bReady;
	double			value;
	BOOL				bNot;
	TProposition*	pProposition;
	TWorldState*	pState;
	BOOL				bFirstConjunct;

	// read the goal
	if (!parse.ParseString(temp))
	{
		message.Format("No goal detected ! Goal ignored \r\n \r\n %s", goal);
		AfxMessageBox(message);
		return FALSE;
	}
	if (parse.Find(STRING_THEN) < 0)
		// it is an oldfashioned goal with no relevance conditions (and no then)
		return ParseGoalName(parse, pPerceptionFactory);

	// read preconditions
	bReady = FALSE;
	bFirstConjunct = TRUE;
	while (!bReady)
	{
		// not or name
		bNot = FALSE;
		parse.ParseString(temp);
		if (temp.CompareNoCase(STRING_THEN) == 0)
			bReady = TRUE;
		else if (bFirstConjunct || (temp.CompareNoCase(STRING_AND) == 0))
		{
			// it is another precondition
			if (!bFirstConjunct)
				// overread and
				parse.ParseString(temp);
			
			if (temp.CompareNoCase(STRING_NOT) == 0)
			{
				bNot = TRUE;
				parse.ParseString(temp);
			}
			pState = pPerceptionFactory->GetPerception(temp);
			if (pState == NULL)
			{
				message.Format("Proposition '%s' not existing! \r\n Goal ignored.", temp);
				AfxMessageBox(message);
				return FALSE;
			}
			// value 
			value = parse.ParseDouble();
			if (parse.HasErrorOccured())
				// value not written, so default is 1.0
				value = 1.0;

			// create the new precondition
			pProposition = new TProposition(this, pState, value, bNot);
			m_pRelevanceList->AddTail(pProposition);
			bFirstConjunct = FALSE;
			pState->SetUsed();
		}
		else
			return FALSE;
	}

	// read goalname (the 'then' has been read above)
	if (!ParseGoalName(parse, pPerceptionFactory))
		return FALSE;

	// read effects
	parse.ParseString(temp);
	if (temp.CompareNoCase(STRING_EFFECT) != 0)
	{
		message.Format("No effect detected ! Rule ignored \r\n\r\n %s", goal);
		AfxMessageBox(message);
		return FALSE;
	}

	bReady = FALSE;
	bFirstConjunct = TRUE;
	while (!bReady)
	{
		// not or name
		bNot = FALSE;
		parse.ParseString(temp);
		if (temp.CompareNoCase(STRING_END_GOAL) == 0)
			bReady = TRUE;
		else if (bFirstConjunct || (temp.CompareNoCase(STRING_AND) == 0))
		{
			// it is another effect
			if (!bFirstConjunct)
				// overread and
				parse.ParseString(temp);

			if (temp.CompareNoCase(STRING_NOT) == 0)
			{
				bNot = TRUE;
				parse.ParseString(temp);
			}
			pState = pPerceptionFactory->GetPerception(temp);
			if (pState == NULL)
			{
				message.Format("Proposition '%s' not existing! \r\n Rule ignored.", temp);
				AfxMessageBox(message);
				return FALSE;
			}
			// value 
			value = parse.ParseDouble();
			if (parse.HasErrorOccured())
				// value not written, so default is 1.0
				value = 1.0;

			// create the new effect
			pProposition = new TProposition(this, pState, value, bNot);
			m_pEffectsList->AddTail(pProposition);

			bFirstConjunct = FALSE;
		}
	}
	return TRUE;
}

/*-----------------------------------------------------------------------------
Function: void TDorer2Goal::CalculateSituationRelevance()
Date    : 13.11.98
-------------------------------------------------------------------------------
Parameter: true, if not using fuzzy
			  if false, dRound indicates at which threshold to round fuzzy to 0 or 1
Returnval: (void)
Description: updates the relevance of this goal. The relevance is a function
				 of the fullfilment of the relevance conditions of the goal
-----------------------------------------------------------------------------*/
void TDorer2Goal::CalculateSituationRelevance()
{
	POSITION			pos;
	TProposition*	pProp;

	m_dRelevance = 1.0;
	pos = m_pRelevanceList->GetHeadPosition();
	while (pos != NULL)
	{
		pProp = (TProposition*) m_pRelevanceList->GetNext(pos);
		// !!! this is another place where Fuzzy and is used. Dirty !
		m_dRelevance *= (double) pProp->GetValue();
	}
}

/*-----------------------------------------------------------------------------
Function: BOOL TDorer2Goal::AddRelevanceCondition(LPCTSTR name, BOOL bNegated)
Date    : 03.02.99
-------------------------------------------------------------------------------
Parameter: name of the relevance
			  flag for state of negation
Returnval: (void)
Description: adds a relevance condition to the condition list
-----------------------------------------------------------------------------*/
BOOL TDorer2Goal::AddRelevanceCondition(LPCTSTR name, BOOL bNegated, TPercept* pPerceptionFactory)
{
	CString			message;
	CString			strRelevance = name;
	TProposition*	pProposition;
	TWorldState*	pState;

	// lookup relevance proposition (need not be there)
	if (!strRelevance.IsEmpty())
	{
		pState = pPerceptionFactory->GetPerception(strRelevance);
		if (pState == NULL)
		{
			message.Format("Proposition '%s' not existing! \r\n Goal ignored.", strRelevance);
			AfxMessageBox(message);
			return FALSE;
		}
		// create the new relevance condition
		pProposition = new TProposition(this, pState, 1.0, bNegated);
		m_pRelevanceList->AddTail(pProposition);
		pState->SetUsed();
	}
	
	return TRUE;
}


/*-----------------------------------------------------------------------------
Function: void TDorer2Goal::DrawModule(CDC* pDC, TScreenParamsNetwork* pScreen)
Date    : 03.02.98
-------------------------------------------------------------------------------
Parameter: pointer to the device context
			  pointer to the screen parameters (translation, scaling, ...)
			  maximal reachable activation (for color scaling)
Returnval: (void)
Description: draws the relevance of the goal and the goal's importance
-----------------------------------------------------------------------------*/
void TDorer2Goal::DrawModule(CDC* pDC, TScreenParamsNetwork* pScreen)
{
	// set the screen coordinates of this module
//	SetCoordinates(pScreen);
	if ((m_tScreenCoord.x < 0) || (m_tScreenCoord.x > pScreen->GetSize().cx))
		return;
	if ((m_tScreenCoord.y < 0) || (m_tScreenCoord.y > pScreen->GetSize().cy))
		return;

	// Draw the relevance
	COLORREF cCol;
	CBrush	brush;
	CBrush	*oldBrush;
	int		size;

	// set the color red = active, blue = not active
	cCol = RGB((int) (m_dRelevance * 220.0), 0, 0);
	brush.CreateSolidBrush(cCol);
	oldBrush = pDC->SelectObject(&brush);

	// draw a Ellipse at the modules position
	size = (int) (MODULE_SIZE * pScreen->GetScale() * 1.5);
	pDC->Ellipse(m_tScreenCoord.x - size, m_tScreenCoord.y - size
					, m_tScreenCoord.x + size, m_tScreenCoord.y + size);

	pDC->SelectObject(oldBrush);
	brush.DeleteObject();

	// draw the module itself
	TEnergySpreader::DrawModule(pDC, pScreen);
}


/*-----------------------------------------------------------------------------
Function: 
Date    : 09.11.99
-------------------------------------------------------------------------------
Parameter: none
Returnval: value of the goal as function of relevance and importance
Description: _
-----------------------------------------------------------------------------*/
double TDorer2Goal::GetActivation()
{
	#ifdef VALUE_FUNCTION_RISIKOWAHL
		// risky choice model
		// phi is used as risk parameter: 0.0 - very conservative, 1.0 - very risky
		if (m_dImportance < 0.0)
			return -pow(-m_dImportance, 2.0 * m_pNetwork->GetNetworkParams()->GetPhi()); 
		else
			return pow(m_dImportance, 2.0 * m_pNetwork->GetNetworkParams()->GetPhi()); 
	#else
		// standard value function is product
		return m_dImportance * m_dRelevance;
	#endif
}