// TDorer2GoalTrack.cpp: implementation of the TDorer2GoalTrack class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TScreenParamsNetwork.h"
#include "TDorer2GoalTrack.h"

#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TDorer2GoalTrack::TDorer2GoalTrack(int iNoOfGoals)
{
	m_iNoOfGoals = iNoOfGoals;
	m_pdActivation = new double[iNoOfGoals];
	m_pdNewActivation = new double[iNoOfGoals];
	Initialize();
}


TDorer2GoalTrack::~TDorer2GoalTrack()
{
	delete [] m_pdNewActivation;
	delete [] m_pdActivation;
}

/*-----------------------------------------------------------------------------
Function: void TDorer2GoalTrack::Initialize()
Date    : 23.11.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: _
-----------------------------------------------------------------------------*/
void TDorer2GoalTrack::Initialize()
{
	int	i;

	for (i = 0; i < m_iNoOfGoals; i++)
	{
		m_pdActivation[i] = 0.0;
		m_pdNewActivation[i] = 0.0;
	}
}

/*-----------------------------------------------------------------------------
Function: void TDorer2GoalTrack::SetToNewActivation()
Date    : 19.11.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: newActivation is used as a buffer to store the new incoming
				 activation. After all competence modules calculated their activation, 
				 current activation is set to new one and new one is reset
-----------------------------------------------------------------------------*/
void TDorer2GoalTrack::SetToNewActivation()
{
	int	i;

	for (i = 0; i < m_iNoOfGoals; i++)
	{
		m_pdActivation[i] = m_pdNewActivation[i];
		m_pdNewActivation[i] = 0.0;
	}
}

/*-----------------------------------------------------------------------------
Function: void TDorer2GoalTrack::SetActivation(int i, double val)
Date    : 08.07.98
-------------------------------------------------------------------------------
Parameter: index of the goal
			  value for the activation
Returnval: (void)
Description: if abs of new value is larger than old  value, the old value is replaced
-----------------------------------------------------------------------------*/
void TDorer2GoalTrack::SetActivation(int i, double val)
{
	ASSERT ((i >= 0) && (i < m_iNoOfGoals)); 
	// take maximum value
	if (fabs(val) > fabs(m_pdNewActivation[i]))
		m_pdNewActivation[i] = val;
}

/*-----------------------------------------------------------------------------
Function: void TDorer2GoalTrack::GetSumActivation()
Date    : 23.11.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: sums up the activations of each goal
-----------------------------------------------------------------------------*/
double TDorer2GoalTrack::GetSumActivation() const
{
	int		i;
	double	dSum = 0.0;

	for (i = 0; i < m_iNoOfGoals; i++)
		dSum += m_pdActivation[i];

	return dSum;
}

/*-----------------------------------------------------------------------------
Function: void TDorer2GoalTrack::Draw(CDC* pDC, TScreenParamsNetwork* pScreen, CPoint& coords)
Date    : 08.07.98
-------------------------------------------------------------------------------
Parameter: device context
			  screnn parameters
			  coordinates of the center of the module
Returnval: (void)
Description: draws the influcence of each goal to this module
-----------------------------------------------------------------------------*/
void TDorer2GoalTrack::Draw(CDC* pDC, TScreenParamsNetwork* pScreen, CPoint& coords)
{
	int		i;
	int		nPointSize;
	CFont		font;
	CFont*	oldFont;
	CRect		field;
	COLORREF	oldTextColor;
	COLORREF	oldBkColor;
	int		sizeX;
	CString	strActivation;
	int		x = coords.x;
	int		y = coords.y + 3;

	// create the font
	nPointSize = (int) (30.0 * pScreen->GetScale());
	if (!font.CreatePointFont(nPointSize, _T("Times New Roman"), pDC))
		font.CreateStockObject(SYSTEM_FONT);
	oldFont = pDC->SelectObject(&font);

	// calculate the rectangle of the text
	oldBkColor = pDC->SetBkColor(0x00ffffff);
	oldTextColor = pDC->SetTextColor(0x00000000);
	sizeX = (int) pScreen->GetScale() * 5 * nPointSize / 20;

	for (i = 0; i < m_iNoOfGoals; i++)
	{
		// write each goal
		y += nPointSize / 7 + 2;
		field.SetRect(x, y, x + sizeX, y + nPointSize / 7);
		strActivation.Format("%4.3f", m_pdActivation[i]);
		pDC->DrawText(strActivation, field, DT_SINGLELINE | DT_VCENTER);
	}

	pDC->SetTextColor(oldTextColor);
	pDC->SetBkColor(oldBkColor);

	pDC->SelectObject(oldFont);
	font.DeleteObject();
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TDorer2NoGoalTrack::TDorer2NoGoalTrack(int iNoOfGoals)
{
	m_dActivation = 0.0;
	m_dNewActivation = 0.0;
}

void TDorer2NoGoalTrack::SetToNewActivation()
{
	m_dActivation = m_dNewActivation;
	m_dNewActivation = 0.0;
}

void TDorer2NoGoalTrack::SetActivation(int i, double val)
{
	m_dNewActivation += val;
}

double TDorer2NoGoalTrack::GetSumActivation() const
{
	return m_dActivation;
}

void TDorer2NoGoalTrack::Draw(CDC* pDC, TScreenParamsNetwork* pScreen, CPoint& coords)
{
}