// TDorer2GoalTrack.h: interface for the TDorer2GoalTrack class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TDorer2GoalTrack_H__E3189C41_157A_11D2_A16C_006008E88C8E__INCLUDED_)
#define AFX_TDorer2GoalTrack_H__E3189C41_157A_11D2_A16C_006008E88C8E__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

class TScreenParamsNetwork;


/*-----------------------------------------------------------------------------
Class   : TDorer2BaseGoalTrack
Date    : 08.01.98
-------------------------------------------------------------------------------
Base Class: CObject
Description: abstract base class for goaltracking in Dorer2
-----------------------------------------------------------------------------*/

class TDorer2BaseGoalTrack : public CObject
{
	public:
		virtual void	Draw(CDC* pDC, TScreenParamsNetwork* pScreen, CPoint& coords) = 0;
		virtual double	GetSumActivation(void) const = 0;
		virtual void	SetActivation(int i, double val) = 0; 
		virtual double	GetActivation(int i) = 0;
		virtual int		GetNoOfGoals(void) const = 0;
		virtual void	SetToNewActivation() = 0;	
};

/*-----------------------------------------------------------------------------
Class   : TDorer2GoalTrack
Date    : 07.07.98
-------------------------------------------------------------------------------
Base Class: CObject

Description: encapsules the goal tracking
-----------------------------------------------------------------------------*/

class TDorer2GoalTrack : public TDorer2BaseGoalTrack
{
	public:
		TDorer2GoalTrack(int iNoOfGoals);
		virtual ~TDorer2GoalTrack();

		// public methods
		void			Initialize();
		void			Draw(CDC* pDC, TScreenParamsNetwork* pScreen, CPoint& coords);

		// attribute functions		
		int			GetNoOfGoals(void) const { return m_iNoOfGoals; };
		double		GetSumActivation(void) const;
		void			SetActivation(int i, double val); 
		double		GetActivation(int i) { ASSERT ((i >= 0) && (i < m_iNoOfGoals)); 
															return m_pdActivation[i]; };
		void			SetToNewActivation();

	private:
		int				m_iNoOfGoals;			// no of goals in the goal track
		double*			m_pdActivation;		// procentual values of goal influences
		double*			m_pdNewActivation;	// procentual values of goal influences
};

/*-----------------------------------------------------------------------------
Class   : TDorer2NoGoalTrack
Date    : 23.02.99
-------------------------------------------------------------------------------
Base Class: CObject
Description: class that satisfies the interface to goaltracking
				 but does not perform it (faster as class above)
-----------------------------------------------------------------------------*/

class TDorer2NoGoalTrack : public TDorer2BaseGoalTrack
{
	public:
		TDorer2NoGoalTrack(int iNoOfGoals);

		// public methods
		void			Initialize();
		void			Draw(CDC* pDC, TScreenParamsNetwork* pScreen, CPoint& coords);

		// attribute functions		
		int			GetNoOfGoals(void) const { return 1; };
		double		GetSumActivation(void) const;
		void			SetActivation(int i, double val); 
		double		GetActivation(int i) { return m_dActivation; };
		void			SetToNewActivation();

	private:
		double			m_dActivation;	
		double			m_dNewActivation;
};

#endif // !defined(AFX_TDorer2GoalTrack_H__E3189C41_157A_11D2_A16C_006008E88C8E__INCLUDED_)
