// TDorer2Network.cpp: implementation of the TDorer2Network class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "NetworkHeader.h"
#include "TParseString.h"
#include "TRuleView.h"
#include "TNetworkView.h"
#include "TNetworkParams.h"
#include "DNetworkParamsSlider.h"
#include "TScreenParamsNetwork.h"
#include "TResource.h"
#include "TResourceFactory.h"
#include "TAction.h"
#include "TDorer2Competence.h"
#include "TDorer2Network.h"
#include "TDorer2Goal.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TDorer2Network::TDorer2Network(TNetworkView* pNView, TRuleView* pRView)
					: TBehaviorNetwork(pNView, pRView) 
{
	m_pRulesView->SetBehaviorNetwork(this);
	m_pNetworkView->SetNetwork(this);

	m_pResourceList = new CObList();
	m_pResourceFactory = new TResourceFactory();

	Initialize();
}


TDorer2Network::~TDorer2Network()
{
	// delete the resources and the list
	while (!m_pResourceList->IsEmpty())
		delete m_pResourceList->RemoveHead();
	delete m_pResourceList;

	delete m_pResourceFactory;
}

void TDorer2Network::Initialize()
{
	m_pNetworkParams->SetGamma(GAMMA_SIGMOID);
	m_pNetworkParams->SetPhi(PHI_SIGMOID);
	m_pNetworkParams->SetDelta(DELTA_SIGMOID);
	m_pNetworkParams->SetBeta(BETA);
	m_pNetworkParams->SetTheta(THETA_SIGMOID);
	m_pNetworkParams->SetThetaReduction(TH_REDUCTION_SIGMOID);
	m_pNetworkParams->SetSigma(0.55);
	m_pNetworkParams->SetGain(5.0);

	TBehaviorNetwork::Initialize();
}

/*-----------------------------------------------------------------------------
Function: void TMaesNetwork::ParamsDialog()
Date    : 19.11.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: allows to change the parameters of the network easily. The parameters are
				changed immediately, so if user presses Cancel afterwards, the old 
				parameters have to be saved and restored at Cancel
-----------------------------------------------------------------------------*/
void TDorer2Network::ParamsDialog()
{
	TNetworkParams			oldParams(m_pNetworkParams);
	DNetworkParamsSlider	tDialog(NULL, m_pNetworkParams);

	tDialog.m_dMin = MIN_DORER;
	tDialog.m_dMax = MAX_DORER;
	tDialog.m_dRange = RANGE_DORER;
	tDialog.m_dMinGamma = 0.0;
	tDialog.m_dMaxBeta = MAX_BETA;
	tDialog.m_dMaxThetaReduction = MAX_DORER;

	tDialog.m_dPi = m_pNetworkParams->GetBeta();
	tDialog.m_dPhi = m_pNetworkParams->GetPhi();
	tDialog.m_dGamma = m_pNetworkParams->GetGamma();
	tDialog.m_dDelta = m_pNetworkParams->GetDelta();
	tDialog.m_dTheta = m_pNetworkParams->GetTheta();
	tDialog.m_dThetaReduction = m_pNetworkParams->GetThetaReduction();
	tDialog.m_dFuzzyRound = m_pNetworkParams->GetFuzzyRound();
	if (tDialog.DoModal() != IDOK)
		// restore old params, because user canceled
		m_pNetworkParams->SetNetworkParams(&oldParams);
}

/*-----------------------------------------------------------------------------
Function: TCompetence* TDorer2Network::CreateNewCompetence()
Date    : 03.02.99
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: virtual factory function
-----------------------------------------------------------------------------*/
TCompetence* TDorer2Network::CreateNewCompetence()
{
	return new TDorer2Competence(m_pNetworkParams, ""
										, GetNoOfRules(), m_pGoalList->GetCount());
}

/*-----------------------------------------------------------------------------
Function: TGoal* TDorer2Network::CreateNewGoal()
Date    : 03.02.99
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: virtual factory function
-----------------------------------------------------------------------------*/
TGoal* TDorer2Network::CreateNewGoal()
{
	TDorer2Goal*	newGoal = new TDorer2Goal();
	newGoal->SetNetwork(this);
	return newGoal;
}

/*-----------------------------------------------------------------------------
Function: void TDorer2Network::UpdatePerception(BOOL bFromNetwork)
Date    : 03.02.99
-------------------------------------------------------------------------------
Parameter: TRUE, if info is from the network (real perception), false, if it is
			  from a logfile
Returnval: (void)
Description: updates the relevance of the goals
-----------------------------------------------------------------------------*/
void TDorer2Network::UpdatePerception(BOOL bFromNetwork)
{ 
	TBehaviorNetwork::UpdatePerception(bFromNetwork);
	
	// update all goal relevances
	POSITION pos = m_pGoalList->GetHeadPosition();
	while (pos != NULL)
		((TDorer2Goal*) m_pGoalList->GetNext(pos))->CalculateSituationRelevance();
}

/*-----------------------------------------------------------------------------
Function: void TBehaviorNetwork::SetNormCoordinates()
Date    : 04.02.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: sets the origin coordinates of all modules
-----------------------------------------------------------------------------*/
void TDorer2Network::SetNormCoordinates()
{
	int	sx = m_pNetworkView->GetScreenParams()->GetStretchX();
	int	sy = m_pNetworkView->GetScreenParams()->GetStretchY();
	int	x;

	TBehaviorNetwork::SetNormCoordinates();
	
	// perception
	x = sx * m_pGoalList->GetCount() * (int) MODULE_SIZE + 40;
	m_pResourceFactory->SetAllNormCoordinates(CPoint(x, 20), CSize(sx, sy), 20000);
}

/*-----------------------------------------------------------------------------
Function: void TDorer2Network::SetCoordinates(TScreenParamsNetwork* pScreen)
Date    : 04.02.98
-------------------------------------------------------------------------------
Parameter: pointer to the scaling and translation and so on
Returnval: (void)
Description: sets the new coordinates of all modules
-----------------------------------------------------------------------------*/
void TDorer2Network::SetCoordinates(TScreenParamsNetwork* pScreen)
{
	TBehaviorNetwork::SetCoordinates(pScreen);

	m_pResourceFactory->SetAllCoordinates(pScreen);
}

/*-----------------------------------------------------------------------------
Function: void TDorer2Network::DrawNetwork(CDC* pDC, TScreenParamsNetwork* pScreen)
Date    : 02.02.98
-------------------------------------------------------------------------------
Parameter: pointer to the device context
			  parameters defining the screen (translation, scaling, ..)
Returnval: (void)
Description: draws the competence network or the action history
-----------------------------------------------------------------------------*/
void TDorer2Network::DrawNetwork(CDC* pDC, TScreenParamsNetwork* pScreen)
{
	TBehaviorNetwork::DrawNetwork(pDC, pScreen);

	if (!pScreen->GetDrawActionHistory())
	{
		// draw the resources
		m_pResourceFactory->DrawAll(pDC, pScreen);
	}
}

// for blocksworld
/*-----------------------------------------------------------------------------
Function: BOOL TDorer2Network::ParseGoals(TParseString &str)
Date    : 23.10.98
-------------------------------------------------------------------------------
Parameter: reference to the string to parse. Parse position is after keyword 'goals'
Returnval: TRUE, if no error occured, FALSE otherwise
Description: parses the list of goals 
-----------------------------------------------------------------------------*/
BOOL TDorer2Network::ParseGoals(TParseString &str)
{
	CString			temp;
	BOOL				bNegated = FALSE;
	BOOL				bReady;
	TGoal*			pNewGoal;

	while (str.ParseString(temp))
	{
		// finished goals
		if (temp.CompareNoCase(STRING_END_GOALS) == 0)
			return TRUE;

		// create the goal
		pNewGoal = CreateNewGoal();
		pNewGoal->SetIndex(m_pGoalList->GetCount());

		// parse a relevance proposition
		if(temp.CompareNoCase(STRING_IF) == 0)
		{
			// read preconditions
			bReady = FALSE;
			while (!bReady)
			{
				// not or name
				bNegated = FALSE;
				str.ParseString(temp);
				
				if (temp.CompareNoCase(STRING_THEN) == 0)
				{
					str.ParseString(temp);
					bReady = TRUE;
				}

				else if (temp.CompareNoCase(STRING_AND) == 0)
					str.ParseString(temp);

				if (temp.CompareNoCase(STRING_NOT) == 0)
				{
					str.ParseString(temp);
					bNegated = TRUE;
				}

				// create the new precondition
				if (!bReady)
					((TDorer2Goal*) pNewGoal)->AddRelevanceCondition(temp, bNegated, m_pPercept);
			}
		}

		if (temp.CompareNoCase(STRING_NOT) == 0)
		{
			// it is a negated goal
			bNegated = TRUE;
			str.ParseString(temp);
		}
		
		// add the goal to the goal list
		if (pNewGoal->SetBlockworldGoal(temp, bNegated, m_pPercept))
			m_pGoalList->AddTail(pNewGoal);
		else
		{
			delete pNewGoal;
			return FALSE;
		}
	}
	return FALSE;
}
// end for blocksworld

/*-----------------------------------------------------------------------------
Function: void TDorer2Network::AddResource(LPCTSTR name, CObject* pObj
											 , PCallBackPerception pFn, int iAmount)
Date    : 22.02.99
-------------------------------------------------------------------------------
Parameter: name of the resource
			  pointer to the object that contains the callback function
			  pointer to the callback function that calculates the current amount of 
			  resource unit available
			  amount of resource units needed
Returnval: (void)
Description: adds a resource to the resource factory
-----------------------------------------------------------------------------*/
void TDorer2Network::AddResource(LPCTSTR name, CObject* pObj
											 , PCallBackPerception pFn, int iAmount)
{
	TResource* pResource = m_pResourceFactory->AddResource(name, pObj, pFn, iAmount);
	pResource->SetActivation(m_pNetworkParams->GetTheta());
	pResource->SetMaxActivation(m_pNetworkParams->GetTheta());
}

/*-----------------------------------------------------------------------------
Function: BOOL TDorer2Network::ParseCompetenceRule(TCompetence* pCompetence, LPCTSTR rule)
Date    : 16.04.99
-------------------------------------------------------------------------------
Parameter: pointer to the competence module, that should contain the rule
			  string that contains the rule to parse
Returnval: (void)
Description: calls the modules parse function. Needed because ParseRule interface
				 differs from network to network
-----------------------------------------------------------------------------*/
BOOL TDorer2Network::ParseCompetenceRule(TCompetence* pCompetence, LPCTSTR rule)
{
	return ((TDorer2Competence*) pCompetence)->ParseRule(rule, m_pAct, m_pPercept, m_pResourceFactory);
}

/*-----------------------------------------------------------------------------
Function: void TDorer2Network::InitializeActionSelection()
Date    : 15.05.99
-------------------------------------------------------------------------------
Parameter: none
Returnval: void
Description: releases resource binding of all modules
-----------------------------------------------------------------------------*/
void TDorer2Network::InitializeActionSelection()
{
	POSITION					pos;
	TDorer2Competence*	module;

	// reset the resource usage for modules no longer active
	pos = m_pCompetenceList->GetHeadPosition();
	while (pos != NULL)
	{
		module = (TDorer2Competence*) m_pCompetenceList->GetNext(pos);
		module->UpdateResourceUsage();
	}
	// !!! this should not be neccessary
	ResetResourceUsage();
}

/*-----------------------------------------------------------------------------
Function: BOOL TDorer2Network::DoActionSelection()
Date    : 15.04.99
-------------------------------------------------------------------------------
Parameter: none
Returnval: true, if an action has been selected
Description: first : calculates the activation of each module
				 second: checks each module for execution
-----------------------------------------------------------------------------*/
BOOL TDorer2Network::DoActionSelection()
{
	POSITION					pos;
	TDorer2Competence*	module;
	TDorer2Competence*	pExecutedModule = NULL;
	int						iNoOfExecutableModules = 0;
	int						iNoOfExecutedModules = 0;

	InitializeActionSelection();

	// check each module for execution
	pos = m_pCompetenceList->GetHeadPosition();
	while (pos != NULL)
	{
		module = (TDorer2Competence*) m_pCompetenceList->GetNext(pos);

		// check module for execution
		if (module->CheckForExecution(m_dThreshold, m_bBlocksworld))
		{
			// the module has been executed
			iNoOfExecutedModules++;
			pExecutedModule = module;
		

			#ifndef CONCURRENT_ACTIONS
				// stop after the first executed action
				pos = NULL;
			#endif
		}

		// for statistics
		if (module->GetExecutability() > 0.01)
			iNoOfExecutableModules++;
	}

	// statistics
	m_dAvgNoOfExecutableModules = (m_dAvgNoOfExecutableModules * m_lNoOfActions 
											+ iNoOfExecutableModules) / (double)(m_lNoOfActions + 1L);
	m_lNoOfActions++;
	
	// !!! this does not work anymore with multiple actions
	if ((pExecutedModule != NULL) && (!m_bBlocksworld))
	{
		// calculate the number of action changes
		if (m_pLastAction != pExecutedModule->GetAction())
			m_lNoOfActionChanges++;
		m_pLastAction = pExecutedModule->GetAction();

		if (m_lNoOfActions < MAX_CYCLES)
			m_aiActionExecuted[m_lNoOfActions] = m_pLastAction->GetIndex();
	}

	if (iNoOfExecutedModules > 0)
		return TRUE;
	else
		return FALSE;
}

/*-----------------------------------------------------------------------------
Function: void TDorer2Network::ReduceThreshold()
Date    : 16.04.99
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: calls the resources threshold reduction method to do
-----------------------------------------------------------------------------*/
void TDorer2Network::ReduceThreshold()
{
	m_pResourceFactory->ReduceAllThresholds(m_pNetworkParams->GetThetaReduction());
}

/*-----------------------------------------------------------------------------
Function: void TDorer2Competence::UpdateResourceUsage()
Date    : 16.04.99
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: releases the resources used by this module, if it has been executed
-----------------------------------------------------------------------------*/
void TDorer2Network::ResetResourceUsage()
{
	m_pResourceFactory->ResetAllResourceUsages();
}

