// TDorerCompetence.h: interface for the TDorerCompetence class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TDorerCompetence_H__76B7A465_8821_11D1_B242_0000C09B5CBE__INCLUDED_)
#define AFX_TDorerCompetence_H__76B7A465_8821_11D1_B242_0000C09B5CBE__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "TCompetence.h"

class TGoal;
class TConnection;
class TNetworkParams;
class TGoalTrack;

/*-----------------------------------------------------------------------------
Class   : TDorerCompetence
Date    : 08.01.98
-------------------------------------------------------------------------------
Base Class: TEnergySpreader

Description: represents a competence module, a module of level 2
-----------------------------------------------------------------------------*/

class TDorerCompetence : public TCompetence  
{
	public:
		TDorerCompetence(TNetworkParams* params, LPCTSTR name, int iNoOfRules, int iNoOfGoals);
		virtual ~TDorerCompetence();

		// functions for different algorithms (haesslich!!)
		
		virtual void	CalculateActivation(int iNoOfModules = 0, double dSumOfActivation = 0.0);
		virtual void	CalculateExecutability();
		virtual double	GetActivationAndExecutability();
		virtual double	Transfer(double dVal);
		virtual double	SetToNewActivation();
		virtual void	DrawModule(CDC* pDC, TScreenParamsNetwork* pScreen);
		
		virtual void	AddPropositionLink(TProposition* source, TProposition* dest);
		virtual void	AddGoalLink(TProposition* source, TProposition* prop);
		virtual void	AddProtectedGoalLink(TProposition* source, TProposition* prop);
		virtual void	AddSuccessor(TProposition* source, TProposition* dest);
		virtual void	AddPredecessor(TProposition* source, TProposition* dest);
		virtual void	AddConflictor(TProposition* source, TProposition* dest);

		// attribute methods
		TGoalTrack* GetGoalTrack() { return m_pGoalTrack; };
		BOOL		GetGoalTrackingOn() const { return m_bGoalTrackingOn; };
		void		SetGoalTrackingOn(BOOL bVal = TRUE) { m_bGoalTrackingOn = bVal; };
		void		SetAdaptActivationAreaOn(BOOL bVal);
		BOOL		GetAdaptActivationAreaOn() const { return m_bAdaptActivationAreaOn; };

	protected:
		void		RecalculateGainAndSigma();
		double	CalculatePreviousActivation();
		double	CalculateExternActivation();
		double	CalculateSpreadingActivation();

		// for goal tracking

		TGoalTrack*		m_pGoalTrack;			// object for goal tracking
		BOOL				m_bGoalTrackingOn;	// true if goaltracking is switched on

		// for activation function
		double			m_dPointOnSigmoidal1X;
		double			m_dPointOnSigmoidal1Y;
		double			m_dPointOnSigmoidal2X;
		double			m_dPointOnSigmoidal2Y;
		BOOL				m_bAdaptActivationAreaOn;
};

#endif // !defined(AFX_TDorerCompetence_H__76B7A465_8821_11D1_B242_0000C09B5CBE__INCLUDED_)
