// TConnection.cpp: implementation of the TConnection class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "NetworkHeader.h"
#include "TEnergySpreader.h"
#include "TProposition.h"
#include "TScreenParamsNetwork.h"
#include "TNetworkParams.h"
#include "TDorerCompetence.h"
#include "TDorerConnection.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//--------------------------------------------------------------------
// Dorer
//--------------------------------------------------------------------

//////////////////////////////////////////////////////////////////////
// TDorerSituationLink
//////////////////////////////////////////////////////////////////////

TDorerSituationLink::TDorerSituationLink(TNetworkParams* params, TProposition* src, TProposition* dest)
						: TConnection(params, src, dest)
{
}

double TDorerSituationLink::GetActivation() 
{ 
	double			dProposition;

	ASSERT(m_pDestinationProp != NULL);

	dProposition = m_pDestinationProp->GetValue();
	return m_dActivation = m_pNetworkParams->GetPhi() 
									* dProposition;
//									* m_pDestinationProp->GetWeigh());
}

BOOL TDorerSituationLink::IsDrawn(TScreenParamsNetwork* pScreen)
{
	return pScreen->GetDrawPredecessors();
}

void TDorerSituationLink::SetMaxActivation()
{
	m_dMaxActivation = m_pNetworkParams->GetPhi();
}

//////////////////////////////////////////////////////////////////////
// TDorerGoalLink
//////////////////////////////////////////////////////////////////////

TDorerGoalLink::TDorerGoalLink(TNetworkParams* params, TProposition* src, TProposition* dest)
						: TConnection(params, src, dest)
{
}

double TDorerGoalLink::GetActivation() 
{ 
	ASSERT(m_pSourceProp != NULL);
	ASSERT(m_pDestinationProp != NULL);

	m_dActivation = m_pNetworkParams->GetGamma() 
						* (m_pSourceProp->GetModule()->GetActivation() 
						* m_pDestinationProp->GetWeigh());
	return m_dActivation;
}

BOOL TDorerGoalLink::IsDrawn(TScreenParamsNetwork* pScreen)
{
	return pScreen->GetDrawSuccessors();
}

void  TDorerGoalLink::SetMaxActivation()
{
	m_dMaxActivation = m_pNetworkParams->GetGamma();
}

//////////////////////////////////////////////////////////////////////
// TDorerProtectedGoalLink
//////////////////////////////////////////////////////////////////////

TDorerProtectedGoalLink::TDorerProtectedGoalLink(TNetworkParams* params, TProposition* src, TProposition* dest)
							: TConnection(params, src, dest)
{
}

double TDorerProtectedGoalLink::GetActivation() 
{ 
	ASSERT(m_pSourceProp != NULL);
	ASSERT(m_pDestinationProp != NULL);

	m_dActivation = -m_pNetworkParams->GetDelta() 
						* (m_pSourceProp->GetModule()->GetActivation() 
						* m_pDestinationProp->GetWeigh());
	return m_dActivation;
}

BOOL TDorerProtectedGoalLink::IsDrawn(TScreenParamsNetwork* pScreen)
{
	return pScreen->GetDrawConflictors();
}

void  TDorerProtectedGoalLink::SetMaxActivation()
{
	m_dMaxActivation = m_pNetworkParams->GetDelta();
}

//////////////////////////////////////////////////////////////////////
// TDorerSuccessorLink
//////////////////////////////////////////////////////////////////////

TDorerSuccessorLink::TDorerSuccessorLink(TNetworkParams* params, TProposition* src, TProposition* dest)
							: TConnection(params, src, dest)
{
}

double TDorerSuccessorLink::GetActivation() 
{ 
	double dActivation;
	double dProposition;

	ASSERT(m_pSourceProp != NULL);
	ASSERT(m_pDestinationProp != NULL);

	dActivation = ((TCompetence*) m_pSourceProp->GetModule())->GetTransferedActivation();
	dProposition = m_pSourceProp->GetValue();

	m_dActivation = m_pNetworkParams->GetGammaSquare() 
						* (1.0 - dProposition) 
						* dActivation
						* m_pDestinationProp->GetWeigh();
	return m_dActivation;
}

BOOL TDorerSuccessorLink::IsDrawn(TScreenParamsNetwork* pScreen)
{
	return pScreen->GetDrawSuccessors();
}

void  TDorerSuccessorLink::SetMaxActivation()
{
	m_dMaxActivation = m_pNetworkParams->GetGammaSquare();
}

//////////////////////////////////////////////////////////////////////
// TDorerPredecessorLink
//////////////////////////////////////////////////////////////////////

TDorerPredecessorLink::TDorerPredecessorLink(TNetworkParams* params, TProposition* src, TProposition* dest)
							: TConnection(params, src, dest)
{
}

double TDorerPredecessorLink::GetActivation() 
{ 
	double dActivation;
	double dProposition;

	ASSERT(m_pSourceProp != NULL);
	ASSERT(m_pDestinationProp != NULL);

	dActivation = ((TCompetence*) m_pSourceProp->GetModule())->GetTransferedActivation();
	dProposition = m_pDestinationProp->GetValue();
	m_dActivation = m_pNetworkParams->GetPhiTimesGamma()
					* (1.0 - dProposition) 
					* (((TCompetence*) m_pSourceProp->GetModule())->GetExecutability() 
					* dActivation
					* m_pSourceProp->GetWeigh());
	return m_dActivation;
}

BOOL TDorerPredecessorLink::IsDrawn(TScreenParamsNetwork* pScreen)
{
	return pScreen->GetDrawPredecessors();
}

void TDorerPredecessorLink::SetMaxActivation()
{
	m_dMaxActivation = m_pNetworkParams->GetPhiTimesGamma();
}

//////////////////////////////////////////////////////////////////////
// TDorerConflictorLink
//////////////////////////////////////////////////////////////////////

TDorerConflictorLink::TDorerConflictorLink(TNetworkParams* params, TProposition* src, TProposition* dest)
							: TConnection(params, src, dest)
{
}

double TDorerConflictorLink::GetActivation() 
{ 
	double dProposition;

	ASSERT(m_pSourceProp != NULL);
	ASSERT(m_pDestinationProp != NULL);

	// if the precondition is false anyway, there is no inhibition: m_pSourceProp->GetValue()
	dProposition = m_pSourceProp->GetValue();
	m_dActivation = -m_pNetworkParams->GetDeltaTimesGamma() 
					* dProposition 
					* ((TCompetence*) m_pSourceProp->GetModule())->GetTransferedActivation()
					* m_pDestinationProp->GetWeigh();
	return m_dActivation;
}

BOOL TDorerConflictorLink::IsDrawn(TScreenParamsNetwork* pScreen)
{
	return pScreen->GetDrawConflictors();
}

void TDorerConflictorLink::SetMaxActivation()
{
	m_dMaxActivation = m_pNetworkParams->GetDeltaTimesGamma();
}
