// TDorerKnowledge.cpp: implementation of the TDorerKnowledge class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "NetworkHeader.h"
#include "TRuleView.h"
#include "TNetworkView.h"
#include "TDorerCompetence.h"
#include "TNetworkParams.h"
#include "DNetworkParamsSlider.h"
#include "TDorerNetwork.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TDorerNetwork::TDorerNetwork(TNetworkView* pNView, TRuleView* pRView)
					: TBehaviorNetwork(pNView, pRView) 
{
	m_pRulesView->SetBehaviorNetwork(this);
	m_pNetworkView->SetNetwork(this);

	Initialize();
}


TDorerNetwork::~TDorerNetwork()
{
}

void TDorerNetwork::Initialize()
{
	m_pNetworkParams->SetGamma(GAMMA_SIGMOID);
	m_pNetworkParams->SetPhi(PHI_SIGMOID);
	m_pNetworkParams->SetDelta(DELTA_SIGMOID);
	m_pNetworkParams->SetBeta(BETA);
	m_pNetworkParams->SetTheta(THETA_SIGMOID);
	m_pNetworkParams->SetThetaReduction(TH_REDUCTION_SIGMOID);
	m_pNetworkParams->SetSigma(SIGMA);
	m_pNetworkParams->SetGain(GAIN);

	TBehaviorNetwork::Initialize();
}

TCompetence* TDorerNetwork::CreateNewCompetence()
{
	return new TDorerCompetence(m_pNetworkParams, ""
										, GetNoOfRules(), m_pGoalList->GetCount());
}

/*-----------------------------------------------------------------------------
Function: void TMaesNetwork::ParamsDialog()
Date    : 19.11.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: allows to change the parameters of the network easily. The parameters are
				changed immediately, so if user presses Cancel afterwards, the old 
				parameters have to be saved and restored at Cancel
-----------------------------------------------------------------------------*/
void TDorerNetwork::ParamsDialog()
{
	TNetworkParams			oldParams(m_pNetworkParams);
	DNetworkParamsSlider	tDialog(NULL, m_pNetworkParams);

	tDialog.m_dMin = MIN_DORER;
	tDialog.m_dMax = MAX_DORER;
	tDialog.m_dRange = RANGE_DORER;
	tDialog.m_dMinGamma = 0.0;
	tDialog.m_dMaxBeta = MAX_BETA;
	tDialog.m_dMaxThetaReduction = MAX_DORER;

	tDialog.m_dPi = m_pNetworkParams->GetBeta();
	tDialog.m_dPhi = m_pNetworkParams->GetPhi();
	tDialog.m_dGamma = m_pNetworkParams->GetGamma();
	tDialog.m_dDelta = m_pNetworkParams->GetDelta();
	tDialog.m_dTheta = m_pNetworkParams->GetTheta();
	tDialog.m_dThetaReduction = m_pNetworkParams->GetThetaReduction();
	tDialog.m_dFuzzyRound = m_pNetworkParams->GetFuzzyRound();
	if (tDialog.DoModal() != IDOK)
		// restore old params, because user canceled
		m_pNetworkParams->SetNetworkParams(&oldParams);
}

void TDorerNetwork::SetGoalTrackingOn(BOOL bVal)
{
	POSITION			pos;

	// set all competence modules goaltracking
	pos = m_pCompetenceList->GetHeadPosition();
	while (pos != NULL)
		((TDorerCompetence*) m_pCompetenceList->GetNext(pos))->SetGoalTrackingOn(bVal);
}

BOOL TDorerNetwork::GetGoalTrackingOn() const
{
	POSITION		pos = m_pCompetenceList->GetHeadPosition();

	if (pos != NULL)
		return ((TDorerCompetence*) m_pCompetenceList->GetNext(pos))->GetGoalTrackingOn();
	else
		return FALSE;
}

void TDorerNetwork::SetAdaptActivationAreaOn(BOOL bVal)
{
	POSITION			pos;

	// set all competence modules activation area adaption for Dorer kind of spreading
	pos = m_pCompetenceList->GetHeadPosition();
	while (pos != NULL)
		((TDorerCompetence*) m_pCompetenceList->GetNext(pos))->SetAdaptActivationAreaOn(bVal);
}

BOOL TDorerNetwork::GetAdaptActivationAreaOn() const
{
	POSITION		pos = m_pCompetenceList->GetHeadPosition();

	if (pos != NULL)
		return ((TDorerCompetence*) m_pCompetenceList->GetNext(pos))->GetAdaptActivationAreaOn();
	else
		return FALSE;
}

