// TEnergySpreader.cpp: implementation of the TEnergySpreader class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "NetworkHeader.h"
#include "TConnection.h"
#include "TProposition.h"
#include "TScreenParamsNetwork.h"
#include "TEnergySpreader.h"

#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
TEnergySpreader::TEnergySpreader()
{
	Initialize();
}

TEnergySpreader::TEnergySpreader(LPCTSTR name, int index)
{
	m_strName = name;
	m_iIndex = index;
	m_dMainActivation = 0.0;
	m_dActivationChange = 0.0;
	m_dMaxActivation = 1.0;
	m_iNoOfDestinations = 0;
	m_bSelected = FALSE;
}

void TEnergySpreader::Initialize()
{
	m_strName.Empty();
	m_iIndex = 0;
	m_dMainActivation = 0.0;
	m_dActivationChange = 0.0;
	m_dMaxActivation = 1.0;
	m_iNoOfDestinations = 0;
	m_bSelected = FALSE;
	
	#ifdef ACTIVATION_HISTORY
		for (int i = 0; i < MAX_HISTORY; i++)	
			m_adHistory[i] = 0.0;
		m_iNoOfHistory = 0;
	#endif

	#ifdef ACTIVATION_DISTRIBUTION
		for (int j = 0; j < NO_OF_SLOTS; j++)
			m_aiDistribution[j] = 0;
	#endif
}

TEnergySpreader::~TEnergySpreader()
{
}

/*-----------------------------------------------------------------------------
Function: void TEnergySpreader::DrawModule(CDC* pDC, TScreenParamsNetwork* pScreen)
Date    : 02.02.98
-------------------------------------------------------------------------------
Parameter: pointer to the device context
			  pointer to the screen parameters such as scaling, translation
Returnval: (void)
Description: _
-----------------------------------------------------------------------------*/
void TEnergySpreader::DrawModule(CDC* pDC, TScreenParamsNetwork* pScreen)
{
	// screen coordinates are already calculated but clip outside modules
	if ((m_tScreenCoord.x < 0) || (m_tScreenCoord.x > pScreen->GetSize().cx))
		return;
	if ((m_tScreenCoord.y < 0) || (m_tScreenCoord.y > pScreen->GetSize().cy))
		return;

	// draw module as circle
	COLORREF cCol;
	CBrush	brush;
	CBrush	*oldBrush;
	double	dActivation = GetMainActivation();
	int		size;
	int		red = (int) (dActivation / m_dMaxActivation * 220.0);
	int		green = 0;
	int		blue = 0;
	
	// set the color red = active, blue = not active
	if (red > 255)
		red = 255;
	if (dActivation < 0.0)
	{
		// negative activation
		blue = -red;
		if (blue > 255)
			blue = 255;
		red = 0;
	}
	
	cCol = RGB(red , green, blue);

	brush.CreateSolidBrush(cCol);
	oldBrush = pDC->SelectObject(&brush);

	// draw a Ellipse at the modules position
	size = (int) (MODULE_SIZE * pScreen->GetScale());
	pDC->Ellipse(m_tScreenCoord.x - size, m_tScreenCoord.y - size
					, m_tScreenCoord.x + size, m_tScreenCoord.y + size);

	// write name of module
	DrawName(pDC, pScreen);

	pDC->SelectObject(oldBrush);
	brush.DeleteObject();
}

/*-----------------------------------------------------------------------------
Function: void TEnergySpreader::DrawName(CDC* pDC, TScreenParamsNetwork* pScreen)
Date    : 21.04.98
-------------------------------------------------------------------------------
Parameter: device context and scaling info
Returnval: (void)
Description: draws the name of the module, and its activation
-----------------------------------------------------------------------------*/
void TEnergySpreader::DrawName(CDC* pDC, TScreenParamsNetwork* pScreen)
{
	if (pScreen->GetDrawNames() || m_bSelected)
	{
		int		nPointSize;
		CFont		font;
		CFont*	oldFont;
		CRect		field;
		COLORREF	oldTextColor;
		COLORREF	oldBkColor;
		int		sizeX;
		CString	strActivation;

		// create the font
		nPointSize = (int) (30.0 * pScreen->GetScale());
		if (!font.CreatePointFont(nPointSize, _T("Times New Roman"), pDC))
			font.CreateStockObject(SYSTEM_FONT);
		oldFont = pDC->SelectObject(&font);

		// calculate the rectangle of the text
		sizeX = (int) pScreen->GetScale() * m_strName.GetLength() * nPointSize / 20;
		field.SetRect(m_tScreenCoord.x, m_tScreenCoord.y
						  , m_tScreenCoord.x + sizeX, m_tScreenCoord.y + nPointSize / 5);
		
		// highlight the executed module
		oldBkColor = pDC->SetBkColor(GetNameBackgroundColor());
		
		oldTextColor = pDC->SetTextColor(0x00000000);
		pDC->DrawText(m_strName, field, DT_SINGLELINE | DT_VCENTER);

		pDC->SetTextColor(oldTextColor);
		pDC->SetBkColor(oldBkColor);

		// draw the activation
		if (pScreen->GetDrawActivations() || m_bSelected)
		{
			field.SetRect(m_tScreenCoord.x, m_tScreenCoord.y - nPointSize / 5
							  , m_tScreenCoord.x + sizeX, m_tScreenCoord.y);
			
			// highlight the executed module
			oldTextColor = pDC->SetTextColor(0x00000000);
			strActivation.Format("%4.3f", GetMainActivation());
			pDC->DrawText(strActivation, field, DT_SINGLELINE | DT_VCENTER);
			pDC->SetTextColor(oldTextColor);
		}

		pDC->SelectObject(oldFont);
		font.DeleteObject();
	}
}

/*-----------------------------------------------------------------------------
Function: void GetCoordinates(int &x, int &y, TScreenParamsNetwork *tScreen)
Date    : 14.11.97
-------------------------------------------------------------------------------
Parameter: reference to the coordinates to return
Returnval: (void)
Description: calculates screen coordinates from distance and direction
-----------------------------------------------------------------------------*/
void TEnergySpreader::SetCoordinates(TScreenParamsNetwork *tScreen)
{
	// the perception is drawn from a top view
	m_tScreenCoord.x = (int) ((double) m_tNormCoord.x * tScreen->GetScale()) + tScreen->GetTranslation().x;
	m_tScreenCoord.y = (int) ((double) m_tNormCoord.y * tScreen->GetScale()) + tScreen->GetTranslation().y;
}

/*-----------------------------------------------------------------------------
Function: void TEnergySpreader::SetActivation(double activation)
Date    : 09.04.98
-------------------------------------------------------------------------------
Parameter: new activation of this module
Returnval: new activation
Description: sets the new activation of the module. Before it calculates the
				 change in activation. all methods should use this function instead 
				 of manipulating m_dMainActivation directly
-----------------------------------------------------------------------------*/
double TEnergySpreader::SetActivation(double activation)
{ 
	m_dActivationChange = activation - m_dMainActivation;

	#ifdef ACTIVATION_HISTORY
		m_adHistory[m_iNoOfHistory] = activation;
		m_iNoOfHistory++;
		if (m_iNoOfHistory >= MAX_HISTORY)
			m_iNoOfHistory = MAX_HISTORY - 1;
	#endif

	#ifdef ACTIVATION_DISTRIBUTION
		int slot = (int) (activation * 10.0);
		if (slot >= NO_OF_SLOTS)
			slot = NO_OF_SLOTS - 1;
		if (slot < 0)
			slot = 0;
		m_aiDistribution[slot]++;
	#endif
	
	return (m_dMainActivation = activation); 
}

/*-----------------------------------------------------------------------------
Function: void TEnergySpreader::WriteActivationHistory(CArchive& ar)
Date    : 16.02.99
-------------------------------------------------------------------------------
Parameter: archive to write to
Returnval: (void)
Description: writes activation history into an archive
-----------------------------------------------------------------------------*/
void TEnergySpreader::WriteActivationHistory(CArchive& ar)
{
#ifdef ACTIVATION_HISTORY
#endif
}

/*-----------------------------------------------------------------------------
Function: void TEnergySpreader::WriteActivationDistribution(CArchive& ar)
Date    : 16.02.99
-------------------------------------------------------------------------------
Parameter: archive to write to
Returnval: (void)
Description: writes activation distribution into an archive
-----------------------------------------------------------------------------*/
void TEnergySpreader::WriteActivationDistribution(CArchive& ar)
{
#ifdef ACTIVATION_DISTRIBUTION
	int		i;
	CString	line;
	CString	temp;

	line = m_strName + " ";
	for (i = 0; i < NO_OF_SLOTS; i++)
	{
		temp.Format("%d ", m_aiDistribution[i]);
		line += temp;
	}
	line += "\r\n";
	ar.WriteString(line);
#endif
}

/*-----------------------------------------------------------------------------
Function: BOOL TEnergySpreader::IsPointWithinModule(CPoint point, double size)
Date    : 14.05.98
-------------------------------------------------------------------------------
Parameter: point (screen coordinates)
			  size of the module with respect to scaling
Returnval: true, if the point is within the module
Description: checks, if the passed point on the screen is within the module
-----------------------------------------------------------------------------*/
BOOL TEnergySpreader::IsPointWithinModule(CPoint point, double size)
{
	CPoint	distance;
	double	dist;

	// size is the size of the module (for competences it is without executability circle
	distance = m_tScreenCoord - point;
	dist = sqrt((double)((distance.x * distance.x) + (distance.y * distance.y)));
	if (dist < size)
		// we fount the module
		return TRUE;
	else
		return FALSE;
}
