// TGoal.cpp: implementation of the TGoal class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "NetworkHeader.h"
#include "TProposition.h"
#include "TPercept.h"
#include "TParseString.h"
#include "TScreenParamsNetwork.h"
#include "TGoal.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TGoal::TGoal() 
{
	m_pCondition = NULL;
	SetImportance(1.0);
}

TGoal::TGoal(int index) 
{
	m_pCondition = NULL;
	SetIndex(index);
	SetImportance(1.0);
}

TGoal::~TGoal()
{
	if (m_pCondition != NULL)
		delete m_pCondition;
}


/*-----------------------------------------------------------------------------
Function: BOOL TGoal::ParseGoal(LPCTSTR goal)
Date    : 29.04.98
-------------------------------------------------------------------------------
Parameter: string containing the goal (Goal name importance Endgoal)
Returnval: (void)
Description: parses a goalstring and sets the parameters of the goal
-----------------------------------------------------------------------------*/
BOOL TGoal::ParseGoal(LPCTSTR goal, TPercept* pPerceptionFactory)
{	
	TParseString	parse(goal);
	CString			temp;
	CString			message;

	// read the goal
	if (!parse.ParseString(temp))
	{
		message.Format("No goal detected ! Goal ignored \r\n \r\n %s", goal);
		AfxMessageBox(message);
		return FALSE;
	}
	
	return ParseGoalName(parse, pPerceptionFactory);
}

/*-----------------------------------------------------------------------------
Function: BOOL TGoal::ParseGoalName(LPCTSTR goal)
Date    : 13.11.98
-------------------------------------------------------------------------------
Parameter: pointer to a string containing the goalstring
Returnval: TRUE, if no error occured
Description: parses the name of the goal and its importance
-----------------------------------------------------------------------------*/
BOOL TGoal::ParseGoalName(TParseString& parse, TPercept* pPerceptionFactory)
{
	CString			name;
	CString			message;
	double			value;
	BOOL				bNegated = FALSE;
	TWorldState*	pState;

	// read name
	if (!parse.ParseString(name))
	{
		message.Format("No name of the goal detected! goal ignored.\r\n\r\n %s", parse);
		AfxMessageBox(message);
		return FALSE;
	}
	else if (name.CompareNoCase(STRING_NOT) == 0)
	{
		// it is a negated goal
		bNegated = TRUE;
		parse.ParseString(name);
	}

	// get the index of the proposition 'name'
	pState = pPerceptionFactory->GetPerception(name);
	if (pState == NULL)
	{
		message.Format("Proposition '%s' not existing! \r\n Goal ignored.", name);
		AfxMessageBox(message);
		return FALSE;
	}

	// get the importance of the goal
	value = parse.ParseDouble();
	if (parse.HasErrorOccured())
		// value not written, so default is 1.0
		value = 1.0;

	// set the goal
	m_pCondition = new TProposition(this, pState, value, bNegated);
	SetName(name);
	SetImportance(value);
	return TRUE;
}

/*-----------------------------------------------------------------------------
Function: void TGoal::SetImportance(double imp)
Date    : 29.04.98
-------------------------------------------------------------------------------
Parameter: importance of the goal
Returnval: (void)
Description: sets the importance and the activation of the goal
-----------------------------------------------------------------------------*/
void TGoal::SetImportance(double imp)
{
	m_dImportance = imp;
	SetActivation(imp);
}


// for blocksworld
BOOL TGoal::SetBlockworldGoal(LPCTSTR name, BOOL bNegated, TPercept* pPerceptionFactory)
{
	CString			message;
	TWorldState*	pState;

	SetName(CString(name));

	// lookup the proposition
	pState = pPerceptionFactory->GetPerception(name);
	if (pState == NULL)
	{
		message.Format("Proposition '%s' not existing! \r\n Rule ignored.", name);
		AfxMessageBox(message);
		return FALSE;
	}
	m_pCondition = new TProposition(this, pState, 1.0, bNegated);
	
	return TRUE;
}

/*-----------------------------------------------------------------------------
Function: void TGoal::DrawModule(CDC* pDC, TScreenParamsNetwork* pScreen)
Date    : 03.02.98
-------------------------------------------------------------------------------
Parameter: pointer to the device context
			  pointer to the screen parameters (translation, scaling, ...)
			  maximal reachable activation (for color scaling)
Returnval: (void)
Description: draws the relevance of the goal and the goal's importance
-----------------------------------------------------------------------------*/
void TGoal::DrawModule(CDC* pDC, TScreenParamsNetwork* pScreen)
{
	// set the screen coordinates of this module
	SetCoordinates(pScreen);
	if ((m_tScreenCoord.x < 0) || (m_tScreenCoord.x > pScreen->GetSize().cx))
		return;
	if ((m_tScreenCoord.y < 0) || (m_tScreenCoord.y > pScreen->GetSize().cy))
		return;

	// draw the module itself
	TEnergySpreader::DrawModule(pDC, pScreen);
}
