// TConnection.cpp: implementation of the TConnection class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "NetworkHeader.h"
#include "TEnergySpreader.h"
#include "TProposition.h"
#include "TScreenParamsNetwork.h"
#include "TNetworkParams.h"
#include "TMaesCompetence.h"
#include "TMaesConnection.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//--------------------------------------------------------------------
// Maes
//--------------------------------------------------------------------

//////////////////////////////////////////////////////////////////////
// TMaesSituationLink
//////////////////////////////////////////////////////////////////////

TMaesSituationLink::TMaesSituationLink(TNetworkParams* params, TProposition* src, TProposition* dest)
						: TConnection(params, src, dest)
{
}

double TMaesSituationLink::GetActivation() 
{ 
	ASSERT(m_pDestinationProp != NULL);
	
	m_dActivation = m_pNetworkParams->GetPhi() * m_pDestinationProp->GetValue();
	return m_dActivation;
}

BOOL TMaesSituationLink::IsDrawn(TScreenParamsNetwork* pScreen)
{
	return pScreen->GetDrawPredecessors();
}

void TMaesSituationLink::SetMaxActivation()
{
	m_dMaxActivation = m_pNetworkParams->GetPhi();
}

//////////////////////////////////////////////////////////////////////
// TMaesGoalLink
//////////////////////////////////////////////////////////////////////

TMaesGoalLink::TMaesGoalLink(TNetworkParams* params, TProposition* src, TProposition* dest)
						: TConnection(params, src, dest)
{
}

double TMaesGoalLink::GetActivation() 
{ 
	return (m_dActivation = m_pNetworkParams->GetGamma());
}

BOOL TMaesGoalLink::IsDrawn(TScreenParamsNetwork* pScreen)
{
	return pScreen->GetDrawSuccessors();
}

void  TMaesGoalLink::SetMaxActivation()
{
	m_dMaxActivation = m_pNetworkParams->GetGamma();
}

//////////////////////////////////////////////////////////////////////
// TMaesProtectedGoalLink
//////////////////////////////////////////////////////////////////////

TMaesProtectedGoalLink::TMaesProtectedGoalLink(TNetworkParams* params, TProposition* src, TProposition* dest)
							: TConnection(params, src, dest)
{
}

double TMaesProtectedGoalLink::GetActivation() 
{ 
	return (m_dActivation = m_pNetworkParams->GetDelta());
}

BOOL TMaesProtectedGoalLink::IsDrawn(TScreenParamsNetwork* pScreen)
{
	return pScreen->GetDrawConflictors();
}

void  TMaesProtectedGoalLink::SetMaxActivation()
{
	m_dMaxActivation = m_pNetworkParams->GetDelta();
}

//////////////////////////////////////////////////////////////////////
// TMaesSuccessorLink
//////////////////////////////////////////////////////////////////////

TMaesSuccessorLink::TMaesSuccessorLink(TNetworkParams* params, TProposition* src, TProposition* dest)
							: TConnection(params, src, dest)
{
}

double TMaesSuccessorLink::GetActivation() 
{ 
	ASSERT(m_pSourceProp != NULL);

	// > 0.5 works, because maes uses binary propositions
	if (m_pSourceProp->GetValue() < 0.5)
		return m_dActivation = ((TMaesCompetence*) m_pSourceProp->GetModule())->GetExternActivation();
	else
		return m_dActivation = 0.0;
}

BOOL TMaesSuccessorLink::IsDrawn(TScreenParamsNetwork* pScreen)
{
	return pScreen->GetDrawSuccessors();
}

void  TMaesSuccessorLink::SetMaxActivation()
{
	m_dMaxActivation = MAX_ACTIVATION;
}

//////////////////////////////////////////////////////////////////////
// TMaesPredecessorLink
//////////////////////////////////////////////////////////////////////

TMaesPredecessorLink::TMaesPredecessorLink(TNetworkParams* params, TProposition* src, TProposition* dest)
							: TConnection(params, src, dest)
{
}

double TMaesPredecessorLink::GetActivation() 
{ 
	ASSERT(m_pSourceProp != NULL);

	// the source module has to be executable and the destination proposition true
	if ((((TMaesCompetence*) m_pSourceProp->GetModule())->GetExecutability() > 0.5)
		 && (m_pSourceProp->GetValue() < 0.5))
		// the module is executable and the proposition is false,so spread activation
		return m_dActivation = m_pNetworkParams->GetPhiByGamma()
							* ((TMaesCompetence*) m_pSourceProp->GetModule())->GetExternActivation();
	else
		return m_dActivation = 0.0;
}

BOOL TMaesPredecessorLink::IsDrawn(TScreenParamsNetwork* pScreen)
{
	return pScreen->GetDrawPredecessors();
}

void TMaesPredecessorLink::SetMaxActivation()
{
	m_dMaxActivation = m_pNetworkParams->GetPhiByGamma() * MAX_ACTIVATION;
}

//////////////////////////////////////////////////////////////////////
// TMaesConflictorLink
//////////////////////////////////////////////////////////////////////

TMaesConflictorLink::TMaesConflictorLink(TNetworkParams* params, TProposition* src, TProposition* dest)
							: TConnection(params, src, dest)
{
	m_bMutualConflictor = FALSE;
}

double TMaesConflictorLink::GetActivation() 
{ 
	ASSERT(m_pSourceProp != NULL);
	ASSERT(m_pDestinationProp != NULL);

	// if the precondition is false anyway, there is no inhibition
	if (m_pSourceProp->GetValue() > 0.5)
	{
		// get the activation of the module that likes to inhibit this
		double source = ((TCompetence*) m_pSourceProp->GetModule())->GetExternActivation();
		if (m_bMutualConflictor)
		{
			// there is a conflictor to this module, so only let inhibit, if activation of
			// source is greater
			double destination = ((TCompetence*) m_pDestinationProp->GetModule())->GetExternActivation();
			if (source > destination)
				return m_dActivation = -m_pNetworkParams->GetDeltaByGamma() * source;
			else
				return m_dActivation = 0.0;
		}
		else
			return m_dActivation = -m_pNetworkParams->GetDeltaByGamma() * source;
	}
	else
		return m_dActivation = 0.0;
}

BOOL TMaesConflictorLink::IsDrawn(TScreenParamsNetwork* pScreen)
{
	return pScreen->GetDrawConflictors();
}

void TMaesConflictorLink::SetMaxActivation()
{
	m_dMaxActivation = m_pNetworkParams->GetDelta();
}



