// TConnection.h: interface for the TConnection class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TMAESCONNECTION_H__76B7A462_8821_11D1_B242_0000C09B5CBE__INCLUDED_)
#define AFX_TMAESCONNECTION_H__76B7A462_8821_11D1_B242_0000C09B5CBE__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "TConnection.h"

class TProposition;
class TScreenParamsNetwork;
class TEnergySpreader;
class TNetworkParams;

/*-----------------------------------------------------------------------------
Class   : Maes Connections
Date    : 19.11.98
-------------------------------------------------------------------------------
Description: special classes to treat Maes connections
-----------------------------------------------------------------------------*/

class TMaesSituationLink : public TConnection
{
	public:
		TMaesSituationLink(TNetworkParams* params, TProposition* src, TProposition *dest);

		virtual double		GetActivation();
		virtual BOOL		IsDrawn(TScreenParamsNetwork* pScreen);
		virtual void		SetMaxActivation();
};


class TMaesGoalLink : public TConnection
{
	public:
		TMaesGoalLink(TNetworkParams* params, TProposition* src, TProposition *dest);

		virtual double		GetActivation();
		virtual BOOL		IsDrawn(TScreenParamsNetwork* pScreen);
		virtual void		SetMaxActivation();
};


class TMaesProtectedGoalLink : public TConnection
{
	public:
		TMaesProtectedGoalLink(TNetworkParams* params, TProposition* src, TProposition *dest);

		virtual double		GetActivation();
		virtual BOOL		IsDrawn(TScreenParamsNetwork* pScreen);
		virtual void		SetMaxActivation();
};


class TMaesSuccessorLink : public TConnection
{
	public:
		TMaesSuccessorLink(TNetworkParams* params, TProposition* src, TProposition *dest);

		virtual double		GetActivation();
		virtual BOOL		IsDrawn(TScreenParamsNetwork* pScreen);
		virtual void		SetMaxActivation();
		virtual void		DrawLink(CDC* pDC, TScreenParamsNetwork* pScreen) { DrawInternalLink(pDC, pScreen); };
		virtual	int		GetBending() { return TConnection::GetBending() + 10; };
};


class TMaesPredecessorLink : public TConnection
{
	public:
		TMaesPredecessorLink(TNetworkParams* params, TProposition* src, TProposition *dest);

		virtual double		GetActivation();
		virtual BOOL		IsDrawn(TScreenParamsNetwork* pScreen);
		virtual void		SetMaxActivation();
		virtual void		DrawLink(CDC* pDC, TScreenParamsNetwork* pScreen) { DrawInternalLink(pDC, pScreen); };
		virtual void		DoCreatePen(CPen& pen, COLORREF cCol) { pen.CreatePen(PS_DASHDOT, 1, cCol); };
		virtual	int		GetBending() { return TConnection::GetBending() + 5; };
};


class TMaesConflictorLink : public TConnection
{
	public:
		TMaesConflictorLink(TNetworkParams* params, TProposition* src, TProposition *dest);

		virtual double		GetActivation();
		virtual BOOL		IsDrawn(TScreenParamsNetwork* pScreen);
		virtual void		SetMaxActivation();
		virtual void		DrawLink(CDC* pDC, TScreenParamsNetwork* pScreen) { DrawInternalLink(pDC, pScreen); };
		virtual void		DoCreatePen(CPen& pen, COLORREF cCol) { pen.CreatePen(PS_DOT, 1, cCol); };

		void					SetMutualConflictor(BOOL conf = TRUE) { m_bMutualConflictor = conf; };
		BOOL					HasMutualConflictor() const { return m_bMutualConflictor; };

	private:
		BOOL					m_bMutualConflictor;
};

#endif // !defined(AFX_TMAESCONNECTION_H__76B7A462_8821_11D1_B242_0000C09B5CBE__INCLUDED_)
