// TNetworkView.cpp : implementation file
//

#include "stdafx.h"
#include "NetworkHeader.h"
#include "TEnergySpreader.h"
#include "TProposition.h"
#include "TBehaviorNetwork.h"
#include "TPercept.h"
#include "TScreenParamsNetwork.h"
#include "TWorldState.h"
#include "TNetworkParams.h"
#include "TNetworkView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// TNetworkView

IMPLEMENT_DYNCREATE(TNetworkView, CView)

TNetworkView::TNetworkView()
{
	m_pScreen = new TScreenParamsNetwork();
	m_pNetwork = NULL;
	m_bDrawAtEndOfActionSelection = TRUE;
	m_pSelectedModule = NULL;
}

TNetworkView::~TNetworkView()
{
	delete m_pScreen;
}


BEGIN_MESSAGE_MAP(TNetworkView, CView)
	//{{AFX_MSG_MAP(TNetworkView)
	ON_WM_VSCROLL()
	ON_WM_HSCROLL()
	ON_COMMAND(IDC_NETNAMES, OnNetnames)
	ON_UPDATE_COMMAND_UI(IDC_NETNAMES, OnUpdateNetnames)
	ON_COMMAND(IDC_NETZOOM_IN, OnNetzoomIn)
	ON_UPDATE_COMMAND_UI(IDC_NETZOOM_IN, OnUpdateNetzoomIn)
	ON_COMMAND(IDC_NETZOOM_OUT, OnNetzoomOut)
	ON_UPDATE_COMMAND_UI(IDC_NETZOOM_OUT, OnUpdateNetzoomOut)
	ON_COMMAND(ID_VIEW_CONFLICTORS, OnViewConflictors)
	ON_UPDATE_COMMAND_UI(ID_VIEW_CONFLICTORS, OnUpdateViewConflictors)
	ON_COMMAND(ID_VIEW_PREDECESSORS, OnViewPredecessors)
	ON_UPDATE_COMMAND_UI(ID_VIEW_PREDECESSORS, OnUpdateViewPredecessors)
	ON_COMMAND(ID_VIEW_SUCCESSORS, OnViewSuccessors)
	ON_UPDATE_COMMAND_UI(ID_VIEW_SUCCESSORS, OnUpdateViewSuccessors)
	ON_COMMAND(IDC_NETVIEW_AT_END, OnNetviewAtEnd)
	ON_UPDATE_COMMAND_UI(IDC_NETVIEW_AT_END, OnUpdateNetviewAtEnd)
	ON_WM_LBUTTONDOWN()
	ON_COMMAND(IDC_NET_VIEW_SELECTED, OnNetViewSelected)
	ON_UPDATE_COMMAND_UI(IDC_NET_VIEW_SELECTED, OnUpdateNetViewSelected)
	ON_COMMAND(ID_VIEW_NET_ACTIVATION, OnViewNetActivation)
	ON_UPDATE_COMMAND_UI(ID_VIEW_NET_ACTIVATION, OnUpdateViewNetActivation)
	ON_COMMAND(ID_NET_PARAMS, OnNetParams)
	ON_UPDATE_COMMAND_UI(ID_NET_PARAMS, OnUpdateNetParams)
	ON_COMMAND(ID_NET_DRAW_GOAL_TRACKING, OnNetDrawGoalTracking)
	ON_UPDATE_COMMAND_UI(ID_NET_DRAW_GOAL_TRACKING, OnUpdateNetDrawGoalTracking)
	ON_COMMAND(ID_VIEW_NET_CON_ACTIVATION, OnViewNetConActivation)
	ON_UPDATE_COMMAND_UI(ID_VIEW_NET_CON_ACTIVATION, OnUpdateViewNetConActivation)
	ON_COMMAND(ID_NET_SINGLE_STEP, OnNetSingleStep)
	ON_UPDATE_COMMAND_UI(ID_NET_SINGLE_STEP, OnUpdateNetSingleStep)
	ON_COMMAND(ID_NET_SQEEZE_Y, OnNetSqeezeY)
	ON_UPDATE_COMMAND_UI(ID_NET_SQEEZE_Y, OnUpdateNetSqeezeY)
	ON_COMMAND(ID_NET_SQUEEZE_X, OnNetSqueezeX)
	ON_UPDATE_COMMAND_UI(ID_NET_SQUEEZE_X, OnUpdateNetSqueezeX)
	ON_COMMAND(ID_NET_STRETCH_X, OnNetStretchX)
	ON_UPDATE_COMMAND_UI(ID_NET_STRETCH_X, OnUpdateNetStretchX)
	ON_COMMAND(ID_NET_STRETCH_Y, OnNetStretchY)
	ON_UPDATE_COMMAND_UI(ID_NET_STRETCH_Y, OnUpdateNetStretchY)
	ON_COMMAND(ID_VIEW_NET_ZERO_LINK, OnViewNetZeroLink)
	ON_UPDATE_COMMAND_UI(ID_VIEW_NET_ZERO_LINK, OnUpdateViewNetZeroLink)
	ON_COMMAND(ID_NET_ACTION, OnNetAction)
	ON_UPDATE_COMMAND_UI(ID_NET_ACTION, OnUpdateNetAction)
	ON_COMMAND(ID_NET_ACTION_HISTORY, OnNetActionHistory)
	ON_UPDATE_COMMAND_UI(ID_NET_ACTION_HISTORY, OnUpdateNetActionHistory)
	ON_COMMAND(ID_DRAW_RESOURCE_LINK, OnDrawResourceLink)
	ON_UPDATE_COMMAND_UI(ID_DRAW_RESOURCE_LINK, OnUpdateDrawResourceLink)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// TNetworkView drawing

void TNetworkView::OnDraw(CDC* pDC)
{
	RedrawView();
}

/////////////////////////////////////////////////////////////////////////////
// TNetworkView diagnostics

#ifdef _DEBUG
void TNetworkView::AssertValid() const
{
	CView::AssertValid();
}

void TNetworkView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// TNetworkView message handlers
void TNetworkView::RedrawView()
{
	CClientDC		pDC(this);
	CDC				dcMem;
	CBitmap			myBitmap;
	CRect				windowRect;
	int				sizeX, sizeY;

	if (m_bNoGraphics)
		return;

	if (m_pNetwork == NULL)
		return;

	// prepare the memory device context
	dcMem.CreateCompatibleDC(&pDC);
	
	// get Device Coordinates
	pDC.GetWindow()->GetClientRect(&windowRect);
	windowRect.NormalizeRect();
	sizeX = windowRect.Width();
	sizeY = windowRect.Height();
	myBitmap.CreateCompatibleBitmap(&pDC, sizeX, sizeY);
	// set the size of the bitmap, to be read by the drawing functions
	myBitmap.SetBitmapDimension(sizeX, sizeY);
	
	ASSERT(myBitmap.m_hObject != NULL);

	CBitmap* pOldBitmap = dcMem.SelectObject(&myBitmap);

	// erase the memory device context
	dcMem.FillSolidRect(0, 0, sizeX, sizeY, 0x00ffffff);

	// draw the things
	m_pScreen->SetMid(sizeX / 2, sizeY / 2);
	m_pScreen->SetSize(CSize(sizeX, sizeY));
	m_pNetwork->DrawNetwork(&dcMem, m_pScreen);
	if (m_pSelectedModule != NULL)
		m_pSelectedModule->DrawModule(&dcMem, m_pScreen);

	// copy the memory bitmap into the window
	pDC.BitBlt(0, 0, sizeX, sizeY, &dcMem, 0, 0, SRCCOPY);

	// remove the created bitmap
	dcMem.SelectObject(pOldBitmap);
	dcMem.DeleteDC();
	myBitmap.DeleteObject();
}

BOOL TNetworkView::PreCreateWindow(CREATESTRUCT& cs) 
{
	BOOL ret = CView::PreCreateWindow(cs);
	
	cs.style = AFX_WS_DEFAULT_VIEW | WS_VSCROLL | WS_HSCROLL | ES_AUTOHSCROLL |
		ES_AUTOVSCROLL | ES_NOHIDESEL;

	m_pScreen->SetScale(2.0);
	
	return ret;
}

void TNetworkView::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	switch (nSBCode)
	{
		case SB_ENDSCROLL:
			break;

		case SB_LINEUP:
			m_pScreen->Translate(CPoint(0, 10));
			if (m_pScreen->GetTranslation().y > 0)
				m_pScreen->SetTranslation(CPoint(m_pScreen->GetTranslation().x, 0));
			break;

		case SB_LINEDOWN:
			m_pScreen->Translate(CPoint(0, -10));
			break;

		case SB_PAGEUP:
			m_pScreen->Translate(CPoint(0, 100));
			if (m_pScreen->GetTranslation().y > 0)
				m_pScreen->SetTranslation(CPoint(m_pScreen->GetTranslation().x, 0));
			break;

		case SB_PAGEDOWN:
			m_pScreen->Translate(CPoint(0, 100));
			break;

//		case SB_TOP:
//			m_pScreen->SetTranslation(CPoint(m_pScreen->GetTranslation().x, 0));
//			break;

		case SB_BOTTOM:
			break;

		case SB_THUMBPOSITION:
		case SB_THUMBTRACK:
			m_pScreen->SetTranslation(CPoint(m_pScreen->GetTranslation().x, nPos * -10));
			break;
	}
	
	CView::OnVScroll(nSBCode, nPos, pScrollBar);
	SetScrollPos( SB_VERT, m_pScreen->GetTranslation().y / -10);
	m_pNetwork->SetCoordinates(m_pScreen);
	RedrawView();
}

void TNetworkView::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	switch (nSBCode)
	{
//		case SB_LEFT:
//			m_pScreen->SetTranslation(CPoint(0, m_pScreen->GetTranslation().y));
//			break;

		case SB_ENDSCROLL:
			break;

		case SB_LINERIGHT:
			m_pScreen->Translate(CPoint(-10, 0));
			break;

		case SB_LINELEFT:
			m_pScreen->Translate(CPoint(10, 0));
			if (m_pScreen->GetTranslation().x > 0)
				m_pScreen->SetTranslation(CPoint(0, m_pScreen->GetTranslation().y));
			break;

		case SB_PAGERIGHT:
			m_pScreen->Translate(CPoint(-100, 0));
			break;

		case SB_PAGELEFT:
			m_pScreen->Translate(CPoint(100, 0));
			if (m_pScreen->GetTranslation().x > 0)
				m_pScreen->SetTranslation(CPoint(m_pScreen->GetTranslation().y));
			break;

		case SB_RIGHT:
			break;

		case SB_THUMBPOSITION:
		case SB_THUMBTRACK:
			m_pScreen->SetTranslation(CPoint(nPos * -10, m_pScreen->GetTranslation().y));
			break;
	}

	CView::OnHScroll(nSBCode, nPos, pScrollBar);
	SetScrollPos( SB_HORZ, m_pScreen->GetTranslation().x / -10);
	m_pNetwork->SetCoordinates(m_pScreen);
	RedrawView();
}


void TNetworkView::OnNetSingleStep() 
{
//	if (m_pAgent->IsConnected())
//		return;

	m_pNetwork->UpdatePerception(TRUE);
	m_pNetwork->DoActivationSpreading();
	RedrawView();
}

void TNetworkView::OnNetParams() 
{
	m_pNetwork->ParamsDialog();
}

void TNetworkView::OnNetnames() 
{
	m_pScreen->SetDrawNames(!m_pScreen->GetDrawNames());
	// activations can only be drawn, when names are drawn as well
	if (!m_pScreen->GetDrawNames())
		m_pScreen->SetDrawActivations(FALSE);
	RedrawView();
}

void TNetworkView::OnViewNetActivation() 
{
	// activations can only be drawn, when names are drawn as well
	m_pScreen->SetDrawActivations(!m_pScreen->GetDrawActivations());
	if (!m_pScreen->GetDrawNames())
		m_pScreen->SetDrawNames(TRUE);
	RedrawView();
}

void TNetworkView::OnViewNetConActivation() 
{
	m_pScreen->SetDrawConnectionActivations(!m_pScreen->GetDrawConnectionActivations());
	RedrawView();	
}

void TNetworkView::OnNetDrawGoalTracking() 
{
	m_pScreen->SetDrawGoalTracking(!m_pScreen->GetDrawGoalTracking());
	RedrawView();
}


void TNetworkView::OnNetzoomIn() 
{
	m_pScreen->Scale(1.2);
	m_pNetwork->SetCoordinates(m_pScreen);
	RedrawView();
}

void TNetworkView::OnNetzoomOut() 
{
	m_pScreen->Scale(0.83);
	m_pNetwork->SetCoordinates(m_pScreen);
	RedrawView();
}

void TNetworkView::OnNetViewSelected() 
{
	m_pScreen->SetJustDrawSelectedConnections(!m_pScreen->GetJustDrawSelectedConnections());	
	RedrawView();
}

void TNetworkView::OnViewSuccessors() 
{
	m_pScreen->SetDrawSuccessors(!m_pScreen->GetDrawSuccessors());	
	RedrawView();
}

void TNetworkView::OnViewPredecessors() 
{
	m_pScreen->SetDrawPredecessors(!m_pScreen->GetDrawPredecessors());	
	RedrawView();
}

void TNetworkView::OnViewConflictors() 
{
	m_pScreen->SetDrawConflictors(!m_pScreen->GetDrawConflictors());	
	RedrawView();
}

void TNetworkView::OnDrawResourceLink() 
{
	m_pScreen->SetDrawResourceLinks(!m_pScreen->GetDrawResourceLinks());	
	RedrawView();
}

void TNetworkView::OnNetviewAtEnd() 
{
	m_bDrawAtEndOfActionSelection = !m_bDrawAtEndOfActionSelection;
}

void TNetworkView::OnUpdateNetSingleStep(CCmdUI* pCmdUI) 
{
}

void TNetworkView::OnUpdateNetnames(CCmdUI* pCmdUI) 
{
	if (m_pScreen->GetDrawNames())
		pCmdUI->SetRadio(TRUE);
	else
		pCmdUI->SetRadio(FALSE);
}

void TNetworkView::OnUpdateViewNetActivation(CCmdUI* pCmdUI) 
{
	if (m_pScreen->GetDrawActivations())
		pCmdUI->SetRadio(TRUE);
	else
		pCmdUI->SetRadio(FALSE);
}


void TNetworkView::OnUpdateViewNetConActivation(CCmdUI* pCmdUI) 
{
	if (m_pScreen->GetDrawConnectionActivations())
		pCmdUI->SetRadio(TRUE);
	else
		pCmdUI->SetRadio(FALSE);
}

void TNetworkView::OnUpdateNetDrawGoalTracking(CCmdUI* pCmdUI) 
{
	if (m_pScreen->GetDrawGoalTracking())
		pCmdUI->SetRadio(TRUE);
	else
		pCmdUI->SetRadio(FALSE);	
}

void TNetworkView::OnUpdateNetParams(CCmdUI* pCmdUI) 
{
}

void TNetworkView::OnUpdateNetzoomIn(CCmdUI* pCmdUI) 
{
}

void TNetworkView::OnUpdateNetzoomOut(CCmdUI* pCmdUI) 
{
}

void TNetworkView::OnUpdateNetViewSelected(CCmdUI* pCmdUI) 
{
	if (m_pScreen->GetJustDrawSelectedConnections())
		pCmdUI->SetCheck(TRUE);
	else
		pCmdUI->SetCheck(FALSE);
}

void TNetworkView::OnUpdateViewSuccessors(CCmdUI* pCmdUI) 
{
	if (m_pScreen->GetDrawSuccessors())
		pCmdUI->SetCheck(TRUE);
	else
		pCmdUI->SetCheck(FALSE);
}

void TNetworkView::OnUpdateViewPredecessors(CCmdUI* pCmdUI) 
{
	if (m_pScreen->GetDrawPredecessors())
		pCmdUI->SetCheck(TRUE);
	else
		pCmdUI->SetCheck(FALSE);
}

void TNetworkView::OnUpdateViewConflictors(CCmdUI* pCmdUI) 
{
	if (m_pScreen->GetDrawConflictors())
		pCmdUI->SetCheck(TRUE);
	else
		pCmdUI->SetCheck(FALSE);
}

void TNetworkView::OnUpdateDrawResourceLink(CCmdUI* pCmdUI) 
{
	if (m_pScreen->GetDrawResourceLinks())
		pCmdUI->SetCheck(TRUE);
	else
		pCmdUI->SetCheck(FALSE);
}

void TNetworkView::OnUpdateNetviewAtEnd(CCmdUI* pCmdUI) 
{
	if (m_bDrawAtEndOfActionSelection)
		pCmdUI->SetCheck(TRUE);
	else
		pCmdUI->SetCheck(FALSE);

	if (m_bNoGraphics)
		pCmdUI->Enable(FALSE);
	else
		pCmdUI->Enable(TRUE);
}


void TNetworkView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// deselect previously selected module
	if (m_pSelectedModule != NULL)
		m_pSelectedModule->SetSelected(FALSE);

	// calculate, which module is selected
	m_pSelectedModule = m_pNetwork->GetModuleFromCoordinates(point, m_pScreen);

	// and select it
	if (m_pSelectedModule != NULL)
		m_pSelectedModule->SetSelected();

	// redraw the view
	RedrawView();

	CView::OnLButtonDown(nFlags, point);
}



void TNetworkView::OnNetSqueezeX() 
{
	m_pScreen->StretchX(-1);
	m_pNetwork->SetNormCoordinates();
	m_pNetwork->SetCoordinates(m_pScreen);
	RedrawView();
}

void TNetworkView::OnNetStretchX() 
{
	m_pScreen->StretchX(1);
	m_pNetwork->SetNormCoordinates();
	m_pNetwork->SetCoordinates(m_pScreen);
	RedrawView();
}

void TNetworkView::OnNetSqeezeY() 
{
	m_pScreen->StretchY(-1);
	m_pNetwork->SetNormCoordinates();
	m_pNetwork->SetCoordinates(m_pScreen);
	RedrawView();
}

void TNetworkView::OnNetStretchY() 
{
	m_pScreen->StretchY(1);
	m_pNetwork->SetNormCoordinates();
	m_pNetwork->SetCoordinates(m_pScreen);
	RedrawView();
}


void TNetworkView::OnUpdateNetSqeezeY(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	
}

void TNetworkView::OnUpdateNetSqueezeX(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	
}

void TNetworkView::OnUpdateNetStretchX(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	
}

void TNetworkView::OnUpdateNetStretchY(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	
}

void TNetworkView::OnViewNetZeroLink() 
{
	m_pScreen->SetDrawZeroLinks(!m_pScreen->GetDrawZeroLinks());
	RedrawView();
}

void TNetworkView::OnUpdateViewNetZeroLink(CCmdUI* pCmdUI) 
{
	if (m_pScreen->GetDrawZeroLinks())
		pCmdUI->SetCheck(FALSE);
	else
		pCmdUI->SetCheck(TRUE);
}

void TNetworkView::OnNetAction() 
{
	m_pNetwork->SetThreshold(-100.0);
	if (m_pNetwork->IsBlocksworld())
		m_pNetwork->TBehaviorNetwork::DoActionSelection();
	else
		m_pNetwork->DoActionSelection();
	RedrawView();
}

void TNetworkView::OnUpdateNetAction(CCmdUI* pCmdUI) 
{
}

void TNetworkView::OnNetActionHistory() 
{
	m_pScreen->SetDrawActionHistory(!m_pScreen->GetDrawActionHistory());	
	RedrawView();
}

void TNetworkView::OnUpdateNetActionHistory(CCmdUI* pCmdUI) 
{
	if (m_pScreen->GetDrawActionHistory())
		pCmdUI->SetCheck(TRUE);
	else
		pCmdUI->SetCheck(FALSE);
}

void TNetworkView::SetSelectedModule(TEnergySpreader* current) 
{
	m_pSelectedModule = current;
}

void TNetworkView::SetNetwork(TBehaviorNetwork* pNet)
{
	m_pNetwork = pNet;
}

TScreenParamsNetwork* TNetworkView::GetScreenParams() 
{
	return m_pScreen;
}

void TNetworkView::SetNoGraphics(BOOL val)
{
	m_bNoGraphics = val;
}

BOOL TNetworkView::GetDrawAtEndOfActionSelection() 
{
	return m_bDrawAtEndOfActionSelection;
}

