// TResourceFactory.cpp: implementation of the TResourceFactory class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "NetworkHeader.h"
#include "TResource.h"
#include "TResourceFactory.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TResourceFactory::TResourceFactory()
{

}

TResourceFactory::~TResourceFactory()
{
	// the factory objects are deleted by the base class
}

/*-----------------------------------------------------------------------------
Function: int TResourceFactory::AddResource(LPCTSTR name, CObject* pObj, PCallBackPerception pCallback)
Date    : 15.04.99
-------------------------------------------------------------------------------
Parameter: name of the Resource
			  pointer to the object that contains the truth value function
			  pointer to the truth value function to call
			  amount of resource units needed
Returnval: pointer to the added Resource
Description: creates and adds a new TResource for this proposition
-----------------------------------------------------------------------------*/
TResource* TResourceFactory::AddResource(LPCTSTR name, CObject* pObj
											 , PCallBackPerception pCallback, int iAmount)
{
	TResource* pState;

	pState = new TResource(name, m_pFactoryObjects->GetCount(), pObj, pCallback, iAmount);
	m_pFactoryObjects->AddTail(pState);
	return pState;
}

/*-----------------------------------------------------------------------------
Function: void TResourceFactory::ReduceAllThresholds(double dDeltaTheta)
Date    : 16.04.99
-------------------------------------------------------------------------------
Parameter: threshold reduction parameter
Returnval: (void)
Description: calls all resources threshold reduction method
-----------------------------------------------------------------------------*/
void TResourceFactory::ReduceAllThresholds(double dDeltaTheta)
{
	POSITION	pos = m_pFactoryObjects->GetHeadPosition();
	while (pos != NULL)
		((TResource*) m_pFactoryObjects->GetNext(pos))->ReduceActivationLevel(dDeltaTheta);
}


/*-----------------------------------------------------------------------------
Function: void TResourceFactory::ResetAllResourceUsages()
Date    : 16.04.99
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: calls all resources usage initilizing method
-----------------------------------------------------------------------------*/
void TResourceFactory::ResetAllResourceUsages()
{
	POSITION	pos = m_pFactoryObjects->GetHeadPosition();
	while (pos != NULL)
		((TResource*) m_pFactoryObjects->GetNext(pos))->ResetAmountAvailable();
}
