// TWorldState.cpp: implementation of the TWorldState class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "NetworkHeader.h"
#include "TProposition.h"
#include "TWorldState.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TWorldState::TWorldState(LPCTSTR name, int index, CObject* pObj
								 , PCallBackPerception pCall)
				: TEnergySpreader(name, index)
{
	m_pProposition = new TProposition(this, this, 0.0, FALSE);
	Initialize();
	m_pObject = pObj;
	m_pTruthFunction = pCall;
	m_bFuzzy = TRUE;
	m_dFuzzyRound = 0.5;
}

TWorldState::~TWorldState()
{
	delete m_pProposition;
}

void TWorldState::Initialize()
{
	m_iNoOfOccurencesInPreconditions = 0;
	m_iNoOfOccurencesInInhibitionLinks = 0;
	m_iNoOfOccurencesInExcitationLinks = 0;
	m_bUsed = FALSE;
	SetActivation(0.0);
}

double TWorldState::GetTruthValue()
{
	double	value;

	if ((m_pObject != NULL) && (m_pTruthFunction != NULL))
		value = (m_pObject->*m_pTruthFunction)();

	m_pProposition->SetValue(value);
	return value;
}

double TWorldState::GetMainActivation() const
{
	double	value = TEnergySpreader::GetMainActivation();

#ifdef INPUT_NOISE_ON
	// equal error
	//	value += (1.0 - (double) rand() / ((double) RAND_MAX / 2.0)) * INPUT_NOISE_VALUE;
	// percent error
	value += value * (1.0 - (double) rand() / ((double) RAND_MAX / 2.0)) * INPUT_NOISE_VALUE;
	if (value < 0.0)
		value = 0.0;
	if (value > 1.0)
		value = 1.0;
#endif

	if (m_bFuzzy)
		return value;
	else if (value < m_dFuzzyRound)
		return 0.0;
	else
		return 1.0;
}