// editbar.cpp : implementation file
//
// This is a part of the Microsoft Foundation Classes C++ library.
// Copyright (C) 1992-1997 Microsoft Corporation
// All rights reserved.
//
// This source code is only intended as a supplement to the
// Microsoft Foundation Classes Reference and related
// electronic documentation provided with the library.
// See these sources for detailed information regarding the
// Microsoft Foundation Classes product.

#include "stdafx.h"
#include "NetworkHeader.h"
#include "editbar.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define COMBOBOX_ACTION			4
#define COMBOBOX_PERCEPTION	3
#define COMBOBOX_WIDTH			150
#define COMBOBOX_HEIGHT			200
#define COMBOBOX_BITMAP_A		4
#define COMBOBOX_BITMAP_P		3

static UINT BASED_CODE EditButtons[] =
{
	// same order as in the bitmap 'toolbar.bmp'
	ID_FILE_OPEN_RULES,
	ID_FILE_SAVE_RULES,
	ID_PARSE_RULES,
		ID_SEPARATOR,
		ID_SEPARATOR,
};

/////////////////////////////////////////////////////////////////////////////
// CEditBar

CEditBar::CEditBar()
{
	m_bColor = TRUE;
}

CEditBar::~CEditBar()
{
}

BOOL CEditBar::Init(CWnd* pParentWnd, BOOL bColor, BOOL bToolTips)
{
	m_bColor = bColor;
	m_bVertical = FALSE;

	// start out with no borders
	DWORD dwStyle = WS_CHILD | WS_VISIBLE | CBRS_TOP | CBRS_SIZE_DYNAMIC;
	if (bToolTips)
		dwStyle |= (CBRS_TOOLTIPS | CBRS_FLYBY);
	if (!Create(pParentWnd, dwStyle, IDW_EDIT_BAR))
	{
		return FALSE;
	}
	
	LoadBitmap(IDB_BITMAP_RULE);
	
	if(!SetButtons(EditButtons, sizeof(EditButtons)/sizeof(UINT)))
		return FALSE;
	
	CRect rect(-COMBOBOX_WIDTH, -COMBOBOX_HEIGHT, 0, 0);
	// The ID of the ComboBox is important for two reasons.  One, so you
	// can receive notifications from the control.  And also for ToolTips.
	// During HitTesting if the ToolBar sees that the mouse is one a child
	// control, the toolbar will lookup the controls ID and search for a
	// string in the string table with the same ID to use for ToolTips
	// and StatusBar info.
	if (!m_tActionBox.Create(WS_CHILD | CBS_DROPDOWNLIST |
		CBS_AUTOHSCROLL | CBS_SORT | WS_VSCROLL | CBS_HASSTRINGS, rect, this,
		IDC_EDITBAR_ACTIONBOX))
	{
		return FALSE;
	}

	if (!m_tPerceptionBox.Create(WS_CHILD | CBS_DROPDOWNLIST |
		CBS_AUTOHSCROLL | CBS_SORT | WS_VSCROLL | CBS_HASSTRINGS, rect, this,
		IDC_EDITBAR_PERCEPTIONBOX))
	{
		return FALSE;
	}

	HFONT hFont = (HFONT)GetStockObject(DEFAULT_GUI_FONT);
	if (hFont == NULL)
		hFont = (HFONT)GetStockObject(ANSI_VAR_FONT);

  	m_tActionBox.SendMessage(WM_SETFONT, (WPARAM)hFont);
  	m_tPerceptionBox.SendMessage(WM_SETFONT, (WPARAM)hFont);

	if(!SetHorizontal())
		return FALSE;

	return TRUE;
}

BOOL CEditBar::SetHorizontal()
{
	m_bVertical = FALSE;

	SetBarStyle(GetBarStyle() | CBRS_ALIGN_TOP);

	SetButtonInfo(COMBOBOX_PERCEPTION, IDC_EDITBAR_PERCEPTIONBOX, TBBS_SEPARATOR, COMBOBOX_WIDTH);
	if (m_tPerceptionBox.m_hWnd != NULL)
	{
		CRect rect;
		GetItemRect(COMBOBOX_PERCEPTION, rect);
		
		m_tPerceptionBox.SetWindowPos(NULL, rect.left, rect.top, 0, 0, SWP_NOZORDER|SWP_NOACTIVATE|SWP_NOSIZE|SWP_NOCOPYBITS);
		m_tPerceptionBox.ShowWindow(SW_SHOW);
	}
	
	SetButtonInfo(COMBOBOX_ACTION, IDC_EDITBAR_ACTIONBOX, TBBS_SEPARATOR, COMBOBOX_WIDTH);
	if (m_tActionBox.m_hWnd != NULL)
	{
		CRect rect;
		GetItemRect(COMBOBOX_ACTION, rect);
		
		m_tActionBox.SetWindowPos(NULL, rect.left, rect.top, 0, 0, SWP_NOZORDER|SWP_NOACTIVATE|SWP_NOSIZE|SWP_NOCOPYBITS);
		m_tActionBox.ShowWindow(SW_SHOW);
	}
	
	return TRUE;
}

BOOL CEditBar::SetVertical()
{
	m_bVertical = TRUE;

	SetButtonInfo(COMBOBOX_ACTION, ID_EDIT_FIND, TBBS_BUTTON, COMBOBOX_BITMAP_A);
	if (m_tActionBox.m_hWnd != NULL)
		m_tActionBox.ShowWindow(SW_HIDE);

	SetButtonInfo(COMBOBOX_PERCEPTION, ID_EDIT_FIND, TBBS_BUTTON, COMBOBOX_BITMAP_P);
	if (m_tPerceptionBox.m_hWnd != NULL)
		m_tPerceptionBox.ShowWindow(SW_HIDE);

	return TRUE;
}

CSize CEditBar::CalcDynamicLayout(int nLength, DWORD dwMode)
{
	// if we're committing set the buttons appropriately
	if (dwMode & LM_COMMIT)
	{
		if (dwMode & LM_VERTDOCK)
		{
			if (!m_bVertical)
				SetVertical();
		}
		else
		{
			if (m_bVertical)
				SetHorizontal();
		}

		return CToolBar::CalcDynamicLayout(nLength, dwMode);
	}
	else
	{
		BOOL bOld = m_bVertical;
		BOOL bSwitch = (dwMode & LM_HORZ) ? bOld : !bOld;

		if (bSwitch)
		{
			if (bOld)
				SetHorizontal();
			else
				SetVertical();
		}
		

		CSize sizeResult = CToolBar::CalcDynamicLayout(nLength, dwMode);
		
		if (bSwitch)
		{
			if (bOld)
				SetHorizontal();
			else
				SetVertical();
		}

		return sizeResult;
	}
}

//CSize CEditBar::CalcFixedLayout(BOOL bStretch, BOOL bHorz)
//{
//  	if (!bHorz && (m_dwStyle & CBRS_SIZE_DYNAMIC) && !(m_dwStyle & CBRS_FLOATING))
//	{
//		CSize size;
//
//		size = CControlBar::CalcFixedLayout(bStretch, bHorz);
//
//		CRect rect;
//		rect.SetRectEmpty();
//
//		CalcInsideRect(rect, bHorz);
//
//		size.cx = m_sizeVert.cx + (m_rectInsideVert.Width() - rect.Width());
//		size.cy = m_sizeVert.cy + (m_rectInsideVert.Height() - rect.Height());
//		return size;
//	}
//	else
//	{
//		if (m_bVertical)
//			SetHorizontal();
//		
//		return CToolBar::CalcFixedLayout(bStretch, bHorz);
//	}
//}


CString CEditBar::GetSelectedAction()
{
	CString	text;
	int		id = m_tActionBox.GetCurSel();
	
	if (id >= 0)
		m_tActionBox.GetLBText(id, text);
	return text;
}

CString CEditBar::GetSelectedPerception()
{
	CString	text;
	int		id = m_tPerceptionBox.GetCurSel();
	
	if (id >= 0)
		m_tPerceptionBox.GetLBText(id, text);
	return text;
}

BEGIN_MESSAGE_MAP(CEditBar, CToolBar)
	//{{AFX_MSG_MAP(CEditBar)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CEditBar message handlers

