// GeneticCode.cpp: implementation of the GeneticCode class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Basis.h"
#include ".\BehaviorNet\TParseString.h"
#include "GeneticCode.h"

#include <time.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

IMPLEMENT_SERIAL( GeneticCode, CObject, 1 )

GeneticCode::GeneticCode()
{
	m_age		= 0;
	m_quality	= 0.0;
	m_varianz	= 0.0;
	for (int i = 0; i < SIZE_OF_CODE; i++)
	{
		m_parameters[i] = 0.0;
		m_range[i] = 0.0;
	}
}

GeneticCode::GeneticCode(GeneticCode* code)
{
	m_age		= code->m_age + 1;
	m_quality	= code->m_quality;
	m_varianz	= code->m_varianz;
	for (int i = 0; i < SIZE_OF_CODE; i++)
	{
		m_parameters[i] = code->m_parameters[i];
		m_range[i] = code->m_range[i];
	}
}

GeneticCode::GeneticCode(double quality, double* code, double* range)
{
	m_age		= 0;
	m_quality = quality;
	m_varianz = 0.0;
	for (int i = 0; i < SIZE_OF_CODE; i++)
	{
		m_parameters[i] = code[i];
		m_range[i] = range[i];
	}
}

GeneticCode::~GeneticCode()
{
}


void GeneticCode::Serialize( CArchive& ar )
{
	if (ar.IsStoring())
	{
		// save to archive
		ar << m_quality;
		ar << m_varianz;
		for (int i = 0; i < SIZE_OF_CODE; i++)
		{
			ar << m_parameters[i];
			ar << m_range[i];
		}
	}
	else
	{
		// load from archive
		ar >> m_quality;
		ar >> m_varianz;
		for (int i = 0; i < SIZE_OF_CODE; i++)
		{
			ar >> m_parameters[i];
			ar >> m_range[i];
		}
	}
}

GeneticCode* GeneticCode::CrossOver(GeneticCode* other)
{
	GeneticCode*	newCode = new GeneticCode();

	// randomize the point where the crossing over is done
	int random = (int) ((double) rand() / (double) RAND_MAX * 
						(double)(SIZE_OF_CODE - 2)) + 1;
	
	for (int i = 0; i < SIZE_OF_CODE; i++)
	{
		if (i < random)
		{
			newCode->m_parameters[i] = Mutate(m_parameters[i], m_range[i]);
			newCode->m_range[i] = m_range[i];
		}
		else
		{
			newCode->m_parameters[i] = Mutate(other->m_parameters[i], other->m_range[i]);
			newCode->m_range[i] = other->m_range[i];
		}
	}

	return newCode;
}

double GeneticCode::Mutate(double value, double range)
{
	// first see if we mutate at all
	double random = (double) rand() / (double) RAND_MAX;
	if (random < MUTATION_PROBABILITY)
		// we have a mutation
		return CreateRandomValue(value, range);
	else
		return value;
}

GeneticCode* GeneticCode::CreateRandomCode()
{
	double	parameter[SIZE_OF_CODE];

	// change all parameters passed by a random percentage
	for (int i = 0; i < SIZE_OF_CODE; i++)
	{
		// evenly distribute
		parameter[i] = CreateRandomValue(m_parameters[i], m_range[i]);
	}

	return new GeneticCode(0.0, parameter, m_range);
}

double GeneticCode::CreateRandomValue(double value, double range)
{
	return 	value + range *	((MUTATION_RATE * (double) rand() / (double) RAND_MAX) - MUTATION_RATE / 2.0);
}

CString GeneticCode::ToString()
{
	CString value;
	CString temp;

	value.Format("%5.4f %5.4f %d", m_quality, m_varianz, m_age);

	for (int i = 0; i < SIZE_OF_CODE; i++)
	{
		temp.Format(" %5.4f", m_parameters[i]);
		value += temp;
	}
	value += "\r\n";

	return value;
}

void GeneticCode::FromString(TParseString& code)
{
	m_quality = code.ParseDouble();
	m_varianz = code.ParseDouble();
	m_age = code.ParseInteger();
	
	for (int i = 0; i < SIZE_OF_CODE; i++)
	{
		m_parameters[i] = code.ParseDouble();
		if (code.HasErrorOccured())
		{
			#ifdef _DEBUG
				TRACE1("Error Parsing parameters  %s\r\n", code);
			#endif
			return;
		}
	}
}