// TPerception.cpp: implementation of the TPerception class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "stdbnx.h"
#include "Basis.h"
#include "BasisView.h"
#include "TPerceptionView.h"
#include "TViewingAgent.h"
#include "TThisPlayer.h"
#include "TScreenParams.h"
#include "TAccessMap.h"
#include "TPerceptionParser.h"
#include "TPerception.h"

#include <math.h>
#include <stdio.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TPerception::TPerception(TViewingAgent* pAgent)
{
	m_pAgent = pAgent;
	CreatePerception();
	m_iSide = LEFT_TO_RIGHT;
}

TPerception::TPerception(TViewingAgent* pAgent, TPerceptionView* pView)
{
	m_iSide = LEFT_TO_RIGHT;
	m_pAgent = pAgent;
	m_pView = pView;
	m_pView->SetPerception(this);
	CreatePerception();
}

void TPerception::CreatePerception()
{
	int	i;
	double	dFlagX[NO_OF_FLAGS] = FLAG_POSITIONS_X;
	double	dFlagY[NO_OF_FLAGS] = FLAG_POSITIONS_Y;

	// create the needed objects
	// players
	m_pThisPlayer = new TThisPlayer();
	m_pThisPlayer->SetPlayer(0, 0.0, 0, 0.0, 0, 0, 0, TRUE, m_iSide, 0, FALSE);
	for (i = 0; i < MAX_PLAYERS; i++)
		m_pVisiblePlayer[i] = new TPlayer();

	for (i = 0; i < PLAYERS_PER_TEAM; i++)
	{
		m_pOwnTeam[i] = new TPlayer();
		m_pOtherTeam[i] = new TPlayer();
	}

	// goals (the x coordinates may be swaped later, when it is clear
	// which team the player is
	m_pOwnGoal = new TSoccerGoal(-52.5, 0.0);
	m_pOtherGoal = new TSoccerGoal(52.5, 0.0);
	m_pOwn11mPoint = new TFlag(-41.5, 0.0);
	m_pOther11mPoint = new TFlag(41.5, 0.0);

	// flags
	for (i = 0; i < NO_OF_FLAGS; i++)
		m_pFlag[i] = new TFlag(dFlagX[i], dFlagY[i]);

	// lines
	m_pLine = new TLine();

	// ball
	m_pBall = new TBall();
	
	for(i = 0; i < MAX_FUTURE_POSITIONS; i++)
		m_pFutureBallPosition[i] = new TVector(0.0, 0.0);

	// access map
	m_pAccessMap = new TAccessMap();
	m_pDirectionMap = new TAccessMap();

	m_bWaitForLastSenseBody = FALSE;
	m_bReceivedSee = FALSE;
	m_iNoOfPlayers = 0;
	m_iPlayMode = BEFORE_KICK_OFF;
	m_iViewQuality = 1;
	m_iViewAngle = 90;
	m_iSeeTime = 0;
	m_iHearTime = 0;
	m_iSenseTime = 0;
	m_iSeeThisCycle = 0;
	m_iLastCatchTime = 0;
	m_bTimeProceeding = FALSE;
	m_iLastHearNumber = 0;
	m_bGoalieCaught = FALSE;
	m_bKnowNotInOffside = TRUE;
	m_dInOffside = 0.0;
	m_dOffsideMinus = 0.1;
	m_iCheckOffsideCounter = 0;

	SetStamina(MAX_STAMINA);
	m_dEffort = 1.0;
	m_dSpeed = 0.0;
	m_iSpeedAngle = 0;
	m_iNoOfKick = 0;
	m_iNoOfDash = 0;
	m_iNoOfTurn = 0;
	m_iNoOfSay = 0;
	m_iNoOfTurnNeck = 0;
	m_iHeadAngle = 0;

	m_iExpectedNoOfKick = 0;
	m_iExpectedNoOfDash = 0;
	m_iExpectedNoOfTurn = 0;
	m_iExpectedNoOfSay = 0;
	m_iExpectedNoOfTurnNeck = 0;
	m_iTurnAngle = 0;
	m_iTurnAngleNeck = 0;

	m_pTeammateAtBall = NULL;
	m_p2TeammateAtBall = NULL;
	m_pTeammateAtMe = NULL;
	m_pFrontmostTeammate = NULL;
	m_pBackmostTeammate = NULL;
	m_p2BackmostTeammate = NULL;
	m_pFreeTeammate = NULL;

	m_pOpponentAtBall = NULL;
	m_pOpponentAtMe = NULL;
	m_p2OpponentAtMe = NULL;
	m_p3OpponentAtMe = NULL;
	m_pOpponentToMark = NULL;
	m_pBackmostOpponent = NULL;
	m_p2BackmostOpponent = NULL;

	m_dNoise = PERCEPTION_NOISE_VALUE;
	m_iNoOfCommandsNotExecuted = 0;
	m_tExpectedBodyDirection = 0;
	m_tExpectedViewDirection = 0;

	m_perceptionParser = new TPerceptionParser();

	for (i = 0; i < NO_OF_PLAYER_TYPES; i++)
	{
		m_playerType[i][player_speed_max]	= PLAYER_SPEED_MAX;
		m_playerType[i][stamina_inc_max]	= STAMINA_INCREMENT;
		m_playerType[i][player_decay]		= PLAYER_DECAY;
		m_playerType[i][inertia_moment]		= 5.0;
		m_playerType[i][dash_power_rate]	= DASH_POWER_RATE;
		m_playerType[i][player_size]		= PLAYER_SIZE;
		m_playerType[i][kickable_margin]	= KICKABLE_MARGIN;
		m_playerType[i][kick_rand]			= 0.0;
		m_playerType[i][extra_stamina]		= 0.0;
		m_playerType[i][effort_max]			= 1.0;
		m_playerType[i][effort_min]			= 0.6;
	}

	m_haveAllPlayerTypes = FALSE;
	m_currentPlayerType = 0;
	m_iSawAll = 0;
}

TPerception::~TPerception()
{
	int	i;

	// delete visible objects
	delete m_pThisPlayer;
	for (i = 0; i < MAX_PLAYERS; i++)
		delete m_pVisiblePlayer[i];

	for (i = 0; i < PLAYERS_PER_TEAM; i++)
	{
		delete m_pOwnTeam[i];
		delete m_pOtherTeam[i];
	}

	// delete the flags
	for (i = 0; i < NO_OF_FLAGS; i++)
		delete m_pFlag[i];
	
	// delete the lines
	delete m_pLine;

	delete m_pOwn11mPoint;
	delete m_pOther11mPoint;
	delete m_pOwnGoal;
	delete m_pOtherGoal;
	delete m_pBall;
	for(i = 0; i < MAX_FUTURE_POSITIONS; i++)
		delete m_pFutureBallPosition[i];

	// access map
	delete m_pAccessMap;
	delete m_pDirectionMap;

	// version 7
	delete m_perceptionParser;
}

/*-----------------------------------------------------------------------------
Function: void TPerception::Initialize()
Date    : 6.5.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: initialize does reset the time the agent has seen the ball or any player
				 this is needed for the absolute view of perceptions and mind
-----------------------------------------------------------------------------*/
void TPerception::Initialize()
{
	int	i;

	// initialize the ball
	m_pBall->Initialize();

	// initializes the players
	for (i = 0; i < PLAYERS_PER_TEAM; i++)
	{
		m_pOwnTeam[i]->Initialize();
		m_pOtherTeam[i]->Initialize();
	}	

	// empty the message pane
}

/*-----------------------------------------------------------------------------
Function: void TPerception::InvalidateAll()
Date    : 17.12.97
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: invalidates all visible objects. Invalidate means, they are not drawn
				 if they are not perceived in this cycle
-----------------------------------------------------------------------------*/
void TPerception::InvalidateAll()
{
	int	i;

	// ThisPlayer does not get invalidated, because it is always drawn
	// Invalidate Players
	m_iNoOfPlayers = 0;

	// invalidate remembered players
	for (i = 0; i < PLAYERS_PER_TEAM; i++)
	{
		m_pOwnTeam[i]->SetValid(FALSE);
		m_pOtherTeam[i]->SetValid(FALSE);
	}

	// and other objects
	for (i = 0; i < NO_OF_FLAGS; i++)
		m_pFlag[i]->SetValid(FALSE);

	m_pLine->SetValid(FALSE);

	m_pOwnGoal->SetValid(FALSE);
	m_pOtherGoal->SetValid(FALSE);
	m_pBall->SetValid(FALSE);
}


/*-----------------------------------------------------------------------------
Function: void TPerception::UpdatePerception()
Date    : 23.03.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: called each 100 ms (when an action should be selected) to update
				 the moveable objects positions, if no perception has been received
				 since the last action selection
-----------------------------------------------------------------------------*/
void TPerception::UpdatePerception()
{
	int	i;
	int	iDataAge = m_iSenseTime - m_iSeeTime;

	// update all moveable objects
	m_pBall->UpdatePosition(iDataAge, m_iSeeTime);
//	m_pAgent->DisplayBallPosition(m_pBall->GetPosition(), m_pBall->GetAbsSpeed(), FALSE);

	for (i = 0; i < PLAYERS_PER_TEAM; i++)
	{
		m_pOwnTeam[i]->UpdatePosition(iDataAge, m_iSeeTime);
		m_pOtherTeam[i]->UpdatePosition(iDataAge, m_iSeeTime);

		// and debug positions
		if (m_pOwnTeam[i]->IsInMind())
			m_pAgent->DisplayPlayerPosition(m_pOwnTeam[i]->GetPosition(), i, FALSE);
		if (m_pOtherTeam[i]->IsInMind())
			m_pAgent->DisplayPlayerPosition(m_pOtherTeam[i]->GetPosition(), i + 11, FALSE);
	}

	// update the players own position and direction according to the
	// current speed.
	m_pThisPlayer->UpdatePlayerCoordinates(iDataAge, m_dSpeed, m_pAgent->IsDashingBackward());
//	m_pAgent->DisplayPosition(m_pThisPlayer->GetPosition(), m_pThisPlayer->GetAbsBodyDir()
//										, m_pThisPlayer->GetAbsDir(), FALSE);
}

/*-----------------------------------------------------------------------------
Function: void TPerception::DrawPerception(CDC* pDC, TScreenParams *tScreen)
Date    : 13.11.97
-------------------------------------------------------------------------------
Parameter: pointer to the device context to draw into
Returnval: (void)
Description: draws the last received perception
-----------------------------------------------------------------------------*/
void TPerception::DrawPerception(CDC* pDC, TScreenParams *tScreen)
{
	int		i;

	tScreen->SetViewAngle(m_iViewAngle);
	if (tScreen->IsRelativeView())
	{
		// draw from the players perspective
		// sort the objects depending on their distance
		CObList	drawQueue;
		
		Insert(drawQueue, m_pOwnGoal);
		Insert(drawQueue, m_pOtherGoal);

		// and the flags
		for (i = 0; i < NO_OF_FLAGS; i++)
			Insert(drawQueue, m_pFlag[i]);

		// draw the directly perceived line tags
		Insert(drawQueue, m_pLine);

		// draw the visible players
		for (i = 0; i < m_iNoOfPlayers; i++)
			Insert(drawQueue, m_pVisiblePlayer[i]);

		// draw the ball
		Insert(drawQueue, m_pBall);

		// draw the visible area
		m_pThisPlayer->DrawVisibleArea(pDC, tScreen);

		// and draw them
		POSITION pos = drawQueue.GetHeadPosition();
		while (pos != NULL)
			((TVisibleObject*) drawQueue.GetNext(pos))->DrawObject(pDC, tScreen);
	}
	else if (tScreen->IsTopView())
	{
		// draw the visible area
		m_pThisPlayer->DrawVisibleArea(pDC, tScreen);

		// draw the goals
		m_pOwnGoal->DrawGoal(pDC, tScreen, GetObjectDistance(m_pOwnGoal));
		m_pOtherGoal->DrawGoal(pDC, tScreen, GetObjectDistance(m_pOtherGoal));

		// and the flags
		for (i = 0; i < NO_OF_FLAGS; i++)
			m_pFlag[i]->DrawObject(pDC, tScreen);

		// draw the directly perceived line tags
		m_pLine->DrawObject(pDC, tScreen);

		// draw the visible players
		for (i = 0; i < m_iNoOfPlayers; i++)
			m_pVisiblePlayer[i]->DrawObject(pDC, tScreen);

		// draw this player
		m_pThisPlayer->DrawObject(pDC, tScreen);

		// draw the ball
		m_pBall->DrawObject(pDC, tScreen);
	}
	else
	{
		TFlag		pFlag;
		TVector	pos;
		TVector	seePos;

		// set the current time (for absolute view)
		tScreen->SetTime(m_iSeeTime);

		// draw the visible area
		m_pThisPlayer->DrawVisibleArea(pDC, tScreen);

		// draw the access map
		if (tScreen->GetDrawAccessMap())
		{
			m_pAccessMap->DrawMap(pDC, tScreen);
		}

		// draw the direction map
		if (tScreen->GetDrawDirectionMap())
		{
			m_pDirectionMap->DrawMap(pDC, tScreen);
		}

		// draw the directions to free players
		if (tScreen->GetDrawFreePlayers())
		{
			for (i = 0; i < PLAYERS_PER_TEAM; i++)
			{
				if (!m_pOwnTeam[i]->IsThisAgent() 
					&& (m_pOwnTeam[i]->GetFreePassDegree() > 0.1))			
						m_pThisPlayer->DrawPassArea(pDC, tScreen, m_pOwnTeam[i]);
			}
		}

		// and the flags
		for (i = 0; i < NO_OF_FLAGS; i++)
		{
			// draw real position
			m_pFlag[i]->DrawObject(pDC, tScreen);

			// and perceived position
			seePos = m_pFlag[i]->GetAbsPosition(m_pThisPlayer->GetPosition()
															, m_pThisPlayer->GetAbsDir());
			pFlag.SetPosition(seePos);
			pFlag.SetValid(m_pFlag[i]->IsValid());
			pFlag.SetTime(m_iSeeTime);
			pFlag.SetUsedForLocalization(FALSE);
			pFlag.DrawObject(pDC, tScreen);
		}

		// draw the players of the agents mind
		for (i = 0; i < PLAYERS_PER_TEAM; i++)
		{
			if (!m_pOwnTeam[i]->IsThisAgent())			
				m_pOwnTeam[i]->DrawObject(pDC, tScreen);
			m_pOtherTeam[i]->DrawObject(pDC, tScreen);
		}

		// draw this player
		m_pThisPlayer->DrawObject(pDC, tScreen);

		// draw future positions of the ball
		if (tScreen->GetDrawFutureBall())
		{
			TBall		fBall;
			TPlayer	fAgent;
			BOOL		bDrawText = tScreen->GetDrawDegreeOfFreeness();
			TVector	position;
			int		timeAhead;

			tScreen->SetDrawDegreeOfFreeness(FALSE);
			for (i = 0; i < MAX_FUTURE_POSITIONS / 2 - 1; i++)
			{
				timeAhead = i * 2 + 2;
				fBall.SetValid(FALSE);
				fBall.SetOnceSeen();
				fBall.SetInMind();
				fBall.SetPosition(*m_pFutureBallPosition[timeAhead]);
				fBall.DrawObject(pDC, tScreen);
				m_pThisPlayer->GetFuturePosition(timeAhead, position);
				fAgent.SetValid(FALSE);
				fAgent.SetOnceSeen();
				fAgent.SetInMind();
				fAgent.SetPosition(position);
				fAgent.DrawObject(pDC, tScreen);
			}
			tScreen->SetDrawDegreeOfFreeness(bDrawText);
		}

		// draw the ball
		m_pBall->DrawObject(pDC, tScreen);
	}
}

/*-----------------------------------------------------------------------------
Function: void TPerception::Insert(CObList &list, TVisibleObject* obj)
Date    : 12.02.98
-------------------------------------------------------------------------------
Parameter: reference to the draw list
			  pointer to the object, that has to be inserted
Returnval: (void)
Description: inserts sorted, by distance of the object, descending. used for drawing
-----------------------------------------------------------------------------*/
void TPerception::Insert(CObList &list, TVisibleObject* obj)
{
	TVisibleObject*	compare;
	
	// only valid objects are drawn
	if (!obj->IsValid())
		return;

	POSITION oldPos;
	POSITION	pos = list.GetHeadPosition();
	while(pos != NULL)
	{
		oldPos = pos;
		compare = (TVisibleObject*) list.GetNext(pos);
		if (obj->GetDistance() >= compare->GetDistance())
		{
			list.InsertBefore(oldPos, obj);
			return;
		}
	}

	// it is the nearest object so far
	list.AddTail(obj);
}

/*-----------------------------------------------------------------------------
Function: void TPerception::SetSide(int side) 
Date    : 08.03.99
-------------------------------------------------------------------------------
Parameter: side on which the player is playing
Returnval: (void)
Description: 
-----------------------------------------------------------------------------*/
void TPerception::SetSide(int side) 
{ 
	m_iSide = side; 
	m_pThisPlayer->SetTeam(TRUE, side); 
}

/*-----------------------------------------------------------------------------
Function: void TPerception::PlayStep(LPCTSTR msg)
Date    : 19.05.98
-------------------------------------------------------------------------------
Parameter: pointer to a string containing the message (perception string)
Returnval: (void)
Description: plays a step of the stored logfile, by calling the agents onreceivedmessage
-----------------------------------------------------------------------------*/
void TPerception::PlayStep(CStringList* pList)
{
	POSITION pos = pList->GetHeadPosition();
	CString	temp;

/*	
	// parse the see part
	if (pos != NULL)
	{
		temp = pList->GetNext(pos);
		m_pAgent->OnReceivedMessage(temp, FALSE);
	}
*/
	// parse all messages of this cycle
	while (pos != NULL)
	{
		temp = pList->GetNext(pos);
		m_pAgent->OnReceivedMessage(temp, FALSE);
	}

	// print all other messages of this cycle
	pos = pList->GetHeadPosition();
	while (pos != NULL)
	{
		temp = pList->GetNext(pos);
		m_pAgent->OnSentMessage(temp);
	}
}

/*-----------------------------------------------------------------------------
Function: int	TPerception::SwitchViewAngle(void)
Date    : 19.12.97
-------------------------------------------------------------------------------
Parameter: none
Returnval: the new view angle
Description: switches the internal representation of the viewing angle from 
				 180 degrees to 90 degrees to 45 degrees
-----------------------------------------------------------------------------*/
int	TPerception::SwitchViewAngle()
{ 
	if (m_iViewAngle == 180)
		m_iViewAngle = 90;
	else if (m_iViewAngle == 90)
		m_iViewAngle = 45;
	else
		m_iViewAngle = 180;
	
	return m_iViewAngle;
}	

/*-----------------------------------------------------------------------------
Function: void	TPerception::SwitchViewQuality(void)
Date    : 19.12.97
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: switches the view quality from high to low and back to high
-----------------------------------------------------------------------------*/
int TPerception::SwitchViewQuality()
{
	if (m_iViewQuality == 0)
		m_iViewQuality = 1;
	else
		m_iViewQuality = 0;

	return m_iViewQuality;
}

/*-----------------------------------------------------------------------------
Function: void TPerception::NewMessage(LPCTSTR str)
Date    : 05.02.98
-------------------------------------------------------------------------------
Parameter: pointer to the message
Returnval: (void)
Description: calls the views function for new message,which stores it and calls 
				 ParseMessage
-----------------------------------------------------------------------------*/
void TPerception::NewMessage(LPCTSTR str)
{
	m_pView->NewMessage(str);
}

/*-----------------------------------------------------------------------------
Function: BOOL TPerception::ParseMessage(LPCTSTR str)
Date    : 07.11.97
-------------------------------------------------------------------------------
Parameter: string containing the perception (max 1024 char)
Returnval: TRUE, if no parsing errors occured
Description: distinguishes between see and hear messages and calls the appropriate
				 specialized parsing functions
-----------------------------------------------------------------------------*/
BOOL TPerception::ParseMessage(LPCTSTR str)
{
	switch(str[1])
	{
	case 's':
		switch(str[3])
		{
		case 'n':
			// a sense body information
			return ParseSenseBody(str);

		case 'e':
			if (str[4] != '_')
			{
				if (SERVER_VERSION < 7)
				{
					if (m_pAgent->IsCoach())
					{
						// a see (see_global in version 7) of the coach
						if (!ParseSeeGlobal(str))
							return FALSE;	
						return TRUE;
					}
				}

				// a see message
				if (!ParseSee(str))
					return FALSE;

				WorldModel(str);
				return TRUE;
			}
			else
			{
				// a see_global of the coach
				if (!ParseSeeGlobal(str))
					return FALSE;	
				return TRUE;
			}

		case 'o':
			// a score message

		default:
			// a server param message
			break;
		}
		break;

	case 'h':
		// something to hear
		return ParseHear(str);
	
	case 'c':
	{
		// changed a player
		int	changePlayer[2] = {-1, -1};
		if (m_perceptionParser->ParseChangedPlayer(str, changePlayer))
		{
			if (changePlayer[0] == m_pAgent->GetNumber())
			{
				// it was this player so change the type
				m_currentPlayerType = changePlayer[1];

				CString msg;
				msg.Format("Changed Type: %d  speedMax: %4.2f  extraStamina: %d", 
					m_currentPlayerType, 
					m_playerType[m_currentPlayerType][player_speed_max], 
					m_playerType[m_currentPlayerType][extra_stamina] 
					);
				DisplayDebug(msg, KATEGORY_PERCEPTION, DEBUG_HIGHPERCEPTION);

				m_pThisPlayer->SetMaxSpeed(m_playerType[m_currentPlayerType][player_speed_max]);
			}
		}
		else
		{
			// other team changed
		}

		break;
	}

	case 'i':
		// message at init
		ParseInit(str);
		break;

	case 'p':
		// player parameters
		if (str[8] == 'p')
		{
			// player_param
			m_perceptionParser->ParseParameters(str, PLAYER_PARAM_COUNT, m_playerParam);
		}
		else
		{
			// player_type
			// get the typeid
			CString message = str;
			int type = message.GetAt(13) - '0';
			ASSERT((type >= 0) && (type < NO_OF_PLAYER_TYPES));
			m_perceptionParser->ParseParameters(str, PLAYER_TYPE_COUNT, m_playerType[type]);
			if (type == NO_OF_PLAYER_TYPES - 1)
				m_haveAllPlayerTypes = TRUE;
		}

		break;

	case 'r':
		// message at reconnect
		return TRUE;
	}
	return TRUE;
}


/*-----------------------------------------------------------------------------
Function: BOOL TPerception::ParseMessage(LPCTSTR str)
Date    : 07.11.97
-------------------------------------------------------------------------------
Parameter: string containing the perception (max 1024 char)
Returnval: TRUE, if no parsing errors occured
Description: distinguishes between see and hear messages and calls the appropriate
				 specialized parsing functions
-----------------------------------------------------------------------------*/
void TPerception::WorldModel(LPCTSTR str)
{		
	// there is a working connection
	if (!m_bReceivedSee)
	{
		m_bReceivedSee = TRUE;
		m_pAgent->OnReceivedSee();
	}

	// agent got a see
	m_pAgent->SetWaitForNextSee(FALSE);

	// get the absoulte localization of the player
	Localize();
//				LocalizeCassini();

	// check if we received a see that does not take our last turn_neck into account
	if ((m_iNoOfTurnNeck < m_iExpectedNoOfTurnNeck) && (m_iSeeTime == m_iSenseTime)
		&& (m_iSeeTime > 0))
	{
			// one thing more here:
			// the Localize method uses the new expected head angle to calculate 
			// the body direction. Therefore we have to add the head angle again
			// since the turn neck has not been executed by now
			m_pThisPlayer->SetAbsBodyDirection(m_pThisPlayer->GetAbsBodyDir() 
						+ m_iTurnAngleNeck);

			UpdatePerceptionAfterTurnNeck(m_iTurnAngleNeck);
	}

/*				CString	msg;
	int		iTurnPrecision = m_iTurnAngleNeck / 10 + 3;

	if (iTurnPrecision < 0)
		iTurnPrecision = -iTurnPrecision;
	
	if (m_iNoOfTurnNeck < m_iExpectedNoOfTurnNeck)
	{
		// if turn and turn neck are same absolute value but different direction
		// we can not detect, if turn worked, so we use old style detection
		
		TDirection viewDir = m_pThisPlayer->GetAbsDir();
		// check if we look into the expected direction
		if (!viewDir.LiesWithin(m_tExpectedViewDirection - iTurnPrecision
									, m_tExpectedViewDirection + iTurnPrecision))
		{
			// our turn neck is not executed on the server, but we do it in mind
			msg.Format("Received old see: view dir: %d expected: %d"
						, (int) viewDir, (int) m_tExpectedViewDirection);
			DisplayDebug(msg, KATEGORY_PERCEPTION, DEBUG_LOWPERCEPTION);

			// one thing more here:
			// the Localize method uses the new expected head angle to calculate 
			// the body direction. Therefore we have to add the head angle again
			// since the turn neck has not been executed by now
			m_pThisPlayer->SetAbsBodyDirection(m_pThisPlayer->GetAbsBodyDir() 
						+ m_iTurnAngleNeck);

			UpdatePerceptionAfterTurnNeck(m_iTurnAngleNeck);
		}
	}
*/				
	// same for turn
	if ((m_iNoOfTurn < m_iExpectedNoOfTurn) && (m_iSeeTime == m_iSenseTime) 
			&& (m_iSeeTime > 0))
			UpdatePerceptionAfterTurn(m_iTurnAngle);
/*
	iTurnPrecision = m_iTurnAngle / 10 + 3;
	if (iTurnPrecision < 0)
		iTurnPrecision = -iTurnPrecision;

	if (m_iNoOfTurn < m_iExpectedNoOfTurn)
	{
		TDirection bodyDir = m_pThisPlayer->GetAbsBodyDir();
		// check if we look into the expected direction
		if (!bodyDir.LiesWithin(m_tExpectedBodyDirection - iTurnPrecision
									, m_tExpectedBodyDirection + iTurnPrecision))
		{
			msg.Format("Received old see: body dir: %d expected: %d"
						, (int) bodyDir, (int) m_tExpectedBodyDirection);
			DisplayDebug(msg, KATEGORY_PERCEPTION, DEBUG_LOWPERCEPTION);

			UpdatePerceptionAfterTurn(m_iTurnAngle);
		}
	}
*/
	
	// and for dash
	if ((m_iNoOfDash < m_iExpectedNoOfDash) && (m_iSeeTime == m_iSenseTime)
		&& (m_iSeeTime > 0))
		UpdatePerceptionAfterDash(m_iDashPower);

	// calculate the absolute positions of the other moveable objects perceived
	GlobalMap();

	// for kick it is done after GlobalMap, because it changes speed vector
	// of the ball which would be overwritten by GlobalMap
	if ((m_iNoOfKick < m_iExpectedNoOfKick) && (m_iSeeTime == m_iSenseTime)
		&& (m_iSeeTime > 0) && m_pBall->IsValid())
		UpdatePerceptionAfterKick(m_iKickPower, m_iKickDirection);
		
	// try to match the perceived players to remembered players
	MatchPlayers();

	// remove objects not seen, but should be seen
	CheckVisualPlausibility();

	// create access map
	CreateAccessMap();

	// calculate the objects functions in the world such as opponentNearBall
	CalculateFunctionalObjects();
}


/*-----------------------------------------------------------------------------
Function: BOOL TPerception::ParseSee(LPCTSTR str)
Date    : 07.11.97
-------------------------------------------------------------------------------
Parameter: reference of the string to parse
			  reference to the current position
Returnval: TRUE, if no parsing errors occured
Description: parses a see perception
-----------------------------------------------------------------------------*/
BOOL TPerception::ParseSee(LPCTSTR str)
{
	TParseString	msg(str);
	TParseString	strObject;
	int				time;
	
	m_seeCount++;
	m_iSeeThisCycle--;
	m_pAgent->SetExpectedSee(m_iSeeThisCycle);

	if (m_seeCount >= 2)
	{
		// ignore second see in a cycle, since it is almost always identical
		CString message;
		message.Format("ignoer see: %d", m_seeCount);
		DisplayDebug(message, KATEGORY_PERCEPTION, DEBUG_LOWPERCEPTION);
		return FALSE;
	}

	// delete previously seen players
	InvalidateAll();

	// get the time
	msg.SetPosition(5);
	time = msg.ParseInteger();
	if (msg.HasErrorOccured())
	{
		#ifdef _DEBUG
			TRACE1("Error Parsing seetime  %s\r\n", msg);
		#endif
		return FALSE;
	}

	// if the time is running, then the game is on
//	if ((time > m_iSeeTime) && (time > m_iHearTime + 1))
//		m_iPlayMode = PLAY_ON;

	m_iSeeTime = time;
	m_pThisPlayer->SetTime(m_iSeeTime);
	if (this->GetViewAngle() == 180)
		m_pAgent->IncreaseSeeWideCount();

	// get the visible objects
	while (msg.ParseBracket(strObject))
	{
		if (!ParseObject(strObject))
		{
			#ifdef _DEBUG
				CString error;
				error.Format(" : error parsing object info at position %d !\n", strObject.GetPosition());
				afxDump << error << strObject;
			#endif
		}
	};

	// may be not true to return True because of parsebracket
	return TRUE;
}

/*-----------------------------------------------------------------------------
Function: BOOL TPerception::ParseHear(LPCTSTR str)
Date    : 10.11.97
-------------------------------------------------------------------------------
Parameter: reference to the current position in the string which is 0
Returnval: TRUE, if no parse error occured
Description: parses a hear perception
-----------------------------------------------------------------------------*/
BOOL TPerception::ParseHear(LPCTSTR str)
{
	TParseString	msg(str);
	CString			tmp;

	// get the time
	msg.SetPosition(5);
	m_iHearTime = msg.ParseInteger();
	if (msg.HasErrorOccured())
	{
		#ifdef _DEBUG
			TRACE1("Error Parsing heartime  %s\r\n", msg);
		#endif
		return FALSE;
	}

	// get the source
	switch (msg.GetAt(msg.GetPosition()))
	{
		case 'r':
			// it is from the referee
			if (!msg.ParseString(tmp))
				return FALSE;
			// get the message
			if (!msg.ParseString(tmp))
				return FALSE;
	
			if (tmp.Compare(m_pAgent->GetSideIdentifier()) == 0)
				// it is a message from the offline coach (only version < 7)
				ParseOfflineCoachMessage(msg);
			else
				ParsePlayMode(tmp);

			break;
		
		case 's':
			// it has been sent by myself
			break;
		
		case 'c':
			// it is from the referee
			if (!msg.ParseString(tmp))
				return FALSE;
			// get the message
			if (!msg.ParseString(tmp))
				return FALSE;
			// it has been sent by the coach
			ParseOfflineCoachMessage(msg);
			break;
		
		default:
			// it is a say message
			if (!ParseSayMessage(&msg))
				return FALSE;
			break;
	}

	return TRUE;
}

void TPerception::ParsePlayMode(CString tmp)
{
	int				mode = 0;
	char*			strPlayMode[] = PLAYMODE_STRINGS;

	while ((mode < NO_OF_PLAYMODES) && (tmp.Compare(strPlayMode[mode])))
		mode++;

	if (mode < NO_OF_PLAYMODES)
	{
		// set the playmode
		m_iPlayMode = mode;
		if ((mode == CATCH_L) && (GetSide() == LEFT_TO_RIGHT))
			m_bGoalieCaught = TRUE;
		else if ((mode == CATCH_R) && (GetSide() == RIGHT_TO_LEFT))
			m_bGoalieCaught = TRUE;
		else if (mode == PLAY_ON)
			m_bGoalieCaught = FALSE;
		if ((mode == GOAL_L) || (mode == GOAL_R))
		{
			// set the new ball position
			m_pBall->SetPosition(TVector(0, 0));
			m_pBall->SetPosition(TVector(0, 0));
			m_pBall->SetPosition(TVector(0, 0));
			m_pBall->SetLastSeePosition();
			m_pBall->SetAbsSpeed(TVector(0.0, 0.0));
		}
	}
	else
	{
		// check if it is a goal, because goals have different text
		if (tmp.Left(6).Compare(strPlayMode[GOAL_L]) == 0)
			m_iPlayMode = GOAL_L;
		else if (tmp.Left(6).Compare(strPlayMode[GOAL_R]) == 0)
			m_iPlayMode = GOAL_R;
	}
}

BOOL TPerception::ParseOfflineCoachMessage(TParseString& msg)
{
	// it is a genetic code message from the coach
	m_pAgent->SetGeneticCode(msg);
	return TRUE;
}


/*-----------------------------------------------------------------------------
Parameter: reference of the string to parse
Returnval: TRUE, if no parsing errors occured
Description: parses a see_global perception of the coach
-----------------------------------------------------------------------------*/
BOOL TPerception::ParseSeeGlobal(LPCTSTR str)
{
	TParseString	msg(str);
	TParseString	strObject;
	
	// get the time
	if (SERVER_VERSION < 7)
		msg.SetPosition(4);
	else
		msg.SetPosition(11);

	m_iSeeTime = msg.ParseInteger();
	m_iSenseTime = m_iSeeTime;

	// get the visible objects
	while (msg.ParseBracket(strObject))
	{
		if (!ParseObjectGlobal(strObject))
		{
			#ifdef _DEBUG
				CString error;
				error.Format(" : error parsing object info at position %d !\n", strObject.GetPosition());
				afxDump << error << strObject;
			#endif
		}
	}

	// may be not true to return True because of parsebracket
	return TRUE;
}

/*-----------------------------------------------------------------------------
Function: ParseSayMessage(TParseString msg)
Date    : 27.05.1999
From    : Markus Plewinski
-------------------------------------------------------------------------------
Parameter: pointer on the message
Returnval: true if no error
Description: parses a say message from a teammate
-----------------------------------------------------------------------------*/
BOOL TPerception::ParseSayMessage(TParseString *msg)
{
	int		iRecipient;
	int		iDispatcher;
	int		iStamina;
	CString	tmp;

	//overread direction
	msg->ParseInteger();

	//check if message from own team
	if (!msg->ParseString(tmp))
		return FALSE;
	if (m_pAgent->GetSideIdentifier().Compare(tmp.Right(tmp.GetLength() - 1)))
		return TRUE;

	iRecipient = msg->ParseInteger();	//number of recipient 0 means for all
	if ( (iRecipient > 11) || (iRecipient < 0) )
		return FALSE;

	iDispatcher = msg->ParseInteger();	//dispatcher of the message
	if ( (iDispatcher > 11) || (iDispatcher < 1) || (iDispatcher == m_pThisPlayer->GetNum()))
		return FALSE;

	// store last speaker
	m_iLastHearNumber = iDispatcher;

	iStamina = msg->ParseInteger();		//dispatchers stamina
	m_pOwnTeam[iDispatcher-1]->SetStamina(iStamina);
	m_pOwnTeam[iDispatcher -1]->SetHearTimePlayer(m_iHearTime);

	//type of the message
	if (!msg->ParseString(tmp))
		return FALSE;
	switch (tmp.GetAt(0))
	{
		case 'P':		//say position message
//			if (m_pOwnTeam[iRecipient]->IsThisAgent())
				if (!ParseMsgPosition(msg))
					return FALSE;
			break;

		default:
		// it is a unknown message
			break;
	}

	return TRUE;
}

/*-----------------------------------------------------------------------------
Function: ParseMsgPosition(TParseString msg)
Date    : 27.05.1999
From    : Markus Plewinski
-------------------------------------------------------------------------------
Parameter: hearn message
Returnval: true in no error
Description: parses a position body
-----------------------------------------------------------------------------*/
BOOL TPerception::ParseMsgPosition(TParseString *msg)
{
	int		i;
	int		length;
	int		iPlayerNr;
	double	dXpos;
	double	dYpos;
	int		iXpos;
	int		iYpos;
	int		iAge;
	int		sawAll;

	//number of objects 2*11 players + 1 ball
	length = msg->ParseInteger();
	if ( (length > 23) || (length < 0) )
		return FALSE;

	// if > 0 the information should be complete field 
	sawAll = msg->ParseInteger();
	// if we have a complete overview, we can be sure that we are in offside if
	// no player or ball is preventing us
	if (sawAll > m_iSawAll)
	{
		KnowOffside();
		m_iSawAll = sawAll;
	}

	for (i = 0; i < length; i++)
	{
		//gets the position and the object number
		//0 for ball
		//1 - 11 for own team
		//21 - 31 for other team
		iPlayerNr = msg->ParseInteger();
		iXpos = msg->ParseInteger();
		iYpos = msg->ParseInteger();
		iAge  = msg->ParseInteger() + 1;
		dXpos = (double) iXpos / 10.0;
		dYpos = (double) iYpos / 10.0;

		if ( (iPlayerNr < 0) || (iPlayerNr > 31) 
			|| ((iPlayerNr > 11) && (iPlayerNr < 21)) )
			return FALSE;
		if (iPlayerNr == 0)			//ball
		{
//			if (!m_pBall->IsValid())
			if (m_iSeeTime - m_pBall->GetTime() > iAge)
			{
				m_pBall->SetPosition(TVector(dXpos, dYpos));
				m_pBall->SetInMind();
				m_pBall->SetTime(m_iHearTime - iAge);
			}
		}
		else if (iPlayerNr < 20)	//players from own team
		{
			iPlayerNr -= 1;
//			if (!m_pOwnTeam[iPlayerNr]->IsValid() )
			if (m_iSeeTime - m_pOwnTeam[iPlayerNr]->GetTime() > iAge)
			{
				m_pOwnTeam[iPlayerNr]->SetPosition(TVector(dXpos, dYpos));
				m_pOwnTeam[iPlayerNr]->SetInMind();
				m_pOwnTeam[iPlayerNr]->SetTime(m_iHearTime - iAge);	
				m_pOwnTeam[iPlayerNr]->SetNum(iPlayerNr + 1);
			}
		}
		else								//players from other team
		{
			iPlayerNr -= 21;
//			if (!m_pOtherTeam[iPlayerNr]->IsValid() )
			if (m_iSeeTime - m_pOtherTeam[iPlayerNr]->GetTime() > iAge)
			{
				m_pOtherTeam[iPlayerNr]->SetPosition(TVector(dXpos, dYpos));
				m_pOtherTeam[iPlayerNr]->SetInMind();
				m_pOtherTeam[iPlayerNr]->SetTime(m_iHearTime - iAge);
				m_pOtherTeam[iPlayerNr]->SetNum(iPlayerNr + 1);
			}
		}
	}

	return TRUE;
}


/*-----------------------------------------------------------------------------
Function: BOOL TPerception::ParseSenseBody(LPCTSTR str)
Date    : 27.01.98
-------------------------------------------------------------------------------
Parameter: string to parse
Returnval: FALSE, if an error occured
Description: parses a sense body information (sense_body TIME ((view_mode QUALITY
				 WIDTH) (stamina STAMINA EFFORT) (speed AMOUNT) (kick COUNT) (dash COUNT)
				 (turn COUNT) (say COUNT))
-----------------------------------------------------------------------------*/
BOOL TPerception::ParseSenseBody(LPCTSTR str)
{
	TParseString	msg(str);
	CString			temp;
	int				time;
	CString			debug;

	m_oldOldSeeCount = m_oldSeeCount;
	m_oldSeeCount = m_seeCount;
	m_seeCount = 0;

	// read the time
	msg.SetPosition(12);
	time = msg.ParseInteger();
	if (msg.HasErrorOccured())
	{
		#ifdef _DEBUG
			TRACE1("Error Parsing sensetime  %s\r\n", msg);
		#endif
		return FALSE;
	}
	if (time > m_iSenseTime)
		m_bTimeProceeding = TRUE;
	else
		m_bTimeProceeding = FALSE;
	m_iSenseTime = time;	

/*
	#ifdef _DEBUG
		TRACE1("Time: %d\n", time);
	#endif
*/
	// read the view mode
	msg.ParseString(temp);
	msg.ParseString(temp);
	if (msg.HasErrorOccured())
	{
		#ifdef _DEBUG
			TRACE1("Error Parsing viewmode  %s\r\n", msg);
		#endif
		return FALSE;
	}
	if (temp.GetAt(0) == 'l')
		m_iViewQuality = 0;
	else
		m_iViewQuality = 1;
	msg.ParseString(temp);
	if (msg.HasErrorOccured())
	{
		#ifdef _DEBUG
			TRACE1("Error Parsing viewmode  %s\r\n", msg);
		#endif
		return FALSE;
	}
	// check if we will receive a see this cycle
	if (temp.GetAt(1) == 'a')
	{
		if (m_iViewAngle == 45)
		{
			if ((m_oldSeeCount == 1) && (m_oldOldSeeCount <= 1))
				// second see is too late usually, so we always take first
				//m_iSeeThisCycle	= 2;
				m_iSeeThisCycle	= 1;
			else
				m_iSeeThisCycle	= 1;
		}
		else 
			m_iSeeThisCycle	= 1;

		m_iViewAngle = 45;
	}
	else if (temp.GetAt(0) == 'w')
	{
		// 180 deg always
		if ((m_oldOldSeeCount == 0) && (m_oldSeeCount == 0))
			m_iSeeThisCycle	= 1;
		else
			m_iSeeThisCycle	= 0;

		m_iViewAngle = 180;
	}
	else
	{
		if (m_iViewAngle == 90)
		{
			if (m_oldSeeCount == 1)
				m_iSeeThisCycle	= 0;
			else
				m_iSeeThisCycle	= 1;
		}
		else
			m_iSeeThisCycle	= 0;

		m_iViewAngle = 90;
	}

	// inform agent that we have narrow view and act on see
	m_pAgent->SetExpectedSee(m_iSeeThisCycle);
	CString message;
	message.Format("expected see: %d", m_iSeeThisCycle);
	DisplayDebug(message, KATEGORY_PERCEPTION, DEBUG_LOWPERCEPTION);


	// read the stamina
	msg.ParseString(temp);
	SetStamina((int) msg.ParseDouble());
	if (msg.HasErrorOccured())
	{
		#ifdef _DEBUG
			TRACE1("Error Parsing stamina  %s\r\n", msg);
		#endif
		return FALSE;
	}

	// read the effort
	m_dEffort = msg.ParseDouble();
	if (msg.HasErrorOccured())
	{
		#ifdef _DEBUG
			TRACE1("Error Parsing effort  %s\r\n", msg);
		#endif
		return FALSE;
	}
	if (m_dEffort > m_playerType[m_currentPlayerType][effort_max])
		m_dEffort = m_playerType[m_currentPlayerType][effort_max];
	if (m_dEffort < m_playerType[m_currentPlayerType][effort_min])
		m_dEffort = m_playerType[m_currentPlayerType][effort_min];

	// read the speed
	msg.ParseString(temp);
	// abs value
	m_dSpeed = msg.ParseDouble();
	if (msg.HasErrorOccured())
	{
		#ifdef _DEBUG
			TRACE1("Error Parsing speed  %s\r\n", msg);
		#endif
		return FALSE;
	}
//	if (m_dSpeed > PLAYER_SPEED_MAX * PLAYER_DECAY)
//		m_dSpeed = PLAYER_SPEED_MAX * PLAYER_DECAY;
	if (m_dSpeed > m_playerType[m_currentPlayerType][player_speed_max] * m_playerType[m_currentPlayerType][player_decay])
		m_dSpeed = m_playerType[m_currentPlayerType][player_speed_max] * m_playerType[m_currentPlayerType][player_decay];
	
	if (SERVER_VERSION > 6)
	{
		// speed angle
		m_iSpeedAngle = msg.ParseInteger();
		if (msg.HasErrorOccured())
		{
			#ifdef _DEBUG
				TRACE1("Error Parsing speedAngle  %s\r\n", msg);
			#endif
			return FALSE;
		}
	}

	// read the headAngle
	msg.ParseString(temp);
	m_iHeadAngle = (int) msg.ParseDouble();
	if (msg.HasErrorOccured())
	{
		#ifdef _DEBUG
			TRACE1("Error Parsing headangle  %s\r\n", msg);
		#endif
		return FALSE;
	}

	// read the counts
	msg.ParseString(temp);
	m_iNoOfKick = msg.ParseInteger();
	if (msg.HasErrorOccured())
	{
		#ifdef _DEBUG
			TRACE1("Error Parsing noOfKicks  %s\r\n", msg);
		#endif
		return FALSE;
	}
	if (m_iNoOfKick < m_iExpectedNoOfKick)
	{
		// a turn has not been executed
		#ifdef _DEBUG
			TRACE0("Kick not executed\n");
		#endif
		debug += "Kick not executed";
		m_iNoOfCommandsNotExecuted++;
	}
	m_iExpectedNoOfKick = m_iNoOfKick;
	
	msg.ParseString(temp);
	m_iNoOfDash = msg.ParseInteger();
	if (msg.HasErrorOccured())
	{
		#ifdef _DEBUG
			TRACE1("Error Parsing noOfDash  %s\r\n", msg);
		#endif
		return FALSE;
	}
	if (m_iNoOfDash < m_iExpectedNoOfDash)
	{
		// a dash has not been executed
		#ifdef _DEBUG
			TRACE0("Dash not executed\n");
		#endif
		debug += "Dash not executed";
		m_iNoOfCommandsNotExecuted++;
	}
	m_iExpectedNoOfDash = m_iNoOfDash;
	
	msg.ParseString(temp);
	m_iNoOfTurn = msg.ParseInteger();
	if (msg.HasErrorOccured())
	{
		#ifdef _DEBUG
			TRACE1("Error Parsing noOfTurn  %s\r\n", msg);
		#endif
		return FALSE;
	}
	if (m_iNoOfTurn < m_iExpectedNoOfTurn)
	{
		// a turn has not been executed
		#ifdef _DEBUG
			TRACE0("Turn not executed\n");
		#endif
		debug += "Turn not executed";
		m_iNoOfCommandsNotExecuted++;
	}
	m_iExpectedNoOfTurn = m_iNoOfTurn;
	
	msg.ParseString(temp);
	m_iNoOfSay = msg.ParseInteger();
	if (msg.HasErrorOccured())
	{
		#ifdef _DEBUG
			TRACE1("Error Parsing noOfSay  %s\r\n", msg);
		#endif
		return FALSE;
	}
	
	msg.ParseString(temp);
	m_iNoOfTurnNeck = msg.ParseInteger();
	if (msg.HasErrorOccured())
	{
		#ifdef _DEBUG
			TRACE1("Error Parsing noOfTurnNeck  %s\r\n", msg);
		#endif
		return FALSE;
	}
	if (m_iNoOfTurnNeck < m_iExpectedNoOfTurnNeck)
	{
		// a turn has not been executed
		#ifdef _DEBUG
			TRACE0("TurnNeck not executed\n");
		#endif
		debug += "TurnNeck not executed";
		m_iNoOfCommandsNotExecuted++;
	}
	m_iExpectedNoOfTurnNeck = m_iNoOfTurnNeck;


	// since server verion 7 also catch, move and changeView


/*	char		buf1[200];
	char		buf2[200];
	double	dStamina;
	// (sense_body 3000 (view_mode high normal) (stamina 3500 1) (speed 0) (head_angle 0) (kick 0) (dash 0) (turn 7) (say 0) (turn_neck 0))
	int n = sscanf(str, "(sense_body %d (view_mode %s %s (stamina %f %f) (speed %f) (head_angle %d) (kick %d) (dash %d) (turn %d) (say %d) (turn_neck %d))"
						, &m_iSenseTime, buf1, buf2, &dStamina, &m_dEffort, &m_dSpeed, &m_iHeadAngle, &m_iNoOfKick, &m_iNoOfDash, &m_iNoOfTurnNeck);
*/	
	// are we waiting for this result (autorun)
	if (m_bWaitForLastSenseBody)
	{
		m_bWaitForLastSenseBody = FALSE;
		m_pAgent->OnReceivedLastSenseBody();
	}

	if (m_iSawAll > 0)
		m_iSawAll--;

	if (debug.GetLength() > 0)
	{
		CString msg;
		msg.Format("Missed commands: %d ", m_iNoOfCommandsNotExecuted);
		DisplayDebug(msg + debug, KATEGORY_PERCEPTION, DEBUG_LOWPERCEPTION);
	}

	return TRUE;
}

/*-----------------------------------------------------------------------------
Function: BOOL TPerception::ParseInit(LPCTSTR str)
Date    : 29.01.98
-------------------------------------------------------------------------------
Parameter: string containing the init message (init side number playmode)
Returnval: FALSE, if an error occured
Description: parses the init message to get the side and players number
-----------------------------------------------------------------------------*/
BOOL TPerception::ParseInit(LPCTSTR str)
{
	TParseString	msg(str);
	CString			tmp;
	int				number;
	int				mode;
	char*				strPlayMode[] = PLAYMODE_STRINGS;
	int				posLX[PLAYERS_PER_TEAM] = POS_KICKOFF_LEFT_X;
	int				posLY[PLAYERS_PER_TEAM] = POS_KICKOFF_LEFT_Y;

	// get the side
	if ((msg.GetLength() > 6) && (msg.GetAt(6) == 'r'))
	{
		// the player is in the team playing from right to left
		SetSide(RIGHT_TO_LEFT);
		// swap x coordinates of own and other goal
		double tmp = m_pOtherGoal->GetAbsX();
		m_pOtherGoal->SetAbsX(m_pOwnGoal->GetAbsX());
		m_pOwnGoal->SetAbsX(tmp);
		// and of 11m points
		tmp = m_pOther11mPoint->GetAbsX();
		m_pOther11mPoint->SetAbsX(m_pOwn11mPoint->GetAbsX());
		m_pOwn11mPoint->SetAbsX(tmp);
	}
	else
		SetSide(LEFT_TO_RIGHT);

	// get the number of the player
	msg.SetPosition(8);
	number = msg.ParseInteger();
	if (msg.HasErrorOccured())
		return FALSE;
	ASSERT((number > 0) && (number <= PLAYERS_PER_TEAM));
	m_pAgent->SetNumber(number, m_iSide);
	m_pThisPlayer->SetTeam(TRUE, m_iSide);
	m_pThisPlayer->SetNum(number);
	// mark the player in our team with this number as this agent
	m_pOwnTeam[number - 1]->SetThisAgent(TRUE);

	// initialize the expected coordinates of own team' players
	for (int i = 0; i < PLAYERS_PER_TEAM; i++)
	{
		if (m_iSide == LEFT_TO_RIGHT)
		{
			// it is a player of the left team
			m_pOwnTeam[i]->SetAbsX(posLX[i]);
			m_pOwnTeam[i]->SetAbsY(posLY[i]);
		}
		else
		{
			// it is a player of the right team, so mirror x coordinates
			m_pOwnTeam[i]->SetAbsX(-posLX[i]);
			m_pOwnTeam[i]->SetAbsY(-posLY[i]);
		}	
	}

	// get the playmode
	msg.ParseString(tmp);
	mode = 0;
	while ((mode < NO_OF_PLAYMODES) && (tmp.Compare(strPlayMode[mode])))
		mode++;

	if (mode < NO_OF_PLAYMODES)
		m_iPlayMode = mode;

	return TRUE;
}

/*-----------------------------------------------------------------------------
Function: BOOL TPerception::ParseObject(LPCTSTR str)
Date    : 07.11.97
-------------------------------------------------------------------------------
Parameter: reference to the current position in the string, which must be the 
			  opening bracket of the objinfo
Returnval: TRUE, if no parse error occured
Description: parses an object and stores the information
-----------------------------------------------------------------------------*/
BOOL TPerception::ParseObject(LPCTSTR str)
{
	TParseString	msg(str);
	TParseString	name;
	CString		teamname;
	CString		goalie;
	BOOL			bMyTeam;
	int			team;
	int			iNum;
	BOOL			bGoalie		= FALSE;
	double		dist;
	int			dir;
	double		distchng;
	double		dirchng;
	int			faceDir;
	int			bodyDir;
	int			index;
	int			type;
	char			tag1;
	char			tag2;
	int			flagNum;
	int			iFlag;
	TFlag*		pFlag;

	// read the objects name bracket
	if (!msg.ParseBracket(name))
		return FALSE;
	
	// read the distance, direction and distchngection (the last may be not existing)
	if (m_iViewQuality == 0)
	{
		// we have got low view quality, so no distances
		dist = NOT_EXISTING;
		dir = msg.ParseInteger();
		if (msg.HasErrorOccured())
			return FALSE;
	}
	else
	{
		// we have got high view quality
		dist = msg.ParseDouble();
		if (msg.HasErrorOccured())
			return FALSE;
		dir = msg.ParseInteger();
		if (msg.HasErrorOccured())
			return FALSE;

		#ifdef PERCEPTION_NOISE_ON
			// perception may be filled with noise
			dir += (int) ((double) dir * (1.0 - (double) rand() / ((double) RAND_MAX / 2.0)) * m_dNoise);
			dist += dist * (1.0 - (double) rand() / ((double) RAND_MAX / 2.0)) * m_dNoise;
		#endif
	}
	dirchng = NOT_EXISTING;
	distchng = msg.ParseDouble();
	if (msg.HasErrorOccured())
		distchng = NOT_EXISTING;
	else 
	{	
		dirchng = msg.ParseDouble();
		if (msg.HasErrorOccured())
			dirchng = NOT_EXISTING;
		else 
		{	
			faceDir = msg.ParseInteger();
			if (msg.HasErrorOccured())
				faceDir = NOT_EXISTING;
			else 
			{	
				bodyDir = msg.ParseInteger();
				if (msg.HasErrorOccured())
					bodyDir = NOT_EXISTING;
			}
		}
	}

	// switch on the kind of object
	switch (name.GetAt(0))
	{
		// it is a player
		case 'P':
		case 'p':
			// read the name
			if (name.GetLength() > PLAYER_NAME_POSITION)
			{
				name.SetPosition(PLAYER_NAME_POSITION);
				// read the teamname
				name.ParseString(teamname);
				// is it my team ?
				if (teamname.Mid(1, teamname.GetLength() - 2).Compare(m_pAgent->GetTeam()) == 0)
					team = m_pAgent->GetSide();
				else if (m_pAgent->GetSide() == LEFT_TO_RIGHT)
					team = RIGHT_TO_LEFT;
				else
					team = LEFT_TO_RIGHT;

				// read player number
				iNum = name.ParseInteger();
				if (name.HasErrorOccured())
					iNum = NOT_EXISTING;
				else
				{
					// parse goalie info
					name.ParseString(goalie);
					if (goalie.Compare("goalie") == 0)
						bGoalie = TRUE;
				}
			}
			else
			{
				team = NOT_EXISTING;
				iNum = NOT_EXISTING;
			} 

			bMyTeam = (team == m_pThisPlayer->GetTeam());
			m_pVisiblePlayer[m_iNoOfPlayers]->SetPlayer(m_iSeeTime, dist, dir, distchng
													, dirchng, faceDir, bodyDir, bMyTeam,
													team, iNum, bGoalie);
			m_iNoOfPlayers++;

			break;


		case 'G':
		case 'g':
			// it is a goal
			if (name.GetLength() > GOAL_NAME_POSITION)
			{
				// goal with name
				if (name.GetAt(GOAL_NAME_POSITION) == 'l')
					if (m_iSide == LEFT_TO_RIGHT)
						index = 0;
					else
						index = 1;
				else
					if (m_iSide == LEFT_TO_RIGHT)
						index = 1;
					else
						index = 0;
			}
			else
			{
				// it is a G goal (do not know which)
				if (m_pThisPlayer->GetAbsX() < 0.0)
				{
					if (m_iSide == LEFT_TO_RIGHT)
						index = 0;
					else
						index = 1;
				}
				else
				{
					if (m_iSide == LEFT_TO_RIGHT)
						index = 1;
					else
						index = 0;
				}
			}
			
			// store the values
			if (index == 0)
				m_pOwnGoal->SetVisibleObject(m_iSeeTime, dist, dir);
			else
				m_pOtherGoal->SetVisibleObject(m_iSeeTime, dist, dir);
						
			break;


		case 'B':
		case 'b':
			// it is the ball
			m_pBall->SetBall(m_iSeeTime, dist, dir, distchng, dirchng);
			break;

		case 'F':
			break;
		case 'f':
			// it is a flag
			type = FIELD_FLAG;
			if (name.GetLength() < 3)
				return FALSE;
			tag1 = name.GetAt(FLAG_TAG1_POSITON);
			if (name.GetLength() < FLAG_TAG1_POSITON + 3)
			{
				// it must be the center flag
				if (tag1 != 'c')
					return FALSE;
				iFlag = CENTER_MARK;
			}
			else
			{
				tag2 = name.GetAt(FLAG_TAG2_POSITON);
				switch (tag1)
				{	
					case 'c':
						// it is a center flag 
						if (tag2 == 't')
							iFlag = CENTER_TOP;
						else
							iFlag = CENTER_BOTTOM;
						break;

					case 'g':
						// it is a goal flag
						tag1 = tag2;
						if (name.GetLength() < FLAG_TAG2_POSITON + 3)
							return FALSE;
						tag2 = name.GetAt(FLAG_TAG3_POSITON);
						if (tag1 == 'l')
						{
							if (tag2 == 't')
								iFlag = GOAL_POST_L;
							else
								iFlag = GOAL_POST_L + 1;
						}
						else
						{
							if (tag2 == 't')
								iFlag = GOAL_POST_R;
							else
								iFlag = GOAL_POST_R + 1;
						}
						break;

					case 'p':
						// it is a flag of the penalty area
						type = PENALTY_AREA_FLAG;
						tag1 = tag2;
						if (name.GetLength() < FLAG_TAG2_POSITON + 3)
							return FALSE;
						tag2 = name.GetAt(FLAG_TAG3_POSITON);
						if (tag1 == 'l')
						{
							if (tag2 == 't')
								iFlag = FLAG_PENALTY_L;
							else if (tag2 == 'c')
								iFlag = FLAG_PENALTY_L + 1;
							else
								iFlag = FLAG_PENALTY_L + 2;
						}
						else
						{
							if (tag2 == 't')
								iFlag = FLAG_PENALTY_R;
							else if (tag2 == 'c')
								iFlag = FLAG_PENALTY_R + 1;
							else
								iFlag = FLAG_PENALTY_R + 2;
						}
						
						break;

					case 'l':
						// it is one of the left flags of the field
						if (tag2 == '0')
							// it is the left center flag
							iFlag = FLAG_OUTSIDE_LC;
						else
						{
							name.SetPosition(FLAG_TAG3_POSITON);
							flagNum = name.ParseInteger();
							if (name.HasErrorOccured())
								// it is an old lt or lb flag
								return TRUE;
							flagNum /= 10;
							flagNum--;
							if ((flagNum < 0) || flagNum > 2)
								return FALSE;
							if (tag2 == 't')
								// it is a left top outside flag
								iFlag = FLAG_OUTSIDE_LT + flagNum;
							else
								// it is a left bottom outside flag
								iFlag = FLAG_OUTSIDE_LB + flagNum;
						}
						break;

					case 'r':
						// it is one of the right flags of the field
						if (tag2 == '0')
							// it is the right center flag
							iFlag = FLAG_OUTSIDE_RC;
						else
						{
							name.SetPosition(FLAG_TAG3_POSITON);
							flagNum = name.ParseInteger();
							if (name.HasErrorOccured())
								// it is an old rt or rb flag
								return TRUE;
							flagNum /= 10;
							flagNum--;
							if ((flagNum < 0) || flagNum > 2)
								return FALSE;
							if (tag2 == 't')
								// it is a right top outside flag
								iFlag = FLAG_OUTSIDE_RT + flagNum;
							else
								// it is a right bottom outside flag
								iFlag = FLAG_OUTSIDE_RB + flagNum;
						}
						break;

					case 't':
						// it is one of the top flags of the field
						if (tag2 == '0')
							// it is the top center flag
							iFlag = FLAG_OUTSIDE_TC;
						else
						{
							name.SetPosition(FLAG_TAG3_POSITON);
							flagNum = name.ParseInteger();
							if (name.HasErrorOccured())
								return FALSE;
							flagNum /= 10;
							flagNum--;
							if ((flagNum < 0) || flagNum > 4)
								return FALSE;
							if (tag2 == 'l')
								// it is a top left outside flag
								iFlag = FLAG_OUTSIDE_TL + flagNum;
							else
								// it is a top right outside flag
								iFlag = FLAG_OUTSIDE_TR + flagNum;
						}
						break;

					case 'b':
						// it is one of the bottom flags of the field
						if (tag2 == '0')
							// it is the bottom center flag
							iFlag = FLAG_OUTSIDE_BC;
						else
						{
							name.SetPosition(FLAG_TAG3_POSITON);
							flagNum = name.ParseInteger();
							if (name.HasErrorOccured())
								return FALSE;
							flagNum /= 10;
							flagNum--;
							if ((flagNum < 0) || flagNum > 4)
								return FALSE;
							if (tag2 == 'l')
								// it is a bottom left outside flag
								iFlag = FLAG_OUTSIDE_BL + flagNum;
							else
								// it is a bottom right outside flag
								iFlag = FLAG_OUTSIDE_BR + flagNum;
						}
						break;

					default:
						iFlag = TRASH_FLAG;
				}
			}

			pFlag = m_pFlag[iFlag];
			pFlag->SetVisibleObject(m_iSeeTime, dist, dir);
			pFlag->SetFlag(type, tag1, tag2);

			break;

		case 'l':
			// it is a line
			if (name.GetLength() < 3)
				break;
			tag1 = name.GetAt(2);
			m_pLine->SetLine(m_iSeeTime, dist, dir, tag1);
			break;
	}

	return TRUE;
}


/*-----------------------------------------------------------------------------
Parameter: object string to parse
Returnval: TRUE, if no parse error occured
Description: parses an object of a see_global information and stores the information
-----------------------------------------------------------------------------*/
BOOL TPerception::ParseObjectGlobal(LPCTSTR str)
{
	TParseString	msg(str);
	TParseString	name;
	CString			teamname;
	CString			goalie;
	int				team;
	int				iNum;
	BOOL			bGoalie		= FALSE;
	double			x, y;
	double			dx, dy;
//	int				bodyangle;
//	int				neckangle;
	char			switchCharacter;
	

	// read the objects name bracket
	if (!msg.ParseBracket(name))
		return FALSE;
	
	// read the coordinates of the object
	x = msg.ParseDouble();
	if (msg.HasErrorOccured())
		return FALSE;
	y = msg.ParseDouble();
	if (msg.HasErrorOccured())
		return FALSE;

	// switch on the kind of object
	switchCharacter = name.GetAt(0);
	if (switchCharacter == 'g')
	{
		// it is a goal
	}
	else 
	{	
		// read the coordinates of the object
		dx = msg.ParseDouble();
		if (msg.HasErrorOccured())
			return FALSE;
		dy = msg.ParseDouble();
		if (msg.HasErrorOccured())
			return FALSE;


		if (switchCharacter == 'b')
		{
			// it is the ball
			m_pBall->SetAbsX(x);
			m_pBall->SetAbsY(y);
			m_pBall->SetAbsSpeed(TVector(dx, dy));
			m_pBall->SetValid(TRUE);
			m_pBall->SetInMind(TRUE);
			m_pBall->SetOnceSeen(TRUE);
		}
		else
		{
			// it is a player
			TPlayer* player;

			// parse the neckangle

			// read the name
			name.SetPosition(PLAYER_NAME_POSITION);

			// read the teamname
			name.ParseString(teamname);
			// is it my team ?
			if (teamname.Mid(1, teamname.GetLength() - 2).Compare(m_pAgent->GetTeam()) == 0)
				team = m_pAgent->GetSide();
			else if (m_pAgent->GetSide() == LEFT_TO_RIGHT)
				team = RIGHT_TO_LEFT;
			else
				team = LEFT_TO_RIGHT;

			// read player number
			iNum = name.ParseInteger();
			iNum--;
			ASSERT((iNum >= 0) && (iNum < 11));

			// parse goalie info
			name.ParseString(goalie);
			if (goalie.Compare("goalie") == 0)
				bGoalie = TRUE;

			if (team == m_pThisPlayer->GetTeam())
				player = this->m_pOwnTeam[iNum];
			else
				player = this->m_pOtherTeam[iNum];

			player->SetAbsX(x);
			player->SetAbsY(y);
			player->SetAbsSpeed(TVector(dx, dy));
			player->SetValid(TRUE);
			player->SetInMind(TRUE);
			player->SetOnceSeen(TRUE);
			player->SetTeam(m_pThisPlayer->GetTeam(), team);
		}
	}

	return TRUE;
}

/*-----------------------------------------------------------------------------
Function: BOOL TPerception::Localize()
Date    : 16.02.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: TRUE, if the absolute position of the agent could be calculated
Description: calculates the absolute position of the agent. Two flags are 
				 necessary (with this algorithm) to locate the position. It is at one
				 of the two intersection points of the circles surounding the flags
				 in the distance of the player. Which intersection is decided by the 
				 relative angle the player sees the flags.
				 To calculate the intersection points, first the inbtersection point P3 of 
				 the line from flag1 to flag2 and the line from intersection point 1
				 and 2 is calculated. 
				 
-----------------------------------------------------------------------------*/
BOOL TPerception::Localize()
{
	int		iFlag1;
	int		iFlag2;
	int		iFlag1Direction;
	int		iFlag2Direction;
	double	r1, r2;				// the distance of the player to the flags 
	double	dist, dist2;		// the distance (square) of the two flags
	double	a;						// distance from one flag to intersection point P3
	double	h;						// distance from P3 to the intersection Points P1 and P2 of the two circles
	double	x, x1, x2, x3;
	double	y, y1, y2, y3;
	double	ratio;
	int		iAbsDirection;
	TDirection absDir;
	int		beta;
	TVector	position;

	// get the two flags with maximum angle difference
	GetTwoFlags(iFlag1, iFlag2);	
//	GetTwoSidemostFlags(iFlag1, iFlag2);

	if (iFlag2 == -1)
		// there are not 2 visible flags
		return FALSE;

	// mark flags as beeing used for localization
	m_pFlag[iFlag1]->SetUsedForLocalization();
	m_pFlag[iFlag2]->SetUsedForLocalization();

	// do the calculations
	iFlag1Direction = m_pFlag[iFlag1]->GetDirection();
	iFlag2Direction = m_pFlag[iFlag2]->GetDirection();
	
	r1 = m_pFlag[iFlag1]->GetDistance();
	r2 = m_pFlag[iFlag2]->GetDistance();

	x1 = m_pFlag[iFlag1]->GetAbsX();
	x2 = m_pFlag[iFlag2]->GetAbsX();

	y1 = m_pFlag[iFlag1]->GetAbsY();
	y2 = m_pFlag[iFlag2]->GetAbsY();

	// calculate the square distance of the two flags
	dist2 = (x2 - x1)*(x2 - x1) + (y2 - y1)*(y2 - y1);	
	dist = sqrt(dist2);
	if (dist > r1 + r2)
	{
		// the circles would not intersect
		dist = r1 + r2;
		dist2 = dist * dist;
	}
	else if ((r1 > r2) && (dist + r2 < r1))
	{
		// the circles would not intersect
		dist = r1 - r2;
		dist2 = dist * dist;
	}
	else if ((r2 > r1) && (dist + r1 < r2))
	{
		// the circles would not intersect
		dist = r2 - r1;
		dist2 = dist * dist;
	}

	r1 *= r1;
	r2 *= r2;
	
	// a = (r1^2 - r2^2 + d^2 ) / (2 d) 
	a = (r1 - r2 + dist2) / (2.0 * dist);
	// h^2 = r1^2 - a^2
	h = r1 - a*a;
	if (h < 0.0)
		h = 0.0;
	else
		h = sqrt(h);

	// P3 = P1 + a ( P2 - P1 ) / d
	x3 = x1 + a * (x2 - x1) / dist;
	y3 = y1 + a * (y2 - y1) / dist;

	// two circles intersect usually in 2 points. Find out which one to select
	if (iFlag1Direction > iFlag2Direction)
	{
		// result x = x3 + h ( y2 - y1 ) / d 
		x = x3 + h * (y2 - y1) / dist;
		// result y = y3 - h ( x2 - x1 ) / d 
		y = y3 - h * (x2 - x1) / dist;
	}
	else
	{
		x = x3 - h * (y2 - y1) / dist;
		y = y3 + h * (x2 - x1) / dist;
	}

	position = TVector(x, y);


	// calculate the absolute direction
	r1 = m_pFlag[iFlag1]->GetDistance();
	ratio = (y1 - y) / r1;
	beta = (int) (asin(ratio) * RAD2DEG);
	if (x > x1)
		if (y > y1)
			iAbsDirection = -180 - beta - iFlag1Direction;
		else
			iAbsDirection = 180 - beta - iFlag1Direction;
	else
		iAbsDirection = beta - iFlag1Direction;

	// improve position estimation by adding the errors to other flags
	double		sumError = 0.0;
	TDirection	dir;
	int			n = 0;
	for (int i = 0; i < NO_OF_FLAGS; i++)
	{
		if (m_pFlag[i]->IsValid())
		{
			dir = m_pFlag[i]->GetAngleTo(position, TDirection(iAbsDirection - 180));
			dir -= m_pFlag[i]->GetDirection();
			sumError += (int)dir;
			n ++;
		}
	}
	if (n > 1)
		sumError /= n;
	if (sumError > 0.5)
	{
		CString msg;
		msg.Format("Improve Absolute Angle: before %d, Error %4.2f, HeadAngle %d"
			, iAbsDirection, sumError, m_iHeadAngle);
		DisplayDebug(msg, KATEGORY_PERCEPTION, DEBUG_LEVEL_4);
		// conversion from double to int the decimals are truncated so add 0.5
		iAbsDirection += (int) (sumError + 0.5);
	}
		
	// set this players coordinates
	m_pThisPlayer->SetPlayerAbsCoordinates(position, iAbsDirection, m_iHeadAngle, 
							m_dSpeed, m_iViewAngle,	m_pAgent->IsDashingBackward()
							, (m_seeCount >= 2) //(m_iSeeTime > m_iSenseTime)
							, m_pAgent->HaveTurned());

//	m_pAgent->DisplayPosition(m_pThisPlayer->GetPosition(), m_pThisPlayer->GetAbsBodyDir()
//										, m_pThisPlayer->GetAbsDir(), TRUE);
	return TRUE;
}

/*-----------------------------------------------------------------------------
Function: BOOL TPerception::LocalizeCassini(void)
Date    : 11.01.2000
-------------------------------------------------------------------------------
Parameter: none
Returnval: TRUE, if enough flags are perceived to calculate position
Description: calculates the absolute position and orientation of the agent by
				 using the directions to three visible flags (Cassini-method).
-----------------------------------------------------------------------------*/
BOOL TPerception::LocalizeCassini(void)
{
	double	x1, x2, x3;
	double	y1, y2, y3;
	double	x, y;
	double	y5y2, x5x2;
	double	y4y2, x4x2;
	double	tant45;
	double	alpha, beta;
	double	nenner;
	double	distance;
	double	ratio;
	double	gamma;
	int		iFlag1Direction;
	int		iFlag2Direction;
	int		iFlag3Direction;
	double	dAbsDirection;
	double	dAbsDir = 0.0;
	CObList	flagList;
	TFlag*	flag1;
	TFlag*	flag2;
	TFlag*	flag3;
	TVector	position;
	TVector	AbsPos;
	POSITION	pos1;
	POSITION	pos2;
	POSITION	pos3;

	// create a list of flags ordered from left to right
	OrderFlags(&flagList);

	if (flagList.GetCount() < 3)
		// there are not 3 visible flags
		return FALSE;

	// do some averaging
	int loops = flagList.GetCount() / 3;
	pos1 = flagList.GetHeadPosition();
	pos2 = flagList.FindIndex(flagList.GetCount() / 3);
	pos3 = flagList.FindIndex(flagList.GetCount() / 3 * 2);
	for (int i = 0; i < loops; i++)
	{
		// left flag
		flag1 = (TFlag*) flagList.GetNext(pos1);
		// mid flag
		flag2 = (TFlag*) flagList.GetNext(pos2);
		// right flag
		flag3 = (TFlag*) flagList.GetNext(pos3);

		// mark flags as used for localize
		if (i == 0)
		{
			flag1->SetUsedForLocalization();
			flag2->SetUsedForLocalization();
			flag3->SetUsedForLocalization();
		}

		// set parameters
		x1 = flag1->GetAbsX();
		x2 = flag2->GetAbsX();
		x3 = flag3->GetAbsX();
		y1 = flag1->GetAbsY();
		y2 = flag2->GetAbsY();
		y3 = flag3->GetAbsY();

		iFlag1Direction = flag1->GetDirection();
		iFlag2Direction = flag2->GetDirection();
		iFlag3Direction = flag3->GetDirection();

		alpha = iFlag3Direction - iFlag2Direction;
		beta = 360 - (iFlag2Direction - iFlag1Direction);
		ASSERT((alpha > D_ZERO_P) && (alpha < 180.0));
		ASSERT((beta > D_ZERO_P) && (beta < 360.0));
		alpha *= DEG2RAD;
		beta *= DEG2RAD;

		// do calculations 
		x5x2 = (x3 - x2) - (y3 - y2) / tan(alpha);
		y5y2 = (y3 - y2) + (x3 - x2) / tan(alpha);
		x4x2 = (x1 - x2) - (y1 - y2) / tan(beta);
		y4y2 = (y1 - y2) + (x1 - x2) / tan(beta);

		nenner = (x4x2 - x5x2);
		ASSERT ((nenner < D_ZERO_N) || (nenner > D_ZERO_P));
		tant45 = (y4y2 - y5y2) / nenner;
		
		// calculate coordinates
		nenner = (1 + tant45 * tant45);
		ASSERT ((nenner < D_ZERO_N) || (nenner > D_ZERO_P));
		y = (y5y2 - x5x2 * tant45) / nenner + y2;
		x = -(y - y2) * tant45 + x2;
		position = TVector(x, y);

		// calculate absoulte direction
		distance = flag1->GetDistanceTo(position);
		ratio = (y1 - y) / distance;
		gamma = asin(ratio) * RAD2DEG;
		if (x > x1)
			if (y > y1)
				dAbsDirection = -180.0 - gamma - iFlag1Direction;
			else
				dAbsDirection = 180.0 - gamma - iFlag1Direction;
		else
			dAbsDirection = gamma - iFlag1Direction;
	
		// do some filtering
		AbsPos += position;
		dAbsDir += dAbsDirection;
	}

	// there are i points to measure
	if (i > 1)
	{
		AbsPos *= 1.0 / (double) i;
		dAbsDir /= (double) i;
	}

	// set this players coordinates
	m_pThisPlayer->SetPlayerAbsCoordinates(AbsPos, (int) dAbsDir, m_iHeadAngle, 
							m_dSpeed, m_iViewAngle,	m_pAgent->IsDashingBackward()
							, (m_iSeeTime > m_iSenseTime), m_pAgent->HaveTurned());

//	m_pAgent->DisplayPosition(m_pThisPlayer->GetPosition(), m_pThisPlayer->GetAbsBodyDir()
//										, m_pThisPlayer->GetAbsDir(), TRUE);

	return TRUE;
}

/*-----------------------------------------------------------------------------
Function: void TPerception::GetTwoFlags(int &iFlag1, int &iFlag2)
Date    : 08.07.99
-------------------------------------------------------------------------------
Parameter: references to the two flags
Returnval: (void)
Description: returns the indices of the first visible flag and a second flag
				 that has biggest angle difference to the first
-----------------------------------------------------------------------------*/
void TPerception::GetTwoFlags(int &iFlag1, int &iFlag2)
{		
	int		i;
	double	dist;
	int		iFlag1Direction;
	int		iMaxAngle = -1000;
	int		iDeltaAngle;

	iFlag1 = -1;
	iFlag2 = -1;
	
	// search for two visible flags with maximal difference of the view angle
	for (i = 0; i < NO_OF_FLAGS; i++)
	{
		if (m_pFlag[i]->IsValid() && ((dist = m_pFlag[i]->GetDistance()) > 4.0))
		{
			if (iFlag1 == -1)
			{
				// found the first visible flag
				iFlag1 = i;
				iFlag1Direction = m_pFlag[iFlag1]->GetDirection();
			}
			else
			{
				iDeltaAngle = iFlag1Direction - m_pFlag[i]->GetDirection();
				if (iDeltaAngle < 0)
					iDeltaAngle = -iDeltaAngle;
				if (iDeltaAngle > iMaxAngle)
				{
					// this flag has a bigger angle to the first flag
					iFlag2 = i;
					iMaxAngle = iDeltaAngle;
				}
			}
		}
	}
}

/*-----------------------------------------------------------------------------
Function: void TPerception::GetTwoSidemostFlags(int &iFlag1, int &iFlag2)
Date    : 10.01.2000
-------------------------------------------------------------------------------
Parameter: references to the two flags
Returnval: (void)
Description: returns the indices of leftmost and rightmost flag
-----------------------------------------------------------------------------*/
void TPerception::GetTwoSidemostFlags(int &iFlag1, int &iFlag2)
{		
	int			i;
	int			iMinFlagDirection = 360;
	int			iMaxFlagDirection = -360;
	TDirection	tFlagDirection;

	iFlag1 = -1;
	iFlag2 = -1;
	
	for (i = 0; i < NO_OF_FLAGS; i++)
	{
		// do not look at too close flags
		if (m_pFlag[i]->IsValid() && (m_pFlag[i]->GetDistance() > 4.0))
		{
			tFlagDirection = m_pFlag[i]->GetDirection();
			if (tFlagDirection < iMinFlagDirection)
			{
				// found the leftmost flag so far
				iFlag1 = i;
				iMinFlagDirection = tFlagDirection;
			}
			else if (tFlagDirection > iMaxFlagDirection)
			{
				// found the rightmost flag so far
				iFlag2 = i;
				iMaxFlagDirection = tFlagDirection;
			}
		}
	}
}

/*-----------------------------------------------------------------------------
Function: void TPerception::OrderFlags(CObList* flagList)
Date    : 14.01.2000
-------------------------------------------------------------------------------
Parameter: pointer to the list that returns the result
Returnval: void
Description: orders the flags in the list from left to right
-----------------------------------------------------------------------------*/
void TPerception::OrderFlags(CObList* flagList)
{
	int			i;
	POSITION		pos;
	POSITION		oldPos;
	TFlag*		pFlag;
	TDirection	thisDir;
	TDirection	dir;

	ASSERT(flagList != NULL);
	for (i = 0; i < NO_OF_FLAGS; i++)
	{
		// do not look at too close flags
		if (m_pFlag[i]->IsValid() && (m_pFlag[i]->GetDistance() > 2.0))
		{
			thisDir = m_pFlag[i]->GetDirection();

			// insert sorted
			BOOL	bFound = FALSE;
			pos = flagList->GetHeadPosition();
			while (pos != NULL)
			{
				oldPos = pos;
				pFlag = (TFlag*) (flagList->GetNext(pos));
				dir = pFlag->GetDirection();
				if (thisDir < dir)
				{
					// found proper place
					flagList->InsertBefore(oldPos, m_pFlag[i]);
					pos = NULL;
					bFound = TRUE;
				}
			}
			if (!bFound)
				flagList->AddTail(m_pFlag[i]);
		}
	}
}


/*-----------------------------------------------------------------------------
Function: void TPerception::GlobalMap()
Date    : 18.05.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: calculates the absolute position of all seen moveable objects
-----------------------------------------------------------------------------*/
void TPerception::GlobalMap()
{
	int		i;
	TPlayer*	pPlayer;
	int		dir = m_pThisPlayer->GetAbsDir();
//	BOOL		bTimeDifference = (m_iSeeTime > m_iSenseTime);
	BOOL		bTimeDifference = m_seeCount >= 2;
	BOOL		playerNearBall = FALSE;
	CString	msg;

	// check, if a player has been close to the ball (used for filtering)
	// the functional object playerAtBall is from last cycle !
	pPlayer = GetAnyPlayerAtBall();
	if ((pPlayer != NULL) && (!pPlayer->IsThisAgent()))
	{
		if (pPlayer->GetDistanceTo(m_pBall) < GetPlayerSize() + BALL_SIZE + GetKickableMargin() + 0.5)
			playerNearBall = TRUE;
	}

	if (m_pBall->IsValid())
	{
		m_pBall->SetAbsCoordinates(m_pThisPlayer, dir, bTimeDifference, playerNearBall, this);
//		m_pAgent->DisplayBallPosition(m_pBall->GetPosition(), m_pBall->GetAbsSpeed(), TRUE);
		msg.Format("bspeed x:%4.2f y:%4.2f bx:%4.2f by:%4.2f ", 
				m_pBall->GetAbsSpeed().GetX(), m_pBall->GetAbsSpeed().GetY(),
				m_pBall->GetPosition().GetX(), m_pBall->GetPosition().GetY());
		DisplayDebug(msg, KATEGORY_PERCEPTION, DEBUG_LOWPERCEPTION);
	}
	
	for (i = 0; i < m_iNoOfPlayers; i++)
		if (m_pVisiblePlayer[i]->IsValid())
			m_pVisiblePlayer[i]->SetAbsCoordinates(m_pThisPlayer, dir, bTimeDifference, FALSE, this);	
}


/*-----------------------------------------------------------------------------
Function: void TPerception::CheckVisualPlausibility()
Date    : 03.06.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: removes players from the mind map, that should be in the visible area, but
				 are not seen
-----------------------------------------------------------------------------*/
void TPerception::CheckVisualPlausibility()
{
	int		i;
	int		iMaxAngle = m_iViewAngle / 2 - 22;
	int		side = this->GetSide();
	CString	msg;

	// check if ball should be visible, but is not
	if (m_pThisPlayer->CheckVisuality(m_pBall, iMaxAngle, m_iSeeTime, side))
		DisplayDebug("Ball should be visible", KATEGORY_PERCEPTION, DEBUG_LOWPERCEPTION);
		

	// check players
	for (i = 0; i < PLAYERS_PER_TEAM; i++)
	{
		if (!m_pOwnTeam[i]->IsThisAgent())
		{
			if (m_pThisPlayer->CheckVisuality(m_pOwnTeam[i], iMaxAngle, m_iSeeTime, side))
			{
				msg.Format("Our player %d should be visible", i);
				DisplayDebug(msg, KATEGORY_PERCEPTION, DEBUG_LOWPERCEPTION);
			}
		}
		if (m_pThisPlayer->CheckVisuality(m_pOtherTeam[i], iMaxAngle, m_iSeeTime, side))
		{
			msg.Format("Other player %d should be visible", i);
			DisplayDebug(msg, KATEGORY_PERCEPTION, DEBUG_LOWPERCEPTION);
		}
	}
}

/*-----------------------------------------------------------------------------
Function: BOOL TPerception::GetIsAt(int x, int y)
Date    : 17.03.98
-------------------------------------------------------------------------------
Parameter: coordinates to check against
Returnval: TRUE, if the agent is, within some boundary, at x, y
Description: coordinates are compared with the coordinates, the agent beliefes
				 to be in. if the difference is less than dist, True is returned
-----------------------------------------------------------------------------*/
BOOL TPerception::GetIsAt(int x, int y, int dist1) const
{
	double	dx = (double) x - m_pThisPlayer->GetAbsX();
	double	dy;
	double	dist;

	// check x coordinate
	dist = (double) dist1;
	if ((dx < -dist) || (dx > dist))
		return FALSE;

	// check y coordinate
	dy = (double) y - m_pThisPlayer->GetAbsY();
	if ((dy < -dist) || (dy > dist))
		return FALSE;
	
	return TRUE;
}

/*-----------------------------------------------------------------------------
Function: void TPerception::MatchPlayers()
Date    : 12.03.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: tries to match the perceived players to players remembered
				 May be this is not very important in eleven players matches, because
				 there are enough important players fully visible
-----------------------------------------------------------------------------*/
void TPerception::MatchPlayers()
{
	int	i;
	int	team;
	int	iNum;

	// invalidate match info
	for (i = 0; i < PLAYERS_PER_TEAM; i++)
	{
		m_pOwnTeam[i]->SetMatched(FALSE);
		m_pOtherTeam[i]->SetMatched(FALSE);
	}

	// treat all the players thats numbers are seen
	for (i = 0; i < m_iNoOfPlayers; i++)
	{
		team = m_pVisiblePlayer[i]->GetTeam();
		iNum = m_pVisiblePlayer[i]->GetNum();

		// try to identify the player
		if (iNum != NOT_EXISTING)
		{
			// team and number can be seen
			ASSERT((iNum > 0) && (iNum <= PLAYERS_PER_TEAM));
			if (team == m_iSide)
				m_pOwnTeam[iNum - 1]->SetPlayer(m_pVisiblePlayer[i]);
			else
				m_pOtherTeam[iNum - 1]->SetPlayer(m_pVisiblePlayer[i]);
		}
	}

	// treat all the players thats numbers are not seen
	for (i = 0; i < m_iNoOfPlayers; i++)
	{
		team = m_pVisiblePlayer[i]->GetTeam();
		iNum = m_pVisiblePlayer[i]->GetNum();

		// try to identify the player
		if (iNum == NOT_EXISTING)
			// find the remembered player closest to the viewed player
			MatchClosestPlayer(m_pVisiblePlayer[i], team);
	}
}

/*-----------------------------------------------------------------------------
Function: void TPerception::MatchClosestPlayer(TPlayer* pViewedPlayer, int team)
Date    : 12.03.98
-------------------------------------------------------------------------------
Parameter: pointer to the player that has been perceived
			  team index: 0 if own team, 1 if other team, NOT_EXISTING if unknown
Returnval: (void)
Description: tries to find the remembered player closest to the perceived player
-----------------------------------------------------------------------------*/
void TPerception::MatchClosestPlayer(TPlayer* pViewedPlayer, int team)
{
	int		i;
	int		iNearest = 0;
	double	dNearestDist = 10000.0;
	double	dDist;
	int		iFreeOpponent;
	CString	message;

	if (team == m_iSide) 
	{
		// it is a player of our team
		for (i = 0; i < PLAYERS_PER_TEAM; i++)
		{
			if (!m_pOwnTeam[i]->IsThisAgent() && !m_pOwnTeam[i]->IsMatched())
			{
				// it is not me self and not a player already matched
				dDist = pViewedPlayer->GetDistanceTo(m_pOwnTeam[i]);
				if (dDist < dNearestDist)
				{
					iNearest = i;
					dNearestDist = dDist;
				}
			}
		}
	}
	else if (team != NOT_EXISTING) 
	{
		// it is a player of the opponent team
		for (i = 0; i < PLAYERS_PER_TEAM; i++)
		{
			if (!m_pOtherTeam[i]->IsMatched())
			{
				dDist = pViewedPlayer->GetDistanceTo(m_pOtherTeam[i]);
				if (dDist < dNearestDist)
				{
					iNearest = i;
					dNearestDist = dDist;
				}
			}
		}
	}
	
	// look if it is a player close to me
	else if (m_pThisPlayer->GetDistanceTo(pViewedPlayer) < 3.0)
	{
		// it is a player close behind me. Opponents close behind me have to 
		// be taken into account
		message.Format("Matching close player: dist: %4.2f, dir: %d"
							, pViewedPlayer->GetDistance(), (int) pViewedPlayer->GetDirection());
		DisplayDebug(message, KATEGORY_PERCEPTION, DEBUG_LOWPERCEPTION);
		
		// we do not know the team, so look at both teams players
		for (i = 0; i < PLAYERS_PER_TEAM; i++)
		{
			if (!m_pOwnTeam[i]->IsThisAgent() && !m_pOwnTeam[i]->IsMatched())
			{
				// it is not me self
				dDist = pViewedPlayer->GetDistanceTo(m_pOwnTeam[i]);
				if (dDist < dNearestDist)
				{
					team = m_iSide;
					iNearest = i;
					dNearestDist = dDist;
				}
			}
		}

		for (i = 0; i < PLAYERS_PER_TEAM; i++)
		{
			if (!m_pOtherTeam[i]->IsMatched())
			{
				dDist = pViewedPlayer->GetDistanceTo(m_pOtherTeam[i]);
				if (dDist < dNearestDist)
				{
					// other team
					if (m_iSide == LEFT_TO_RIGHT)
						team = RIGHT_TO_LEFT;
					else
						team = LEFT_TO_RIGHT;
					iNearest = i;
					dNearestDist = dDist;
				}
			}
		}
		message.Format("Matched to player %d of team %d", iNearest + 1, team + 1);
		DisplayDebug(message, KATEGORY_PERCEPTION, DEBUG_LOWPERCEPTION);
		
	}
	else
		// we do not match distant players
		return;
	
	if (dNearestDist > 200.0)
	{
		iNearest = GetFreeOtherTeam();
		if (iNearest >= PLAYERS_PER_TEAM)
			// all opponents matched
			return;
		if (m_iSide == LEFT_TO_RIGHT)
			team = RIGHT_TO_LEFT;
		else
			team = LEFT_TO_RIGHT;
	}

	// only match close matches
//	if (dNearestDist > 10.0)
//		return;

	// the player's parameters have to be set	
	if (team == m_iSide)
	{
		m_pOwnTeam[iNearest]->SetPlayer(pViewedPlayer);
		m_pOwnTeam[iNearest]->SetTeam(TRUE, team);
		m_pOwnTeam[iNearest]->SetNum(iNearest + 1);
		m_pOwnTeam[iNearest]->SetOnlyInMind();
	}
	else
	{
		if (m_pOtherTeam[iNearest]->IsValid())
		{
			// the player iNearest has been matched previously (errorneously)
			iFreeOpponent = GetFreeOtherTeam();
			if (iFreeOpponent < PLAYERS_PER_TEAM)
			{
				m_pOtherTeam[iFreeOpponent]->SetPlayer(m_pOtherTeam[iNearest]);
				m_pOtherTeam[iFreeOpponent]->SetNum(iFreeOpponent + 1);
			}
		}
		m_pOtherTeam[iNearest]->SetPlayer(pViewedPlayer);
		m_pOtherTeam[iNearest]->SetTeam(FALSE, team);
		m_pOtherTeam[iNearest]->SetNum(iNearest + 1);
		m_pOtherTeam[iNearest]->SetOnlyInMind();
	}
}

/*-----------------------------------------------------------------------------
Function: void TPerception::CalculateFutureBallPositions()
Date    : 22.07.99
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: calculates the future ball positions
-----------------------------------------------------------------------------*/
void TPerception::CalculateFutureBallPositions()
{
	// calculate the new future positions of the ball
	for (int i = 0; i < MAX_FUTURE_POSITIONS; i++)
		m_pBall->GetFuturePosition(i, *m_pFutureBallPosition[i]);
}

/*-----------------------------------------------------------------------------
Function: int TPerception::GetFreeOtherTeam()
Date    : 26.08.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: number of a m_pOtherTeam opponent, that has not been matched (is not valid)
Description: tries to find an opponent, that has not been matched this perception.
				 All players of unknown team are opponents !
-----------------------------------------------------------------------------*/
int TPerception::GetFreeOtherTeam()
{
	int	i = 0;

	while ((i < PLAYERS_PER_TEAM) && (m_pOtherTeam[i]->IsValid()))
		i++;

	return i;
}

/*-----------------------------------------------------------------------------
Function: int TPerception::GetPlayersWithin(TDirection& dir1, TDirection& dir2)
Date    : 26.08.98
-------------------------------------------------------------------------------
Parameter: left direction of sector
			  right border of sector
Returnval: number of players within the specified sector
Description: counts the players within the specified sector
-----------------------------------------------------------------------------*/
int TPerception::GetPlayersWithin(TDirection dir1, TDirection dir2)
{
	int			i;
	TDirection	dir;
	int			count = 0;

	// first count the opponents
	count = GetOpponentsWithin(dir1, dir2);

	// now count own team
	for (i = 0; i < PLAYERS_PER_TEAM; i++)
	{
		if (m_pOwnTeam[i]->IsInMind())
		{
			dir = GetObjectDirection(m_pOwnTeam[i]);
			if (dir.LiesWithin(dir1, dir2))
				count++;
		}
	}

	return count;
}

/*-----------------------------------------------------------------------------
Function: int TPerception::GetOpponentsWithin(TDirection dir1, TDirection dir2)
Date    : 31.08.98
-------------------------------------------------------------------------------
Parameter: left direction of slice
			  right direction of slice
Returnval: (void)
Description: counts the numbers of opponent players, that lie within the two directions
-----------------------------------------------------------------------------*/
int TPerception::GetOpponentsWithin(TDirection dir1, TDirection dir2)
{
	int			i;
	TDirection	dir;
	int			count = 0;

	for (i = 0; i < PLAYERS_PER_TEAM; i++)
	{
		if (m_pOtherTeam[i]->IsInMind())
		{
			dir = GetObjectDirection(m_pOtherTeam[i]);
			if (dir.LiesWithin(dir1, dir2))
				count++;
		}
	}
	return count;
}

/*-----------------------------------------------------------------------------
Function: Functions returning properties of the world
Date    : 24.02.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: property
Description: 
-----------------------------------------------------------------------------*/
TDirection TPerception::GetObjectDirection(TVisibleObject* obj)
{
	// 06.07.99 allways take global map
	// if the player can see the object, then return visible direction
//	if (obj->IsValid() && (m_iSenseTime - m_iSeeTime == 0))
//		return obj->GetDirection(); 
//	else
		// the object is not visible, so use the absolute position of the agent
		return m_pThisPlayer->GetAngleTo(obj);
}

TDirection TPerception::GetObjectDirectionToBody(TVisibleObject* obj)
{
	// 06.07.99 allways take global map
	// returns the objects direction with respect to the body direction of the agent
//	if (obj->IsValid() && (m_iSenseTime - m_iSeeTime == 0))
//		return obj->GetDirection() + m_iHeadAngle;
//	else
		// the object is not visible, so use the absolute position of the agent
		return m_pThisPlayer->GetAngleTo(obj) + m_iHeadAngle;
}

double TPerception::GetObjectDistance(TVisibleObject* obj)
{ 
//	if (obj->IsValid() && (m_iSenseTime - m_iSeeTime == 0))
//		return obj->GetDistance(); 
//	else
		return m_pThisPlayer->GetDistanceTo(obj);
}

TDirection TPerception::GetObjectDeltaDir(TVisibleObject* obj)
{
	return obj->GetDeltaDir(); 
}

double TPerception::GetObjectDeltaDist(TVisibleObject* obj)
{ 
	return obj->GetDeltaDist(); 
}

BOOL TPerception::GetSeeObject(TVisibleObject* obj) const
{
	if (m_iSeeTime == obj->GetTime()) 
		if ((m_iSeeTime == 0) && (obj->GetAbsX() < -500.0))
			return FALSE;
		else
			return TRUE; 
	else 
		return FALSE; 
}

TSoccerGoal* TPerception::GetOwnGoal()
{	
	return m_pOwnGoal;
}

TSoccerGoal* TPerception::GetOtherGoal()
{	
	return m_pOtherGoal;
}

BOOL TPerception::GetPlaymodeOffsideOther()
{ 
	if ((GetSide() == LEFT_TO_RIGHT) && (GetPlayMode() == OFFSIDE_R))
		return TRUE;
	else if ((GetSide() == RIGHT_TO_LEFT) && (GetPlayMode() == OFFSIDE_L))
		return TRUE;
	else
		return FALSE;
}

BOOL TPerception::GetPlaymodeOffsideOwn()
{ 
	if ((GetSide() == LEFT_TO_RIGHT) && (GetPlayMode() == OFFSIDE_L))
		return TRUE;
	else if ((GetSide() == RIGHT_TO_LEFT) && (GetPlayMode() == OFFSIDE_R))
		return TRUE;
	else
		return FALSE;
}

BOOL TPerception::GetPlaymodeFreekickOther()
{ 
	if ((GetSide() == LEFT_TO_RIGHT) && (GetPlayMode() == FREE_KICK_R))
		return TRUE;
	else if ((GetSide() == RIGHT_TO_LEFT) && (GetPlayMode() == FREE_KICK_L))
		return TRUE;
	else
		return FALSE;
}

BOOL TPerception::GetPlaymodeKickInOwn()
{ 
	if ((GetSide() == LEFT_TO_RIGHT) && (GetPlayMode() == KICK_IN_L))
		return TRUE;
	else if ((GetSide() == RIGHT_TO_LEFT) && (GetPlayMode() == KICK_IN_R))
		return TRUE;
	else
		return FALSE;
}

BOOL TPerception::GetPlaymodeKickInOther()
{ 
	if ((GetSide() == LEFT_TO_RIGHT) && (GetPlayMode() == KICK_IN_R))
		return TRUE;
	else if ((GetSide() == RIGHT_TO_LEFT) && (GetPlayMode() == KICK_IN_L))
		return TRUE;
	else
		return FALSE;
}

BOOL TPerception::GetPlaymodeFreekickOwn()
{ 
	if ((GetSide() == LEFT_TO_RIGHT) && (GetPlayMode() == FREE_KICK_L))
		return TRUE;
	else if ((GetSide() == RIGHT_TO_LEFT) && (GetPlayMode() == FREE_KICK_R))
		return TRUE;
	else
		return FALSE;
}

TFlag* TPerception::GetRightMidline()
{	
	if (m_pAgent->GetSide() == LEFT_TO_RIGHT) 
		return m_pFlag[CENTER_BOTTOM];
	else
		return m_pFlag[CENTER_TOP]; 
}

TFlag* TPerception::GetLeftMidline()
{	
	if (m_pAgent->GetSide() == LEFT_TO_RIGHT) 
		return m_pFlag[CENTER_TOP];
	else
		return m_pFlag[CENTER_BOTTOM]; 
}

TFlag* TPerception::GetOtherLeftCorner()
{	
	if (m_pAgent->GetSide() == LEFT_TO_RIGHT) 
		return m_pFlag[FLAG_OUTSIDE_TR + 4];
	else
		return m_pFlag[FLAG_OUTSIDE_BL + 4]; 
}

TFlag* TPerception::GetOtherRightCorner()
{	
	if (m_pAgent->GetSide() == LEFT_TO_RIGHT) 
		return m_pFlag[FLAG_OUTSIDE_BR + 4];
	else
		return m_pFlag[FLAG_OUTSIDE_TL + 4]; 
}

TFlag* TPerception::GetOwnLeftPenaltyCorner()
{	
	if (m_pAgent->GetSide() == LEFT_TO_RIGHT) 
		return m_pFlag[FLAG_PENALTY_L];
	else
		return m_pFlag[FLAG_PENALTY_R + 2]; 
}

TFlag* TPerception::GetOwnRightPenaltyCorner()
{	
	if (m_pAgent->GetSide() == LEFT_TO_RIGHT) 
		return m_pFlag[FLAG_PENALTY_L + 2];
	else
		return m_pFlag[FLAG_PENALTY_R]; 
}

TFlag* TPerception::GetOtherLeftPenaltyCorner()
{	
	if (m_pAgent->GetSide() == LEFT_TO_RIGHT) 
		return m_pFlag[FLAG_PENALTY_R];
	else
		return m_pFlag[FLAG_PENALTY_L + 2]; 
}

TFlag* TPerception::GetOtherRightPenaltyCorner()
{	
	if (m_pAgent->GetSide() == LEFT_TO_RIGHT) 
		return m_pFlag[FLAG_PENALTY_R + 2];
	else
		return m_pFlag[FLAG_PENALTY_L]; 
}

TFlag* TPerception::GetOtherLeftGoalPost()
{	
	if (m_pAgent->GetSide() == LEFT_TO_RIGHT) 
		return m_pFlag[GOAL_POST_R];
	else
		return m_pFlag[GOAL_POST_L + 1]; 
}

TFlag* TPerception::GetOtherRightGoalPost()
{	
	if (m_pAgent->GetSide() == LEFT_TO_RIGHT) 
		return m_pFlag[GOAL_POST_R + 1];
	else
		return m_pFlag[GOAL_POST_L]; 
}

TFlag* TPerception::GetOwnLeftGoalPost()
{	
	if (m_pAgent->GetSide() == LEFT_TO_RIGHT) 
		return m_pFlag[GOAL_POST_L];
	else
		return m_pFlag[GOAL_POST_R + 1]; 
}

TFlag* TPerception::GetOwnRightGoalPost()
{	
	if (m_pAgent->GetSide() == LEFT_TO_RIGHT) 
		return m_pFlag[GOAL_POST_L + 1];
	else
		return m_pFlag[GOAL_POST_R]; 
}

int TPerception::GetBodyAngle() const
{
	// returns the absolute body direction
	TDirection dir = m_pThisPlayer->GetAbsDir() - m_iHeadAngle;
	return dir;
}

int TPerception::GetNoOfOpponentsInMind()
{
	int	count = 0;

	// returns the number of opponents that are in the global map
	for (int i = 0; i < PLAYERS_PER_TEAM; i++)
	{
		if (m_pOtherTeam[i]->IsInMind())
			count++;
	}
	return count;
}

TPlayer* TPerception::GetOtherGoalie()
{
	for (int i = 0; i < PLAYERS_PER_TEAM; i++)
	{
		if ((m_pOtherTeam[i] != NULL) && (m_pOtherTeam[i]->IsGoalie()))
			return m_pOtherTeam[i];
	}
	
	return NULL;
}

// returns true, if the closest opponent at the ball is closer than distance passed
BOOL TPerception::GetOpponentNearBall(double dist)
{
	TPlayer*		pOpponent = GetOpponentAtBall();
	if (pOpponent != NULL)
	{
		double dBallOpponentDistance = pOpponent->GetDistanceTo(GetBall()); 
		if (dBallOpponentDistance < dist)
			return TRUE;
		else
			return FALSE;
	}
	
	// no opponent at all
	return FALSE;
}

/*-----------------------------------------------------------------------------
Function: TPlayer* TPerception::GetPlayerInWay(int dir, double p_dDist)
Date    : 13.03.98
-------------------------------------------------------------------------------
Parameter: direction which is relevant
			  max distance to examine
Returnval: pointer to the player, that is in the direction passed and near this 
			  agent, NULL, if none such exists
Description: searches for a player, that is within a certain direction of this player
				 and is near this player. It is used to avoid collisions
-----------------------------------------------------------------------------*/
TPlayer* TPerception::GetPlayerInWay(int dir, double p_dDist)
{
	int		iMinDir = 1000;
	double	dMinDist;
	double	dist;
	int		iIndexMinDir = -1;
	int		iTeam = 0;
	int		i;
	int		deltaDir;


	for (i = 0; i < PLAYERS_PER_TEAM; i++)
	{
		// check the own team
		if ((m_pOwnTeam[i]->GetAge(m_iSeeTime) < 10) && !m_pOwnTeam[i]->IsThisAgent())
		{
			dist = GetObjectDistance(m_pOwnTeam[i]);
			if (dist < p_dDist)
			{
				// find the player closest to the passed direction, that is within
				// p_dDist meters of this player
				deltaDir = dir - GetObjectDirection(m_pOwnTeam[i]);
				if (deltaDir < 0)
					deltaDir = -deltaDir;
				if (deltaDir < iMinDir)
				{
					iMinDir = deltaDir;
					dMinDist = dist;
					iIndexMinDir = i;
					iTeam = 0;
				}
			}
		}

		// check the other team
		if (m_pOtherTeam[i]->GetAge(m_iSeeTime) < 10)
										{
			dist = GetObjectDistance(m_pOtherTeam[i]);
			if (dist < p_dDist)
			{
				// find the player closest to the passed direction, that is within
				// five meters of this player
				deltaDir = dir - GetObjectDirection(m_pOtherTeam[i]);
				if (deltaDir < 0)
					deltaDir = -deltaDir;
				if (deltaDir < iMinDir)
				{
					iMinDir = deltaDir;
					dMinDist = dist;
					iIndexMinDir = i;
					iTeam = 1;
				}
			}
		}
	}

	if ((iMinDir < 70) && (dMinDist * sin((double)iMinDir * DEG2RAD) < PLAYER_SIZE * 2.5))//3.0))
	{
		if (iTeam == 0)
			return m_pOwnTeam[iIndexMinDir];
		else
			return m_pOtherTeam[iIndexMinDir];
	}
	else
		return NULL;
}

/*-----------------------------------------------------------------------------
Function: TPlayer* TPerception::CalculateFreestTeammate(int dir)
Date    : 22.06.98
From    :
Modified: MP 24.04.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: pointer to the player, that has highest degree of freeness
			  null, if there is no teammate, or the highest degree of freeness is too low
Description: returns a pointer to the teammate, that has got the highest degree
				 of freeness or null, if it is too low
-----------------------------------------------------------------------------*/
TPlayer* TPerception::CalculateFreestTeammate()
{
	int		i;
	double	value;
	double	dHighest = -0.1;
	int		iHighest = -1;

	for (i = 0; i < PLAYERS_PER_TEAM; i++)
	{
		if ((m_pOwnTeam[i]->IsInMind()) && !m_pOwnTeam[i]->IsThisAgent()
			&& (m_pOwnTeam[i]->IsAlife(m_iSenseTime)))
			value = CalculateFreePlayer(m_pOwnTeam[i]);
		else
		{
			value = 0.0;
			m_pOwnTeam[i]->SetFreePassDegree(0.0);
		}

		if (value > dHighest)
		{
			dHighest = value;
			iHighest = i;
		}
	}

	// check for threshold
	if (iHighest < 0)
		return NULL;
	else
		return m_pOwnTeam[iHighest];
}

/*-----------------------------------------------------------------------------
Function: double	CalculateFreePlayer (TPlayer * pTeammate, int x, int y)
Date    : 24.04.99
From    : Markus Plewinski
-------------------------------------------------------------------------------
Parameter: for which teammate is the calculation
Returnval: quality of freeness of this player
Description: 
-----------------------------------------------------------------------------*/
//double	CalculateFreePlayer (TPlayer * pTeammate, int x, int y) ?????
double	TPerception::CalculateFreePlayer (TPlayer * pTeammate)
{
	int	i;
	int	iRangeAccessBall;
	double dQuality = 0.0;
	double dQualityOfPossibility;		// 0.5
	double dQualityOfPassDir;	// 0.2
	double dQualityOfMateDistance;	// 0.1
	double dQualityOfGoalDistance;
	double dDistanceToOwnGoal;
	double dDelta;
	double dFactor;
	double dAccessRange;
	double dAccessQuality;
	double dDist;
	double dQualityDistOpp;
	double dDistOppMate;
	TDirection dir;
	TDirection iOptimalAngle;
	TDirection iDelta;
	TDirection iWidth;
	TVector	tOptimalPassPoint;
	int		iDeltaStart;
	int		iDeltaStop;
	CString	msg;
	CString	msgTmp;
	CString	msgTmp2;

	// init the passable area 
	dDist = GetObjectDistance(pTeammate);
	dir = GetObjectDirection(pTeammate);
	iRangeAccessBall = GetRangeFromDistance(dDist);
	pTeammate->InitFreePassArea(dir, iRangeAccessBall);

	//calculate quality of distance to the teammate
	if (dDist < 20.0)
		dQualityOfMateDistance = m_pAgent->GetLinearFuzzyValue(dDist, 5.0, 10.0);
	else
		dQualityOfMateDistance = m_pAgent->GetLinearFuzzyValue(dDist, 25.0, 40.0, FALSE);

	if (dQualityOfMateDistance > 0.0)
	{
		//set quality of distance to own goal
		dDistanceToOwnGoal = pTeammate->GetDistanceTo(m_pOwnGoal);
		dQualityOfGoalDistance = m_pAgent->GetLinearFuzzyValue(dDistanceToOwnGoal, 10.0, 40.0);

		if (dQualityOfGoalDistance > 0.0)
		{
			//calculate quality of optimal pass direction
			GetOptimalPassPoint(tOptimalPassPoint, iDeltaStart, iDeltaStop);
			iOptimalAngle = m_pThisPlayer->GetAngleTo(tOptimalPassPoint);
			iWidth = dir - iOptimalAngle;
			iWidth.Abs();
			
			// check if teammate is close to goal. then it is free anyhow
			int x = (int) pTeammate->GetAbsX();
			int y = (int) pTeammate->GetAbsY();
			if (GetSide() == RIGHT_TO_LEFT)
			{
				x = -x;
				y = -y;
			}
			if ((x > 34) && (y < 16) && (y > -16))
			{
				// player is in penalty area, so it is a good direction
				iWidth = 0;
			}

			// check if we are attacked by an opponent or tired, which makes
			// us to consider more teammates for passing
			// check the space we have in other goal direction
			dAccessRange = GetAccessDistance(GetObjectDirection(GetOtherGoal()));
			dAccessQuality = m_pAgent->GetLinearFuzzyValue(dAccessRange, 3.0, 6.0, FALSE);
			
			// the more demanding of teammate goal distance and own distance to opponent counts
//			if (dAccessQuality < dQualityOfGoalDistance)
				dFactor = dAccessQuality;
//			else
//				dFactor = dQualityOfGoalDistance;

			// dDelta is used to describe the min distance, an opponent should have
			// from a teammate
			dDelta = 8.0 * (1.15 - dFactor);
			// iDelta Stop is the angle differnce at which a player is no more passable
//			iDeltaStop = (int) ((double) iDeltaStop * dFactor);
			dQualityOfPassDir = m_pAgent->GetLinearFuzzyValue(iWidth, iDeltaStart, iDeltaStop, FALSE);

			if (dQualityOfPassDir > 0.0)
			{
				//calculate quality of possibility
				dQualityDistOpp = 1.0;
				for (i = 0; i < PLAYERS_PER_TEAM; i++)
				{
					if (m_pOtherTeam[i]->IsInMind())
					{
						// look if there are opponents around the player
						dDistOppMate = pTeammate->GetDistanceTo(m_pOtherTeam[i]);
						dQualityDistOpp *= m_pAgent->GetLinearFuzzyValue(dDistOppMate, dDelta, dDelta + 3.0);
	
						// look if the direction is passable
						if (GetObjectDistance(m_pOtherTeam[i]) < dDist + 1.0)
						{
							iRangeAccessBall = GetRangeFromDistance(GetObjectDistance(m_pOtherTeam[i]));
							if (pTeammate->UpdateFreePassArea(GetObjectDirection(m_pOtherTeam[i]), iRangeAccessBall))
								msgTmp.Format("%d* ", i + 1);
							else
								msgTmp.Format("%d ", i + 1);
							msgTmp2 += msgTmp;
						}
					}
				}
				dQualityOfPossibility = m_pAgent->GetLinearFuzzyValue(
											(double) pTeammate->GetFreePassArea(), 15, 45);

				//calculation of the quality
				dQuality = dQualityDistOpp * dQualityOfPossibility * dQualityOfPassDir 
								* dQualityOfMateDistance * dQualityOfGoalDistance;

				if (dQuality > 0.1)
				{
					msg.Format("Free Player %d: %4.1f (%4.1f %4.1f %4.1f %4.1f %4.1f ) angle: %d  optimal: %d, "
							, pTeammate->GetNum(), dQuality
							, dQualityDistOpp, dQualityOfPossibility, dQualityOfPassDir 
							, dQualityOfMateDistance, dQualityOfGoalDistance
							, (int) dir, (int) iOptimalAngle);
					msg += msgTmp2;
					DisplayDebug(msg, KATEGORY_PERCEPTION, DEBUG_LEVEL_4);		
				}
				//				dQuality = pow(dQuality, 0.2);
			}
		}
	}

	pTeammate->SetFreePassDegree(dQuality);
	return dQuality;
}

// returns the angle a player (opponent) covers depending on the distance of the player
int TPerception::GetRangeFromDistance(double dist)
{
	return 15 + (int) (20.0 * m_pAgent->GetLinearFuzzyValue(dist, 8.0, 30.0));
/*	if (dist < 2.0)
		return 10;

	double	speed = m_pAgent->GetPassPower(dist) * KICK_POWER_RATE;
	// this is the time the ball needs to come to the specified distance
	double	time;
	double	bracket = 1.0 - dist / speed * (1.0 - BALL_DECAY);
	if (bracket < D_ZERO_P)
		time = 30;
	else
		time = log(bracket) / log(BALL_DECAY);

	double	range = time * PLAYER_SPEED_MAX;
	int		direction = (int) (atan(range / dist) * RAD2DEG);
	
	CString msg;
	msg.Format("dist:%4.2f dir:%d range:%4.2f speed:%4.2f time:%4.2f",dist, direction, range, speed, time);
	DisplayDebug(msg, KATEGORY_PERCEPTION, DEBUG_LOWPERCEPTION);
	
	return direction;
*/
}

/*-----------------------------------------------------------------------------
Function: void TPerception::GetOptimalPassPoint(TVector& point)
Date    : 08.07.99
-------------------------------------------------------------------------------
Parameter: reference to the vector that gets the result
			  reference to the max delta direction that has value 1.0
			  reference to the min delta direction that has value 0.0
Returnval: (void)
Description: calculates the optimal point the direction to which is optimal to pass
-----------------------------------------------------------------------------*/
void TPerception::GetOptimalPassPoint(TVector& point, int& iDeltaStart, int& iDeltaStop)
{
	int	x = (int) m_pThisPlayer->GetAbsX();
	int	y = (int) m_pThisPlayer->GetAbsY();

	if (GetSide() == RIGHT_TO_LEFT)
	{
		x = -x;
		y = -y;
	}

	if (x < -30)
	{
		// near the own goal, the player should pass to one of the corners
		if (y > 0)
			point = GetRightMidline()->GetPosition();
		else
			point = GetLeftMidline()->GetPosition();

		iDeltaStart = 25;
		iDeltaStop = 45;
	}
	else if (x < 0)
	{
		// in own half player passes to other Penalty corner
		if (y > 15)
		{
			point = GetOtherRightPenaltyCorner()->GetPosition();
			iDeltaStart = 20;
			iDeltaStop = 45;
		}
		else if (y < -15)
		{
			point = GetOtherLeftPenaltyCorner()->GetPosition();
			iDeltaStart = 20;
			iDeltaStop = 45;
		}
		else
		{
			point = GetOther11mPoint()->GetPosition();
			iDeltaStart = 40;
			iDeltaStop = 80;
		}
	}
	else if (x < 30)
	{
		// in other half player passes to 11m
		point = GetOther11mPoint()->GetPosition();
		if (y > 15)
		{
			iDeltaStart = 35;
			iDeltaStop = 60;
		}
		else if (y < -15)
		{
			iDeltaStart = 35;
			iDeltaStop = 60;
		}
		else
		{
			iDeltaStart = 40;
			iDeltaStop = 90;
		}
	}
	else 
	{
		// in other half player passes to other Penalty corner
		point = GetOther11mPoint()->GetPosition();
		if (y > 15)
		{
			iDeltaStart = 20;
			iDeltaStop = 40;
		}
		else if (y < -15)
		{
			iDeltaStart = 20;
			iDeltaStop = 40;
		}
		else
		{
			iDeltaStart = 40;
			iDeltaStop = 90;
		}
	}
}

/*-----------------------------------------------------------------------------
Function: BOOL TPerception::IsPlayerTired(TPlayer* player)
Date    : 12.07.99
-------------------------------------------------------------------------------
Parameter: none
Returnval: true, if the player can not run to the ball, because it is tired
Description: looks if the player is fit enough to run to the ball 
-----------------------------------------------------------------------------*/
BOOL TPerception::IsPlayerTired(TPlayer* player)
{
	double	ballDist = player->GetDistanceTo(m_pBall);
	int		stamina = player->GetStamina();

	if (stamina - ballDist * 60 < LOW_STAMINA - 300)
		return TRUE;
	else
		return FALSE;
}

/*-----------------------------------------------------------------------------
Function: void TPerception::CalculateFunctionalObjects()
Date    : 13.03.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: void
Description: calculates, which object has got which function in the environment. 
				 E.g. the opponent at the ball or the teammate nearest to me
-----------------------------------------------------------------------------*/
void TPerception::CalculateFunctionalObjects()
{
	// check the own team
	CheckOwnTeamForFunctionalObjects();

	// decay offside
	CheckBallForOffside();

	// check the other team
	CheckOtherTeamForFunctionalObjects();

	// if we know all players, we know about offside
	if (m_iNoOfTeammatesInMind + m_iNoOfOpponentsInMind >= 21)
	{
		KnowOffside();
	}
}

/*-----------------------------------------------------------------------------
Function: void TPerception::CheckOwnTeamForFunctionalObjects()
Date    : 14.04.99
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: calculates functional objects of agent's own team
-----------------------------------------------------------------------------*/
void TPerception::CheckOwnTeamForFunctionalObjects()
{
	int		i;
	int		iDistBallTeammate;
	int		iMinDistBallTeammate = 1000;
	int		i2MinDistBallTeammate = 1000;
//	int		iDistMeTeammate;
//	int		iMinDistMeTeammate = 1000;
	int		x = (int) m_pThisPlayer->GetAbsX();
	int		y = (int) m_pThisPlayer->GetAbsY();
	double	dTeammateX;
	double	dFrontmostX = 0.0;
	double	dBackmostX = 0.0;
	double	d2BackmostX = 0.0;
	
	// make coordinates team independent
	if (m_iSide == RIGHT_TO_LEFT)
	{
		x = -x;
		y = -y;
	}

	m_pTeammateAtBall = NULL;
	m_p2TeammateAtBall = NULL;
	m_pTeammateAtMe = NULL;
	m_pFrontmostTeammate = NULL;
	m_pBackmostTeammate = NULL;
	m_p2BackmostTeammate = NULL;
	m_iNoOfTeammatesInMind = 0;

	// check the own team
	for (i = 0; i < PLAYERS_PER_TEAM; i++)
	{
		if ((m_pOwnTeam[i]->IsInMind()) && !m_pOwnTeam[i]->IsThisAgent())
		{
			m_iNoOfTeammatesInMind++;
			// we have a teammate that is in mind and is not this agent itself
			// and we do not take goalie into account during normal play
			// look for the teammate at the ball (where it is in 7 timesteps)
			if (m_pOwnTeam[i]->IsAlife(m_iSenseTime) && !IsPlayerTired(m_pOwnTeam[i])
				&& (((i + 1) != GOALIE_ID) || (GetPlayMode() != PLAY_ON)))
			{
				iDistBallTeammate = (int) m_pOwnTeam[i]->GetDistanceTo(*m_pFutureBallPosition[7]);
				if (iDistBallTeammate < i2MinDistBallTeammate)
				{
					if (iDistBallTeammate < iMinDistBallTeammate)
					{
						i2MinDistBallTeammate = iMinDistBallTeammate;
						iMinDistBallTeammate = iDistBallTeammate;
						m_p2TeammateAtBall = m_pTeammateAtBall;
						m_pTeammateAtBall = m_pOwnTeam[i];
					}
					else
					{
						i2MinDistBallTeammate = iDistBallTeammate;
						m_p2TeammateAtBall = m_pOwnTeam[i];
					}
				}
			}
/*
			// look for teammate at me
			iDistMeTeammate = (int) GetObjectDistance(m_pOwnTeam[i]);
			if (iDistMeTeammate < iMinDistMeTeammate)
			{
				iMinDistMeTeammate = iDistMeTeammate;
				m_pTeammateAtMe = m_pOwnTeam[i];
			}
*/
			// look for frontmost player
			if (m_iSide == RIGHT_TO_LEFT)
				dTeammateX = m_pOwnTeam[i]->GetAbsX();
			else
				dTeammateX = -m_pOwnTeam[i]->GetAbsX();
			if (dTeammateX < dFrontmostX)
			{
				dFrontmostX = dTeammateX;
				m_pFrontmostTeammate = m_pOwnTeam[i];
			}

			// look for backmost players
			if (m_iSide == RIGHT_TO_LEFT)
				dTeammateX = m_pOwnTeam[i]->GetAbsX();
			else
				dTeammateX = -m_pOwnTeam[i]->GetAbsX();
			if (dTeammateX > d2BackmostX)
			{
				if (dTeammateX > dBackmostX)
				{
					// backmost player
					d2BackmostX = dBackmostX;
					dBackmostX = dTeammateX;
					m_p2BackmostTeammate = m_pBackmostTeammate;
					m_pBackmostTeammate = m_pOwnTeam[i];
				}
				else
				{
					d2BackmostX = dTeammateX;
					m_p2BackmostTeammate = m_pOwnTeam[i];
				}
			}
		}
	}
	m_pFreeTeammate = CalculateFreestTeammate();
}

/*-----------------------------------------------------------------------------
Function: void TPerception::CheckOtherTeamForFunctionalObjects()
Date    : 14.04.99
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: checks the other team for functional objects
-----------------------------------------------------------------------------*/
void TPerception::CheckOtherTeamForFunctionalObjects()
{
	int		i;
//	int		j;
	double	dDistBallOpponent;
	double	dMinDistBallOpponent = 1000.0;
	double	dDistMeOpponent;
	double	dMinDistMeOpponent = 1000.0;
	double	d2MinDistMeOpponent = 1000.0;
	double	d3MinDistMeOpponent = 1000.0;
	int		iNoOfOffsidePreventingPlayers;
	double	dOpponentX;
	double	dBackmostX = 100.0;
	double	d2BackmostX = 100.0;
	int		x = (int) m_pThisPlayer->GetAbsX();
	int		y = (int) m_pThisPlayer->GetAbsY();

	// make coordinates team independent
	if (m_iSide == RIGHT_TO_LEFT)
	{
		x = -x;
		y = -y;
	}

	iNoOfOffsidePreventingPlayers = 0;
	m_pOpponentAtBall = NULL;
	m_pOpponentAtMe = NULL;
	m_p2OpponentAtMe = NULL;
	m_p3OpponentAtMe = NULL;
	m_pBackmostOpponent = NULL;
	m_p2BackmostOpponent = NULL;
	m_iNoOfOpponentsInMind = 0;

		// check the other team
	for (i = 0; i < PLAYERS_PER_TEAM; i++)
	{
		if (m_pOtherTeam[i]->IsInMind())
		{
			m_iNoOfOpponentsInMind++;

			// look for the opponent at the ball
			dDistBallOpponent = m_pOtherTeam[i]->GetDistanceTo(m_pBall);
			if (dDistBallOpponent < dMinDistBallOpponent)
			{
				dMinDistBallOpponent = dDistBallOpponent;
				m_pOpponentAtBall = m_pOtherTeam[i];
			}

			// look for Opponent at me
			dDistMeOpponent = GetObjectDistance(m_pOtherTeam[i]);
			if (dDistMeOpponent < dMinDistMeOpponent)
			{
				d3MinDistMeOpponent = d2MinDistMeOpponent;
				d2MinDistMeOpponent = dMinDistMeOpponent;
				dMinDistMeOpponent = dDistMeOpponent;
				m_p3OpponentAtMe = m_p2OpponentAtMe;
				m_p2OpponentAtMe = m_pOpponentAtMe;
				m_pOpponentAtMe = m_pOtherTeam[i];
			}
			else if (dDistMeOpponent < d2MinDistMeOpponent)
			{
				d3MinDistMeOpponent = d2MinDistMeOpponent;
				d2MinDistMeOpponent = dDistMeOpponent;
				m_p3OpponentAtMe = m_p2OpponentAtMe;
				m_p2OpponentAtMe = m_pOtherTeam[i];
			}
			else if (dDistMeOpponent < d3MinDistMeOpponent)
			{
				d3MinDistMeOpponent = dDistMeOpponent;
				m_p3OpponentAtMe = m_pOtherTeam[i];
			}

			// check if this player prevents me from beeing offside
			if (!m_bKnowNotInOffside && m_pThisPlayer->CheckPreventOffside(m_pOtherTeam[i]))
			{
				// there have to be two players behind this agent (goalie + another)
				iNoOfOffsidePreventingPlayers++;
				if (iNoOfOffsidePreventingPlayers > 1)
				{
					m_bKnowNotInOffside = TRUE;
					m_dInOffside = 0.0;
					m_iCheckOffsideCounter = 0;
				}
			}

			// look for 2nd backmost player
			if (m_iSide == RIGHT_TO_LEFT)
				dOpponentX = m_pOtherTeam[i]->GetAbsX();
			else
				dOpponentX = -m_pOtherTeam[i]->GetAbsX();
			if (dOpponentX < d2BackmostX)
			{
				if (dOpponentX < dBackmostX)
				{
					// backmost player
					d2BackmostX = dBackmostX;
					dBackmostX = dOpponentX;
					m_p2BackmostOpponent = m_pBackmostOpponent ;
					m_pBackmostOpponent = m_pOtherTeam[i];
				}
				else
				{
					d2BackmostX = dOpponentX;
					m_p2BackmostOpponent = m_pOtherTeam[i];
				}
			}
		}
	}

	// look for opponent to be marked 
	m_pOpponentToMark = NULL;
	if (m_pOpponentAtMe != NULL)
	{
		TPlayer*	o1 = m_pOpponentAtMe;
		TPlayer*	o2 = m_p2OpponentAtMe;
		TPlayer*	o3 = m_p3OpponentAtMe;
		TPlayer*	temp;
		double		d1;
		double		d2;
		double		d3;
		double		dtemp;
		
		if (o2 == NULL)
		{
			o2 = o1;
		}
		if (o3 == NULL)
		{
			o3 = o2;
		}
		
		if (GetObjectDistance(GetOwnGoal()) > 16.0)
		{
			// order them by distance to own goal
			d1 = o1->GetDistanceTo(GetOwnGoal());
			d2 = o2->GetDistanceTo(GetOwnGoal());
			d3 = o3->GetDistanceTo(GetOwnGoal());

			if (d2 < d1)
			{
				temp = o2; o2 = o1;	o1 = temp;
				dtemp = d2; d2 = d1; d1 = dtemp;
			}
			if (d3 < d1)
			{
				temp = o3; o3 = o1;	o1 = temp;
				dtemp = d3; d3 = d1; d1 = dtemp;
			}
			if (d3 < d2)
			{
				temp = o2; o2 = o3;	o3 = temp;
				dtemp = d2; d2 = d3; d3 = dtemp;
			}
		}

		// remember which teammate is already marking another opponent
		int mark1 = 0;
		int mark2 = 0;

		if ((mark1 = CheckMarked(o1, mark1, mark2)) >= PLAYERS_PER_TEAM)
			m_pOpponentToMark = o1;
		else if ((mark2 = CheckMarked(o2, mark1, mark2)) >= PLAYERS_PER_TEAM)
			m_pOpponentToMark = o2;
		else if (CheckMarked(o3, mark1, mark2) >= PLAYERS_PER_TEAM)
			m_pOpponentToMark = o3;
	}
}

// checks if an opponent is marked
int TPerception::CheckMarked(TPlayer* pOpp, int mark1, int mark2)
{
	if (pOpp == NULL)
		// no opponent, so no one to mark
		return TRUE;

	// only mark opponents close to the own goal
	if (pOpp->GetDistanceTo(GetOwnGoal()) > 62.0)
		return TRUE;
		
	// check if another player is marking it
	double	distMeOpp = m_pThisPlayer->GetDistanceTo(pOpp);
	double	distHimOpp;
	int		i;

	for (i = 0; i < PLAYERS_PER_TEAM; i++)
	{
		if ((!m_pOwnTeam[i]->IsThisAgent()) && (m_pOwnTeam[i]->IsInMind()) 
			&& (i != mark1) && (i != mark2))
		{
			distHimOpp = m_pOwnTeam[i]->GetDistanceTo(pOpp);
			if (distHimOpp < distMeOpp)
				break;
		}
	}
	return i;

/*
	if (i < PLAYERS_PER_TEAM)
		return TRUE;
	else
		return FALSE;
*/
}

/*-----------------------------------------------------------------------------
Function: void TPerception::CreateAccessMap()
Date    : 24.02.2000
-------------------------------------------------------------------------------
Parameter: none
Returnval: void
Description: creates a map that gives the agent an idea on what the area is
				 he would be fastest
-----------------------------------------------------------------------------*/
void TPerception::CreateAccessMap()
{
	m_pAccessMap->Initialize(m_pThisPlayer->GetPosition(), m_pThisPlayer->GetAbsDir()
									, GetDistanceByStamina());
	m_pDirectionMap->Initialize(m_pThisPlayer->GetPosition(), m_pThisPlayer->GetAbsDir());

	for (int i = 0; i < PLAYERS_PER_TEAM; i++)
	{
		if (m_pOtherTeam[i]->IsInMind())
		{
			// add to the map (take into account the players facing direction)
			TDirection	bodyDir = m_pOtherTeam[i]->GetBodyDir();
			double		dist = GetObjectDistance(m_pOtherTeam[i]);
			TDirection	dir = GetObjectDirection(m_pOtherTeam[i]);
			BOOL			isGoalie = m_pOtherTeam[i]->IsGoalie();

			// calculate relative direction from absolute one
			bodyDir = GetThisPlayer()->GetRelativeFromAbsoluteDirection(bodyDir);
			m_pAccessMap->AddPlayer(dist, dir, bodyDir, isGoalie);
			m_pDirectionMap->AddPlayerDir(dist, dir);
		}
	}
}

double TPerception::GetAccessDistance(int iRelDir)
{
	return m_pAccessMap->GetDistanceFromIndex(m_pAccessMap->GetIndexFromAngle(iRelDir));
}

double TPerception::GetDirectionDistance(int iRelDir)
{
	return m_pDirectionMap->GetDistanceFromIndex(m_pDirectionMap->GetIndexFromAngle(iRelDir));
}

/*-----------------------------------------------------------------------------
Function: BOOL TPerception::KnowOffside()
Date    : 31.07.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: TRUE, if the player is in offside
Description: called, when the player has seen complete field to other goal, so 
				 he should know about offside. This function then decides if he is 
				 in offside
-----------------------------------------------------------------------------*/
BOOL TPerception::KnowOffside()
{
	if (!m_bKnowNotInOffside)
	{
		// the farer the player is in offside, the longer he can be sure he will
		// need to leave offside
//		double dDistanceToOffsideLine = Get2OpponentAtMe()->GetAbsX() - GetThisPlayer()->GetAbsX();
		TPlayer*	opponent = Get2BackmostOpponent();
		double		dDistanceToOffsideLine = 5.0;
		if (opponent != NULL)
			dDistanceToOffsideLine = opponent->GetAbsX() - GetThisPlayer()->GetAbsX();

		if (dDistanceToOffsideLine < 0)
			dDistanceToOffsideLine = -dDistanceToOffsideLine;
		
		// know 1 sec if 5m in offside, 2 sec for 10m and so on
		m_dOffsideMinus = m_pAgent->GetLinearFuzzyValue(dDistanceToOffsideLine, 0.0, 20.0) * 50.0;
		if (m_dOffsideMinus < 20.0)
			m_dOffsideMinus = 0.05;
		else
			m_dOffsideMinus = 1.0 / m_dOffsideMinus;

		// player is not sure if in offside or not, now he can be: offside
		m_dInOffside = 1.0;
	}

	// we saw the complete field. the following counter will store this info for some cycles
	m_iSawAll = MAX_SAY_DELAY;
	m_iCheckOffsideCounter = 0;
	return FALSE;
}

/*-----------------------------------------------------------------------------
Function: void TPerception::CheckBallForOffside()
Date    : 14.04.99
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: decays offside truth value and checks if ball prevents agent from
				 beeing in offside
-----------------------------------------------------------------------------*/
void TPerception::CheckBallForOffside()
{
	m_dInOffside -= m_dOffsideMinus;
	if (m_dInOffside < 0.0)
		m_dInOffside = 0.0;

	m_bKnowNotInOffside = FALSE;

	// check if ball prevents offside
	if (m_iSide == LEFT_TO_RIGHT)	
	{
		if (m_pBall->GetAbsX() - m_pThisPlayer->GetAbsX() > 0)
		{
			m_bKnowNotInOffside = TRUE;
			m_dInOffside = 0.0;
			m_iCheckOffsideCounter = 0;
		}
	}	
	else
	{
		if (m_pBall->GetAbsX() - m_pThisPlayer->GetAbsX() < 0)
		{
			m_bKnowNotInOffside = TRUE;
			m_dInOffside = 0.0;
			m_iCheckOffsideCounter = 0;
		}
	}	
}

/*-----------------------------------------------------------------------------
Function: TPlayer* TPerception::GetAnyPlayerAtBall()
Date    : 27.05.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: the teammate or opponent, that is visible and is closest to this
Description: compares the two InFront players in distance and takes the distance
				 of this agent to the ball into account
-----------------------------------------------------------------------------*/
TPlayer* TPerception::GetAnyPlayerAtBall()
{
	if (m_pTeammateAtBall == NULL)
		return m_pOpponentAtBall;
	if (m_pOpponentAtBall == NULL)
		return m_pTeammateAtBall;
	if (GetObjectDistance(m_pTeammateAtBall) < GetObjectDistance(m_pOpponentAtBall))
		return m_pTeammateAtBall;
	else
		return m_pOpponentAtBall;
}


/*-----------------------------------------------------------------------------
Function: void TPerception::SetStamina(int stamina)
Date    : 27.08.98
-------------------------------------------------------------------------------
Parameter: stamina of the agent
Returnval: (void)
Description: sets the stamina and the stamina of thisPlayer for graphics
-----------------------------------------------------------------------------*/
void TPerception::SetStamina(int stamina)
{
	m_iStamina = stamina;
	m_pThisPlayer->SetStamina(stamina);

	// check if we get see messages
	// a server error sometimes prevents agents with 180 view from getting see
	if ((m_iSenseTime - m_iSeeTime > 20) && m_pAgent->IsConnected())
		m_pAgent->ChangeView(NORMAL_VIEW, 1);
}

/*-----------------------------------------------------------------------------
Function: void TPerception::GetListVisiblePlayers(CObList & pList)
Date    : 13.05.99
From    : Markus Plewinski
-------------------------------------------------------------------------------
Parameter: list start pointer
Description: creates a list of all visible players
-----------------------------------------------------------------------------*/

void TPerception::GetListVisiblePlayers(CObList & pList)
{
	int		i;

	for (i = 0; i < m_iNoOfPlayers; i++)
	{
		Insert(pList,m_pVisiblePlayer[i]);
	}
}


/*-----------------------------------------------------------------------------
Function: CString TPerception::GetValidSeen()
Date    : 11.06.99
From    : Markus Plewinski
-------------------------------------------------------------------------------
Parameter: - 
Description: creates a string with number of objects:
  ball, own and other players position who are valid. divided by " ".
-----------------------------------------------------------------------------*/

CString TPerception::GetValidSeen()
{
	CString	sMsg;
	CString	sStr1;
	CString	sStr2;
	char  	cSep = ' ';
	int		i;
	int		j = 0;
	int		iNum;
	int		xPos;
	int		yPos;
	int		age;
	TVector	pos;

	//position for the ball - object number 0
	if (m_iSeeTime - m_pBall->GetTime() <= MAX_SAY_DELAY)
	{
		iNum = 0;
		xPos = (int) (m_pFutureBallPosition[1]->GetX() * 10.0);
		yPos = (int) (m_pFutureBallPosition[1]->GetY() * 10.0);
		age  = m_iSeeTime - m_pBall->GetTime();
		if ((age < 0) || (age > 9))
			age = 9;

		sStr2.Format("%d %d %d %d ", iNum, xPos, yPos, age);
		sStr1 += sStr2;
		j++;
	}

	// position from dispatcher
	iNum = m_pThisPlayer->GetNum();
	xPos = (int) (m_pThisPlayer->GetAbsX() * 10.0);
	yPos = (int) (m_pThisPlayer->GetAbsY() * 10.0);
	age  = 0;

	sStr2.Format("%d %d %d %d ", iNum, xPos, yPos, age);
	sStr1 += sStr2;

	j++;

	//position of all seen players - own and other
/*
	for (i = 0; i < m_iNoOfPlayers; i++)
	{
		iNum = m_pVisiblePlayer[i]->GetNum();
		if( iNum != NOT_EXISTING )
		{
			xPos = (int) (m_pVisiblePlayer[i]->GetAbsX() * 10.0);
			yPos = (int) (m_pVisiblePlayer[i]->GetAbsY() * 10.0);

			if( !m_pVisiblePlayer[i]->IsOwnTeam() )
				iNum += 20;

			sStr2.Format("%d %d %d ", iNum, xPos, yPos);
			sStr1 += sStr2;
			j++;
		}
	}
*/
	for (i = 0; i < PLAYERS_PER_TEAM; i++)
	{
		// own team
		iNum = i + 1;
		if (m_iSeeTime - m_pOwnTeam[i]->GetTime() <= MAX_SAY_DELAY)
		{
/*			xPos = (int) (m_pOwnTeam[i]->GetAbsX() * 10.0);
			yPos = (int) (m_pOwnTeam[i]->GetAbsY() * 10.0);
*/
			m_pOwnTeam[i]->GetFuturePosition(1, pos);
			xPos = (int) (pos.GetX() * 10.0);
			yPos = (int) (pos.GetY() * 10.0);
			age  = m_iSeeTime - m_pOwnTeam[i]->GetTime();
			if ((age < 0) || (age > 9))
				age = 9;
			sStr2.Format("%d %d %d %d ", iNum, xPos, yPos, age);
			sStr1 += sStr2;
			j++;
		}

		// other team
		iNum = i + 21;
		if (m_iSeeTime - m_pOtherTeam[i]->GetTime() <= MAX_SAY_DELAY)
		{
/*			xPos = (int) (m_pOtherTeam[i]->GetAbsX() * 10.0);
			yPos = (int) (m_pOtherTeam[i]->GetAbsY() * 10.0);
*/
			m_pOtherTeam[i]->GetFuturePosition(1, pos);
			xPos = (int) (pos.GetX() * 10.0);
			yPos = (int) (pos.GetY() * 10.0);
			age  = m_iSeeTime - m_pOtherTeam[i]->GetTime();
			if ((age < 0) || (age > 9))
				age = 9;
			sStr2.Format("%d %d %d %d ", iNum, xPos, yPos, age);
			sStr1 += sStr2;
			j++;
		}
	}

	// store length and if player saw all
	sStr2.Format("%d %d ", j, m_iSawAll);
	sMsg = sMsg + sStr2;
	sMsg = sMsg + sStr1;

	return sMsg;
}

/*-----------------------------------------------------------------------------
Function: void TPerception::OnDashed(int power)
Date    : 07.07.99
-------------------------------------------------------------------------------
Parameter: power with which the player kicked
Returnval: (void)
Description: called when the player sent a dash command
-----------------------------------------------------------------------------*/
void TPerception::OnDashed(int power)
{
	UpdatePerceptionAfterDash(power);
	m_iExpectedNoOfDash = m_iNoOfDash + 1;
	m_iDashPower = power;
}

/*-----------------------------------------------------------------------------
Function: void TPerception::OnKicked(int iPower, int iDirection)
Date    : 29.06.99
-------------------------------------------------------------------------------
Parameter: power (0 .. 100)
			  relative direction to body of kick
Returnval: (void)
Description: updates the ball
-----------------------------------------------------------------------------*/
void TPerception::OnKicked(int iPower, int iDirection)
{
	UpdatePerceptionAfterKick(iPower, iDirection);
	m_iExpectedNoOfKick = m_iNoOfKick + 1;
	m_iKickPower = iPower;
	m_iKickDirection= iDirection;
}

/*-----------------------------------------------------------------------------
Function: void TPerception::OnTurned(int dir)
Date    : 08.03.99
-------------------------------------------------------------------------------
Parameter: angle that has been turned
Returnval: (void)
Description: called after the agent executed a turn command
-----------------------------------------------------------------------------*/
void TPerception::OnTurned(int dir)
{
	UpdatePerceptionAfterTurn(dir);
	m_iExpectedNoOfTurn = m_iNoOfTurn + 1;
	m_iTurnAngle = dir;
}

/*-----------------------------------------------------------------------------
Function: void TPerception::OnTurnedNeck(int dir)
Date    : 08.03.99
-------------------------------------------------------------------------------
Parameter: angle that has been turned
Returnval: (void)
Description: called after the agent executed a turn_neck command
-----------------------------------------------------------------------------*/
void TPerception::OnTurnedNeck(int dir)
{
	UpdatePerceptionAfterTurnNeck(dir);
	m_iExpectedNoOfTurnNeck = m_iNoOfTurnNeck + 1;
	m_iTurnAngleNeck = dir;
	m_iHeadAngle += dir;
}

/*-----------------------------------------------------------------------------
Function: void TPerception::UpdatePerceptionAfterDash()
Date    : 07.07.99
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: updates the agent's speed and position according to the executed dash command
-----------------------------------------------------------------------------*/
void TPerception::UpdatePerceptionAfterDash(int power)
{
	double	dSpeed;

	if (power < 0)
		power = -power;
	if (power > 100)
		power = 100;

	// update position after a dash is done once sense body is received
	// but since sense_body passes the decayed speed, we have to update the position
	// for the not decayed part.
//	dSpeed = DASH_POWER_RATE * GetEffort() * power;
//	dSpeed -= (dSpeed + GetSpeed()) * PLAYER_DECAY - GetSpeed();
	dSpeed = m_playerType[m_currentPlayerType][dash_power_rate] * GetEffort() * power;
	dSpeed -= (dSpeed + GetSpeed()) * m_playerType[m_currentPlayerType][player_decay] - GetSpeed();
	m_pThisPlayer->SetPushSpeed(dSpeed);
}

/*-----------------------------------------------------------------------------
Function: void TPerception::UpdatePerceptionAfterKick(int iPower, int iDirection)
Date    : 29.06.99
-------------------------------------------------------------------------------
Parameter: power and relative direction of the kick
Returnval: (void)
Description: updates the ball speed according to the executed kick command
-----------------------------------------------------------------------------*/
void TPerception::UpdatePerceptionAfterKick(int iPower, int iDirection)
{
	// we assume, that the kick has been successfull
	TVector		newSpeed;
	TVector		ballSpeed = m_pBall->GetAbsSpeed();
	double		dBallDist = GetObjectDistance(m_pBall) - GetPlayerSize() - BALL_SIZE;
	double		dDeltaDir = (double) m_pThisPlayer->GetAngleToBody(m_pBall->GetPosition()) / 180.0;
	double		dAcceleration;
	TDirection	iDir = iDirection + m_pThisPlayer->GetAbsBodyDir();
	CString		msg;

	if (dBallDist > GetKickableMargin())
		// the ball is too far to kick
		return;

	if (dBallDist < 0.0)
		// should not happen
		dBallDist = 0.0;

	if (dDeltaDir < 0.0)
		dDeltaDir = -dDeltaDir;

	if (dDeltaDir > 1.0)
		// should not happen
		dDeltaDir = 1.0;

	// calculate the acceleration the ball is expected to get
	dAcceleration = iPower * KICK_POWER_RATE 
							* (1.0 - 0.25 * dDeltaDir - 0.25 * dBallDist / GetKickableMargin());
	
	newSpeed.SetVectorByDirection(dAcceleration, iDir);

	ballSpeed += newSpeed;
/*
	msg.Format("UpdatePerception after Kick: dAcceleration: %4.2f dDeltaDir: %4.2f dBallDistFaktor: %4.2f new ball speed: %4.2f"
				, dAcceleration, dDeltaDir, dBallDist / GetKickableMargin(), ballSpeed.GetLength());
	DisplayDebug(msg, KATEGORY_PERCEPTION, DEBUG_LOWPERCEPTION);
*/
	// set absolute ball speed
	m_pBall->SetAbsSpeed(ballSpeed);
	
	// ball position is updated next UpdatePerception
}

/*-----------------------------------------------------------------------------
Function: void TPerception::UpdatePerceptionAfterTurn(int angle)
Date    : 23.06.99
-------------------------------------------------------------------------------
Parameter: angle the agent turned or turned_neck
Returnval: (void)
Description: all objects' directions except lines and flags are updated
-----------------------------------------------------------------------------*/
void TPerception::UpdatePerceptionAfterTurn(int angle)
{
	TDirection	direction;
	CString		msg;
	
	// update the agent's absolute body direction
	direction = m_pThisPlayer->GetAbsBodyDir();
/*	
	msg.Format("UpdatePerception after Turn: angle: %d oldDir: %d newDir: %d  seeTime: %d senseTime: %d noOfTurn: %d expected: %d"
				, angle, (int) direction, (int) direction + angle, m_iSeeTime, m_iSenseTime, m_iNoOfTurn, m_iExpectedNoOfTurn);
	DisplayDebug(msg, KATEGORY_PERCEPTION, DEBUG_LOWPERCEPTION);
*/
	direction += angle;
	m_pThisPlayer->SetAbsBodyDirection(direction);

	// remember the expected body direction to check at the next see, if
	// the turn has already been executed
	m_tExpectedBodyDirection = direction;

	UpdatePerceptionAfterTurnNeck(angle);
}

/*-----------------------------------------------------------------------------
Function: void TPerception::UpdatePerceptionAfterTurnNeck(int angle)
Date    : 23.06.99
-------------------------------------------------------------------------------
Parameter: angle the agent turned or turned_neck
Returnval: (void)
Description: all objects' directions except lines and flags are updated
-----------------------------------------------------------------------------*/
void TPerception::UpdatePerceptionAfterTurnNeck(int angle)
{
	int			i;
	TDirection	direction;
	CString		msg;

	// update the agent's absolute view direction
	direction = m_pThisPlayer->GetAbsDir();
	
/*	msg.Format("UpdatePerception after TurnNeck: angle: %d oldHeadDir: %d newHeadDir: %d  seeTime: %d senseTime: %d noOfTurnNeck: %d expected: %d"
				, angle, (int) direction, (int) direction + angle, m_iSeeTime, m_iSenseTime, m_iNoOfTurnNeck, m_iExpectedNoOfTurnNeck);
	DisplayDebug(msg, KATEGORY_PERCEPTION, DEBUG_LOWPERCEPTION);
*/
	direction += angle;
	m_pThisPlayer->SetAbsDirection(direction);

	// remember the expected view direction to check at the next see, if
	// the turn has already been executed
	m_tExpectedViewDirection = direction;

	// update visible players
	for (i = 0; i < m_iNoOfPlayers; i++)
		m_pVisiblePlayer[i]->UpdateDirection(angle);

	// update remembered players (global map)
	for (i = 0; i < PLAYERS_PER_TEAM; i++)
	{
		m_pOwnTeam[i]->UpdateDirection(angle);
		m_pOtherTeam[i]->UpdateDirection(angle);
	}

	// and other objects
	m_pOwnGoal->UpdateDirection(angle);
	m_pOtherGoal->UpdateDirection(angle);
	m_pBall->UpdateDirection(angle);
}

/*-----------------------------------------------------------------------------
Function: BOOL TPerception::IsOutside(TVector& pos)
Date    : 24.06.99
-------------------------------------------------------------------------------
Parameter: position that is checked
Returnval: true, if the position is not within the field
Description: checks the position to be outside or inside field
-----------------------------------------------------------------------------*/
BOOL TPerception::IsOutside(TVector& pos)
{
	double sizeX = PITCH_LENGTH / 2.0;
	double sizeY = PITCH_WIDTH / 2.0;

	if ((pos.GetX() < -sizeX) || (pos.GetX() > sizeX)
		 || (pos.GetY() < -sizeY) || (pos.GetY() > sizeY))
		return TRUE;
	else
		return FALSE;
}


/*-----------------------------------------------------------------------------
Function: TVector TPerception::GetAbsCoordinates(TVector &origin, double dist, int dir)
Date    : 10.02.2000
-------------------------------------------------------------------------------
Parameter: origin from where to start
			  distance and direction (degrees) where to go
Returnval: Vector containing the point described by radial coordinates
Description: returns a point that is dist away from origin in dir direction (abs)
-----------------------------------------------------------------------------*/
TVector TPerception::GetAbsCoordinates(TVector &origin, double dist, int dir)
{
	double	dDir = dir * DEG2RAD;
	return TVector(origin + TVector(cos(dDir), sin(dDir)) * dist);
}

/*-----------------------------------------------------------------------------
Function: double TPerception::GetDistanceByStamina() const
Date    : 29.02.2000
-------------------------------------------------------------------------------
Parameter: none
Returnval: distance, how far the agent can maximally go with 100 power
Description: 
-----------------------------------------------------------------------------*/
double TPerception::GetDistanceByStamina() const
{
	// 200 because the agent usually does not need to dash 100 when having ball
	double	dist = (double) (GetStamina() - LOW_STAMINA + 200) / 60.0;
	if (dist < 1.0)
		dist = 1.0;
	return dist;
}

/*-----------------------------------------------------------------------------
Function: void TViewingAgent::DisplayDebug(LPCTSTR msg, int category, int level)
Date    : 20.08.2000
-------------------------------------------------------------------------------
Parameter: msg to display
			  category action / perception
			  debug level
Returnval: void
Description: 
-----------------------------------------------------------------------------*/
void TPerception::DisplayDebug(LPCTSTR msg, int category, int level)
{
	m_pAgent->DisplayDebug(msg, category, level);
}

void TPerception::SetViewParams(int angle, int quality)
{ 
	m_iViewAngle = angle; 
	m_iViewQuality = quality; 

	// when changing the viewmode, we may not see something
	if (angle == NARROW_VIEW) 
		m_iSeeThisCycle	= 1;
	else
		m_iSeeThisCycle	= 0;

	m_pAgent->SetExpectedSee(m_iSeeThisCycle);
	
	CString message;
	message.Format("expected see after change: %d", m_iSeeThisCycle);
	DisplayDebug(message, KATEGORY_PERCEPTION, DEBUG_LOWPERCEPTION);
};