// TPerception.h: interface for the TPerception class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TPERCEPTION_H__DA8AA141_577A_11D1_B242_0000C09B5CBE__INCLUDED_)
#define AFX_TPERCEPTION_H__DA8AA141_577A_11D1_B242_0000C09B5CBE__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000


#include "TVisibleObject.h"
#include "TDirection.h"
#include "TPerceptionParser.h"

// no of stored future positions of the ball
#define MAX_FUTURE_POSITIONS	20
// max age of objects that are said to other players
#define MAX_SAY_DELAY			6

#define NARROW_VIEW		45
#define NORMAL_VIEW		90
#define WIDE_VIEW			180

#define NO_OF_FLAGS		50
#define CENTER_MARK		0
#define CENTER_TOP		1
#define CENTER_BOTTOM	2
#define GOAL_POST_L		3
#define GOAL_POST_R		5
#define FLAG_OUTSIDE_TL	7
#define FLAG_OUTSIDE_TC	12
#define FLAG_OUTSIDE_TR	13
#define FLAG_OUTSIDE_BL	18
#define FLAG_OUTSIDE_BC	23
#define FLAG_OUTSIDE_BR	24
#define FLAG_OUTSIDE_LT	29
#define FLAG_OUTSIDE_LC	32
#define FLAG_OUTSIDE_LB	33
#define FLAG_OUTSIDE_RT	36
#define FLAG_OUTSIDE_RC	39
#define FLAG_OUTSIDE_RB	40
#define FLAG_PENALTY_L	43
#define FLAG_PENALTY_R	46
#define TRASH_FLAG		49

#define FLAG_POSITIONS_X { 0.0, 0.0, 0.0, -52.5, -52.5, 52.5, 52.5\
				, -10.0, -20.0, -30.0, -40.0, -50.0, 0.0\
				, 10.0, 20.0, 30.0, 40.0, 50.0\
				, -10.0, -20.0, -30.0, -40.0, -50.0, 0.0\
				, 10.0, 20.0, 30.0, 40.0, 50.0\
				, -57.5, -57.5, -57.5, -57.5\
				, -57.5, -57.5, -57.5\
				, 57.5, 57.5, 57.5, 57.5\
				, 57.5, 57.5, 57.5\
				, -36.0, -36.0, -36.0, 36.0, 36.0, 36.0, 0.0 }

// Achtung attention
// goal posts are not positioned correct in server version 4.03b !!!, which
// is used by me. So y coordinates of goal posts 3-6 should be -7.01 and 7.01
// but are here -3.66 and 3.66
//#define FLAG_POSITIONS_Y {	0.0, - 34.0, 34.0, -3.66, 3.66, -3.66, 3.66 
#define FLAG_POSITIONS_Y {	0.0, - 34.0, 34.0, -7.01, 7.01, -7.01, 7.01\
				, -39.0, -39.0, -39.0, -39.0, -39.0, -39.0\
				, -39.0, -39.0, -39.0, -39.0, -39.0\
				, 39.0, 39.0, 39.0, 39.0, 39.0, 39.0\
				, 39.0, 39.0, 39.0, 39.0, 39.0\
				, -10.0, -20.0, -30.0, 0.0\
				, 10.0, 20.0, 30.0\
				, -10.0, -20.0, -30.0, 0.0\
				, 10.0, 20.0, 30.0\
				, -20.16, 0.0, 20.16, -20.16, 0.0, 20.16, 0.0 }


// positions within the perception string from server

#if (SERVER_VERSION > 5)
	#define PLAYER_NAME_POSITION	2
	#define	GOAL_NAME_POSITION		2
	#define FLAG_TAG1_POSITON		2
	#define	FLAG_TAG2_POSITON		4
	#define FLAG_TAG3_POSITON		6
#else
	#define PLAYER_NAME_POSITION	7
	#define	GOAL_NAME_POSITION		5
	#define FLAG_TAG1_POSITON		5
	#define	FLAG_TAG2_POSITON		7
	#define FLAG_TAG3_POSITON		9
#endif

class TScreenParams;
class TPerceptionView;
class TViewingAgent;
class TThisPlayer;
class TParseString;
class TAccessMap;
class TPerceptionParser;

/*-----------------------------------------------------------------------------
Class   : TPerception
Date    : 07.11.97
-------------------------------------------------------------------------------
Base Class: CObject

Description: contains the methods for parsing the perceptions
-----------------------------------------------------------------------------*/

class TPerception : public CObject  
{

	public:
		TPerception(TViewingAgent* pAgent);
		TPerception(TViewingAgent* pAgent, TPerceptionView* pView);
		virtual ~TPerception();
		
		void	Initialize();
		void	CreatePerception();
		void	NewMessage(LPCTSTR msg);
		BOOL	ParseMessage(LPCTSTR msg);
		void	DrawPerception(CDC* pDC, TScreenParams *tScreen);
		void	UpdatePerception();
		void	CalculateFutureBallPositions();
		void	InvalidateAll(void);
		void	PlayStep(CStringList* pList);

		// parameter functions
		double		GetPlayerSpeedMax()	{ return m_playerType[m_currentPlayerType][player_speed_max]; };
		int			GetStaminaIncMax()	{ return (int) m_playerType[m_currentPlayerType][stamina_inc_max]; };
		double		GetPlayerDecay()	{ return m_playerType[m_currentPlayerType][player_decay]; };
		double		GetInertiaMoment()	{ return m_playerType[m_currentPlayerType][inertia_moment]; };
		double		GetDashPowerRate()	{ return m_playerType[m_currentPlayerType][dash_power_rate]; };
		double		GetPlayerSize()		{ return m_playerType[m_currentPlayerType][player_size]; };
		double		GetKickableMargin() { return m_playerType[m_currentPlayerType][kickable_margin]; };
		double		GetKickRand()		{ return m_playerType[m_currentPlayerType][kick_rand]; };
		int			GetExtraStamina()	{ return (int) m_playerType[m_currentPlayerType][extra_stamina]; };
		int			GetCurrentPlayerType() { return m_currentPlayerType; };
		
		// Attribute functions
		BOOL		HasReceivedSee() const { return m_bReceivedSee; };
		int		GetSide() { return m_iSide; };
		void		SetPlayMode(int mode) { m_iPlayMode = mode; };
		int		GetPlayMode(void) const { return m_iPlayMode; };
		BOOL		GetPlaymodeOffsideOther();
		BOOL		GetPlaymodeOffsideOwn();
		BOOL		GetPlaymodeFreekickOther();
		BOOL		GetPlaymodeFreekickOwn();
		BOOL		GetPlaymodeKickInOther();
		BOOL		GetPlaymodeKickInOwn();
		int		GetSeeTime(void) const { return m_iSeeTime; };
		int		GetSenseTime(void) const { return m_iSenseTime; };
		BOOL		GetTimeProceeding(void) const { return m_bTimeProceeding; };
		int		GetLastHearNumber(void) const { return m_iLastHearNumber; };
		void		SetSide(int side);
		void		SetViewParams(int angle, int quality);
		int		SwitchViewAngle(void); 
		int		SwitchViewQuality(void);
		int		GetViewQuality(void) const { return m_iViewQuality; };
		int		GetViewAngle(void) const { return m_iViewAngle; };
		TViewingAgent*	GetAgent() { return m_pAgent; };

		int		GetNoOfKick() const { return m_iNoOfKick; };
		int		GetNoOfDash() const { return m_iNoOfDash; };
		int		GetNoOfTurn() const { return m_iNoOfTurn; };
		int		GetNoOfSay() const { return m_iNoOfSay; };
		int		GetNoOfTurnNeck() const { return m_iNoOfTurnNeck; };
		int		GetStamina() const { return m_iStamina; };
		void		SetStamina(int val);
		double	GetDistanceByStamina() const;
		void		SetSpeed(double val) { m_dSpeed = val; };
		double	GetSpeed() const { return m_dSpeed; };
		double	GetEffort() const { return m_dEffort; };
		int		GetNoOfCommandsNotExecuted() const { return m_iNoOfCommandsNotExecuted ; };
		
		void		SetNoise(double noise) { m_dNoise = noise; };
		
		void		OnDashed(int iPower);
		void		OnKicked(int iPower, int iAngle);
		void		OnTurned(int iAngle);
		void		OnTurnedNeck(int Angle);

		void		SetWaitForLastSenseBody(BOOL val) { m_bWaitForLastSenseBody = val; };
		int		GetLastCatchTime(void) const { return m_iLastCatchTime; };
		void		SetLastCatchTime(void) { m_iLastCatchTime = m_iSenseTime; };
		void		SetLastCatchTime(int time) { m_iLastCatchTime = time; };
		int		GetBodyAngle(void) const;
		int		GetHeadAngle(void) const { return m_iHeadAngle; };
		void		SetHeadAngle(int iAngle) { m_iHeadAngle = iAngle; };
		int		GetPlayersWithin(TDirection dir1, TDirection dir2);
		int		GetOpponentsWithin(TDirection dir1, TDirection dir2);

		// offside
		double	GetInOffside(void) const { return m_dInOffside; };
		BOOL		GetKnowNotInOffside(void) const { return m_bKnowNotInOffside; };
		int		GetCheckOffsideCounter(void) const { return m_iCheckOffsideCounter; };
		void		SetCheckOffsideCounter(int count) { m_iCheckOffsideCounter = count; };
		BOOL		KnowOffside();
		void		CheckBallForOffside();

		BOOL		GetGoalieCaught() const { return m_bGoalieCaught; };
		void		SetGoalieCaught(BOOL val) { m_bGoalieCaught = val; };

		// proposition functions
		BOOL			GetIsAt(int x, int y, int dist = 4) const;
		TDirection	GetObjectDirection(TVisibleObject* obj);
		TDirection	GetObjectDirectionToBody(TVisibleObject* obj);
		double		GetObjectDistance(TVisibleObject* obj);
		TDirection	GetObjectDeltaDir(TVisibleObject* obj);
		double		GetObjectDeltaDist(TVisibleObject* obj);
		BOOL			GetSeeObject(TVisibleObject* obj) const;
		int			GetObjectTime(TVisibleObject* obj) const { return obj->GetTime(); };

		// object functions
		TBall*			GetBall() { return m_pBall; };		
		TVector*			GetFutureBallPosition(int i) { ASSERT((i >= 0) && (i < MAX_FUTURE_POSITIONS));
																	return m_pFutureBallPosition[i]; };
		TSoccerGoal*	GetOwnGoal(); 
		TSoccerGoal*	GetOtherGoal();
		TThisPlayer*	GetThisPlayer() { return m_pThisPlayer; };
		TFlag*			GetOwnRightPenaltyCorner();
		TFlag*			GetOwnLeftPenaltyCorner();
		TFlag*			GetRightMidline();
		TFlag*			GetLeftMidline();
		TFlag*			GetOtherRightPenaltyCorner();
		TFlag*			GetOtherLeftPenaltyCorner();
		TFlag*			GetOtherRightCorner();
		TFlag*			GetOtherLeftCorner();
		TFlag*			GetOtherLeftGoalPost();
		TFlag*			GetOtherRightGoalPost();
		TFlag*			GetOwnLeftGoalPost();
		TFlag*			GetOwnRightGoalPost();
		TFlag*			GetOwn11mPoint() { return m_pOwn11mPoint; };
		TFlag*			GetOther11mPoint() { return m_pOther11mPoint; };

		void		CalculateFunctionalObjects();
		TPlayer*	GetOpponentAtBall() { return m_pOpponentAtBall; };
		TPlayer* GetOpponentAtMe() { return m_pOpponentAtMe; };
		TPlayer* Get2OpponentAtMe() { return m_p2OpponentAtMe; };
		TPlayer* Get3OpponentAtMe() { return m_p3OpponentAtMe; };
		TPlayer*	GetOpponentToMark() { return m_pOpponentToMark; };
		TPlayer* GetBackmostOpponent() { return m_pBackmostOpponent; };
		TPlayer* Get2BackmostOpponent() { return m_p2BackmostOpponent; };
		TPlayer* GetFrontmostTeammate() { return m_pFrontmostTeammate; };
		TPlayer* GetBackmostTeammate() { return m_pBackmostTeammate; };
		TPlayer* Get2BackmostTeammate() { return m_p2BackmostTeammate; };
		TPlayer* GetTeammateAtMe() { return m_pTeammateAtMe; };
		TPlayer* GetTeammateAtBall() { return m_pTeammateAtBall; };
		TPlayer* Get2TeammateAtBall() { return m_p2TeammateAtBall; };
		TPlayer* GetAnyPlayerAtBall();

		TPlayer*	GetPlayerInWay(int dir, double p_dDist);
		TPlayer*	GetFreeTeammate() {return m_pFreeTeammate;};
		TPlayer*	GetOtherGoalie();

		TPlayer*	GetOwnPlayer(int i) { return m_pOwnTeam[i]; };
		
		BOOL		GetOpponentNearBall(double dist);
		
		int		GetNoOfOpponentsInMind();
		void		GetListVisiblePlayers(CObList & pList);		//added by MP List of visible players
		CString	GetValidSeen();										//gives valid seen string added by MP
		BOOL		IsOutside(TVector& pos);
		TVector	GetAbsCoordinates(TVector &origin, double dist, int dir);

		// access map
		double	GetAccessDistance(int iRelDir);
		double	GetDirectionDistance(int iRelDir);
		void		DisplayDebug(LPCTSTR msg, int category, int level);

		double*		GetPlayerParams() { return m_playerParam; };
		double*		GetPlayerType(int i) { return m_playerType[i]; };
		BOOL		HaveAllPlayerTypes() { return m_haveAllPlayerTypes; };

	private:

		BOOL	ParseSee(LPCTSTR str);
		BOOL	ParseHear(LPCTSTR str);
		BOOL	ParseSayMessage(TParseString *msg);	//added by MP 27.05.99
		BOOL	ParseMsgPosition(TParseString *msg); //added by MP 27.05.99
		BOOL	ParseSenseBody(LPCTSTR str);
		BOOL	ParseInit(LPCTSTR str);
		BOOL	ParseObject(LPCTSTR str);
		BOOL	ParseSeeGlobal(LPCTSTR str);
		BOOL	ParseObjectGlobal(LPCTSTR str);
		BOOL	ParseOfflineCoachMessage(TParseString& msg);
		void	ParsePlayMode(CString tmp);

		void	WorldModel(LPCTSTR str);
		void	Insert(CObList &list, TVisibleObject* obj);
		void	OrderFlags(CObList* flagList);
		
		void	CheckOwnTeamForFunctionalObjects();
		void	CheckOtherTeamForFunctionalObjects();

		void	UpdatePerceptionAfterDash(int power);
		void	UpdatePerceptionAfterKick(int power, int dir);
		void	UpdatePerceptionAfterTurn(int angle);
		void	UpdatePerceptionAfterTurnNeck(int angle);
		
		BOOL		Localize(void);
		BOOL		LocalizeCassini(void);
		void		GetTwoFlags(int &iFlag1, int &iFlag2);
		void		GetTwoSidemostFlags(int &iFlag1, int &iFlag2);
		void		GlobalMap(void);
		int			CheckMarked(TPlayer* pOpp, int mark1, int mark2);
		void		MatchPlayers(void);
		void		MatchClosestPlayer(TPlayer* pViewedPlayer, int team);
		void		CheckVisualPlausibility(void);
		int		GetFreeOtherTeam(void);
		void		GetOptimalPassPoint(TVector& point, int& start, int& stop);
		BOOL		IsPlayerTired(TPlayer* player);
		void		CreateAccessMap();

		// best free teammate
		double	CalculateFreePlayer (TPlayer * pTeammate);
		TPlayer*	CalculateFreestTeammate();
		int		GetRangeFromDistance(double dist);

		// internal states
		BOOL		m_bReceivedSee;		// true, once the first see message has been seen
		BOOL		m_bWaitForLastSenseBody; // true, if app waits for sense body results (autorun)
		int			m_seeCount;
		int			m_oldSeeCount;
		int			m_oldOldSeeCount;

		int		m_iSeeTime;				// time received from the last perception
		int		m_iHearTime;
		int		m_iSenseTime;			// time received from the last sense_body
		BOOL	m_bTimeProceeding;	// true if the time is running
		int		m_iLastHearNumber;	// own player that has spoken lately
		int		m_iSeeThisCycle;		// no of see messages expected this cycle

		int		m_iPlayMode;			// mode the game is in (e.g. before_kickoff)
		int		m_iSide;					// side on which playing

		// episodical 
		int		m_iLastCatchTime;		// see time at the last catch command
		double	m_dInOffside;			// 1.0, if player knows he is in offside
		double	m_dOffsideMinus;		// decay (minus) of m_dInOffside
		BOOL		m_bKnowNotInOffside;	// true, if player knows he is not in offside
		int		m_iCheckOffsideCounter;	// counts the steps in checking offside
		BOOL		m_bGoalieCaught;		// true immediately after the goalie caught the ball

		// visible object's information
		int			m_iNoOfPlayers;			// number of currently visible players
		int			m_iNoOfTeammatesInMind;	
		int			m_iNoOfOpponentsInMind;			

		TPlayer*		m_pVisiblePlayer[MAX_PLAYERS];	// max 22 players

		TSoccerGoal* m_pOwnGoal;
		TSoccerGoal* m_pOtherGoal;
		TFlag*		m_pFlag[NO_OF_FLAGS];		// flags of the corners and midline

		TFlag*		m_pOwn11mPoint;
		TFlag*		m_pOther11mPoint;
		TLine*		m_pLine;
		TBall*		m_pBall;			// the ball

		TVector*		m_pFutureBallPosition[MAX_FUTURE_POSITIONS];

		// access map

		TAccessMap*	m_pAccessMap;
		TAccessMap*	m_pDirectionMap;

		// objects in mind

		TPlayer*		m_pOwnTeam[PLAYERS_PER_TEAM];		// players by team

		TPlayer*		m_pOtherTeam[PLAYERS_PER_TEAM];	

		// functional objects

		TPlayer*		m_pOpponentAtBall;
		TPlayer*		m_pOpponentAtMe;
		TPlayer*		m_p2OpponentAtMe;
		TPlayer*		m_p3OpponentAtMe;
		TPlayer*		m_pOpponentToMark;
		TPlayer*		m_pBackmostOpponent;
		TPlayer*		m_p2BackmostOpponent;
		TPlayer*		m_pFrontmostTeammate;
		TPlayer*		m_pBackmostTeammate;
		TPlayer*		m_p2BackmostTeammate;
		TPlayer*		m_pTeammateAtBall;
		TPlayer*		m_p2TeammateAtBall;
		TPlayer*		m_pTeammateAtMe;
		TPlayer*		m_pFreeTeammate;

		// information about the player itself

		TThisPlayer*	m_pThisPlayer;

		// sense body information
		int		m_iViewQuality;
		int		m_iViewAngle;
		int		m_iStamina;
		double	m_dEffort;
		double	m_dSpeed;
		int		m_iSpeedAngle;
		int		m_iNoOfKick;
		int		m_iNoOfDash;
		int		m_iNoOfTurn;
		int		m_iNoOfSay;
		int		m_iNoOfTurnNeck;
		int		m_iNoOfCatch;
		int		m_iNoOfMove;
		int		m_iNoOfChangeView;
		int		m_iHeadAngle;
		int		m_iNoOfCommandsNotExecuted;

		// this is where object oriented programming ended :-)
		int		m_iExpectedNoOfDash;
		int		m_iDashPower;
		int		m_iExpectedNoOfSay;
		int		m_iExpectedNoOfKick;
		int		m_iKickPower;
		int		m_iKickDirection;
		int		m_iExpectedNoOfTurn;
		int		m_iTurnAngle;
		TDirection m_tExpectedBodyDirection;
		int		m_iExpectedNoOfTurnNeck;
		int		m_iTurnAngleNeck;
		TDirection m_tExpectedViewDirection;

		double	m_dNoise;	// only needed for evaluation
		int		m_iSawAll;

		TPerceptionView*	m_pView;
		TViewingAgent*		m_pAgent;


		// server 7
		TPerceptionParser*	m_perceptionParser;
//		double				m_serverParam[SERVER_PARAM_COUNT];
		double				m_playerParam[PLAYER_PARAM_COUNT];
		double				m_playerType[NO_OF_PLAYER_TYPES][PLAYER_TYPE_COUNT];
		int					m_currentPlayerType;
		BOOL				m_haveAllPlayerTypes;
};

#endif // !defined(AFX_TPERCEPTION_H__DA8AA141_577A_11D1_B242_0000C09B5CBE__INCLUDED_)
