// TPerceptionView.cpp : implementation file
//

#include "stdafx.h"
#include "Basis.h"
#include "TScreenParams.h"
#include "TPerception.h"
#include "TDoingAgent.h"
#include "DPerceptionSlider.h"
#include "TThisPlayer.h"
#include "TPerceptionView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// TPerceptionView

IMPLEMENT_DYNCREATE(TPerceptionView, CView)

TPerceptionView::TPerceptionView()
{
	m_pScreen = new TScreenParams();
	m_iNoOfCycles = 0;
	m_iCurrentCycle = 0;
	m_bRunning = FALSE;
	m_bChanged = FALSE;
	m_bReplaying = FALSE;
	m_iTimerID = 0;
}

TPerceptionView::~TPerceptionView()
{
	delete m_pScreen;
}


BEGIN_MESSAGE_MAP(TPerceptionView, CView)
	//{{AFX_MSG_MAP(TPerceptionView)
	ON_WM_CHAR()
	ON_COMMAND(ID_ZOOM_PLUS, OnZoomPlus)
	ON_COMMAND(ID_VIEW_PLAYERS_VIEW, OnViewPlayersView)
	ON_UPDATE_COMMAND_UI(ID_VIEW_PLAYERS_VIEW, OnUpdateViewPlayersView)
	ON_COMMAND(ID_ZOOM_MINUS, OnZoomMinus)
	ON_COMMAND(ID_VIEW_ANGLE, OnViewAngle)
	ON_COMMAND(ID_VIEW_QUALITY, OnViewQuality)
	ON_UPDATE_COMMAND_UI(ID_VIEW_QUALITY, OnUpdateViewQuality)
	ON_COMMAND(ID_LOG_PERCEPT_BACK, OnLogPerceptBack)
	ON_COMMAND(ID_LOG_PERCEPT_GOTO, OnLogPerceptGoto)
	ON_COMMAND(ID_LOG_PERCEPT_ONE_STEP, OnLogPerceptOneStep)
	ON_COMMAND(ID_LOG_PERCEPT_PLAY, OnLogPerceptPlay)
	ON_UPDATE_COMMAND_UI(ID_LOG_PERCEPT_BACK, OnUpdateLogPerceptBack)
	ON_UPDATE_COMMAND_UI(ID_LOG_PERCEPT_GOTO, OnUpdateLogPerceptGoto)
	ON_UPDATE_COMMAND_UI(ID_LOG_PERCEPT_ONE_STEP, OnUpdateLogPerceptOneStep)
	ON_UPDATE_COMMAND_UI(ID_LOG_PERCEPT_PLAY, OnUpdateLogPerceptPlay)
	ON_COMMAND(ID_FILE_SAVE_PERCEPTION, OnFileSavePerception)
	ON_COMMAND(ID_FILE_OPEN_PERCEPTION, OnFileOpenPerception)
	ON_UPDATE_COMMAND_UI(ID_FILE_OPEN_PERCEPTION, OnUpdateFileOpenPerception)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE_PERCEPTION, OnUpdateFileSavePerception)
	ON_COMMAND(ID_SAVE_AS_PERCEPTION, OnFileSaveAsPerception)
	ON_UPDATE_COMMAND_UI(ID_SAVE_AS_PERCEPTION, OnUpdateFileSaveAsPerception)
	ON_COMMAND(ID_LOG_PERCEPT_STOP, OnLogPerceptStop)
	ON_UPDATE_COMMAND_UI(ID_LOG_PERCEPT_STOP, OnUpdateLogPerceptStop)
	ON_WM_TIMER()
	ON_WM_DESTROY()
	ON_COMMAND(ID_VIEW_RELATIVE_PERCEPTION, OnViewRelativePerception)
	ON_UPDATE_COMMAND_UI(ID_VIEW_RELATIVE_PERCEPTION, OnUpdateViewRelativePerception)
	ON_COMMAND(ID_VIEW_ABSOLUTE_MAP, OnViewAbsoluteMap)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ABSOLUTE_MAP, OnUpdateViewAbsoluteMap)
	ON_COMMAND(ID_LOG_PERCEPT_ONE_STEP_BACK, OnLogPerceptOneStepBack)
	ON_UPDATE_COMMAND_UI(ID_LOG_PERCEPT_ONE_STEP_BACK, OnUpdateLogPerceptOneStepBack)
	ON_COMMAND(ID_VIEW_DEGREE_OF_FREENESS, OnViewDegreeOfFreeness)
	ON_UPDATE_COMMAND_UI(ID_VIEW_DEGREE_OF_FREENESS, OnUpdateViewDegreeOfFreeness)
	ON_COMMAND(ID_VIEW_DRAW_FUTURE_BALL, OnViewDrawFutureBall)
	ON_UPDATE_COMMAND_UI(ID_VIEW_DRAW_FUTURE_BALL, OnUpdateViewDrawFutureBall)
	ON_COMMAND(ID_VIEW_ACCESSMAP, OnViewAccessmap)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ACCESSMAP, OnUpdateViewAccessmap)
	ON_COMMAND(ID_VIEW_DIRECTION_MAP, OnViewDirectionMap)
	ON_UPDATE_COMMAND_UI(ID_VIEW_DIRECTION_MAP, OnUpdateViewDirectionMap)
	ON_COMMAND(ID_VIEW_FREE_AREA, OnViewFreeArea)
	ON_UPDATE_COMMAND_UI(ID_VIEW_FREE_AREA, OnUpdateViewFreeArea)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// TPerceptionView drawing

void TPerceptionView::OnDraw(CDC* pDC)
{
	RedrawView();
}

/////////////////////////////////////////////////////////////////////////////
// TPerceptionView diagnostics

#ifdef _DEBUG
void TPerceptionView::AssertValid() const
{
	CView::AssertValid();
}

void TPerceptionView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}


#endif //_DEBUG

void TPerceptionView::NewMessage(LPCTSTR str)
{
	// check upper limit
	if (m_iNoOfCycles >= PERCEPTION_RECORDER_MAX - 1)
	{
		m_MsgList[m_iNoOfCycles].RemoveAll();
		m_iNoOfCycles--;
		m_MsgList[m_iNoOfCycles].RemoveAll();
		m_iCurrentCycle = m_iNoOfCycles;
	}

	if (m_bReplaying)
		// the info is from replaying, so do not store again
		RedrawView();
	
	else if ((str[1] == 's') && (str[3] == 'e'))
	{
		// see messages advance the list counter and are always head of the message list
		m_iNoOfCycles++;
		m_MsgList[m_iNoOfCycles].AddTail(str);
		m_iCurrentCycle = m_iNoOfCycles;
		m_bChanged = TRUE;

		RedrawView();
	}
	else
	{
		// any other message: store
		m_MsgList[m_iNoOfCycles].AddTail(str);
		m_pPerception->ParseMessage(str);
	}
}

void TPerceptionView::RedrawView()
{
	CClientDC		pDC(this);
	CDC				dcMem;
	CBitmap			myBitmap;
	CRect				windowRect;
	int				sizeX, sizeY;

	if (m_pPerception->GetAgent() == NULL)
		return;

	// first call the parse function of perception, to draw the right things
	if (!m_MsgList[m_iCurrentCycle].IsEmpty())
		m_pPerception->ParseMessage(m_MsgList[m_iCurrentCycle].GetHead());
	
	// parsing has to be done in any case, even when not drawing
	if (m_bNoGraphics)
		return;

	// prepare the memory device context
	dcMem.CreateCompatibleDC(&pDC);
	
	// get Device Coordinates
	pDC.GetWindow()->GetClientRect(&windowRect);
	windowRect.NormalizeRect();
	sizeX = windowRect.Width();
	sizeY = windowRect.Height();
	myBitmap.CreateCompatibleBitmap(&pDC, sizeX, sizeY);
	// set the size of the bitmap, to be read by the drawing functions
	myBitmap.SetBitmapDimension(sizeX, sizeY);
	
	ASSERT(myBitmap.m_hObject != NULL);

	CBitmap* pOldBitmap = dcMem.SelectObject(&myBitmap);

	// set screen parameters
	m_pScreen->SetSize(CSize(sizeX, sizeY));
	if (m_pScreen->IsTopView())
		m_pScreen->SetMid(sizeX / 2, sizeY - 30);
	else
	{
		m_pScreen->SetMid(sizeX / 2, sizeY / 2);

		// set the scale for absolute view
		if (m_pScreen->IsAbsoluteView())
		{
			if (((double) sizeX / (double) sizeY) < PITCH_LENGTH / PITCH_WIDTH)
				m_pScreen->SetAbsScale((double) sizeX / (PITCH_LENGTH + PITCH_MARGIN * 2.0));
			else
				m_pScreen->SetAbsScale((double) sizeY / (PITCH_WIDTH + PITCH_MARGIN * 2.0));
		}
	}

	// draw the things
	m_pPerception->DrawPerception(&dcMem, m_pScreen);

	// copy the memory bitmap into the window
	pDC.BitBlt(0, 0, sizeX, sizeY, &dcMem, 0, 0, SRCCOPY);

	// remove the created bitmap
	dcMem.SelectObject(pOldBitmap);
	dcMem.DeleteDC();
	myBitmap.DeleteObject();
}



void TPerceptionView::LoadFromArchive(CArchive &ar)
{
	int		i;
	int		tmp;

	ar >> tmp;
	m_pPerception->SetSide(tmp);
	ar >> tmp;
	m_pPerception->GetThisPlayer()->SetNum(tmp);

	// load the messages from the archive
	ar >> m_iNoOfCycles;
	for (i = 0; i < m_iNoOfCycles; i++)
	{
		// delete the old content of the list
		m_MsgList[i].RemoveAll();
		m_MsgList[i].Serialize(ar);
	}
	m_iCurrentCycle = 0;
	m_pPerception->Initialize();
	m_bChanged = FALSE;
}

void TPerceptionView::SaveToArchive(CArchive &ar)
{
	int		i;
	
	// write team and number of this agent
	ar << m_pPerception->GetThisPlayer()->GetTeam();
	ar << m_pPerception->GetThisPlayer()->GetNum();

	// write the messages to the archive
	ar << m_iNoOfCycles;
	for (i = 0; i < m_iNoOfCycles; i++)
		m_MsgList[i].Serialize(ar);
	m_bChanged = FALSE;
}

void TPerceptionView::SetCurrentCycle(int cycle)
{
	ASSERT((cycle >= 0) && (cycle < m_iNoOfCycles));
	m_iCurrentCycle = cycle;
	SetCurrentCycle();
}

void TPerceptionView::SetCurrentCycle()
{
	m_bReplaying = TRUE;
	m_pPerception->PlayStep(&m_MsgList[m_iCurrentCycle]);
	m_bReplaying = FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// TPerceptionView message handlers
void TPerceptionView::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// switch on the command received
	switch (nChar)
	{
		case '1':
			m_pPerception->GetAgent()->Kick(10, 0);
			break;
		case '2':
			m_pPerception->GetAgent()->Kick(20, 0);
			break;
		case '3':
			m_pPerception->GetAgent()->Kick(30, 0);
			break;
		case '4':
			m_pPerception->GetAgent()->Kick(40, 0);
			break;
		case '5':
			m_pPerception->GetAgent()->Kick(50, 0);
			break;
		case '6':
			m_pPerception->GetAgent()->Kick(60, 0);
			break;
		case '7':
			m_pPerception->GetAgent()->Kick(70, 0);
			break;
		case '8':
			m_pPerception->GetAgent()->Kick(80, 0);
			break;
		case '9':
			m_pPerception->GetAgent()->Kick(90, 0);
			break;
		case ' ':
			m_pPerception->GetAgent()->Kick(100, 0);
			break;
		case 'a':
			m_pPerception->GetAgent()->TurnNeck(-15);
			break;
		case 'd':
			m_pPerception->GetAgent()->TurnNeck(15);
			break;
		case 'j':
			m_pPerception->GetAgent()->Turn(-15);
			break;
		case 'l':
			m_pPerception->GetAgent()->Turn(15);
			break;
		case 'u':
			m_pPerception->GetAgent()->Turn(-90);
			break;
		case 'o':
			m_pPerception->GetAgent()->Turn(90);
			break;
		case 'h':
			m_pPerception->GetAgent()->Turn(180);
			break;
		case 'i':
				m_pPerception->GetAgent()->Dash(100);
			break;
		case 'k':
				m_pPerception->GetAgent()->Dash(-100);
			break;

		case VK_RETURN:
			m_pPerception->GetAgent()->Move(-5, 0);
			break;

		case 's':
			m_pPerception->GetAgent()->SenseBody();			
			break;

		case 'x':
			m_pPerception->GetAgent()->TurnNeck(-(int)m_pPerception->GetHeadAngle());
			break;

		case 'c':
			m_pPerception->GetAgent()->Catch(0);
			break;

		case 'q':
			m_pPerception->GetAgent()->SenseBody();			
			m_pPerception->GetAgent()->TurnNeck(90);
			m_pPerception->GetAgent()->Turn(-90);
			m_pPerception->GetAgent()->Say("Hallo Klausi");
			break;

		case 'w':
			m_pPerception->GetAgent()->SenseBody();			
			m_pPerception->GetAgent()->TurnNeck(-90);
			m_pPerception->GetAgent()->Turn(90);
			m_pPerception->GetAgent()->Say("Hallo Klausi");
			break;
		
		case 't':
			m_pPerception->GetAgent()->Dash(100);
			m_pPerception->GetAgent()->Dash(100, TRUE);
			m_pPerception->GetAgent()->Dash(100, TRUE);
			m_pPerception->GetAgent()->Dash(100, TRUE);
			m_pPerception->GetAgent()->Dash(100, TRUE);
			m_pPerception->GetAgent()->Turn(180, TRUE);
			break;			

		case 'z':
			m_pPerception->GetAgent()->Kick(100, 90);
			break;

		default:
			CView::OnChar(nChar, nRepCnt, nFlags);
	}
}

BOOL TPerceptionView::PreCreateWindow(CREATESTRUCT& cs) 
{
	BOOL ret = CView::PreCreateWindow(cs);
	
	cs.style = AFX_WS_DEFAULT_VIEW | ES_NOHIDESEL;

	// set the initial parameters for displaying the perceptions
	// that is midx, midy, scale
	m_pScreen->SetScale(6.0);
	return ret;
}


void TPerceptionView::OnFileOpenPerception() 
{
	CFile		file;
	CString	message;

	// ask for the file to open
	CFileDialog fileDlg(TRUE, "*.le1", NULL, OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_NOCHANGEDIR
							  , "Perception Files (*.le1)|*.le1|All Files (*.*)|*.*", this);
	if (fileDlg.DoModal() != IDOK)
		return;
	m_strFileName = fileDlg.GetPathName();

	// open the file and archive
	if(!file.Open(m_strFileName, CFile::modeRead))
		return;
	
	CArchive	archive(&file, CArchive::load);

	// and load it
	LoadFromArchive(archive);
	archive.Close();
	file.Close();
	return;
}

void TPerceptionView::OnFileSaveAsPerception() 
{
	// ask for the filename
	CFileDialog fileDlg(FALSE, "*.le1", m_strFileName, OFN_HIDEREADONLY | OFN_NOCHANGEDIR
							  , "Perception Files (*.le1)|*.le1|All Files (*.*)|*.*|", this);
	if (fileDlg.DoModal() != IDOK)
		return;
	m_strFileName = fileDlg.GetPathName();
	
	if (m_strFileName.GetLength() > 0)
		OnFileSavePerception();
}

void TPerceptionView::OnFileSavePerception() 
{
	if (m_strFileName.GetLength() == 0)
		// the file has no name
		OnFileSaveAsPerception();
	else
	{
		CFile		file;
		CString	message;

		// open the file and archive
		if(!file.Open(m_strFileName, CFile::modeCreate | CFile::modeWrite))
		{
			message.Format("Unable to open file %s for writing", m_strFileName);
			AfxMessageBox(message);
			return;
		}
		
		CArchive	archive(&file, CArchive::store);

		// and save it
		SaveToArchive(archive);
		archive.Close();
		file.Close();
	}
}

void TPerceptionView::OnLogPerceptBack() 
{
	m_iCurrentCycle = 0;
	m_pPerception->Initialize();
	SetCurrentCycle();
}

void TPerceptionView::OnLogPerceptGoto() 
{
	DPerceptionSlider	tDialog(NULL, this);

	tDialog.m_strStart.Format("%d", 0);
	tDialog.m_strStop.Format("%d", m_iNoOfCycles);
	tDialog.m_iMin = 0;
	tDialog.m_iMax = m_iNoOfCycles - 1;
	tDialog.m_iPos = m_iCurrentCycle;

	tDialog.DoModal();
}

void TPerceptionView::OnLogPerceptOneStep() 
{
	m_iCurrentCycle++;
	if (m_iCurrentCycle >= m_iNoOfCycles)
		m_iCurrentCycle = m_iNoOfCycles - 1;
	SetCurrentCycle();
}


void TPerceptionView::OnLogPerceptOneStepBack() 
{
	m_iCurrentCycle--;
	if (m_iCurrentCycle < 0)
		m_iCurrentCycle = 0;
	SetCurrentCycle();
}

void TPerceptionView::OnLogPerceptPlay() 
{
	if (m_bRunning)
		return;

	m_bRunning = TRUE;
	m_iTimerID = 1;
	m_iTimerID = SetTimer(m_iTimerID, 150, NULL);
}


void TPerceptionView::OnLogPerceptStop() 
{
	if (!m_bRunning)
		return;

	KillTimer(m_iTimerID);
	m_bRunning = FALSE;
	m_iTimerID = 0;
}

void TPerceptionView::OnTimer(UINT nIDEvent) 
{
	m_iCurrentCycle++;
	if (m_iCurrentCycle >= m_iNoOfCycles)
	{
		m_iCurrentCycle = m_iNoOfCycles - 1;
		OnLogPerceptStop();
	}
	else
		SetCurrentCycle();

	CView::OnTimer(nIDEvent);
}

void TPerceptionView::OnViewAngle() 
{
	m_pPerception->GetAgent()->SwitchViewAngle();
	RedrawView();
}

void TPerceptionView::OnViewQuality() 
{
	m_pPerception->GetAgent()->SwitchViewQuality();
	RedrawView();
}

void TPerceptionView::OnZoomMinus() 
{
	// zoom out of the players perceptions
	m_pScreen->Scale(0.75);
}

void TPerceptionView::OnZoomPlus() 
{
	// zoom in into the players perceptions
	m_pScreen->Scale(1.333);
}

void TPerceptionView::OnViewPlayersView() 
{
	// switch to top view
	m_pScreen->SetTopView();
	RedrawView();
}

void TPerceptionView::OnViewRelativePerception() 
{
	// switch to top view
	m_pScreen->SetRelativeView();
	RedrawView();
}

void TPerceptionView::OnViewAbsoluteMap() 
{
	// switch to top view
	m_pScreen->SetAbsoluteView();
	RedrawView();
}

void TPerceptionView::OnViewDegreeOfFreeness() 
{
	m_pScreen->SetDrawDegreeOfFreeness(!m_pScreen->GetDrawDegreeOfFreeness());
	RedrawView();
}

void TPerceptionView::OnViewDrawFutureBall() 
{
	m_pScreen->SetDrawFutureBall(!m_pScreen->GetDrawFutureBall());
	RedrawView();
}


void TPerceptionView::OnUpdateViewPlayersView(CCmdUI* pCmdUI) 
{
	if (m_pScreen->IsTopView())
		pCmdUI->SetRadio(TRUE);
	else
		pCmdUI->SetRadio(FALSE);
}

void TPerceptionView::OnUpdateViewRelativePerception(CCmdUI* pCmdUI) 
{
	if (m_pScreen->IsRelativeView())
		pCmdUI->SetRadio(TRUE);
	else
		pCmdUI->SetRadio(FALSE);
}

void TPerceptionView::OnUpdateViewAbsoluteMap(CCmdUI* pCmdUI) 
{
	if (m_pScreen->IsAbsoluteView())
		pCmdUI->SetRadio(TRUE);
	else
		pCmdUI->SetRadio(FALSE);
}

void TPerceptionView::OnUpdateViewQuality(CCmdUI* pCmdUI) 
{
	if (m_pPerception->GetViewQuality() == 1)
		pCmdUI->SetRadio(TRUE);
	else
		pCmdUI->SetRadio(FALSE);
}


void TPerceptionView::OnUpdateLogPerceptBack(CCmdUI* pCmdUI) 
{
	if (m_iCurrentCycle > 0)
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void TPerceptionView::OnUpdateLogPerceptGoto(CCmdUI* pCmdUI) 
{
	if (m_iNoOfCycles > 0)
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void TPerceptionView::OnUpdateLogPerceptOneStep(CCmdUI* pCmdUI) 
{
	if ((m_iNoOfCycles > 0) && (m_iCurrentCycle < m_iNoOfCycles - 1))
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void TPerceptionView::OnUpdateLogPerceptOneStepBack(CCmdUI* pCmdUI) 
{
	if ((m_iNoOfCycles > 0) && (m_iCurrentCycle > 0))
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void TPerceptionView::OnUpdateLogPerceptPlay(CCmdUI* pCmdUI) 
{
	if ((m_iNoOfCycles > 0) && !m_bRunning)
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);

	if (m_bRunning)
		pCmdUI->SetRadio(TRUE);
	else
		pCmdUI->SetRadio(FALSE);
}

void TPerceptionView::OnUpdateLogPerceptStop(CCmdUI* pCmdUI) 
{
	if (m_bRunning)
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void TPerceptionView::OnUpdateFileOpenPerception(CCmdUI* pCmdUI) 
{
}

void TPerceptionView::OnUpdateFileSavePerception(CCmdUI* pCmdUI) 
{
	if (m_bChanged)
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void TPerceptionView::OnUpdateFileSaveAsPerception(CCmdUI* pCmdUI) 
{
}

void TPerceptionView::OnUpdateViewDegreeOfFreeness(CCmdUI* pCmdUI) 
{
	if (m_pScreen->GetDrawDegreeOfFreeness())
		pCmdUI->SetCheck(TRUE);
	else
		pCmdUI->SetCheck(FALSE);
}


void TPerceptionView::OnUpdateViewDrawFutureBall(CCmdUI* pCmdUI) 
{
	if (m_pScreen->GetDrawFutureBall())
		pCmdUI->SetCheck(TRUE);
	else
		pCmdUI->SetCheck(FALSE);
}

void TPerceptionView::OnDestroy() 
{
	CView::OnDestroy();
	
	// stop the timer
	if (m_bRunning)
		OnLogPerceptStop();	
}



void TPerceptionView::OnViewAccessmap() 
{
	m_pScreen->SetDrawAccessMap(!m_pScreen->GetDrawAccessMap());
	RedrawView();
}

void TPerceptionView::OnUpdateViewAccessmap(CCmdUI* pCmdUI) 
{
	if (m_pScreen->GetDrawAccessMap())
		pCmdUI->SetCheck(TRUE);
	else
		pCmdUI->SetCheck(FALSE);
}

void TPerceptionView::OnViewDirectionMap() 
{
	m_pScreen->SetDrawDirectionMap(!m_pScreen->GetDrawDirectionMap());
	RedrawView();
}

void TPerceptionView::OnUpdateViewDirectionMap(CCmdUI* pCmdUI) 
{
	if (m_pScreen->GetDrawDirectionMap())
		pCmdUI->SetCheck(TRUE);
	else
		pCmdUI->SetCheck(FALSE);
}

void TPerceptionView::OnViewFreeArea() 
{
	m_pScreen->SetDrawFreePlayers(!m_pScreen->GetDrawFreePlayers());
	RedrawView();
}

void TPerceptionView::OnUpdateViewFreeArea(CCmdUI* pCmdUI) 
{
	if (m_pScreen->GetDrawFreePlayers())
		pCmdUI->SetCheck(TRUE);
	else
		pCmdUI->SetCheck(FALSE);
}
