// TThisPlayer.cpp: implementation of the TThisPlayer class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Basis.h"
#include "TScreenParams.h"
#include "TThisPlayer.h"

#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TThisPlayer::TThisPlayer()
{
	m_iAbsDirection = 0;
	m_iAbsBodyDirection = 0;
	m_dPushSpeed = 0.0;
	m_dMaxSpeed = PLAYER_SPEED_MAX;
	SetThisAgent(TRUE);
}

TThisPlayer::~TThisPlayer()
{

}

/*-----------------------------------------------------------------------------
Function: void TThisPlayer::SetPlayerAbsCoordinates(double x, double y, int dir, double dSpeed
														, int iViewMode, BOOL bDashingBackward)
Date    : 27.05.98
-------------------------------------------------------------------------------
Parameter: absolute x and y coordinates of the player as calculated from the current perception
			  absolute direcetion the player is viewing as calculated from cp
			  relative angle of the head to the body
			  abs-speed of the agent percieved by sense body
			  flag if the agent is dashing forward or backward
			  flag, if a dash has not been accounted for
Returnval: (void)
Description: calculates the absolute coordinates and direction of the agent
				 according to the new coordinates calculated and the old coordinates and speed
-----------------------------------------------------------------------------*/
void TThisPlayer::SetPlayerAbsCoordinates(TVector& pos, int dir, int headAngle, double dSpeed
											, int iViewMode, BOOL bDashingBackward, BOOL bAccountDash, BOOL haveTurned)
{
	if (m_tPosition.GetX() < -500.0)
	{
		// this is the first time, this function is called for this agent
		m_tPosition = pos;
		m_iAbsDirection = dir;
		m_iAbsBodyDirection = dir - headAngle;
		return;
	}

	TVector	tNewPos;

	// the player has localized before, so take old values into account
	// if the new position is plausible according to the old speed, take it
	// otherwise, take the average of old and new position
	int		deltaTime = m_iTime - m_iOldTime;
	TVector	tExpectedPos;

	m_iAbsDirection = dir;
	m_iAbsBodyDirection = m_iAbsDirection - headAngle;

	if (bAccountDash)
	{
		// take last dash into account for expected position
		CalculateAbsSpeed(dSpeed + m_dPushSpeed, bDashingBackward);
		m_dPushSpeed = 0.0;

		// and position is advanced
		tExpectedPos = m_tPosition + m_tAbsSpeed;
	}
	else
		tExpectedPos = m_tPosition + GetAbsSpeed();

	// calculate difference of expected and viewd position
	m_tDeltaPos = tExpectedPos - pos;

	if (haveTurned)
	{
		// we do not expect to have moved a lot
		// ??? speed
		tNewPos = m_tPosition * 0.8 + pos * 0.2;
	}
	else
	{
		if (m_tDeltaPos.GetLength() > 4.0)
			// possibly a move
			tNewPos = pos;
		else
			// take average of expected and new position
		//	tNewPos = tExpectedPos * 0.8 + pos * 0.2;
//			tNewPos = tExpectedPos * 0.6 + pos * 0.4;
			tNewPos = tExpectedPos * 0.4 + pos * 0.6;
		//	tNewPos = pos;
	}

	CalculateAbsSpeed(dSpeed, bDashingBackward);
	SetPosition(tNewPos);
}

/*-----------------------------------------------------------------------------
Function: void TPlayer::UpdatePlayerCoordinates(int iDataAge)
Date    : 29.06.99
-------------------------------------------------------------------------------
Parameter: sense_body timesteps, the agent has not perceived see 
			  amount of speed the agent perceived by sensebody
Returnval: (void)
Description: updates the agents position due to its speed perceived by sense_body
				 and current dashing direction
-----------------------------------------------------------------------------*/
void TThisPlayer::UpdatePlayerCoordinates(int iDataAge, double dSpeed, BOOL bDashingBackward)
{
	if (iDataAge == 0)
	{
		// player just got see command
		m_dPushSpeed = 0.0;
		return;
	}

	// take last dash into account
	CalculateAbsSpeed(dSpeed + m_dPushSpeed, bDashingBackward);
	m_dPushSpeed = 0.0;

	// and position is advanced
	SetPosition(m_tPosition + m_tAbsSpeed);
	
	// the absolute speed of the player is always the decayed speed
	CalculateAbsSpeed(dSpeed, bDashingBackward);
}

/*-----------------------------------------------------------------------------
Function: void TThisPlayer::CalculateAbsSpeed(double dSpeed, BOOL bDashingBackward)
Date    : 24.06.99
-------------------------------------------------------------------------------
Parameter: amount of speed the player has got
			  flag that is true if the player is dashing backward
Returnval: (void)
Description: calculates the absolute speed vector for this agent
-----------------------------------------------------------------------------*/
void TThisPlayer::CalculateAbsSpeed(double dSpeed, BOOL bDashingBackward)
{
	double	dirrad;
	double	dSpeedX;
	double	dSpeedY;

	if (dSpeed > m_dMaxSpeed)
		dSpeed = m_dMaxSpeed;

	dirrad = (double)m_iAbsBodyDirection * DEG2RAD;
	dSpeedX = dSpeed * cos(dirrad);
	dSpeedY = dSpeed * sin(dirrad);
	if (bDashingBackward)
		m_tAbsSpeed.SetVector(-dSpeedX, -dSpeedY);
	else
		m_tAbsSpeed.SetVector(dSpeedX, dSpeedY);
/*
	double	dirrad;
	double	dSpeedX;
	double	dSpeedY;
	TVector	newSpeed;
	double	dSpeedAbs;

	// calculate the speed we gained during this dash
	dirrad = (double)m_iAbsBodyDirection * DEG2RAD;
	dSpeedX = m_dPushSpeed * cos(dirrad);
	dSpeedY = m_dPushSpeed * sin(dirrad);
	if (bDashingBackward)
		newSpeed = TVector(-dSpeedX, -dSpeedY);
	else
		newSpeed = TVector(dSpeedX, dSpeedY);

	// add to previous speed
	newSpeed += m_tAbsSpeed;

	// and make length of vector the perceived speed
	dSpeedAbs = newSpeed.GetLength();
	if (dSpeedAbs > 0.05)
		dSpeedAbs = (dSpeed + m_dPushSpeed) / dSpeedAbs;
	else
		dSpeedAbs = dSpeed + m_dPushSpeed;
	m_tAbsSpeed = newSpeed * dSpeedAbs;
*/
}

/*-----------------------------------------------------------------------------
Function: TDirection TThisPlayer::GetRelativeFromAbsolutePlayDirection(TDirection dir)
Date    : 27.05.98
-------------------------------------------------------------------------------
Parameter: absolute direction of players playing from left to right, that is
			  -90 points to the left side of the field, where left is with respect
			  to the direction of play of this agent
Returnval: relative direction calculated from the absolute one
Description: calculates a relative direction of an absolute one taking into
				 account the absolute direction the player is facing as well as the
				 direction the player is playing to (LEFT_TO_RIGHT or RIGHT_TO_LEFT)
-----------------------------------------------------------------------------*/
TDirection TThisPlayer::GetRelativeFromAbsolutePlayDirection(TDirection dir)
{
	TDirection	direction;
	if (m_iTeam == LEFT_TO_RIGHT)
		direction = dir - m_iAbsDirection;
	else
		direction = dir - m_iAbsDirection - 180;
	return direction;
}

/*-----------------------------------------------------------------------------
Function: TDirection TThisPlayer::GetRelativeFromAbsoluteDirection(TDirection dir)
Date    : 09.03.2000
-------------------------------------------------------------------------------
Parameter: absolute direction of player
Returnval: relative direction calculated from the absolute one
Description: calculates a relative direction of an absolute 
-----------------------------------------------------------------------------*/
TDirection TThisPlayer::GetRelativeFromAbsoluteDirection(TDirection dir)
{
	TDirection	direction;
	direction = dir - m_iAbsDirection;
	return direction;
}

/*-----------------------------------------------------------------------------
Function: TDirection TThisPlayer::GetAngleTo(TVector position)
Date    : 15.07.98
-------------------------------------------------------------------------------
Parameter: coordinates to the point, which is of interesst
Returnval: relative angle of the passed object
Description: calculates the rel angle. If the object is ahead it is 0 left is - right is +
				 only makes sense, if the object has got a view direction
-----------------------------------------------------------------------------*/
TDirection TThisPlayer::GetAngleTo(TVector& position)
{
	TVector		delta;
	double		dx, dy;
	double		ratio;
	int			beta;
	TDirection	angle;

	delta = position - GetPosition();
	dx = delta.GetX();
	dy = delta.GetY();
	if ((dx > D_ZERO_N) && (dx < D_ZERO_P))
	{
		// points are on same x coordinates
		if (dy > 0.0)
			angle = 90 - m_iAbsDirection;
		else
			angle = -90 - m_iAbsDirection;
	}
	else
	{
		ratio = dy / dx;
		beta = (int) (atan(ratio) * RAD2DEG);
		if (dx > 0.0)
			angle = beta - m_iAbsDirection;
		else
			angle = 180 + beta - m_iAbsDirection;
	}

	return angle;
}

/*-----------------------------------------------------------------------------
Function: TDirection TThisPlayer::GetAngleToBody(TVector position)
Date    : 15.07.98
-------------------------------------------------------------------------------
Parameter: coordinates to the point, which is of interesst
Returnval: relative angle of the passed object
Description: calculates the rel angle. If the object is ahead it is 0 left is - right is +
				 only makes sense, if the object has got a view direction
				 uses m_iAbsBodyDirection instead of m_iAbsDirection, so it is
				 the angle with respect to the body direction not view direction
-----------------------------------------------------------------------------*/
TDirection TThisPlayer::GetAngleToBody(TVector& position)
{
	TVector		delta;
	double		dx, dy;
	double		ratio;
	int			beta;
	TDirection	angle;

	delta = position - GetPosition();
	dx = delta.GetX();
	dy = delta.GetY();
	if ((dx > D_ZERO_N) && (dx < D_ZERO_P))
	{
		// points are on same x coordinates
		if (dy > 0.0)
			angle = 90 - m_iAbsBodyDirection;
		else
			angle = -90 - m_iAbsBodyDirection;
	}
	else
	{
		ratio = dy / dx;
		beta = (int) (atan(ratio) * RAD2DEG);
		if (dx > 0.0)
			angle = beta - m_iAbsBodyDirection;
		else
			angle = 180 + beta - m_iAbsBodyDirection;
	}

	return angle;
}

/*-----------------------------------------------------------------------------
Function: BOOL TThisPlayer::CheckVisuality(TVisibleObject* object, int iMaxAngle, int time)
Date    : 03.06.98
-------------------------------------------------------------------------------
Parameter: pointer to the visible object to check
			  maximal view area (positive value) as + and - offset to dir
			  current see time
Returnval: TRUE, if the object should be seen but is not
Description: checks if the object lies withing the view cone of the agent. If
				 so, the IsInMind is set to false
-----------------------------------------------------------------------------*/
BOOL TThisPlayer::CheckVisuality(TVisibleObject* object, int iMaxAngle, int time, int side)
{
	int		dir;
	double	dist;
	double	range = VIEW_NEIGHBORHOOD - 0.1;

	// only not seen, but in mind objects are treated
	if(object->IsValid() || !object->IsInMind())
	{
		object->SetOnceNotSeen(FALSE);
		return FALSE;
	}

	// in which direction is the object
	dir = GetAngleTo(object);
	dist = GetDistanceTo(object);
	if ((dir < iMaxAngle) && (dir > -iMaxAngle)
		|| (dist < range))
	{
		// the object is within the visible area
		if (object->IsOnceNotSeen())
		{
			if (dist < range)
			{
				// the object must be somewhere, so move it into own goal direction
				if (side == LEFT_TO_RIGHT)
					object->SetAbsX(object->GetAbsX() - range);
				else
					object->SetAbsX(object->GetAbsX() + range);
			}
			else
			{
				// only after 2 see messages we can be sure
				object->SetInMind(FALSE);
				object->SetOnceNotSeen(FALSE);
				return TRUE;
			}
		}
		else
			object->SetOnceNotSeen(TRUE);
	}
	return FALSE;
}

/*-----------------------------------------------------------------------------
Function: BOOL TPlayer::CheckPreventOffside(TPlayer* other)
Date    : 31.07.98
-------------------------------------------------------------------------------
Parameter: pointer to a player of the opponent team
Returnval: TRUE, if the player passed is nearer to his own goal, or if this player
			  is in its own half which prevents this player to be in offside
Description: checks, if the other player prevents me from beeing offside
				a player can prevent me from beeing offside, if it is nearer to its
				own goal (x-coordinate). The age of the player info should not
				be too old, if the player is only few meters in front
-----------------------------------------------------------------------------*/
BOOL TThisPlayer::CheckPreventOffside(TPlayer* other)
{
	if (IsInOwnHalfOfField())
		return TRUE;
	
	// check if info is not too old
	double	oppX = other->GetAbsX();
	double	ownX = GetAbsX();
	double	deltaX = oppX - ownX;
	int		iAge = GetTime() - other->GetTime();

	if (m_iTeam == RIGHT_TO_LEFT)
	{
		// make coordinates side safe
		deltaX = -deltaX;
	}
	
	// opponent is behind me, so does not prevent me from beeing offside
	if (deltaX <= 2.0)
		return FALSE;

	// if the opponent is very close to me, it only prevents me
	// if we recently saw it
	if (deltaX * 2.0 < iAge)
		return FALSE;	
	
	return TRUE;
}


/*-----------------------------------------------------------------------------
Function: void TThisPlayer::DrawVisibleArea(CDC* pDC, TScreenParams *tScreen)
Date    : 18.12.97
-------------------------------------------------------------------------------
Parameter: pointer to the device context to draw into
			  reference to the sreen paramters (mid, scale)
Returnval: (void)
Description: draws the visible area depending on the view angle
-----------------------------------------------------------------------------*/
void TThisPlayer::DrawVisibleArea(CDC* pDC, TScreenParams *tScreen)
{
	// draw only players having been percepted this cycle
	if (!IsValid())
		return;

	if (tScreen->IsTopView())
	{
		// draw as seen from the top onto the player
		DrawTopView(pDC, tScreen);
		DrawTime(pDC, tScreen);
	}
	else if (tScreen->IsRelativeView())
	{
		// draw perception as seen from the agent
		DrawRelativeView(pDC, tScreen);
		DrawTime(pDC, tScreen);
	}
	else
	{
		DrawAbsoluteView(pDC, tScreen);
	}
}

/*-----------------------------------------------------------------------------
Function: void TThisPlayer::DrawTopView(CDC* pDC, TScreenParams *tScreen)
Date    : 21.04.98
-------------------------------------------------------------------------------
Parameter: pointer to the device context to draw to
			  pointer to the screen parameters such as scaling, translation
Returnval: (void)
Description: draws the visible area of the agent according to the view angle
-----------------------------------------------------------------------------*/
void TThisPlayer::DrawTopView(CDC* pDC, TScreenParams *tScreen)
{
	COLORREF cCol;
	CBrush	brush;
	CBrush	*oldBrush;
	CPoint	point[4];

	// view seen from the top
	pDC->FillSolidRect(0, 0, tScreen->GetSize().cx, tScreen->GetSize().cx, COLOR_FIELD);
	// color of the visible area
	SetCoordinates(tScreen);

	// draw the polygone
	cCol = COLOR_VISIBLE_AREA;
	brush.CreateSolidBrush(cCol);
	oldBrush = pDC->SelectObject(&brush);

	// draw the visible area
	point[0].x = m_iX;
	point[0].y = m_iY;
	if (tScreen->GetViewAngle() == 90)
	{
		// 90 degrees view, it is a equaly sided triangle
		point[1].x = m_iX - m_iY;
		point[1].y = 0;
		point[2].x = m_iX + m_iY;
		point[2].y = 0;
		pDC->Polygon(point, 3);
	}
	else if (tScreen->GetViewAngle() == 180)
	{
		// 180 degrees, it is a rectangle
		point[0].x = 0;
		point[0].y = 0;
		point[1].x = tScreen->GetMitteX() * 2;
		point[1].y = 0;
		point[2].x = point[1].x;
		point[2].y = m_iY;
		point[3].x = 0;
		point[3].y = m_iY;
		pDC->Polygon(point, 4);
	}
	else
	{
		// 45 degrees view
		point[1].x = m_iX - m_iY / 2;
		point[1].y = 0;
		point[2].x = m_iX + m_iY / 2;
		point[2].y = 0;
		pDC->Polygon(point, 3);
	}
	pDC->SelectObject(oldBrush);
	brush.DeleteObject();
}

/*-----------------------------------------------------------------------------
Function: void TThisPlayer::DrawRelativeView(CDC* pDC, TScreenParams *tScreen)
Date    : 21.04.98
-------------------------------------------------------------------------------
Parameter: pointer to the device context to draw to
			  pointer to the screen parameters such as scaling, translation
Returnval: (void)
Description: draws the visible area of the agent as seen
-----------------------------------------------------------------------------*/
void TThisPlayer::DrawRelativeView(CDC* pDC, TScreenParams *tScreen)
{
	COLORREF cCol;
	CBrush	brush;
	CBrush	*oldBrush;

	// view seen from players view
	pDC->FillSolidRect(0, 0, tScreen->GetSize().cx, tScreen->GetSize().cx, COLOR_FIELD);
	// draw the rectangle
	cCol = COLOR_SKY;
	brush.CreateSolidBrush(cCol);
	oldBrush = pDC->SelectObject(&brush);

	// draw the sky
	pDC->Rectangle(0, 0, tScreen->GetMitteX() * 2, tScreen->GetMitteY());

	pDC->SelectObject(oldBrush);
	brush.DeleteObject();
}

/*-----------------------------------------------------------------------------
Function: void TThisPlayer::DrawAbsoluteView(CDC* pDC, TScreenParams *tScreen)
Date    : 30.04.98
-------------------------------------------------------------------------------
Parameter: pointer to the device context
			  pointer to screen parameters (scaling, ...)
Returnval: (void)
Description: draws the field from top view
-----------------------------------------------------------------------------*/
void TThisPlayer::DrawAbsoluteView(CDC* pDC, TScreenParams *tScreen)
{
	int			x1, x2, x3, x5;
	int			y1, y2, y3, y4, y5, y6, y7;
	int			radius;
	CPen			linePen;
	CPen			goalPen;
	CPen*			oldPen;
	COLORREF		col;
	CBrush		brush;
	CBrush*		oldBrush;
	double		dScale = tScreen->GetAbsScale();
	CPoint		point[4];
	int			iAngle;
	double		dAngle;
	double		size;
	int			direction;
	
	// draw visible area
	pDC->FillSolidRect(0, 0, tScreen->GetSize().cx, tScreen->GetSize().cx, COLOR_FIELD_GRAY);
	SetCoordinates(tScreen);

	// calculte points of a triangle, that's end points are at least as far as
	// the furthest corner of the screen
	size = tScreen->GetSize().cx;
	if ((tScreen->GetTime() > 3000) && (tScreen->GetTime() <= 6000))
		direction = m_iAbsDirection + 180;
	else
		direction = m_iAbsDirection;

	iAngle = direction - tScreen->GetViewAngle() / 2;
	dAngle = (double) iAngle * DEG2RAD;
	x1 = m_iX + (int) (size * cos(dAngle));
	y1 = m_iY + (int) (size * sin(dAngle));

	iAngle = direction + tScreen->GetViewAngle() / 2;
	dAngle = (double) iAngle * DEG2RAD;
	x2 = m_iX + (int) (size * cos(dAngle));
	y2 = m_iY + (int) (size * sin(dAngle));
	
	// points of the triangle
	if (tScreen->GetViewAngle() == 180)
	{
		// 180 degrees
		dAngle = (double) m_iAbsDirection * DEG2RAD;
		x3 = m_iX + (int) (size * cos(dAngle));
		y3 = m_iY + (int) (size * sin(dAngle));
		point[0].x = x1;
		point[0].y = y1;
		point[1].x = x3;
		point[1].y = y3;
		point[2].x = x2;
		point[2].y = y2;
		point[3].x = x1;
		point[3].y = y1;
	}
	else
	{
		// 45 and 90 degrees
		point[0].x = m_iX;
		point[0].y = m_iY;
		point[1].x = x1;
		point[1].y = y1;
		point[2].x = x2;
		point[2].y = y2;
		point[3].x = m_iX;
		point[3].y = m_iY;
	}

	// color
	col = COLOR_FIELD;
	brush.CreateSolidBrush(col);
	oldBrush = pDC->SelectObject(&brush);
	
	// draw
	pDC->Polygon(point, 3);
	
	pDC->SelectObject(oldBrush);
	brush.DeleteObject();

	// draw the lines
	linePen.CreatePen(PS_SOLID, 2, COLOR_LINE);
	oldPen = pDC->SelectObject(&linePen);

	// outline
	x1 = (int) (dScale * PITCH_MARGIN);
	y1 = x1;
	x2 = (int) (dScale * (PITCH_MARGIN + PITCH_LENGTH));
	y2 = (int) (dScale * (PITCH_MARGIN + PITCH_WIDTH));
	pDC->MoveTo(x1, y1);
	pDC->LineTo(x2, y1);
	pDC->LineTo(x2, y2);
	pDC->LineTo(x1, y2);
	pDC->LineTo(x1, y1);
	
	// midline and circle
	x3 = (int) (dScale * (PITCH_MARGIN + PITCH_LENGTH / 2.0));
	y3 = (int) (dScale * (PITCH_MARGIN + PITCH_WIDTH / 2.0));
	radius = (int) (dScale * CENTER_CIRCLE_R);
	pDC->Arc(x3 - radius, y3 - radius, x3 + radius, y3 + radius, x3, y3 - radius, x3, y3 - radius);
	pDC->MoveTo(x3, y1);
	pDC->LineTo(x3, y2);
	
	// penalty area left
	// penalty circle
	x3 = (int) (dScale * (PITCH_MARGIN + PENALTY_AREA_LENGTH));
	y3 = (int) (dScale * (PITCH_MARGIN + (PITCH_WIDTH - PENALTY_AREA_WIDTH) / 2.0));
	y4 = (int) (dScale * (PITCH_MARGIN + (PITCH_WIDTH - PENALTY_AREA_WIDTH) / 2.0 + PENALTY_AREA_WIDTH));
	x5 = (int) (dScale * (PITCH_MARGIN + PENALTY_SPOT_DIST));
	y5 = (int) (dScale * (PITCH_MARGIN + PITCH_WIDTH / 2.0));
	y6 = (int) (dScale * (PITCH_MARGIN + PITCH_WIDTH / 2.0 - CIRCLE_ARC));
	y7 = (int) (dScale * (PITCH_MARGIN + PITCH_WIDTH / 2.0 + CIRCLE_ARC));
	radius = (int)(dScale * PENALTY_CIRCLE_R);
	pDC->Arc(x5 - radius, y5 - radius, x5 + radius, y5 + radius, x3, y7, x3, y6);	
	// penalty area
	pDC->MoveTo(x1, y3);
	pDC->LineTo(x3,y3);
	pDC->LineTo(x3,y4);
	pDC->LineTo(x1,y4);
	// penalty points
	pDC->MoveTo(x5, y5 - 1);
	pDC->LineTo(x5, y5 + 1);
	
	// penalty area right
	// penalty circle
	x3 = (int) (dScale * (PITCH_MARGIN + PITCH_LENGTH - PENALTY_AREA_LENGTH));
	x5 = (int) (dScale * (PITCH_MARGIN + PITCH_LENGTH - PENALTY_SPOT_DIST));
	pDC->Arc(x5 - radius, y5 - radius, x5 + radius, y5 + radius, x3, y6, x3, y7);	
	// penalty area
	pDC->MoveTo(x2, y3);
	pDC->LineTo(x3,y3);
	pDC->LineTo(x3,y4);
	pDC->LineTo(x2,y4);
	// penalty points
	pDC->MoveTo(x5, y5 - 1);
	pDC->LineTo(x5, y5 + 1);
	
	// goal area left
	x3 = (int) (dScale * (PITCH_MARGIN + GOAL_AREA_LENGTH));
	y3 = (int) (dScale * (PITCH_MARGIN + (PITCH_WIDTH - GOAL_AREA_WIDTH) / 2.0));
	y4 = (int) (dScale * (PITCH_MARGIN + (PITCH_WIDTH - GOAL_AREA_WIDTH) / 2.0 + GOAL_AREA_WIDTH));
	pDC->MoveTo(x1, y3);
	pDC->LineTo(x3, y3);
	pDC->LineTo(x3, y4);
	pDC->LineTo(x1, y4);
	// goal area right
	x3 = (int) (dScale * (PITCH_MARGIN + PITCH_LENGTH - GOAL_AREA_LENGTH));
	pDC->MoveTo(x2, y3);
	pDC->LineTo(x3, y3);
	pDC->LineTo(x3, y4);
	pDC->LineTo(x2, y4);

	// draw the goals
	goalPen.CreatePen(PS_SOLID, (int) GOAL_DEPTH, COLOR_GOAL);
	pDC->SelectObject(goalPen);
	// goal left
	y3 = (int) (dScale * (PITCH_MARGIN + (PITCH_WIDTH - GOAL_WIDTH) / 2.0));
	y4 = (int) (dScale * (PITCH_MARGIN + (PITCH_WIDTH - GOAL_WIDTH) / 2.0 + GOAL_WIDTH));
	pDC->MoveTo(x1, y3);
	pDC->LineTo(x1, y4);
	// goal right
	pDC->MoveTo(x2, y3);
	pDC->LineTo(x2, y4);

	// select the old gdi objects
	pDC->SelectObject(oldPen);
	goalPen.DeleteObject();
	linePen.DeleteObject();
}

/*-----------------------------------------------------------------------------
Function: void TThisPlayer::DrawPassArea(CDC* pDC, TScreenParams *tScreen, TPlayer* mate)
Date    : 30.04.98
-------------------------------------------------------------------------------
Parameter: pointer to the device context
			  pointer to screen parameters (scaling, ...)
			  pointer to the free player
Returnval: (void)
Description: draws the pass area of a free player
-----------------------------------------------------------------------------*/
void TThisPlayer::DrawPassArea(CDC* pDC, TScreenParams *tScreen, TPlayer* mate)
{
	int			x1, x2;
	int			y1, y2;
	COLORREF		col;
	CBrush		brush;
	CBrush*		oldBrush;
	double		dScale = tScreen->GetAbsScale();
	CPoint		point[4];
	int			iAngle;
	int			direction;
	double		dAngle;
	double		size;

	// calculte points of a triangle, that's end points are the distance of the player
	size = GetDistanceTo(mate) * dScale;
	if ((tScreen->GetTime() > 3000) && (tScreen->GetTime() <= 6000))
		direction = GetAbsDir() + 180;
	else
		direction = GetAbsDir();

	iAngle = mate->GetFreeAreaBordersLeft() + direction;
	dAngle = (double) iAngle * DEG2RAD;
	x1 = m_iX + (int) (size * cos(dAngle));
	y1 = m_iY + (int) (size * sin(dAngle));

	iAngle = mate->GetFreeAreaBordersRight() + direction;
	dAngle = (double) iAngle * DEG2RAD;
	x2 = m_iX + (int) (size * cos(dAngle));
	y2 = m_iY + (int) (size * sin(dAngle));
	
	// points of the triangle
	// 45 and 90 degrees
	point[0].x = m_iX;
	point[0].y = m_iY;
	point[1].x = x1;
	point[1].y = y1;
	point[2].x = x2;
	point[2].y = y2;
	point[3].x = m_iX;
	point[3].y = m_iY;

	// color
	col = COLOR_FREE_AREA;
	brush.CreateSolidBrush(col);
	oldBrush = pDC->SelectObject(&brush);
	
	// draw
	pDC->Polygon(point, 3);
	pDC->SelectObject(oldBrush);
	brush.DeleteObject();
}

/*-----------------------------------------------------------------------------
Function: void TThisPlayer::DrawTime(CDC *pDC, TScreenParams* tScreen)
Date    : 29.04.98
-------------------------------------------------------------------------------
Parameter: pointer to device context
			  pointer to screen parameters (scaling, ...)
Returnval: (void)
Description: draws the time, absolute coordinates and direction of the agent
-----------------------------------------------------------------------------*/
void TThisPlayer::DrawTime(CDC *pDC, TScreenParams* tScreen)
{
	int		nPointSize;
	CFont		font;
	CFont*	oldFont;
	CRect		field;
	COLORREF	oldTextColor;
	COLORREF	oldBkColor;
	int		sizeX;
	CString	time;

	nPointSize = (int) (30.0 * tScreen->GetScale());
	if (!font.CreatePointFont(nPointSize, _T("Times New Roman"), pDC))
		font.CreateStockObject(SYSTEM_FONT);
	oldFont = pDC->SelectObject(&font);

	// calculate the rectangle of the text
	sizeX = (int) tScreen->GetScale() * nPointSize / 5;
	oldTextColor = pDC->SetTextColor(0x00000000);
	oldBkColor = pDC->SetBkColor(COLOR_FIELD);

	// draw time
	field.SetRect(5, 5, sizeX, nPointSize / 5);
	time.Format("%5d", m_iTime);
	pDC->DrawText(time, field, DT_SINGLELINE);

	// draw coordinates
	field.SetRect(tScreen->GetMitteX() * 2 - sizeX, 5, tScreen->GetMitteX() * 2 - 5, nPointSize / 5);
	time.Format("(%4.1f %4.1f) %3d", m_tPosition.GetX(), m_tPosition.GetY(), (int) m_iAbsDirection);
	pDC->DrawText(time, field, DT_SINGLELINE | DT_RIGHT);

	pDC->SetTextColor(oldTextColor);
	pDC->SetBkColor(oldBkColor);
	pDC->SelectObject(oldFont);
	font.DeleteObject();
}

