// TVisibleObject.cpp: implementation of the TVisibleObject class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "Basis.h"
#include "TScreenParams.h"

// !!! just for debugging, to be erased
#include "TPerception.h"
#include "BasisView.h"

#include "TVisibleObject.h"

#include	<math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


/*-----------------------------------------------------------------------------
Function: TVisibleObject::TVisibleObject()
Date    : 07.11.97
-------------------------------------------------------------------------------
Parameter: none
Description: constructor
-----------------------------------------------------------------------------*/
TVisibleObject::TVisibleObject()
{
	m_iTime = -1000;
	m_iOldTime = m_iTime;
	m_dDistance = 0.0;
	m_iDirection = 0;
	m_dDeltaDist = 0.0;
	m_iDeltaDir = 0;
	m_bValid = FALSE;
	m_bInMind = FALSE;
	m_bOnceNotSeen = FALSE;
}

TVisibleObject::~TVisibleObject()
{
}

/*-----------------------------------------------------------------------------
Function: void GetCoordinates(int &x, int &y, TScreenParams *tScreen)
Date    : 14.11.97
-------------------------------------------------------------------------------
Parameter: reference to the coordinates to return
Returnval: (void)
Description: calculates screen coordinates from distance and direction
-----------------------------------------------------------------------------*/
void TVisibleObject::SetCoordinates(TScreenParams *tScreen)
{
	double direction = ((double) (m_iDirection - 90)) * DEG2RAD;
	// rad is x * pi / 180
	if (tScreen->IsTopView())
	{
		// the perception is drawn from a top view
		m_iX = tScreen->GetMitteX() + (int) (tScreen->GetScale() * m_dDistance * cos(direction)); 	
		m_iY = tScreen->GetMitteY() + (int) (tScreen->GetScale() * m_dDistance * sin(direction)); 	
	}
	else if (tScreen->IsRelativeView())
	{
		double x;
		double y;
		// it is drawn as seen by the player
		x = m_dDistance * cos(direction); 	
		y = - m_dDistance * sin(direction); 	

		// m_iX = x * distance of viewplane / y
		if (y > 0.01)
			m_iX = tScreen->GetMitteX() + (int) (x * VIEWPLANE_DIST * tScreen->GetScale()/ y);
		else
			m_iX = 1000;

		// m_iY = distance to view plane * height of eye / distance of object 
		if (m_dDistance > 0.01)
			m_iY = tScreen->GetMitteY() + (int) (VIEWPLANE_DIST * tScreen->GetScale()
															* EYE_HEIGHT / m_dDistance);
		else
			m_iY  = 1000;
	}
	else
	{
		// absolute perception and world model
		if ((tScreen->GetTime() > 3000) && (tScreen->GetTime() <= 6000))
		{
			// second half
			m_iX = (int) (((double) -m_tPosition.GetX() + PITCH_MARGIN + PITCH_LENGTH / 2.0)
								* tScreen->GetAbsScale());
			m_iY = (int) (((double) -m_tPosition.GetY() + PITCH_MARGIN + PITCH_WIDTH / 2.0)
								* tScreen->GetAbsScale());
		}
		else
		{
			// first half or extended time
			m_iX = (int) (((double) m_tPosition.GetX() + PITCH_MARGIN + PITCH_LENGTH / 2.0)
								* tScreen->GetAbsScale());
			m_iY = (int) (((double) m_tPosition.GetY() + PITCH_MARGIN + PITCH_WIDTH / 2.0)
								* tScreen->GetAbsScale());
		}
	}
}

/*-----------------------------------------------------------------------------
Function: void TVisibleObject::SetVisibleObject(TVisibleObject* obj)
Date    : 12.03.98
-------------------------------------------------------------------------------
Parameter: pointer to the object to be copied
Returnval: (void)
Description: copies the passed object to this
-----------------------------------------------------------------------------*/
void TVisibleObject::SetVisibleObject(TVisibleObject* obj)
{
	m_iTime = obj->GetTime();
	m_iOldTime = obj->GetOldTime();
	m_dDistance = obj->GetDistance();
	m_iDirection = obj->GetDirection();
	m_dDeltaDist = obj->GetDeltaDist();
	m_iDeltaDir = obj->GetDeltaDir();
	m_bValid = obj->IsValid();
	m_tPosition = obj->GetPosition();
	m_bInMind = obj->IsInMind();
	m_bOnceNotSeen = obj->IsOnceNotSeen();
}

/*-----------------------------------------------------------------------------
Function: void TVisibleObject::SetVisibleObject(int time, double dist, int dir)
Date    : 12.03.98
-------------------------------------------------------------------------------
Parameter: time, distance and relative direction of the object
Returnval: (void)
Description: copies the passed object to this
-----------------------------------------------------------------------------*/
void TVisibleObject::SetVisibleObject(int time, double dist, int dir)
{
	m_iOldTime = m_iTime;
	m_iTime = time;
	SetDistance(dist);
	SetDirection(dir);
	m_bValid = TRUE;
	m_bInMind = TRUE;
	m_bOnceNotSeen = FALSE;
}

/*-----------------------------------------------------------------------------
Function: void TVisibleObject::SetDistance(double dist)
Date    : 21.04.98
-------------------------------------------------------------------------------
Parameter: new relative distance of the object
Returnval: (void)
Description: calculates the change in distance, sets the new distance and then
				 sets the object as valid (seen this cycle)
-----------------------------------------------------------------------------*/
void TVisibleObject::SetDistance(double dist)
{ 
	// calculate the change in distance
	m_dDeltaDist = dist - m_dDistance;
	m_dDistance = dist; 
}

/*-----------------------------------------------------------------------------
Function: void TVisibleObject::SetDirection(TDirection dir)
Date    : 21.04.98
-------------------------------------------------------------------------------
Parameter: new relative direction of the object
Returnval: (void)
Description: calculates the change in direction and sets the new direciton
-----------------------------------------------------------------------------*/
void TVisibleObject::SetDirection(TDirection dir)
{ 
	// calculate the change in direction
	m_iDeltaDir = dir - m_iDirection;
	m_iDirection = dir; 
}

/*-----------------------------------------------------------------------------
Function: double TVisibleObject::GetDistanceTo(double x, double y) const
Date    : 02.04.98
-------------------------------------------------------------------------------
Parameter: absolute coordinates of a point to which the distance is calculated
Returnval: distance from this object to the passed point
Description: calculates the object's distances using their absolute coordinates
-----------------------------------------------------------------------------*/
double TVisibleObject::GetDistanceTo(TVector& pos)
{
	return GetPosition().GetDistanceTo(pos);
}


/*-----------------------------------------------------------------------------
Function: TDirection TVisibleObject::GetAngleTo(TVector& position, int dir)
Date    : 21.01.2000
-------------------------------------------------------------------------------
Parameter: coordinates to the point, which is of interesst
			  absolute direction to which the angle is calculated
Returnval: relative angle of the passed object
Description: calculates the rel angle. If the object is ahead it is 0 left is - right is +
-----------------------------------------------------------------------------*/
TDirection TVisibleObject::GetAngleTo(TVector& position, int dir)
{
	TVector		delta;
	double		dx, dy;
	double		ratio;
	int			beta;
	TDirection	angle;

	delta = position - GetPosition();
	dx = delta.GetX();
	dy = delta.GetY();
	if ((dx > D_ZERO_N) && (dx < D_ZERO_P))
	{
		// points are on same x coordinates
		if (dy > 0.0)
			angle = 90 - dir;
		else
			angle = -90 - dir;
	}
	else
	{
		ratio = dy / dx;
		beta = (int) (atan(ratio) * RAD2DEG);
		if (dx > 0.0)
			angle = beta - dir;
		else
			angle = 180 + beta - dir;
	}

	return angle;
}


/*-----------------------------------------------------------------------------
Function: void TVisibleObject::SetPosition(TVector& pos)
Date    : 06.07.99
-------------------------------------------------------------------------------
Parameter: the position for the object
Returnval: (void)
Description: sets the objects position and stores the previous position
-----------------------------------------------------------------------------*/
void TVisibleObject::SetPosition(TVector& pos)
{
	m_tOldOldPos = m_tOldPos;
	m_tOldPos = m_tPosition;
	m_tPosition = pos;
}

/*-----------------------------------------------------------------------------
Function: TVector TVisibleObject::GetAbsPosition(TVector& agentPos, int dir)
Date    : 20.07.99
-------------------------------------------------------------------------------
Parameter: position of the agent
			  absolute direction the agent looks to
Returnval: absolute position of this object
Description: calculates abs position by distance and direction information
-----------------------------------------------------------------------------*/
TVector TVisibleObject::GetAbsPosition(TVector& pos, int dir)
{
	double	direction = ((double)(dir + m_iDirection)) * DEG2RAD;
	TVector	tNewPos = pos + TVector(cos(direction), sin(direction)) * m_dDistance;
	return tNewPos;
}

/*-----------------------------------------------------------------------------
Function: TMoveableObject::TMoveableObject()
Date    : 26.03.98
-------------------------------------------------------------------------------
Parameter: none
Description: 
-----------------------------------------------------------------------------*/
TMoveableObject::TMoveableObject()
{
	m_dDistChng = 0.0;
	m_dDirChng = 0.0;
	Initialize();
}

void TMoveableObject::Initialize()
{
	m_iTime = 0;
	m_bOnceSeen = FALSE;
	m_bInMind = FALSE;
	m_tPosition.SetVector(-1000.0, -1000.0);
}
 

/*-----------------------------------------------------------------------------
Function: void TMoveableObject::SetMoveableObject(int time, double dist, int dir, double distchng, int dirchng)
Date    : 01.04.98
-------------------------------------------------------------------------------
Parameter: parameters of a moveable object
Returnval: (void)
Description: sets the parameters
-----------------------------------------------------------------------------*/
void TMoveableObject::SetMoveableObject(int time, double dist, int dir, double distchng, double dirchng)
{
	// set the new parameters
	SetVisibleObject(time, dist, dir);
	m_dDistChng = distchng;
	m_dDirChng = dirchng;
	m_bOnceSeen = TRUE;
}

void TMoveableObject::SetMoveableObject(TMoveableObject *obj)
{
	SetVisibleObject(obj); 
	m_bOnceSeen = TRUE;
	m_tAbsSpeed = obj->GetAbsSpeed();
//	m_tRelSpeed = obj->GetRelSpeed();
	m_dDistChng = obj->GetDistChng();
	m_dDirChng = obj ->GetDirChng();
	m_tLastSeePosition = obj->GetLastSeePosition();
}

/*-----------------------------------------------------------------------------
Function: void TMoveableObject::UpdatePosition(int iDataAge, iSeeTime)
Date    : 23.03.98
-------------------------------------------------------------------------------
Parameter: indicates, how much action cycles (100ms) have been since last perception
			  time of latest see message needed to calculate the age of perception of 
			  this object
Returnval: (void)
Description: updates the position (direction and distance) of an object
				 according to distchange, direction change in a linear way. This
				 function is called every 100 ms before an action has to be selected
-----------------------------------------------------------------------------*/
void TMoveableObject::UpdatePosition(int iDataAge, int iSeeTime)
{
	if (IsValid() && (iDataAge == 0))
		// the object is visible and has just be seen
		return;
	
	// the object is visible, but perception is slower than action, so update
	// also update invisible object's absolute coordinates
	SetPosition(m_tPosition + m_tAbsSpeed);
	m_tAbsSpeed *= GetDecay();

	CheckValid(iSeeTime);
}

void TMoveableObject::CheckValid(int iSeeTime)
{
	if (!IsValid() && (iSeeTime - m_iTime >= MAX_IMAGINE_TIME))
		m_bInMind = FALSE;
}


/*-----------------------------------------------------------------------------
Function: void TMoveableObject::SetAbsCoordinates(TMoveableObject* pAgent, int dir
													 , BOOL bTimeDifference, BOOL playerNearBall)
Date    : 26.03.98
-------------------------------------------------------------------------------
Parameter: pointer to the agent
			  absolute direction the agent is facing
			  flag indicating, if the last see has a higher time than last sense body (TRUE)
			  flag indicating, if a player is close to the ball (able to kick)
Returnval: (void)
Description: calculates the new absolute position of this object according to
				 the absolute position of the agent (x, y) and the relative distance
				 and direction in which this object is perceived
-----------------------------------------------------------------------------*/
void TMoveableObject::SetAbsCoordinates(TMoveableObject* pAgent, int dir
							, BOOL bTimeDifference, BOOL playerNearBall, TPerception* pPerception)
{
	TVector	tNewPos = GetAbsPosition(pAgent->GetPosition(), dir);
	double	distance1 = (GetPosition() - tNewPos).GetLength();
	double	distance2 = (GetLastSeePosition() - tNewPos).GetLength();
	BOOL		bWhich = TRUE;
	double	dFilter = 0.0;
/*
	// check if the new position is closer to the expected position or to the
	// latest see position
	if (distance1 < distance2 + 0.05)
	{
		// only filter if positions are rather close
		if (distance1 < 5.0)
		{
			// if no player is near the ball that may have kicked it, the
			// expected position should be better
			if (playerNearBall)
				dFilter = 0.1;
			else
				dFilter = 0.5;

			tNewPos = dFilter * m_tPosition + (1.0 - dFilter) * tNewPos;
		}
		bWhich = TRUE;
	}
	else
	{
		// ball is closer to last view position
/*		if (distance2 < 5.0)
		{
			// only filter if positions are rather close
			dFilter = 0.2;
			tNewPos = dFilter * GetLastSeePosition() + (1.0 - dFilter) * tNewPos;
		}
		bWhich = FALSE;
	}
*/

	// !!! to be erased, takes time
/*	CString	msg;
	msg.Format("SetAbsCoordinates: distance1: %4.2f distance2: %4.2f playerNearBall: %d Filter: %4.2f"
					, distance1, distance2, playerNearBall, dFilter);
	pPerception->DisplayDebug(msg, KATEGORY_PERCEPTION, DEBUG_LOWPERCEPTION);
*/
	// set the new position taking into account the difference of expected
	// and visible position of the agent. So the error of localization should
	// not be added onto the object speed
	SetPosition(tNewPos);	
	m_tOldOldPos += pAgent->GetDeltaPos();
	m_tOldPos += pAgent->GetDeltaPos();
	// m_tLastSeenPosition is ignored here, because it is usually a big error
	// anyhow, if it has to be used

	// set the relative and absolute speed
	SetSpeed(pAgent, dir, bWhich, bTimeDifference, playerNearBall, pPerception);
	// remember the last position where the object has been seen
	SetLastSeePosition();
}

/*-----------------------------------------------------------------------------
Function: TVector TMoveableObject::GetRelSpeed(TVector& pos, int dir)
Date    : 29.06.99
-------------------------------------------------------------------------------
Parameter:	absolute position of the agent
				absolute body direction of the agent
Returnval: (void)
Description: calculate relative speed values
-----------------------------------------------------------------------------*/
TVector TMoveableObject::GetRelSpeed(TVector& pos, int dir)
{
	// calculate according to the absolute direction the agent is facing
	TVector posObject = m_tPosition;
	TVector posSpeed = posObject + m_tAbsSpeed;

	// translate to agents coordinate system
	posObject -= pos;
	posObject.Rotate(dir);
	posSpeed -= pos;
	posSpeed.Rotate(dir);
	return posSpeed - posObject;
}

/*-----------------------------------------------------------------------------
Function: void TMoveableObject::SetSpeed(TMoveableObject* pAgent, int dir, BOOL bWhich
										 , BOOL bTimeDifference, BOOL bPlayerNearBall)
Date    : 26.03.98
-------------------------------------------------------------------------------
Parameter: pointer to this agent
			  absolute direction the agent is facing
			  flag indicating, if the old (expected) Position (TRUE) or lastSeePosition is used
			  flag indicating, if the last see has a higher time than last sense body (TRUE)
			  flag indicating, if a player is near the ball (only used for ball)
Returnval: (void)
Description: calculates the new absolute speed of this object according to
				 the absolute position of the agent (x, y) and the relative distance
				 and direction in which this object is perceived
-----------------------------------------------------------------------------*/
void TMoveableObject::SetSpeed(TMoveableObject* pAgent, int dir, BOOL bWhich
										 , BOOL bTimeDifference, BOOL bPlayerNearBall
										 , TPerception* pPerception)
{
	double	dMaxSpeed;
	double	dAbsSpeed;
	double	dRatio;
	CString msg;
	
	// collision detection
	if ((GetDistance() < 0.41) && (m_dDistChng < NOT_EXISTING + 1.0))
	{
		m_tAbsSpeed = TVector(0.0, 0.0);

		msg.Format("Collision distance: %4.2f distchn:%4.2f", GetDistance(), m_dDistChng);
		pPerception->DisplayDebug(msg, KATEGORY_PERCEPTION, DEBUG_LOWPERCEPTION);
		return;
	}

	// It seem to be more precise using global coordinates instead of distchng ??
	// if distchng or dirchng is not existing use absolute coordinate change
	if (!(m_dDistChng < NOT_EXISTING + 1.0) && !(m_dDirChng < NOT_EXISTING + 1.0))
	{
		// we have distance and dir change information
		SetAbsSpeedFromChange(pAgent, dir, m_dDistChng, m_dDirChng);
	}
	else
	{
		// take absolute coordinates change
		TVector	tAbsSpeed;

		if (bWhich)
		{
			// take stored previous position
			if (bTimeDifference)
			{
				// the see is newer than last sense body
				tAbsSpeed = GetPosition() - GetOldPos();
				if (IsBall())
				{
					msg.Format("calculate speed old: x:%4.2f y:%4.2f pos x:%4.2f y:%4.2f", 
						GetOldPos().GetX(), GetOldPos().GetY(), GetPosition().GetX(), GetPosition().GetY());
					pPerception->DisplayDebug(msg, KATEGORY_PERCEPTION, DEBUG_LOWPERCEPTION);
				}
			}
			else
			{
				tAbsSpeed = GetPosition() - GetOldOldPos();
				if (IsBall())
				{
					msg.Format("calculate speed oldold: x:%4.2f y:%4.2f pos x:%4.2f y:%4.2f", 
						GetOldOldPos().GetX(), GetOldOldPos().GetY(), GetPosition().GetX(), GetPosition().GetY());
					pPerception->DisplayDebug(msg, KATEGORY_PERCEPTION, DEBUG_LOWPERCEPTION);
				}
			}
		}
		else
		{
			int	deltaTime = m_iTime - m_iOldTime;
			// take last see position
			tAbsSpeed = GetPosition() - m_tLastSeePosition;
			
			if (deltaTime > 1)
				tAbsSpeed *= 1.0 / (double) deltaTime;
		}

		// filter with previous speed
		if (bPlayerNearBall)
			// accept new speed if players are close to the ball
			m_tAbsSpeed = 0.2 * m_tAbsSpeed + 0.8 * tAbsSpeed;
		else if (m_dDistance > 10.0)
			// Far ball needs more filtering
			m_tAbsSpeed = 0.8 * m_tAbsSpeed + 0.2 * tAbsSpeed;
		else
			// otherwise ballspeed does not change immediately
			m_tAbsSpeed = 0.6 * m_tAbsSpeed + 0.4 * tAbsSpeed;
	}

	// treat implausible values
	dMaxSpeed = GetSpeedMax();
	dAbsSpeed = m_tAbsSpeed.GetLength();
	if (dAbsSpeed > dMaxSpeed)
	{
		dRatio = dMaxSpeed / dAbsSpeed;
		m_tAbsSpeed *= dRatio;
	}
}

/*-----------------------------------------------------------------------------
Function: TVector TMoveableObject::SetAbsSpeedFromChange(TMoveableObject* pAgent, int dir, double dDistChng, double dDirChng)
Date    : 15.07.99
-------------------------------------------------------------------------------
Parameter: pointer to the agent
			absolute direction of the agent
			change in distance
			change in direction
Returnval: void
Description: calculates the relative speed of the object due to its change in 
			 distance and direction. 
-----------------------------------------------------------------------------*/
void TMoveableObject::SetAbsSpeedFromChange(TMoveableObject* pAgent, int dir, double dDistChng, double dDirChng)
{
	// we have distance and dir change information
	TVector tRelSpeed;
	double	dDirRad = (double) m_iDirection * PI / 180;
	double	dDirAlt = ((double) m_iDirection - dDirChng) * PI / 180;
	double	dDistAlt = m_dDistance - dDistChng;

	tRelSpeed = m_dDistance * TVector(cos(dDirRad), sin(dDirRad))
				- dDistAlt * TVector(cos(dDirAlt), sin(dDirAlt));

	// calculating absolute speed
	m_tAbsSpeed = tRelSpeed;
	m_tAbsSpeed.Rotate(-dir);
	m_tAbsSpeed += pAgent->GetAbsSpeed();
	// ??? Filter abs speed
}

/*-----------------------------------------------------------------------------
Function: void TMoveableObject::GetFuturePosition(double timeAhead, TVector& pos)
Date    : 02.04.98
-------------------------------------------------------------------------------
Parameter: no of timesteps to calculate ahead (in 100ms units)
			  pointer of the vector that gets the result
Returnval: (void)
Description: calculates the absolute position of this object due to its current 
				 position and the speed of the object
				 takes speed decay into account
-----------------------------------------------------------------------------*/
void TMoveableObject::GetFuturePosition(double timeAhead, TVector& pos)
{
	int		i;
	TVector	speed(m_tAbsSpeed);
	double	decay = GetDecay();

	pos = GetPosition();
	for (i = 0; i < timeAhead; i++)
	{
		pos += speed;
		speed *= decay;
	}
}

/*-----------------------------------------------------------------------------
Function: _
Date    : 21.04.98
-------------------------------------------------------------------------------
Parameter: _
Returnval: (void)
Description: _
-----------------------------------------------------------------------------*/
TPlayer::TPlayer()
{
	m_bThisAgent = FALSE;
	m_bMatched = FALSE;
	m_bOnlyInMind = FALSE;
	m_iHearTimePlayer = 0;
	m_iStamina = MAX_STAMINA;
	m_iFaceDir = 0;
	m_iBodyDir = 0;
	m_dPassDegree = 0.0;
	m_iFreeAreaBordersLeft = 0;
	m_iFreeAreaBordersRight = 0;
	m_bGoalie = FALSE;
}

/*-----------------------------------------------------------------------------
Function: void TPlayer::SetPlayer(int iTime, double dist, int dir, double distchng
							 , int dirchng, BOOL myteam, int team, int num)
Date    : 13.11.97
-------------------------------------------------------------------------------
Parameter: _
Returnval: (void)
Description: sets the players member variables
-----------------------------------------------------------------------------*/
void TPlayer::SetPlayer(int iTime, double dist, int dir, double distchng
							 , double dirchng, int faceDir, int bodyDir, BOOL myTeam
							 , int team, int num, BOOL bGoalie)
{
	// sets the parameters of a player
	SetMoveableObject(iTime, dist, dir, distchng, dirchng);
	m_bOwnTeam = myTeam;
	m_iTeam = team;
	m_iNum = num;
	m_iFaceDir = faceDir;
	m_iBodyDir = bodyDir;
	m_bGoalie = bGoalie;
}

/*-----------------------------------------------------------------------------
Function: void TPlayer::SetPlayer(TPlayer* player)
Date    : 12.03.98
-------------------------------------------------------------------------------
Parameter: pointer
Returnval: (void)
Description: sets the players member variables to the one passed
-----------------------------------------------------------------------------*/
void TPlayer::SetPlayer(TPlayer* player)
{
	// sets inherited params
	SetMoveableObject(player);
	
	// sets the parameters of a player
	m_bThisAgent = player->IsThisAgent();
	m_bOwnTeam = player->IsOwnTeam();
	m_iTeam = player->GetTeam();
	m_iNum = player->GetNum();
	m_iFaceDir = player->GetFaceDir();
	m_iBodyDir = player->GetBodyDir();
	m_bGoalie = player->IsGoalie();
	
	// this function is called, when a seen player is matched to a player 
	// in the global map, so set the player to be matched with a player
	// in the global map
	SetMatched();
	SetOnlyInMind(FALSE);
}

/*-----------------------------------------------------------------------------
Parameter: the current see time
Returnval: void
Description: checks if this player is still valid. goalies are valid a longer time
-----------------------------------------------------------------------------*/
void TPlayer::CheckValid(int iSeeTime)
{
	if (!IsValid())
	{
		if (IsGoalie() && (iSeeTime - m_iTime >= MAX_IMAGINE_TIME_GOALIE))
			m_bInMind = FALSE;
		else if (iSeeTime - m_iTime >= MAX_IMAGINE_TIME)
			m_bInMind = FALSE;
	}
}

/*-----------------------------------------------------------------------------
Function: void TPlayer::UpdateDirection(int angle)
Date    : 31.07.98
-------------------------------------------------------------------------------
Parameter: angle the agent has turned
Returnval: void
Description: updates the players free area information after a turn
-----------------------------------------------------------------------------*/
void TPlayer::UpdateDirection(int angle)
{
	TVisibleObject::UpdateDirection(angle);
	m_iFreeAreaBordersLeft -= angle;
	m_iFreeAreaBordersRight -= angle;
}

/*-----------------------------------------------------------------------------
Function: void TPlayer::DrawObject(CDC* pDC, TScreenParams *tScreen)
Date    : 13.11.97
-------------------------------------------------------------------------------
Parameter: pointer to the device context to draw into
Returnval: (void)
Description: draws the player as a circle (with the number in if existing)
-----------------------------------------------------------------------------*/
void TPlayer::DrawObject(CDC* pDC, TScreenParams *tScreen)
{
	COLORREF cCol;
	CBrush	brush;
	CBrush	*oldBrush;
	int		size;
	int		age;

	// draw only players having been percepted this cycle
	if (tScreen->IsAbsoluteView())
	{
		// do not draw players, that have never been seen before
		if (!GetOnceSeen())
			return;
	}
	else if (!IsValid())
		return;

	// what team is the player
	if (m_iTeam == LEFT_TO_RIGHT)
		cCol = COLOR_MYTEAM;
	else if (m_iTeam == RIGHT_TO_LEFT)
		cCol = COLOR_OTHERTEAM;
	else
		cCol = COLOR_ANY_OBJECT;

	if (tScreen->IsTopView())
		size = PLAYERSIZE;
	else if (tScreen->IsRelativeView())
	{
		if (m_dDistance > EYE_HEIGHT)
			size = (int) (PLAYERSIZE  * SCALE_DIST / m_dDistance);
		else
			size = (int) (PLAYERSIZE  * SCALE_DIST / EYE_HEIGHT);
	}
	else
	{
		// if the player is no longer in the mind of the object do not draw it
		if (!IsInMind())
			return;

		// absolute view
		age = tScreen->GetTime() - m_iTime;
		if (age < 0)
			age = 0;
		age *= 200 / MAX_IMAGINE_TIME;
		if (m_iTeam == LEFT_TO_RIGHT)
			cCol = RGB(200 - age, 200 - age, 20);
		else if (m_iTeam == RIGHT_TO_LEFT)
			cCol = RGB(200 - age, 20, 20);
		else
			cCol = COLOR_ANY_OBJECT;
		size = (int) (PLAYER_SIZE_DRAW * tScreen->GetAbsScale());
	}

	// set color
	brush.CreateSolidBrush(cCol);
	oldBrush = pDC->SelectObject(&brush);
	SetCoordinates(tScreen);

	// draw a Ellipse at the players position
	pDC->Ellipse(m_iX - size, m_iY - size, m_iX + size, m_iY + size);
	
	// draw degree of freeness
	if (tScreen->GetDrawDegreeOfFreeness())
	{
		int		nPointSize;
		CFont		font;
		CFont*	oldFont;
		CRect		field;
		COLORREF	oldTextColor;
		COLORREF	oldBkColor;
		int		sizeX;
		CString	text;

		nPointSize = (int) (20.0 * tScreen->GetAbsScale());
		if (!font.CreatePointFont(nPointSize, _T("Times New Roman"), pDC))
			font.CreateStockObject(SYSTEM_FONT);
		oldFont = pDC->SelectObject(&font);

		// calculate the rectangle of the text
		sizeX = (int) tScreen->GetScale() * nPointSize / 5;
		oldTextColor = pDC->SetTextColor(0x00000000);
		oldBkColor = pDC->SetBkColor(0xffffffff);

		// draw degree of freeness
		field.SetRect(m_iX, m_iY, m_iX + sizeX, m_iY + nPointSize / 5);
		GetObjectText(text);
		pDC->DrawText(text, field, DT_SINGLELINE);

		pDC->SetTextColor(oldTextColor);
		pDC->SetBkColor(oldBkColor);
		pDC->SelectObject(oldFont);
		font.DeleteObject();
	}

	pDC->SelectObject(oldBrush);
	brush.DeleteObject();
}

/*-----------------------------------------------------------------------------
Function: BOOL TPlayer::GetInLeftSideOfField()
Date    : 27.05.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: TRUE, if the player is on the left side of the field (seen in playdirection)
Description: _
-----------------------------------------------------------------------------*/
BOOL TPlayer::IsOnLeftSideOfField()
{
	return ((m_iTeam == LEFT_TO_RIGHT) && (GetAbsY() < 0.0))
				|| ((m_iTeam == RIGHT_TO_LEFT) && (GetAbsY() > 0.0));
}

BOOL TPlayer::IsInOwnHalfOfField()
{
	return ((m_iTeam == LEFT_TO_RIGHT) && (GetAbsX() < 1.0))
				|| ((m_iTeam == RIGHT_TO_LEFT) && (GetAbsX() > 1.0));
}

/*-----------------------------------------------------------------------------
Function: BOOL TPlayer::IsAlife(int time)
Date    : 12.07.99
-------------------------------------------------------------------------------
Parameter: current (sensebody) time
Returnval: true, if we heard something from the player within few time
Description: checks if the specified player did say something recently
				 This only works for own players
				 Players may be too far to hear and nevertheless alife, so only
				 use for players close to this agent
-----------------------------------------------------------------------------*/
BOOL TPlayer::IsAlife(int time)
{
	// true if the player did not say something 10 times
	if (IsOwnTeam() && (time - m_iHearTimePlayer < 220))
		return TRUE;
	else
		return FALSE;
}

/*-----------------------------------------------------------------------------
Function: void TPlayer::InitFreePassArea(int dir, int iSecureBoundary)
Date    : 24.04.99
From    : Markus Plewinski
-------------------------------------------------------------------------------
Parameter: relative direction of this player to the agent from global map
			secure boundary in which the ball flies safe
Returnval: (void)
Description: initialize the two bordes at max values
-----------------------------------------------------------------------------*/
void TPlayer::InitFreePassArea(int dir, int iSecureBoundary)
{
	m_iFreeAreaBordersLeft = dir - iSecureBoundary;
	m_iFreeAreaBordersRight = dir + iSecureBoundary;
}

/*-----------------------------------------------------------------------------
Function: void	UpdateFreePassArea(TPlayer *pOpponent)
Date    : 24.04.99
From    : Markus Plewinski
-------------------------------------------------------------------------------
Parameter: Pointer TPlayer who covers the player
			secure boundary angle by which it is save
Returnval: true if the opponent is hiding this player
Description: calculates the free pass area by the opponent who blocks
				its ensured that the player is blocked by that opponent
-----------------------------------------------------------------------------*/
BOOL TPlayer::UpdateFreePassArea(TDirection pDir, int iSecureBoundary)
{
	if(m_iFreeAreaBordersLeft.IsLeftOf(m_iFreeAreaBordersRight))
	{
		TDirection leftDir = pDir - iSecureBoundary;
		TDirection rightDir = pDir + iSecureBoundary;

		// does opponent hide player ?
		if (leftDir.LiesWithin(m_iFreeAreaBordersLeft, m_iFreeAreaBordersRight))
		{
			if (rightDir.LiesWithin(m_iFreeAreaBordersLeft, m_iFreeAreaBordersRight))
			{
				// the hiding area lies completely within the free pass area
				// take larger side
				TDirection delta1 = leftDir - m_iFreeAreaBordersLeft;
				TDirection delta2 = m_iFreeAreaBordersRight - rightDir;
				if (delta1 < delta2)
					// hides left side
					m_iFreeAreaBordersLeft = rightDir;
				else
					// hides right side
					m_iFreeAreaBordersRight = leftDir;
			}
			else
				// hides right side
				m_iFreeAreaBordersRight = leftDir;
		}
		else if (rightDir.LiesWithin(m_iFreeAreaBordersLeft, m_iFreeAreaBordersRight))
			// hides left side
			m_iFreeAreaBordersLeft = rightDir;
		else
			return FALSE;
		return TRUE;
	}
	return FALSE;
}

/*-----------------------------------------------------------------------------
Function: int	GetFreePassArea()
Date    : 24.04.99
From    : Markus Plewinski
-------------------------------------------------------------------------------
Parameter: none
Returnval: int
Description: gives the range of the free pass area
-----------------------------------------------------------------------------*/
int TPlayer::GetFreePassArea()
{
	if(m_iFreeAreaBordersLeft.IsLeftOf(m_iFreeAreaBordersRight))
	{
		TDirection area(m_iFreeAreaBordersRight - m_iFreeAreaBordersLeft);
		area.Abs();
		return (int) area;
	}
	else
		// should not happen that left border is right of right border
		return 0;
}

/*-----------------------------------------------------------------------------
Function: int	GetFreePassDirection()
Date    : 24.04.99
From    : Markus Plewinski
-------------------------------------------------------------------------------
Parameter: none
Returnval: int
Description: gives the angle of the free pass direction as the middle direction
				 of the free pass area
-----------------------------------------------------------------------------*/
int TPlayer::GetFreePassDirection()
{
	TDirection passDir = (m_iFreeAreaBordersLeft + (GetFreePassArea() / 2));
	return (int) passDir;
}

/*-----------------------------------------------------------------------------
Function: int	GetFreePassDirection(int goaldir)
Date    : 10.03.2000
-------------------------------------------------------------------------------
Parameter: relative (to the agent) direction of the other goal
Returnval: int
Description: gives the angle of the free pass direction. tries to play into
				 run direction of the player to the goal
-----------------------------------------------------------------------------*/
int TPlayer::GetFreePassDirection(int goaldir)
{
// unused
	
	TDirection passDir = GetFreePassDirection();
	TDirection goalDir = goaldir;
	
//	if (GetFreePassArea() < 20)
		// if the player is not very free, we do nothing special
		return passDir;
/*
	// we try to play the ball into the run direction of the teammate
	if (goalDir.IsLeftOf(passDir))
	{
		// we pass a little more to the left
		passDir = passDir - (GetFreePassArea() / 4);
	}
	else
	{
		// we pass a little more to the rightt
		passDir = passDir + (GetFreePassArea() / 4);
	}
	return (int) passDir;
*/
}


/*-----------------------------------------------------------------------------
Function: void TPlayer::GetFuturePositionActive(double timeAhead, int dashpower, int direction
												  , double effort, TVector& pos)
Date    : 24.06.99
-------------------------------------------------------------------------------
Parameter: timesteps in future to calculate the positions of this object
			  power the player is expected to dash
			  absolute direction the player dashes
			  effort of the player from sense_body
			  vector to store the result into
Returnval: (void)
Description: calculates the expected position of a player that is expected to
				 constantly dash with dashpower into current direction
-----------------------------------------------------------------------------*/
void TPlayer::GetFuturePositionActive(double timeAhead, int dashpower, int direction
												  , double effort, double dashpowerRate, TVector& pos)
{
	ASSERT(timeAhead >= 0);
	
	int		i;
	double	dAcc = dashpowerRate * effort * dashpower;

	m_tFutureSpeed = m_tAbsSpeed;
	m_tFutureAcc.SetVectorByDirection(dAcc, direction);
	m_tFuturePos = m_tPosition;
	
	for (i = 0; i < timeAhead; i++)
		GetNextFuturePosition();

	pos = m_tFuturePos;
}

void TPlayer::GetNextFuturePosition()
{
	m_tFutureSpeed += m_tFutureAcc;
	m_tFuturePos += m_tFutureSpeed;
	m_tFutureSpeed *= GetDecay();
}

/*-----------------------------------------------------------------------------
Function: double TPlayer::GetFutureDistanceActive(double timeAhead, int dashpower, int dir, double effort)
Date    : 24.06.99
-------------------------------------------------------------------------------
Parameter: timesteps in future to calculate the positions of this object
			  power the player is expected to dash
			  direction the player is expected to dash
			  effort of the player from sense_body
Returnval: distance the player can go within the specified time
Description: calculates the expected distance a player can go that is expected to
				 constantly dash with dashpower
-----------------------------------------------------------------------------*/
double TPlayer::GetFutureDistanceActive(double timeAhead, int dashpower, int dir, double effort, double dashpowerRate)
{
	ASSERT(timeAhead >= 0);
	
	int		i;
	double	dAcc = dashpowerRate * effort * dashpower;
	double	ret = 0.0;

	m_tFutureSpeed = m_tAbsSpeed;
	m_tFutureAcc.SetVectorByDirection(dAcc, dir);
	m_tFuturePos = TVector(0.0, 0.0);
	
	for (i = 0; i < timeAhead; i++)
		ret = GetNextFutureDistance();
	
	return ret;
}

double TPlayer::GetNextFutureDistance()
{
	m_tFutureSpeed += m_tFutureAcc;
	m_tFuturePos += m_tFutureSpeed;
	m_tFutureSpeed *= GetDecay();
	return m_tFuturePos.GetLength();
}

/*-----------------------------------------------------------------------------
Function: void TBall::SetBall(int time, double dist, int dir, double distchng, int dirchng)
Date    : 01.04.98
-------------------------------------------------------------------------------
Parameter: parameters of the ball
Returnval: (void)
Description: sets the balls parameters
-----------------------------------------------------------------------------*/
void TBall::SetBall(int time, double dist, int dir, double distchng, double dirchng)
{
	SetMoveableObject(time, dist, dir, distchng, dirchng);
}

/*-----------------------------------------------------------------------------
Function: void TBall::DrawObject(CDC* pDC, TScreenParams *tScreen)
Date    : 13.11.97
-------------------------------------------------------------------------------
Parameter: pointer to the device context to draw into
			  reference to the sreen paramters (mid, scale)
Returnval: (void)
Description: draws the ball as a circle 
-----------------------------------------------------------------------------*/
void TBall::DrawObject(CDC* pDC, TScreenParams *tScreen)
{
	COLORREF cCol;
	CBrush	brush;
	CBrush	*oldBrush;
	int		size;
	int		age;

	// draw only players having been percepted this cycle
	if (!tScreen->IsAbsoluteView() && !IsValid())
		return;
	if (tScreen->IsAbsoluteView() && !GetOnceSeen())
		return;

	// draw a Ellipse at the players position
	if (tScreen->IsTopView())
	{
		size = BALLSIZE;
		cCol = COLOR_BALL;
	}
	else if (tScreen->IsRelativeView())
	{
		if (m_dDistance > EYE_HEIGHT)
			size = (int) (BALLSIZE  * SCALE_DIST / m_dDistance);
		else
			size = (int) (BALLSIZE  * SCALE_DIST / EYE_HEIGHT);
		cCol = COLOR_BALL;
	}
	else
	{
		// absolute view
		if (!IsInMind())
			return;

		// color of the ball is fading when it has been seen long time ago
		age = tScreen->GetTime() - m_iTime;
		if (age < -7)
			age = -7;
		cCol = RGB(20, 20, 200 - age * 200 /MAX_IMAGINE_TIME);
		size = (int) (BALL_SIZE_DRAW * tScreen->GetAbsScale());
	}

	// draw the circle
	brush.CreateSolidBrush(cCol);
	oldBrush = pDC->SelectObject(&brush);
	SetCoordinates(tScreen);

	pDC->Ellipse(m_iX - size, m_iY - size, m_iX + size, m_iY + size);
	
	pDC->SelectObject(oldBrush);
	brush.DeleteObject();
}


TSoccerGoal::TSoccerGoal(double x, double y)
{
	SetPosition(TVector(x, y));
}

/*-----------------------------------------------------------------------------
Function: void TSoccerGoal::DrawObject(CDC* pDC, TScreenParams *tScreen)
Date    : 13.11.97
-------------------------------------------------------------------------------
Parameter: pointer to the device context to draw into
			  reference to the sreen paramters (mid, scale)
Returnval: (void)
Description: draws the goal as a black box
-----------------------------------------------------------------------------*/
void TSoccerGoal::DrawObject(CDC* pDC, TScreenParams *tScreen)
{
	COLORREF cCol;
	CBrush	brush;
	CBrush	*oldBrush;
	int		size;

	// draw only players having been percepted this cycle
	if (!IsValid())
		return;

	if (tScreen->IsAbsoluteView())
		// the goal in abs view is drawn by DrawVisibleArea()
		return;

	// color of the goal
	cCol = COLOR_GOAL;

	// draw the polygone
	brush.CreateSolidBrush(cCol);
	oldBrush = pDC->SelectObject(&brush);
	SetCoordinates(tScreen);

	// draw a rectangle
	if (tScreen->IsTopView())
		size = GOALSIZE;
	else if (tScreen->IsRelativeView())
	{
		if (m_dDistance > EYE_HEIGHT)
			size = (int) (GOALSIZE  * SCALE_DIST / m_dDistance);
		else
			size = (int) (GOALSIZE  * SCALE_DIST / EYE_HEIGHT);
	}

	pDC->Rectangle(m_iX - size - size, m_iY - size, m_iX + size + size, m_iY + size);
	
	pDC->SelectObject(oldBrush);
	brush.DeleteObject();
}

// additionally draws the distance of the goal to the player in top view
void TSoccerGoal::DrawGoal(CDC* pDC, TScreenParams *tScreen, double dist)
{
	DrawObject(pDC, tScreen);

	// to test the distance to the goals
	if (tScreen->GetDrawDegreeOfFreeness())
	{
		int		nPointSize;
		CFont		font;
		CFont*	oldFont;
		CRect		field;
		COLORREF	oldTextColor;
		COLORREF	oldBkColor;
		int		sizeX;
		CString	text;

		nPointSize = (int) (20.0 * tScreen->GetAbsScale());
		if (!font.CreatePointFont(nPointSize, _T("Times New Roman"), pDC))
			font.CreateStockObject(SYSTEM_FONT);
		oldFont = pDC->SelectObject(&font);

		// calculate the rectangle of the text
		sizeX = (int) tScreen->GetScale() * nPointSize / 5;
		oldTextColor = pDC->SetTextColor(0x00000000);
		oldBkColor = pDC->SetBkColor(0xffffffff);

		// draw distance to this agent
		field.SetRect(m_iX, m_iY, m_iX + sizeX, m_iY + nPointSize / 5);
		text.Format("%5.2f", dist);
		pDC->DrawText(text, field, DT_SINGLELINE);

		pDC->SetTextColor(oldTextColor);
		pDC->SetBkColor(oldBkColor);
		pDC->SelectObject(oldFont);
		font.DeleteObject();
	}
}

/*-----------------------------------------------------------------------------
Function: TFlag::TFlag(double x, double y)
Date    : 18.12.97
-------------------------------------------------------------------------------
Parameter: none
Description: a flag is by default a field flag
-----------------------------------------------------------------------------*/
TFlag::TFlag(double x, double y)
{
	SetPosition(TVector(x, y));
	m_iType = FIELD_FLAG;
	SetUsedForLocalization(FALSE);
}

/*-----------------------------------------------------------------------------
Function: _
Date    : 14.11.97
-------------------------------------------------------------------------------
Parameter: _
Returnval: (void)
Description: _
-----------------------------------------------------------------------------*/
void TFlag::SetFlag(int type, char tag1, char tag2)
{
	m_iType = type;
	m_cTag1 = tag1;
	m_cTag2 = tag2;
	SetUsedForLocalization(FALSE);
}

/*-----------------------------------------------------------------------------
Function: void TFlag::DrawObject(CDC* pDC, TScreenParams *tScreen)
Date    : 13.11.97
-------------------------------------------------------------------------------
Parameter: pointer to the device context to draw into
			  reference to the sreen paramters (mid, scale)
Returnval: (void)
Description: draws the flag as a triangle
-----------------------------------------------------------------------------*/
void TFlag::DrawObject(CDC* pDC, TScreenParams *tScreen)
{
	COLORREF cCol;
	CBrush	brush;
	CBrush	*oldBrush;
	CPoint	point[3];
	int		size;

	// draw only players having been percepted this cycle
	if (!IsValid())
		return;

	// color of the flag
	if (m_bUsedForLocalization)
		cCol = COLOR_FLAG_RED;
	else
		cCol = COLOR_FLAG;

	// draw the polygone
	brush.CreateSolidBrush(cCol);
	oldBrush = pDC->SelectObject(&brush);
	SetCoordinates(tScreen);

	// draw a Ellipse at the players position
	if (tScreen->IsTopView())
		size = FLAG_SIZE;
	else if (tScreen->IsRelativeView())
	{
		if (m_dDistance > EYE_HEIGHT)
			size = (int) (FLAG_SIZE  * SCALE_DIST / m_dDistance);
		else
			size = (int) (FLAG_SIZE  * SCALE_DIST / EYE_HEIGHT);
	}
	else
		// absolute view
		size = FLAG_SIZE;

	if (m_iType == FIELD_FLAG)
	{
		point[0].x = m_iX - size;
		point[0].y = m_iY - size;
		point[1].x = m_iX + size;
		point[1].y = m_iY;
		point[2].x = m_iX - size;
		point[2].y = m_iY + size;
	}
	else
	{
		point[0].x = m_iX + size;
		point[0].y = m_iY - size;
		point[1].x = m_iX - size;
		point[1].y = m_iY;
		point[2].x = m_iX + size;
		point[2].y = m_iY + size;
	}
	pDC->Polygon(point, 3);

	pDC->SelectObject(oldBrush);
	brush.DeleteObject();
}

/*-----------------------------------------------------------------------------
Function: void	TFlag::DrawLine(TFlag* pFlag, CDC *pDC, TScreenParams *tScreen)
Date    : 14.11.97
-------------------------------------------------------------------------------
Parameter: pointer to the flag to which the line is drawn
			  pointer to the device context on which should be drawn
			  pointer to the screen parameters for scaling and translation
Returnval: (void)
Description: draws a line from this flag to the passed flag, if both flags are
				 seen from this point of view
-----------------------------------------------------------------------------*/
void	TFlag::DrawLine(TFlag* pFlag, CDC *pDC, TScreenParams *tScreen)
{
	CPen	pen;
	CPen* oldPen;
	COLORREF cCol = COLOR_LINE;

	// test if the two flags are valid
	if (!IsValid() || !pFlag->IsValid())
		return;

	// prepare the pen
	pen.CreatePen(PS_SOLID, 1, cCol);
	oldPen = pDC->SelectObject(&pen);

	// get the proper coordinates
	pDC->MoveTo(m_iX, m_iY);
	pDC->LineTo(pFlag->GetX(), pFlag->GetY());

	pDC->SelectObject(oldPen);
	pen.DeleteObject();
}

void TLine::SetLine(int iTime, double dDistance, int iDirection, char tag)
{
	SetVisibleObject(iTime, dDistance, iDirection);
	m_cTag = tag;
}

/*-----------------------------------------------------------------------------
Function: void TLine::DrawObject(CDC* pDC, TScreenParams *tScreen)
Date    : 18.12.97
-------------------------------------------------------------------------------
Parameter: pointer to the device context to draw into
			  reference to the sreen paramters (mid, scale)
Returnval: (void)
Description: draws the flag as a triangle
-----------------------------------------------------------------------------*/
void TLine::DrawObject(CDC* pDC, TScreenParams *tScreen)
{
	COLORREF cCol;
	CBrush	brush;
	CBrush	*oldBrush;
	CPoint	point[4];

	// draw only players having been percepted this cycle
	if (!IsValid())
		return;

	// color of the ball
	cCol = COLOR_LINE;

	// draw the polygone
	brush.CreateSolidBrush(cCol);
	oldBrush = pDC->SelectObject(&brush);
	SetCoordinates(tScreen);

	// draw a raute at the lines position
	point[0].x = m_iX;
	point[0].y = m_iY - FLAG_SIZE;
	point[1].x = m_iX + FLAG_SIZE;
	point[1].y = m_iY;
	point[2].x = m_iX;
	point[2].y = m_iY + FLAG_SIZE;
	point[3].x = m_iX - FLAG_SIZE;
	point[3].y = m_iY;
	pDC->Polygon(point, 4);

	pDC->SelectObject(oldBrush);
	brush.DeleteObject();
}
