// TVisibleObject.h: interface for the TVisibleObject class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TVISIBLEOBJECT_H__DA8AA142_577A_11D1_B242_0000C09B5CBE__INCLUDED_)
#define AFX_TVISIBLEOBJECT_H__DA8AA142_577A_11D1_B242_0000C09B5CBE__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "TVector.h"
#include "TDirection.h"

#define GOALSIZE			10
#define VIEWPLANE_DIST	30.0
#define EYE_HEIGHT		2.0
#define SCALE_DIST		50.0

#define MAX_IMAGINE_TIME		50		// time, a player can trace an invisible object in mind
#define MAX_IMAGINE_TIME_GOALIE	500		// time, a player can trace an invisible goalie in mind

class TScreenParams;
class TPerception;

/*-----------------------------------------------------------------------------
Class   : TVisibleObject
Date    : 11.11.97
-------------------------------------------------------------------------------
Base Class: CObject

Description: class containing information of visible objects of an agent
-----------------------------------------------------------------------------*/

class TVisibleObject : public CObject  
{
	public:
		TVisibleObject();
		virtual ~TVisibleObject();

		virtual void GetObjectText(CString& text) { text = ""; };
		virtual void DrawObject(CDC* pDC, TScreenParams *tScreen) {};
		void		SetCoordinates(TScreenParams *tScreen);
		void		SetVisibleObject(TVisibleObject* obj);
		void		SetVisibleObject(int time, double dist, int dir);
		void		SetTime(int time) { m_iOldTime = m_iTime; m_iTime = time; };

		// get and set functions
		void		SetValid(BOOL val = TRUE) { m_bValid = val; };
		void		SetInMind(BOOL val = TRUE) { m_bInMind = val; };
		void		SetAbsX(double x) { m_tPosition.SetX(x); };
		void		SetAbsY(double y) { m_tPosition.SetY(y); };
		void		SetPosition(TVector& pos);
		void		SetLastSeePosition() { m_tLastSeePosition = GetPosition(); };
		TVector&	GetLastSeePosition() { return m_tLastSeePosition; };
		TVector	GetAbsPosition(TVector& pos, int dir);
		TVector	GetDeltaPos() { return m_tDeltaPos; };

		int			GetTime(void) const { return m_iTime; };
		int			GetOldTime(void) const { return m_iOldTime; };
		double		GetDistance(void) const { return m_dDistance; };
		TDirection&	GetDirection(void) { return m_iDirection; };
		double		GetDeltaDist(void) const { return m_dDeltaDist; };
		TDirection&	GetDeltaDir(void) { return m_iDeltaDir; };
		int			GetAge(int time) const { return (time - m_iTime); };
		BOOL			IsValid(void) const { return m_bValid; };
		BOOL			IsInMind(void) const { return m_bInMind; };
		BOOL			IsOnceNotSeen(void) const { return m_bOnceNotSeen; };
		void			SetOnceNotSeen(BOOL val = TRUE) { m_bOnceNotSeen = val; };
		int			GetX() const { return m_iX; };
		int			GetY() const { return m_iY; };
		double		GetAbsX() const { return m_tPosition.GetX(); };
		double		GetAbsY() const { return m_tPosition.GetY(); };
		TVector		GetPosition() { return m_tPosition; };
		TVector		GetOldPos() { return m_tOldPos; };
		TVector		GetOldOldPos() { return m_tOldOldPos; };
		double		GetDistanceTo(TVisibleObject* obj) { return GetDistanceTo(obj->GetPosition()); }; 
		double		GetDistanceTo(TVector& pos); 
		TDirection	GetAngleTo(TVector& position, int direction);
		virtual void	UpdateDirection(int angle) { m_iDirection -= angle; };

	protected:

		void		SetDistance(double dist);
		void		SetDirection(TDirection dir);

		int			m_iTime;			// time, when this perception has been made
		int			m_iOldTime;		// time of previous perception
		double		m_dDistance;	// rel. distance of the object to the agent

		TDirection	m_iDirection;	// angle with respect to the agents position
		double		m_dDeltaDist;	// change in distance (caused by action of the agent)

		TDirection	m_iDeltaDir;	// change in direction
		BOOL			m_bValid;		// true, if this object has been percepted this cycle
		BOOL			m_bInMind;		// true, if player has been seen recently (< MAX_IMAGINE_TIME)
		BOOL			m_bOnceNotSeen; // true, if this object is no more seen the first cycle

		int			m_iX;				// coordinates for displaying
		int			m_iY;
		TVector		m_tPosition;
		TVector		m_tOldPos;
		TVector		m_tOldOldPos;
		TVector		m_tLastSeePosition;	// position where the object has been seen lately

		TVector		m_tDeltaPos;		// difference of expected and viewd position
};

/*-----------------------------------------------------------------------------
Class   : TMoveableObject
Date    : 13.11.97
-------------------------------------------------------------------------------
Base Class: TVisibleObject

Description: class containing information of movable visible objects of an agent
-----------------------------------------------------------------------------*/

class TMoveableObject : public TVisibleObject
{
	public:
		TMoveableObject();
		TMoveableObject(TMoveableObject& obj) { SetMoveableObject(&obj); };
		virtual ~TMoveableObject() {};

		void		Initialize(void);
		void		UpdatePosition(int iDataAge, int iSeeTime);
		void		GetFuturePosition(double timeAhead, TVector& pos);
		virtual double	GetDecay() const { return 1.0; };
		virtual double	GetSpeedMax() const { return 3.0; };
		virtual void	CheckValid(int iSeeTime);

		virtual void	DrawObject(CDC* pDC, TScreenParams *tScreen) {};
		virtual BOOL	IsBall() { return FALSE; };
		
		void		SetMoveableObject(TMoveableObject* obj); 
		void		SetMoveableObject(int time, double dist, int dir, double distchng, double dirchng);

		void		SetAbsCoordinates(TMoveableObject* pAgent, int dir, BOOL bTimeDifference
											, BOOL playerNearBall, TPerception* pPerception);
		void		SetSpeed(TMoveableObject* pAgent, int dir, BOOL bWhich, BOOL bTimeDifference, BOOL playerNearBall, TPerception* pPerception);
//		void		SetRelativeFromAbsoluteSpeed(TVector& pos, int dir);
		void		SetAbsSpeed(TVector& speed) { m_tAbsSpeed = speed; };
		void		SetAbsSpeedFromChange(TMoveableObject* pAgent, int dir, double dDistChng, double dDirChng);

		double	GetDistChng(void) const { return m_dDistChng; };
		double	GetDirChng(void) { return m_dDirChng; };
		TVector	GetRelSpeed(TVector& pos, int dir);
		TVector&	GetAbsSpeed(void) { return m_tAbsSpeed; };
		BOOL		GetOnceSeen(void) const { return m_bOnceSeen; };
		void		SetOnceSeen(BOOL seen = TRUE) { m_bOnceSeen = seen; };
		void		operator =(TMoveableObject &obj) { SetMoveableObject(&obj); };

	protected:
		
		void		SetDistChng(double distchng) { m_dDistChng = distchng; };
		void		SetDirChng(TDirection vdir) { m_dDistChng = vdir; };

		double		m_dDistChng;	// change of distance (perceived from server)
		double		m_dDirChng;		// change of direction (perceived from server)
//		TVector		m_tRelSpeed;

		TVector		m_tAbsSpeed;
		BOOL			m_bOnceSeen;	// true, if the player is in the game
};

/*-----------------------------------------------------------------------------
Class   : TPlayer
Date    : 13.11.97
-------------------------------------------------------------------------------
Base Class: TMoveableObject

Description: class containing information of movable visible objects of an agent
-----------------------------------------------------------------------------*/

class TPlayer : public TMoveableObject
{
	public:
		TPlayer();
		virtual ~TPlayer() {};

		void		GetFuturePositionActive(double timeAhead, int dashpower, int direction
													, double effort, double dashpowerRate, TVector& pos);
		double	GetFutureDistanceActive(double timeAhead, int dashpower, int direction
													, double effort, double dashpowerRate);
		void		GetNextFuturePosition();
		double	GetNextFutureDistance();

		// ok to leave PLAYER_DECAY here
		virtual double	GetDecay() const { return PLAYER_DECAY; };
		// ok to leave PLAYER_SPEED_MAX here
		virtual double	GetSpeedMax() const { return PLAYER_SPEED_MAX; };
		virtual void	DrawObject(CDC* pDC, TScreenParams *tScreen);
//		virtual void	GetObjectText(CString& text) { text.Format("%d", GetFreePassDirection()); };
//		virtual void	GetObjectText(CString& text) { text.Format("%d", m_iStamina); };
		virtual void	GetObjectText(CString& text) { text.Format("%4.2f", m_dPassDegree); };
//		virtual void	GetObjectText(CString& text) { text.Format("%d", m_iNum); };
//		virtual void	GetObjectText(CString& text) { text.Format("%d", GetTime()); };

		virtual void	UpdateDirection(int angle);
		virtual void	CheckValid(int iSeeTime);

		// get and set functions
		void	SetPlayer(int iTime, double dist, int dir, double distchng
							 , double dirchng, int faceDir, int bodyDir, BOOL myTeam
							 , int team, int num, BOOL bGoalie);
		void	SetPlayer(TPlayer* player);
		void	SetTeam(BOOL myTeam, int team) { m_bOwnTeam = myTeam; m_iTeam = team; };
		void	SetNum(int num) { m_iNum = num; };
		void	SetThisAgent(BOOL agent = TRUE) { m_bThisAgent = agent; };
		void	SetMatched(BOOL matched = TRUE) { m_bMatched = matched; };
		void	SetOnlyInMind(BOOL val = TRUE) { m_bOnlyInMind = val; };

		BOOL	IsThisAgent(void) const { return m_bThisAgent; };
		BOOL	IsOwnTeam(void) const { return m_bOwnTeam; };
		BOOL	IsMatched(void) const { return m_bMatched; };
		BOOL	IsOnlyInMind(void) const { return m_bOnlyInMind; };
		int	GetTeam(void) const { return m_iTeam; };
		int	GetNum(void) const { return m_iNum; };
		BOOL	IsGoalie(void) const { return m_bGoalie; };
		BOOL	IsOnLeftSideOfField();
		BOOL	IsInOwnHalfOfField();
		BOOL	IsAlife(int time);
		BOOL	IsTired(void) const { if (m_iStamina < LOW_STAMINA) return FALSE; else return TRUE; };

		BOOL	UpdateFreePassArea(TDirection pDir, int iSecureBoundary); //added by MP 06.05.99
		int	GetFreePassArea(); // added by MP 24.04.99
		int	GetFreePassDirection(); // added by MP 24.04.99
		int	GetFreePassDirection(int goaldir); 
		void	SetFreePassDegree(double dQuality){	m_dPassDegree = dQuality; }; // added by MP 24.04.99
		double GetFreePassDegree() const { return m_dPassDegree; }; 
		void	InitFreePassArea(int dir, int iSecureBorder); //added by MP 24.04.99
		TDirection& GetFreeAreaBordersLeft() { return m_iFreeAreaBordersLeft; };
		TDirection& GetFreeAreaBordersRight() { return m_iFreeAreaBordersRight; };

		void		operator =(TMoveableObject &obj) { SetMoveableObject(&obj); };

		void			SetHearTimePlayer(int time) { m_iHearTimePlayer = time; };
		int			GetHearTimePlayer() const { return m_iHearTimePlayer; };
		void			SetStamina(int val) { m_iStamina = val; }; //added by MP 27.05.99
		int			GetStamina() { return m_iStamina; }; //added by MP 27.05.99
		void			SetAbsDirection(TDirection dir) { m_iAbsDirection = dir; };
		void			SetAbsBodyDirection(TDirection dir) { m_iAbsBodyDirection = dir; };
		TDirection	GetAbsDir() { return m_iAbsDirection; };
		TDirection	GetAbsBodyDir() { return m_iAbsBodyDirection; };
		int			GetBodyDir() const { return m_iBodyDir; };
		int			GetFaceDir() const { return m_iFaceDir; };


	protected:
		BOOL	m_bThisAgent;				// True, if this player is the agent itself
		BOOL	m_bOwnTeam;					// True, if this player is in the team of the agent
		BOOL	m_bMatched;					// True, if the player has been maped
		BOOL	m_bOnlyInMind;				// True, if the player is not seen
		int	m_iTeam;						// RIGHT_TO_LEFT or LEFT_TO_RIGHT
		int	m_iNum;						// number of player
		BOOL	m_bGoalie;					// true if it is goalie
		int	m_iFaceDir;					// direction the player is looking
		int	m_iBodyDir;					// direction the player is running
		double m_dPassDegree;			// quality of freeness - added by MP 24.04.99

		TDirection m_iFreeAreaBordersLeft; //added by MP 24.04.99

		TDirection m_iFreeAreaBordersRight; //added by MP 24.04.99

		int			m_iHearTimePlayer;
		int			m_iStamina;				//added by MP 27.05.99

		TDirection	m_iAbsDirection;		// absoulte view direction

		TDirection	m_iAbsBodyDirection;	// running direction of player


	private:
		// variables for calculation of future positions

		TVector	m_tFutureSpeed;
		TVector	m_tFuturePos;
		TVector	m_tFutureAcc;
};

/*-----------------------------------------------------------------------------
Class   : TBall
Date    : 13.11.97
-------------------------------------------------------------------------------
Base Class: TMoveableObject

Description: class containing information of movable visible objects of an agent
-----------------------------------------------------------------------------*/

class TBall : public TMoveableObject
{
	public:
		TBall() {};
		virtual ~TBall() {};
		
		virtual double	GetDecay() const { return BALL_DECAY; };
		virtual double	GetSpeedMax() const { return BALL_SPEED_MAX; };
		virtual void	DrawObject(CDC* pDC, TScreenParams *tScreen);
		void			SetBall(int time, double dist, int dir, double distchng, double dirchng);
		void			operator =(TMoveableObject &obj) { SetMoveableObject(&obj); };
		virtual BOOL	IsBall() { return TRUE; };
};

/*-----------------------------------------------------------------------------
Class   : TSoccerGoal
Date    : 13.11.97
-------------------------------------------------------------------------------
Base Class: TVisibleObject

Description: class containing information of movable visible objects of an agent
-----------------------------------------------------------------------------*/

class TSoccerGoal : public TVisibleObject
{
	public:
		TSoccerGoal(double x, double y);
		virtual ~TSoccerGoal() {};

		virtual void	DrawObject(CDC* pDC, TScreenParams *tScreen);
		void				DrawGoal(CDC* pDC, TScreenParams *tScreen, double dist);
};

/*-----------------------------------------------------------------------------
Class   : TFlag
Date    : 13.11.97
-------------------------------------------------------------------------------
Base Class: TVisibleObject

Description: class containing information of movable visible objects of an agent
-----------------------------------------------------------------------------*/

class TFlag : public TVisibleObject
{
	public:
		TFlag() { SetPosition(TVector(0.0, 0.0)); };
		TFlag(double x, double y);
		virtual ~TFlag() {};

		virtual void DrawObject(CDC* pDC, TScreenParams *tScreen);
		void	DrawLine(TFlag* pFlag, CDC *pDC, TScreenParams *tScreen);
		void	SetFlag(int type, char tag1, char tag2);
		void	SetUsedForLocalization(BOOL bValue = TRUE) { m_bUsedForLocalization = bValue; };

	private:
		int		m_iType;		// penalty area or play area
		char		m_cTag1;		// char indicating which one
		char		m_cTag2;		// char indicating which one
		BOOL		m_bUsedForLocalization;
};

/*-----------------------------------------------------------------------------
Class   : TLine
Date    : 13.11.97
-------------------------------------------------------------------------------
Base Class: TVisibleObject

Description: class containing information of movable visible objects of an agent
-----------------------------------------------------------------------------*/

class TLine : public TVisibleObject
{
	public:
		TLine() {};
		virtual ~TLine() {};

		virtual void DrawObject(CDC* pDC, TScreenParams *tScreen);
		
		void SetLine(int iTime, double dDistance, int iDirection, char tag);
		char GetTag(void) const { return m_cTag; };

	private:
		char	m_cTag;
};

#endif // !defined(AFX_TVISIBLEOBJECT_H__DA8AA142_577A_11D1_B242_0000C09B5CBE__INCLUDED_)
